<?xml version="1.0" encoding="utf-8"?><!-- -*- coding: utf-8;-*- -->
<xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:svg="http://www.w3.org/2000/svg"
>
<xsl:output method = "xml" indent = "yes" />

<xsl:param name='base_path' select=''/>
<!--   this can be: none, new, edit_choose, edit, response -->
<xsl:param name='form_type' select='new'/>
<xsl:param name='accept_name' select='Accept'/>
<xsl:param name='edit_name' select='Edit'/>
<xsl:param name='edit_choose_name' select=''/>
<xsl:param name='edit_choose_value' select=''/>
<xsl:param name='reset_name' select='Reset'/>
<xsl:param name='page' select=''/>

<xsl:template name='att'>
<xsl:for-each select="@*">
  <xsl:attribute name="{translate(name(),'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
		                         'abcdefghijklmnopqrstuvwxyz')}">
  <xsl:value-of select="." /> 
  </xsl:attribute>
</xsl:for-each>
<xsl:apply-templates/>
</xsl:template>

<!-- The Identity Transformation -->
  <!-- Whenever you match any node or any attribute -->
  <xsl:template match="node()|@*">
    <!-- Copy the current node -->
    <xsl:copy>
      <!-- Including any attributes it has and any child nodes -->
      <xsl:apply-templates select="@*|node()"/>
    </xsl:copy>
  </xsl:template>

<xsl:template match="@*">
    <!-- Create the same element with a lowercase name -->
    <xsl:attribute name="{translate(name(),'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
		                         'abcdefghijklmnopqrstuvwxyz')}">
      <!-- Including any attributes it has and any child nodes -->
      <xsl:apply-templates select="@*|node()"/>
	<xsl:value-of select="."/>
    </xsl:attribute>
  </xsl:template>

  <!-- When you match any element -->
  <xsl:template match="*">
    <!-- Create the same element with a lowercase name -->
    <xsl:element name="{translate(local-name(),'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
		                         'abcdefghijklmnopqrstuvwxyz')}">
      <!-- Including any attributes it has and any child nodes -->
      <xsl:apply-templates select="@*|node()"/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="/*">
    <div>
      <xsl:if test="not((//text())[1]/parent::emphasis[@role='header' or @role='strong']) and not(//formulae)">
	<h3>
	  <xsl:value-of select="title"/>
	</h3>
      </xsl:if>
      <xsl:call-template name='att'/>
    </div>
  </xsl:template>
  
<xsl:template match="lognoter[formulae/@only_form_view_mode='1']">
    <div>
      <xsl:apply-templates select="formulae"/>
    </div>
</xsl:template>

<xsl:template match="section">
    <h1>
      <xsl:value-of select="title"/>
    </h1>
    <xsl:call-template name='att'/>
</xsl:template>

<xsl:template match="itemizedlist">
    <ul>
      <xsl:call-template name='att'/>
    </ul>
</xsl:template>
<xsl:template match="listitem">
    <li>
      <xsl:call-template name='att'/>
    </li>
</xsl:template>
  
  <xsl:template match="row">
    <tr>
      <xsl:call-template name='att'/>
    </tr>
  </xsl:template>

  <xsl:template name='lindex'>
    <xsl:param name="list" />
    <xsl:param name="position" />
    <xsl:choose>
      <xsl:when test = "$position &gt; 0">
	<xsl:call-template name="lindex">
	  <xsl:with-param name="list" select="substring-after($list,' ')"/>
	  <xsl:with-param name="position" select="-1+$position"/>
	</xsl:call-template>
      </xsl:when>
      <xsl:when test = "contains($list,' ')">
	<xsl:value-of select="substring-before($list,' ')"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:value-of select="$list"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="entry">
    <td>
      <xsl:attribute name="width">
	<xsl:call-template name="lindex">
	  <xsl:with-param name="list" select="ancestor::table/@relativesizes"/>
	  <xsl:with-param name="position" select="position()"/>
	</xsl:call-template>
	<xsl:text>%</xsl:text>
      </xsl:attribute>
      <xsl:apply-templates>
	<xsl:with-param  name="position" select="position()"/>
      </xsl:apply-templates> 
    </td>
  </xsl:template>
  
  <xsl:template match="para[emphasis[@role='header']]">
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="para">
    <xsl:param name="position"/>
    <p>
      <xsl:attribute name="class">
	<xsl:choose>
	  <xsl:when test = "parent::entry/@cols_justify">
	    <xsl:call-template name="lindex">
	      <xsl:with-param name="list" select="parent::entry/@cols_justify"/>
	      <xsl:with-param name="position" select="0"/>
	    </xsl:call-template>
	  </xsl:when>
	  <xsl:when test = "parent::entry/parent::row/@cols_justify">
	    <xsl:call-template name="lindex">
	      <xsl:with-param name="list" select="parent::entry/parent::row/@cols_justify"/>
	      <xsl:with-param name="position" select="-1+$position"/>
	    </xsl:call-template>
	  </xsl:when>
	  <xsl:when test = "parent::entry/parent::row/parent::table/@cols_justify">
	    <xsl:call-template name="lindex">
	      <xsl:with-param name="list" select="parent::entry/parent::row/parent::table/@cols_justify"/>
	      <xsl:with-param name="position" select="-1+$position"/>
	    </xsl:call-template>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:text>justify</xsl:text>
	  </xsl:otherwise>
	</xsl:choose>        
      </xsl:attribute>
      <xsl:attribute name="style">
	<xsl:choose>
	  <xsl:when test = ".//mediaobject/imageobject/imagedata/@align">
	    <xsl:text>text-align:</xsl:text>
	    <xsl:value-of select=".//mediaobject/imageobject/imagedata/@align"/>
	  </xsl:when>
	</xsl:choose>
      </xsl:attribute>
      <xsl:if test="0 and string(.)='' and not(.//img)">
	<xsl:text>&#160;</xsl:text>
      </xsl:if>
      <xsl:call-template name='att'/>
    </p>
  </xsl:template>

  <xsl:template match="emphasis">
    <xsl:choose>
      <xsl:when test = "@role='header'">
	<h3>
	  <xsl:call-template name='att'/>
	</h3>
      </xsl:when>
      <xsl:when test = "@role='strong'">
	<b>
	  <xsl:call-template name='att'/>
	</b>
      </xsl:when>
      <xsl:when test = "@role='warning'">
	<font color="red">
	  <xsl:call-template name='att'/>
	</font>
      </xsl:when>
      <xsl:otherwise>
	<i>
	  <xsl:call-template name='att'/>
	</i>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="sub|subscript">
    <sub>
      <xsl:call-template name='att'/>
    </sub>
  </xsl:template>
    
  <xsl:template match="sup|superscript">
    <sup>
      <xsl:call-template name='att'/>
    </sup>
  </xsl:template>

  <xsl:template name='url-parameter'>
    <xsl:param name="url" />
    <xsl:param name="param-name"/>
    <xsl:variable name='tmp' select="substring-after($url,concat($param-name,'='))"/>
    <xsl:choose>
      <xsl:when test="substring-before($tmp,'&amp;') != ''">
	<xsl:value-of select="substring-before($tmp,'&amp;')"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:value-of select="$tmp"/>
	</xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name='url-add-parameter'>
    <xsl:param name="url" />
    <xsl:param name="param-name"/>
    <xsl:param name="param-value"/>
    <xsl:choose>
      <xsl:when test="not(contains($url,'?'))">
	<xsl:value-of select="concat($url,'?',$param-name,'=',$param-value)"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:value-of select="concat($url,'&amp;',$param-name,'=',$param-value)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name='url-remove-parameter'>
    <xsl:param name="url" />
    <xsl:param name="param-name"/>
    <xsl:choose>
      <xsl:when test="substring-before($url,concat($param-name,'=')) = ''">
	<xsl:value-of select="$url"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:variable name='tmp1'>
	  <xsl:choose>
	    <xsl:when test="substring-before($url,concat('?',$param-name,'=')) != ''">
	      <xsl:value-of select="substring-before($url,concat('?',$param-name,'='))"/>
	    </xsl:when>
	    <xsl:when test="substring-before($url,concat('&amp;',$param-name,'=')) != ''">
	      <xsl:value-of select="substring-before($url,concat('&amp;',$param-name,'='))"/>
	    </xsl:when>
	    <xsl:otherwise>
	      <xsl:value-of select="substring-before($url,concat($param-name,'='))"/>
	    </xsl:otherwise>
	  </xsl:choose>
	</xsl:variable>
	<xsl:variable name='tmp2' select="substring-after(substring-after($url,concat($param-name,'=')),'&amp;')"/>
	<xsl:choose>
	  <xsl:when test="$tmp2 = ''">
	    <xsl:value-of select="$tmp1"/>
	  </xsl:when>
	  <xsl:when test="not(contains($tmp1,'?'))">
	    <xsl:value-of select="concat($tmp1,'?',$tmp2)"/>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:value-of select="concat($tmp1,'&amp;',$tmp2)"/>
	    </xsl:otherwise>
	</xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  
  <xsl:template match="ulink">
    <xsl:choose>
      <xsl:when test = "@linktype='url' and (contains(@url,'www.youtube.com') or contains(@url,'youtu.be'))">
	<xsl:variable name="url-youtube">
	  <xsl:call-template name='url-add-parameter'>
	    <xsl:with-param name="url">
	      <xsl:call-template name='url-remove-parameter'>
		<xsl:with-param name="url">
		  <xsl:call-template name='url-remove-parameter'>
		    <xsl:with-param name="url" select="@url"/>
		    <xsl:with-param name="param-name" select="'width'"/>
		  </xsl:call-template>
		</xsl:with-param>
		<xsl:with-param name="param-name" select="'height'"/>
	      </xsl:call-template>
	    </xsl:with-param>
	    <xsl:with-param name="param-name" select="'fs'"/>
	    <xsl:with-param name="param-value" select="'1'"/>
	  </xsl:call-template>
	</xsl:variable>
	<xsl:variable name="width">
	  <xsl:call-template name='url-parameter'>
	    <xsl:with-param name="url" select="@url"/>
	    <xsl:with-param name="param-name" select="'width'"/>
	  </xsl:call-template>
	</xsl:variable>
	<xsl:variable name="height">
	  <xsl:call-template name='url-parameter'>
	    <xsl:with-param name="url" select="@url"/>
	    <xsl:with-param name="param-name" select="'height'"/>
	  </xsl:call-template>
	</xsl:variable>
	<object width="{$width}" height="{$height}">
	  <param name="movie" value="{$url-youtube}"></param>
	  <param name="allowFullScreen" value="true"></param>
	  <param name="allowScriptAccess" value="always"></param>
	  <embed src="{$url-youtube}"
		 type="application/x-shockwave-flash"
		 allowfullscreen="true"
		 allowscriptaccess="always"
		 width="{$width}" height="{$height}">
	  </embed>
	</object>
      </xsl:when>
      <xsl:when test = "(@linktype='local' or @linktype='local_ext') and @exists='0'">
	<font color="red">[</font>
	<a href="{concat($base_path,'/',translate(@url,'&gt;','/'))}">
	  <xsl:if test="@linktype='local_ext'">
	    <xsl:attribute name="target">_blank</xsl:attribute>
	  </xsl:if>
	  <xsl:apply-templates/>
	</a>
	<font color="red">]</font>
      </xsl:when>
      <xsl:when test = "@linktype='local' or @linktype='local_ext'">
	<a href="{concat($base_path,'/',translate(@url,'&gt;','/'))}">
	  <xsl:if test="@linktype='local_ext'">
	    <xsl:attribute name="target">_blank</xsl:attribute>
	  </xsl:if>
	  <xsl:apply-templates/>
	</a>
      </xsl:when>
      <xsl:when test = "@linktype='url' or @linktype='url_ext'">
	<a href='{@url}'>
	  <xsl:if test="@linktype='url_ext'">
	    <xsl:attribute name="target">_blank</xsl:attribute>
	  </xsl:if>
	  <xsl:apply-templates/>
	</a>
      </xsl:when>
      <xsl:when test = "@linktype='localfile'">
	<a href="{concat($base_path,'/','intfile/',translate(@url,'&gt;','/'))}">
	  <xsl:apply-templates/>
	</a>
      </xsl:when>
      <xsl:otherwise>
	<xsl:copy>
	  <xsl:call-template name='att'/>
	</xsl:copy>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="mediaobject">
    <xsl:choose>
      <xsl:when test = "substring(imageobject/imagedata/@fileref,1,8)='local://'">
<!--         <span style="text-align:{imageobject/imagedata/@align}"> -->
	  <img src="{concat($base_path,'/img/',substring-after(imageobject/imagedata/@fileref,'local://'))}"
	       align="{imageobject/imagedata/@align}" title="{caption}">
	    <xsl:choose>
	      <xsl:when test="imageobject/imagedata/@width != ''">
		<xsl:attribute name="width">
		  <xsl:value-of select="imageobject/imagedata/@width"/>
		</xsl:attribute>
	      </xsl:when>
	      <xsl:otherwise>
		<xsl:attribute name="width">100%</xsl:attribute>
	      </xsl:otherwise>
	    </xsl:choose>
	  </img>
<!--         </span> -->
      </xsl:when>
      <xsl:when test = "substring(imageobject/imagedata/@fileref,1,11)='internal://'">
	<img src="{concat($base_path,'/imgi/',substring-after(imageobject/imagedata/@fileref,'internal://'))}"
	    align="{imageobject/imagedata/@align}">
	    <xsl:choose>
	      <xsl:when test="imageobject/imagedata/@width != ''">
		<xsl:attribute name="width">
		  <xsl:value-of select="imageobject/imagedata/@width"/>
		</xsl:attribute>
	      </xsl:when>
<!--             Normally, they will be small pictures -->
<!--               <xsl:otherwise> -->
<!--                 <xsl:attribute name="width">100%</xsl:attribute> -->
<!--               </xsl:otherwise> -->
	    </xsl:choose>
	  </img>
<!--         </span> -->
      </xsl:when>
      <xsl:when test = "substring(imageobject/imagedata/@fileref,1,15)='data:image/jpeg'">
<!--         <span style="text-align:{imageobject/imagedata/@align}"> -->
	  <img src="{concat($base_path,'/imgp/',$page,'/',count(preceding::imagedata[@fileref])+1,'.jpeg')}"
	    align="{imageobject/imagedata/@align}">
	    <xsl:choose>
	      <xsl:when test="imageobject/imagedata/@width != ''">
		<xsl:attribute name="width">
		  <xsl:value-of select="imageobject/imagedata/@width"/>
		</xsl:attribute>
	      </xsl:when>
	      <xsl:otherwise>
		<xsl:attribute name="width">100%</xsl:attribute>
	      </xsl:otherwise>
	    </xsl:choose>
	  </img>
<!--         </span> -->
      </xsl:when>
	<xsl:when test = "substring(imageobject/imagedata/@fileref,1,14)='data:image/png'">
<!--         <span style="text-align:{imageobject/imagedata/@align}"> -->
	  <img src="{concat($base_path,'/imgp/',$page,'/',count(preceding::imagedata[@fileref])+1,'.png')}"
	    align="{imageobject/imagedata/@align}">
	    <xsl:choose>
	      <xsl:when test="imageobject/imagedata/@width != ''">
		<xsl:attribute name="width">
		  <xsl:value-of select="imageobject/imagedata/@width"/>
		</xsl:attribute>
	      </xsl:when>
	      <xsl:otherwise>
		<xsl:attribute name="width">100%</xsl:attribute>
	      </xsl:otherwise>
	    </xsl:choose>
	  </img>
<!--         </span> -->
      </xsl:when>
	<xsl:when test = "substring(imageobject/imagedata/@fileref,1,14)='data:image/gif'">
<!--         <span style="text-align:{imageobject/imagedata/@align}"> -->
	  <img src="{concat($base_path,'/imgp/',$page,'/',count(preceding::imagedata[@fileref])+1,'.gif')}"
	    align="{imageobject/imagedata/@align}">
	    <xsl:choose>
	      <xsl:when test="imageobject/imagedata/@width != ''">
		<xsl:attribute name="width">
		  <xsl:value-of select="imageobject/imagedata/@width"/>
		</xsl:attribute>
	      </xsl:when>
	      <xsl:otherwise>
		<xsl:attribute name="width">100%</xsl:attribute>
	      </xsl:otherwise>
	    </xsl:choose>
	  </img>
<!--         </span> -->
      </xsl:when>
      <xsl:when test = "imageobject/svg:svg">
	<object data="{concat($base_path,'/svg/',$page,'/',count(preceding::svg:svg)+1,'.svg')}"
	  type="image/svg+xml" width="350" height="350">
	  <xsl:if test = "imageobject/svg:svg/@width != ''">
	    <xsl:attribute name="width">
	      <xsl:value-of select="imageobject/svg:svg/@width"/>
	    </xsl:attribute>
	  </xsl:if>
	  <xsl:if test = "imageobject/svg:svg/@height != ''">
	    <xsl:attribute name="height">
	      <xsl:value-of select="imageobject/svg:svg/@height"/>
	    </xsl:attribute>
	  </xsl:if>  
	  <img src="{concat($base_path,'/svg/',$page,'/',count(preceding::svg:svg)+1,'.png')}"
	    width="350" height="350">
	    <xsl:if test = "imageobject/svg:svg/@width != ''">
	      <xsl:attribute name="width">
		<xsl:value-of select="imageobject/svg:svg/@width"/>
	      </xsl:attribute>
	    </xsl:if>
	    <xsl:if test = "imageobject/svg:svg/@height != ''">
	      <xsl:attribute name="height">
		<xsl:value-of select="imageobject/svg:svg/@height"/>
	      </xsl:attribute>
	    </xsl:if>
	  </img>
	</object>
      </xsl:when>
      <xsl:otherwise>
	
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="formulae">
    <h3>
      <xsl:value-of select="title"/>
    </h3>
    <xsl:choose>
      <xsl:when test="$form_type!='response'">
	<form action="" method="post">
	  <xsl:apply-templates/>
	  <p align="center">
	    <xsl:choose>
	      <xsl:when test="$form_type='new'">
		<input type="submit" name="new" value="{$accept_name}" style='margin:2pt;'/>
		<input type="reset" name="reset" value="{$reset_name}" style='margin:2pt;'/>
	      </xsl:when>
	      <xsl:when test="$form_type='edit_choose'">
		<input type="submit" name="edit_choose" value="{$edit_name}" style='margin:2pt;'/>
		<input type="hidden" name="edit_choose_name" value="{$edit_choose_name}"/>
	      </xsl:when>
	      <xsl:when test="$form_type='edit'">
		<input type="submit" name="edit" value="{$accept_name}" style='margin:2pt;'/>
		<input type="reset" name="reset" value="{$reset_name}" style='margin:2pt;'/>
		<input type="hidden" name="edit_choose_name" value="{$edit_choose_name}"/>
		<input type="hidden" name="edit_choose_value" value="{$edit_choose_value}"/>
	      </xsl:when>
	    </xsl:choose>
	  </p>
	</form>
      </xsl:when>
      
      <xsl:otherwise>
	<xsl:apply-templates/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="container">
    <xsl:variable name="optsR">,options editable,options non-editable,text,numeric,long text,text expression,formatted expression,"</xsl:variable>
    <xsl:variable name="opts">,options editable,options non-editable,text,numeric,long text,"</xsl:variable>
    <xsl:choose>
      <xsl:when test="$form_type = 'response' and (param[contains($optsR,concat(',',@field_type,','))] or param[not(@field_type)] or param[@n=$edit_choose_name])">
	<table style='font-size:10.0pt;border:solid #6699cc 1.0pt;margin:5.4pt;
	  padding:4pt 4pt 4pt 4pt'>
	  <tr><th colspan="2" style="text-align:left">
	      <xsl:choose>
		<xsl:when test="@pn != ''">
		  <xsl:value-of select="@pn"/>
		</xsl:when>
		<xsl:otherwise>
		  <xsl:value-of select="@n"/>
		</xsl:otherwise>
	      </xsl:choose>
	  </th></tr>
	<xsl:apply-templates/>
	</table>
      </xsl:when>
      <xsl:when test="$form_type != 'response' and (param[not(starts-with(@value,'[')) and contains($opts,concat(',',@field_type,','))] or param[not(@field_type) and not(starts-with(@value,'['))]
		     or param[@n=$edit_choose_name])">
	<table style='font-size:10.0pt;border:solid #6699cc 1.0pt;margin:5.4pt;
	  padding:4pt 4pt 4pt 4pt'>
	  <tr><th colspan="2" style="text-align:left">
	      <xsl:choose>
		<xsl:when test="@pn != ''">
		  <xsl:value-of select="@pn"/>:
		</xsl:when>
		<xsl:otherwise>
		  <xsl:value-of select="@n"/>:
		</xsl:otherwise>
	      </xsl:choose>
	  </th></tr>
	<xsl:apply-templates/>
	</table>
      </xsl:when>
    </xsl:choose>
  </xsl:template>

  <xsl:template name='print-option'>
    <xsl:param name='commaList'/>
    <xsl:param name='defaultValue'/>
    
    <option>
      <xsl:choose>
	<xsl:when test="contains($commaList,',')">
	  <xsl:if test="substring-before($commaList,',') = $defaultValue">
	    <xsl:attribute name="selected">1</xsl:attribute>
	  </xsl:if>
	  <xsl:value-of select="substring-before($commaList,',')"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:if test="$commaList = $defaultValue">
	    <xsl:attribute name="selected">1</xsl:attribute>
	  </xsl:if>
	  <xsl:value-of select="$commaList"/>
	</xsl:otherwise>
      </xsl:choose>
    </option>
    
    <xsl:if test="contains($commaList,',')">
      <xsl:call-template name='print-option'>
	<xsl:with-param name="commaList" select="substring-after($commaList,',')"/>
	<xsl:with-param name="defaultValue" select="$defaultValue"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="param[@state='hidden']">
    <input type="hidden" name="p_{@n}" value='{@value}'/>
  </xsl:template>
  
  <xsl:template match="param">
    <xsl:variable name="inputTypes">,text,long text,date,numeric,options non-editable,options editable,"</xsl:variable>
    <xsl:if test="$form_type = 'response' or ((contains($inputTypes,concat(',',@field_type,',')) or not(@field_type)) and not(starts-with(@value,'['))) or @n=$edit_choose_name">
      <tr>
	<xsl:variable name="n">
	  <xsl:choose>
	    <xsl:when test="@pn != ''">
	      <xsl:value-of select="@pn" disable-output-escaping="yes"/>
	    </xsl:when>
	    <xsl:otherwise>
	      <xsl:value-of select="@n"/>
	    </xsl:otherwise>
	  </xsl:choose>
	</xsl:variable>
	
	<xsl:variable name="colspan">
	  <xsl:choose>
	    <xsl:when test="$n!='-' and ($form_type = 'response' or not(@values) or (@values != '1,0' and @values != '0,1'))">
	      1
	      </xsl:when>
	    <xsl:otherwise>
	      2
	    </xsl:otherwise>
	  </xsl:choose>
	</xsl:variable>
	
	<xsl:if test="$n!='-' and ($form_type = 'response' or not(@values) or (@values != '1,0' and @values != '0,1'))">
	  <td style='padding:0pt 8pt 0pt 8pt'>
	    <xsl:value-of select="$n" disable-output-escaping="yes"/>:
	  </td>
	</xsl:if>
	
	<xsl:choose>
	  
	  <xsl:when test="@n=$edit_choose_name">
	    <td colspan="{$colspan}" style='border-top:none;padding-left:5.4pt' id='p_{@n}'>
	      <xsl:value-of select="$edit_choose_value"/>
	      <xsl:apply-templates/>
	    </td>
	  </xsl:when>
	  
	  <xsl:when test="$form_type = 'response'">
	    <td colspan="{$colspan}" style='border-top:none;padding-left:5.4pt' id='p_{@n}'>
	      <xsl:value-of select="@value"/>
	      <xsl:apply-templates/>
	    </td>
	  </xsl:when>
	  <xsl:when test="@values = '1,0' or @values = '0,1'">
	    <td colspan="{$colspan}"><input type="checkbox" name="p_{@n}" value='1' title="{@help}">
		<xsl:if test="@value='1'">
		  <xsl:attribute name="checked">checked</xsl:attribute>
		</xsl:if>
		<xsl:value-of select="$n"/>
	    </input></td>
	  </xsl:when>
	  <xsl:when test="@values != '' or @field_type='options non-editable'">
	    <td colspan="{$colspan}"><select name="p_{@n}">
		<xsl:choose>
		  <xsl:when test="$form_type = 'edit_choose'">
		    <xsl:attribute name="size">12</xsl:attribute>
		    <xsl:attribute name="style">min-width: 200px</xsl:attribute>
		  </xsl:when>
		  <xsl:otherwise>
		    <xsl:attribute name="size">1</xsl:attribute>
		  </xsl:otherwise>
		</xsl:choose>
		<xsl:call-template name='print-option'>
		  <xsl:with-param name="commaList" select="@values"/>
		  <xsl:with-param name='defaultValue' select="@value"/>
		</xsl:call-template>
	    </select></td>
	  </xsl:when>
	  <xsl:when test="@field_type='long text' and not(starts-with(@value,'['))">
	    <td colspan="{$colspan}"><textarea rows="4" cols="30" name="p_{@n}" title="{@help}">
		<xsl:value-of select="@value"/> 
	    </textarea></td>
	  </xsl:when>
	  <xsl:when test="(contains($inputTypes,concat(',',@field_type,',')) or not(@field_type)) and not(starts-with(@value,'['))">
	    <td colspan="{$colspan}"><input type="text" name="p_{@n}" value="{@value}" size="25" title="{@help}"/></td>
	  </xsl:when>
	</xsl:choose>
	<xsl:if test="@units">
	  <td><xsl:value-of select="@units"/></td>
	</xsl:if> 
      </tr>
    </xsl:if>
  </xsl:template>
  
<xsl:template match="tcl">
<!-- nothing -->
</xsl:template>


</xsl:stylesheet>













