
package require smtp
package require mime
package require tls
package require SASL
package require SASL::NTLM
package require compass_utils
package require tdom
namespace eval send_mail_message {
    variable topdir [file normalize [file dirname [info script]]]
    variable server "smtp.gmail.com"
    variable ports 587
    variable username
    variable password
    variable debug
}
source [file join $send_mail_message::topdir tcl send_fossil_digest.tcl]
proc send_mail_message::init { args } {
    variable username
    variable password
    variable debug
    
    set optional {
	{ -reset "" 0 }
	{ -debug "" 0 }
	{ -check_digest 0|1|auto auto }
	{ -username_password "username pasword" "" }
    }
    parse_args $optional "" $args
    
    set prefs [cu::get_program_preferences -debug $debug send_mail_message]
    
    if { $username_password ne "" } {
	lassign $username_password username password
	dict set prefs username $username
	dict set prefs password $password
    }
    
    if { ![dict exists $prefs username] || $reset } {
	set err [catch { package require Tk } ret]
	if { $err } {
	    if { $debug } {
		parray ::env
	    }
	    puts stderr "It is necessary to enter mail data and there is no DISPLAY ($ret)"
	    exit 2
	}
	package require dialogwin
	wm withdraw .
	set w [dialogwin_snit .password -title [_ "Enter mail data"] -entrytext \
		[_ "Enter mail username (example: myname@compassis.com) and password"] \
	       ]
	set f [$w giveframe]
	ttk::label $f.l1 -text [_ "Username"]:
	ttk::entry $f.e1 -textvariable [$w give_uservar username]
	ttk::label $f.l2 -text [_ "Password"]:
	ttk::entry $f.e2 -textvariable [$w give_uservar password] -show "*"
	
	grid $f.l1 $f.e1 -sticky w -padx 2 -pady 2
	grid $f.l2 $f.e2 -sticky w -padx 2 -pady 2
	grid configure $f.e1 $f.e2 -sticky ew
	grid columnconfigure $f 1 -weight 1
	tk::TabToWindow $f.e1
	bind $w <Return> [list $w invokeok]
	set action [$w createwindow]
	if { $action == 1 } {
	    set username [$w give_uservar_value username]
	    set password [$w give_uservar_value password]
	}
	destroy $w
	if { $action <= 0 } { exit }
	dict set prefs username $username
	dict set prefs password $password
    } else {
	set username [dict get $prefs username]
	set password [dict get $prefs password]
    }
    if { $check_digest != 0 } {
	if { $check_digest == 1 } {
	    set force 1
	} else {
	    set force 0
	}
	set err [catch { send_fossil_digest::check_default_send -prefs_name prefs -force=$force } errstring opts]
	if { $err } {
	    puts stderr [dict get $opts -errorinfo]
	}
	if { $force } {
	    cu::store_program_preferences send_mail_message $prefs
	    exit
	}
    }
    cu::store_program_preferences send_mail_message $prefs
}
proc send_mail_message::send { recipients subject body body_html } {
    variable server
    variable ports
    variable username
    variable password
    variable debug
    
    set token_text [mime::initialize -canonical text/plain -param {charset "utf-8"} -encoding quoted-printable \
	    -string [encoding convertto utf-8 $body]]
    
    if { $body_html ne "" } {
	set token_html [mime::initialize -canonical text/html -param {charset "utf-8"} -encoding quoted-printable \
		-string [encoding convertto utf-8 $body_html]]
	set token [mime::initialize -canonical multipart/alternative -parts [list $token_text $token_html]]
    } else {
	set token $token_text
    }
    mime::setheader $token Subject [mime::word_encode "utf-8" quoted-printable [encoding convertto utf-8 $subject]]
    #mime::setheader $token From $username
    mime::setheader $token To $recipients
    smtp::sendmessage $token -debug $debug -ports $ports -originator $username \
	-recipients $recipients -servers "smtp.gmail.com" -username \
	$username -password $password
    mime::finalize $token
}
unset -nocomplain ::env(GATEWAY_INTERFACE)
set optional {
    { -reset "" 0 }
    { -debug "" 0 }
    { -body_html html "" }
    { -check_digest 0|1|auto auto }
    { -username_password "username pasword" "" }
}
set compulsory "recipients subject body"
set err [catch { parse_args $optional $compulsory $argv } ret]
if { $err } {
    # this is to parse  -digest_force alone
    set err [catch { send_mail_message::init {*}$argv }]
    if { $err } {
	puts stderr $ret
	exit 2
    }
    exit 0
}
send_mail_message::init -reset=$reset -debug=$debug -check_digest $check_digest \
    -username_password $username_password
send_mail_message::send $recipients $subject $body $body_html
exit
