#!/bin/sh -f
# -*-TCL-*- this sets TCL mode in both emacs and RamDebugger

#    OutputFile: $2/$1.info
#    ErrorFile: $2/$1.err
#    WarningFile: $2/$1.war

#env > /tmp/kk.txt

# the next line restarts using command, a tcl shell \
exec "$GIDDEFAULT/command" "$0" "$@"

namespace eval myrun {
    variable pid
    variable remotedir
    variable has_run 0
    variable err
    variable res
    variable inf
    variable war
    variable out
}

proc myrun::exit { basename dir } {
    variable has_run
    variable pid
    variable remotedir
    variable dat
    variable err
    variable res

    if { [info exists pid] } {
	set err [catch { compass_client::send_force end_process \
		    -force 1 $pid } ret]
	if { $err } {
	    tk_messageBox -message [_ "Could not kill process (%s)" $ret]
	}
    }
    if { [info exists remotedir] } {
	catch { compass_client::send_force file delete -force $remotedir }
    }

    set flavia [file join $dir $basename$res]

    if { !$has_run } {
	set fout [open [file join $dir $basename$err] a]
	puts $fout "Program has not been executed"
	close $fout
    } elseif { [file exists $flavia] && [file size $flavia] > 0 } {
	file delete [file join $dir $basename$dat]
    } else {
	set fout [open [file join $dir $basename$err] a]
	puts $fout "Program failed"
	close $fout
    }
}

proc myrun::give_exe { sim problemtypedir } {

    if { $sim eq "seakeeping" } { set sim tdyn }
    set platform [compass_client::server_platform]
    switch -regexp -- $platform {
	{^win32-ix86$} - {^win32-x86_64$} {
	    regexp {\w+$} $platform machine
	    if { [file exists [file join $problemtypedir exec $sim-$machine.exe]] } {
		set exe [file join $problemtypedir exec $sim-$machine.exe]
	    } else {
		set exe [file join $problemtypedir exec $sim.exe]
	    }
	}
	{^linux-.*-x86_64$} - {^linux-.*-ix86$} {
	    regexp {\w+$} $platform machine
	    if { [file exists [file join $problemtypedir exec $sim-$machine]] } {
		set exe [file join $problemtypedir exec $sim-$machine]
	    } else {
		set exe [file join $problemtypedir exec $sim]
	    }
	}
	default {
	    error [_ "unknown server platform '%s'" $platform]
	}
    }
    return $exe
}

proc myrun::get_sysinfo { sim problemtypedir } {
    variable remotedir

    set exe [myrun::give_exe $sim $problemtypedir]
    set localdir [file dirname $exe]
    set remotedir [send create_remote_new_dir compassfem]

    copy_file_to_server -binary 1 -save_cache 1 $exe $remotedir
    myrun::copy_libraries_to_server $localdir $remotedir
    
    set data [send exec_in_dir $remotedir ./[file tail $exe] -outputinfo]
    send file delete -force $remotedir
    unset remotedir
    return $data
}

proc myrun::copy_libraries_to_server { localdir remotedir } {

    set platform [compass_client::server_platform]
    switch -regexp -- $platform {
	{^win32-ix86$} - {^win32-x86_64$} {
	    foreach dll [glob -nocomplain -dir $localdir *.dll] {
		copy_file_to_server -binary 1 $dll $remotedir
	    }
	}
	{^linux-.*-x86_64$} - {^linux-.*-ix86$} {
	    foreach so [glob -nocomplain -dir $localdir *.so] {
		if { $sim ne "ramseries" && [file tail $so] eq "Laminate.so" } { continue }
		copy_file_to_server -save_cache 1 -binary 1 $so $remotedir
	    }
	    foreach so [glob -nocomplain -dir [file join $localdir .. .. .. .. lib] libhdf5.so*] {
		copy_file_to_server -save_cache 1 -binary 1 $so $remotedir
	    }
	    foreach so [glob -nocomplain -dir [file join $localdir .. .. .. .. lib] libmkl*.so*] {
		copy_file_to_server -save_cache 1 -binary 1 $so $remotedir
	    }
	}
    }
}

proc myrun::run_model { problemtypedir sim more_files gid_model password_file } {
    variable pid
    variable exe
    variable remotedir
    variable has_run
    variable dat
    variable res
    variable inf
    variable err
    variable war 
    variable out

    set has_run 1
    set l_base [file join $gid_model [file tail [file root $gid_model]]]
    foreach i [list $res $inf $war $err] {
	file delete $l_base$i
    }
    set datfile $l_base$dat
    set errfile $l_base$err
    set outfile $l_base$out

    set exe [myrun::give_exe $sim $problemtypedir]
    set localdir [file dirname $exe]
    set remotedir [send create_remote_new_dir compassfem]

    copy_file_to_server -save_cache 1 -binary 1 $exe $remotedir
    copy_file_to_server -nocomplain 1 -binary 0 $password_file $remotedir
    copy_file_to_server -binary 0 $datfile $remotedir
    
    myrun::copy_libraries_to_server $localdir $remotedir

    set tcl_file 0
    foreach file $more_files {
	if { [file extension $file] eq ".tcl" } { set tcl_file 1 }
	copy_file_to_server -save_cache 1 -binary 1 $file $remotedir
    }
    
    if { $tcl_file } {
	set remote_tcl_dir [send create_remote_new_dir [file join compassfem tcl]]
	foreach f [glob -nocomplain -dir [file join $localdir tcl] *] {
	    copy_file_to_server -binary 0 $f $remote_tcl_dir
	}
    }

    if { [file tail $password_file] ne "password.txt" } {
	send file rename [file join $remotedir [file tail $password_file]] \
	    [file join $remotedir password.txt]
    }
    
    if { $::tcl_platform(platform) eq "unix"} {
	set oldlibpath [send set ::env(LD_LIBRARY_PATH)]
	send set ::env(LD_LIBRARY_PATH) .
    }

    switch $sim {
	"ramseries" { 
	    set pid [compass_client::exec_in_dir $remotedir ./[file tail $exe] -a \
		    -e [file tail $errfile] [file tail $datfile] >>& [file tail $outfile] &]
	}
	"tdyn" {
	    set pid [compass_client::exec_in_dir $remotedir ./[file tail $exe] -name \
		    [file tail $datfile] >>& [file tail $outfile] &]
	}
	default { 
	    set pid [compass_client::exec_in_dir $remotedir ./[file tail $exe] -name \
		    [file tail $datfile] -SeaWaves >>& [file tail $outfile] &]
	}
    }
    set r_base [file join $remotedir [file tail [file root $gid_model]]]

    after 1000
    set mtime ""
    while 1 {
	if { [send file exists $r_base$inf] } {
	    set new_mtime [send file mtime $r_base$inf]
	    if { $mtime eq "" || $new_mtime > $mtime } {
		copy_file_from_server $r_base$inf $gid_model
		if { $inf ne ".info" } {
		    file copy -force [file join $l_base$inf] [file join $l_base\.info]
		}
	    }
	    set mtime $new_mtime
	}
	if { ![info exists pid] } { break }
	if { ![send process_exists $pid] } { break }
	after 5000 [list set waiting_var 1]
	vwait ::waiting_var
    }
    if { [info exists pid] } { unset pid }
    catch {file delete -force [file join $l_base\.info]}

    copy_file_from_server -nocomplain 1 -binary 1 $r_base$res $gid_model
    copy_file_from_server -nocomplain 1 $r_base$inf $gid_model
    copy_file_from_server -nocomplain 1 $r_base$war $gid_model
    copy_file_from_server -nocomplain 1 $r_base$err $gid_model

    send file delete -force $remotedir
    if { $::tcl_platform(platform) eq "unix"} {
	send set ::env(LD_LIBRARY_PATH) $oldlibpath
    }
    unset remotedir
}

lassign $argv basename dir problemtypedir - sim more_files

set dir [file normalize $dir]
set problemtypedir [file normalize $problemtypedir]

lappend auto_path [file join $problemtypedir exec] \
    [file join $env(GIDDEFAULT) problemtypes common_lib]

set debug 1
if { $debug } {
    lappend auto_path ~/mytcltk ~/mytcltk/compass
}

set err [catch {
	package require Tk
	package require compass_utils
	package require comm
	package require sha1
	package require compass_utils::c
	
	if { $debug } {
	    bind all <F12> { package require RamDebugger }
	}
	auto_load compass_client::init_gui
	namespace import -force compass_client::*
	
	if { $::tcl_platform(platform) eq "windows"} {
	    # Global variable "RunningProcess" is written when GiD kills the process
	    set RunningProcess running
	    trace add variable RunningProcess write "compass_client::gui_exit ;#"
	} else {
	    cu::signal trap TERM compass_client::gui_exit
	}
	
	set sim [string tolower $sim]
	switch $sim {
	    "ramseries" {
		set myrun::dat .dat
		set myrun::err .err
		set myrun::res .flavia.res
		set myrun::inf .info
		set myrun::war .war
		set myrun::out .out
	    }
	    default {
		foreach "from to" [list $basename.dat $basename.flavia] {
		    if { [file exists [file join $dir $from]] } {
		        file rename -force [file join $dir $from] [file join $dir $to]
		    }
		}
		set myrun::dat .flavia
		set myrun::err .flavia.err
		set myrun::res .flavia.res
		set myrun::inf .flavia.inf
		set myrun::war .flavia.war
		set myrun::out .flavia.out
	    }            
	}
	
	compass_client::init_gui -infofile_ext $myrun::inf \
	    -run_handler [list myrun::run_model $sim $problemtypedir $more_files] \
	    -sysinfo_handler [list myrun::get_sysinfo $sim $problemtypedir] \
	    -exit_handler [list myrun::exit $basename $dir] \
	    -hide_model_password -exit_after_analysis \
	    -gid_model $dir \
	    -password_file [file join $problemtypedir password.txt] \
	    -full_error_info $debug
    } ret]

if { $err } {
    set fout [open [file join $dir $basename.err] a]
    puts $fout [_ "Program has not been executed (%s)" $ret]
    close $fout
    exit
}

vwait forever















 
