rem =======================================
rem =======================================
rem COMBINED APPROACH SEAFEM-AND-RAMSERIES:
rem =======================================
rem =======================================


rem SEAFEM DATAFILE:
copy %1.flavia %1.seafem.dat

rem RAMS DATAFILE:
rem copy %1.dat %1.ramseries.dat
rem leave as is.

rem    -*- TCL -*-
rem    OutputFile: %2\%1.seafem.dat.inf
rem    OutputFile: %2\%1.info
rem    ErrorFile: %2\%1.seafem.dat.err
rem    ErrorFile: %2\%1.err
rem    WarningFile: %2\%1.seafem.dat.war
rem    WarningFile: %2\%1.war
tcl

proc LaunchSeaFEMAndRamseries { basename dir problemtypedir } {
    global RunningProcess

    set err [catch {
	foreach i [list .flavia .flavia.res .flavia.err .flavia.error .tdyn.dat.err .flavia.war .flavia.inf \
		       .flaviafnl .flaviafnl.err .flaviafnl.war .flaviafnl.inf] {
	    if { [file exists [file join $dir $basename$i]] } {
		file delete [file join $dir $basename$i]
	    }
	}
	foreach i [list .err .info .war .warnings] {
	    if { [file exists [file join $dir $basename$i]] } {
		file delete [file join $dir $basename$i]
	    }
	}
	} errstring]
    
    for { set port 9300 } { $port <= 9320 } { incr port } {
	set err [catch { socket -server "" $port } sock]
	if { !$err } {
	    close $sock
	    break
	}
    }
    if { $port > 9320 } {
	set fout [open [file join $dir $basename.flavia.err] a]
	puts $fout "Failed to open TCP-IP port in the range 9300-9320."
	puts $fout "Contact your system administrator and check firewalls and anti-virus"
	close $fout
	return
    }

    # NOTE: the timeout 50000 means 50 seconds.

    set exe [file join $problemtypedir . exec tdyn.exe]
    set dat [file join $dir $basename.seafem.dat]
    set out1 [file join $dir $basename.seafem.dat.out]    
    set err1 [file join $dir $basename.seafem.dat.err]
    set pid1 [exec $exe -seawaves -name $dat -coupling $port 50000 > $out1 &]

    set exe [file join $problemtypedir . exec ramseries.exe]  
    set dat [file join $dir $basename.dat]
    set out2 [file join $dir $basename.out]
    set err2 [file join $dir $basename.err]
    set inf2 [file join $dir $basename.info]
    set war2 [file join $dir $basename.war]
    set pid2 [exec $exe -i $inf2 -e $err2 -w $war2 -tdyncoupling $port 100000 $dat &]

    set RunningProcess running
    set ErrorInProcess "no"
    while 1 {
	 if { $RunningProcess == "" } {
	     if { [isalive $pid1] } {  kill $pid1 }
	     if { [isalive $pid2] } {  kill $pid2 }
	     vwait forever
	 }
	 if { ![isalive $pid1] && ![isalive $pid2] } { break }
	 if { ![isalive $pid1] && [file exists $err1] && [file size $err1] > 0 } {
	     kill $pid2
	     set ErrorInProcess "yes"
	    break
	}        
	 if { ![isalive $pid2] && [file exists $err2] && [file size $err2] > 0 } {
	     kill $pid1
	     set ErrorInProcess "yes"
	     break
	 }
	after 200
	update
     }


    if { $ErrorInProcess=="no" } {
	if { ![file exists [file join $dir $basename.flavia.res]] || \
		[file size [file join $dir $basename.flavia.res]] == 0 } {
	    set fout [open [file join $dir $basename.err] a]
	    puts $fout "Ramseries failed. Results are not available."
	    close $fout
	}
	if { ![file exists [file join $dir $basename.seafem.dat.res]]  || \
		[file size [file join $dir $basename.seafem.dat.res]] == 0 } {
	    set fout [open [file join $dir $basename.seafem.dat.err] a]
	    puts $fout "SeaFEM  failed. Results are not available."
	    close $fout
	} 
    }

# joining error messages into one file: case SeaFEM error
    set f [file join $dir $basename.seafem.dat.err]
    if { [file exists $f] && [file size $f] > 0 } {
	set fin [open $f r]
	set fout [open [file join $dir $basename.flavia.err] a]
	puts -nonewline $fout [read $fin]
	set outtext " \"Tdyn has failed\" " 
	puts $fout $outtext
	close $fin
	close $fout
    }
# joining error messages into one file: case RamSeries error
    set f [file join $dir $basename.err]
    if { [file exists $f] && [file size $f] > 0 } {
	set fin [open $f r]
	set fout [open [file join $dir $basename.flavia.err] a]
	puts -nonewline $fout [read $fin]
	set outtext " \"Ramseries has failed\" " 
	puts $fout $outtext
	close $fin
	close $fout
    }
}

foreach "- - basename dir problemtypedir -" $argv break
LaunchSeaFEMAndRamseries $basename $dir $problemtypedir
