
proc doRestartGiD { } {   
    set changedVariables {}

    if { [info exists ::UseLocalOpenGL] } {
	if { $::GidPriv(SoftwareOpenGL) != $::UseLocalOpenGL} {
	    set ::GidPriv(SoftwareOpenGL) $::UseLocalOpenGL
	    if { $::GidPriv(SoftwareOpenGL)} {
		WarnWinDirect [_ "Changing to software rendering."] .themechangewin
	    } else {
		WarnWinDirect [_ "Changing to accelerated OpenGL."] .themechangewin
	    }
	    # if { $::tcl_platform(os) == "Darwin"} {
	    after 1000 GiD_Set OGL_emulateFrontBuffer $::UseLocalOpenGL
	    # }
	}
	lappend changedVariables [ list SoftwareOpenGL $::GidPriv(SoftwareOpenGL) ] \
	    [ list OGL_configured 1 ]
    }
    if { [info exists ::UseLocalTheme] } {        
	#allways apply begining color options (example change from black to black)
	gid_themes::SetCurrentThemeAndApplyBackground $::UseLocalTheme
	
    }

    set args $::argv
    if { $::GidPriv(restartCurrentProject) != "UNNAMED"} {
	lappend args $::GidPriv(restartCurrentProject)
    }

    # saving the preferences:
    # write the SoftwareOpenGL and OGL_configured variables in the user's defaults file
    # together with the default preferences
    set fileDefaults [GiveGidDefaultsFile]
    writedefaults $fileDefaults;#kernel values
    WriteTCLDefaultsInFile $fileDefaults ;#tcl values
    if { $::tcl_platform(platform) != "windows"} {	
	if { [ file exists $fileDefaults] } {
	    # create a new gidDefaults from previous gidDefaults
	    file copy -force $fileDefaults ${fileDefaults}~
	    set fi [ open ${fileDefaults}~]
	} else {
	    # create a new gidDefaults from BestDefaults
	    set fi [ open [ GiveBestGidDefaults]]
	}
	set fo [ open $fileDefaults w]
	
	foreach KeyValue $changedVariables {
	    puts $fo "[lindex $KeyValue 0] [lindex $KeyValue 1]"
	}
	while { ![ eof $fi]} {
	    set line [ gets $fi]
	    set key [ lindex $line 0]
	    if { [ lsearch -index 0 $changedVariables $key ] != -1 } {
		continue
	    }
	    puts $fo $line
	}
	close $fi
	close $fo
	CheckFileOwner $fileDefaults
    }
    
    # WarnWin "argv0 = $::argv0"
    if { [ info exists ::env(INI_ARG_0)]} {
	set exec_prog $::env(INI_ARG_0)
    } else {
	#set exec_prog [ file rootname $::argv0]
	set exec_prog $::argv0
    }
    # WarnWin "$exec_prog 'argv: $::argv'"
    # WarnWin "exec-$exec_prog-$args-"
    # destroy .gid
    update
    update idletasks
    set err [ catch {       
	if { [llength $args] == 0} {
	    exec $exec_prog &
	} else {
	    exec $exec_prog {*}$args &
	}
    } errTxt ]
    # if { $err} {
    #         WarnWin $errTxt
    # }
    exit
}

 proc checkMaxAttemptsRestart { } {
    if { $::GidPriv(restartMaxAttempts) <= 0} {
	# max number of attempts, now aborting restart.
	WarnWinTk [_ "%s could not be restarted automatically.\nPlease,\
 as soon as you can quit the program\nand\
 start it again so that the new settings\ncan\
 be applied." $::GidPriv(ProgName)]
	return 0
    } else {
	return 1
    }
}

proc checkChangesAndRestart { } {
    if { ![ checkMaxAttemptsRestart]} {
	return
    }
    set notSaved [ GiD_Info Project AreChanges]
    if { $notSaved} {
	after 1000 checkChangesAndRestart
    } else {
	doRestartGiD
    }
}

 proc checkIfInPreAndRestart {} {
     if { ![ checkMaxAttemptsRestart]} {
	 return
     }    
     if { [GetCurrentPrePostMode] == "POST" } {
	 incr ::GidPriv(restartMaxAttempts) -1
	 after 1000 checkIfInPreAndRestart
     } else {
	 # ask the user to save any modifications:
	 GiD_Process escape escape escape escape Files New
	 after 1000 checkChangesAndRestart
     }
 }

proc RestartGiD { { w .gid}} {
    set resp [ tk_messageBox -parent $w -default no -title [_ "%s restart" $::GidPriv(ProgName)] \
		   -icon question -type yesno \
		   -message [_ "In order to apply the new settings %s needs to be reinitiated.\nRestart %s now?" \
		                $::GidPriv(ProgName)  $::GidPriv(ProgName)]
	       ]
    if { "$resp" == "no"} {
	return
    }

    # if we are in Post mode, change to pre   
    if { [GetCurrentPrePostMode] == "POST" } {
	GiD_Process escape escape escape escape Preprocess
    }

    # wait at max. 15 seconds to solve for all issues: in post, save changes, ....
    # if not, abort restart
    set ::GidPriv(restartMaxAttempts) 15

    set ::GidPriv(restartCurrentProject) [GiD_Info Project ModelName]

    checkIfInPreAndRestart
}



proc FillPrefValuesFromGid {} {
	CreateWidgetsFromXml::ChangeAllVariablesValues gidPreferences
}


proc UpdateWidgetsLanguage { } {
    global GidPriv        
    SetAllMenusVariables    

    ChangeTopMenusAndWindows
    ChangeVariables
    
    if { [info exists GidPriv(InitWindowList)] } {
	foreach geomwinname $GidPriv(InitWindowList) {
	    if { [lsearch {PrePostTopMenuWindow} $geomwinname] != "-1" } {
		continue
	    }
	    if { [info exists GidPriv($geomwinname)] && [winfo exists $GidPriv($geomwinname)] } {
		catch { eval $GidPriv($geomwinname,InitComm) }
	    }
	}
    }
    #lower command
    if { [info exists GidPriv(PrePostEntryWindowGeom)] } {
	BottomEntryFrame [lindex $GidPriv(PrePostEntryWindowGeom) 0]
    }
    ChangeWindowTitle
    DWInitUserDataOptions
}


 proc ChangeVariables { {groupname ""} { w .gid.gidPreferences } } {    
    if { [winfo exists $w] } {      
       destroy $w
    }
    CreateWidgetsFromXml::ReadXmlFile Preferences.xml $groupname "ChangeVariables"
}

