#---------start problemtypetest-------------------

proc TestCreateWidgetsFromXml { testnum } {
    
    set xmlfile "mytest.xml"
    set contentxml {
        <xml_to_tcl version="1.0">
        <window name="nosegidPreferences" label="Test Preferences Window" dynamicupdate="no">
        <group name="general" label="General">
        <labelframe name="entitiescolour" label="Entities colour" help="This option set the default colour of the geometrical and mesh entities.">
        <colorbutton variable="ColorPoints" whenreadvar="TclColour" needsredraw="1" label="Points"/>
        <colorbutton variable="ColorLines" whenreadvar="TclColour" needsredraw="1" label="Lines"/>
        <colorbutton variable="ColorPolyLines" whenreadvar="TclColour" needsredraw="1" label="PolyLines"/>
        <colorbutton variable="ColorSurfaces" whenreadvar="TclColour" needsredraw="1" label="Surfaces"/>
        <colorbutton variable="ColorSurfsIsoparametric" whenreadvar="TclColour" needsredraw="1" label="Surfaces Iso"/>
        <colorbutton variable="ColorVolumes" whenreadvar="TclColour" needsredraw="1" label="Volumes"/>
        <colorbutton variable="ColorNodes" whenreadvar="TclColour" needsredraw="1" label="Nodes"/>
        <colorbutton variable="ColorElements" whenreadvar="TclColour" needsredraw="1" label="Elements"/>
        </labelframe>
        </group>
        </window>
        </xml_to_tcl>
    }
    
    
    switch $testnum {
        1 {
            #1) Modify preferences window
            set root [CreateWidgetsFromXml::GetPreferencesXml]
            set checkbuttontoadd {
                <checkbutton variable="My_ProgressBarInMainWindow" variablemanager="Mitestproc" label="My_Embedded mi_progress mi_bar" help="hi!!"/>
            }
            
            set labelframetoadd {
                <labelframe name="pp" label="TestAddAfter">
                <combobox variable="My_Language" variablemanager="Mitestproc" label="My_Language" help="hi world">
                <option value="en" nottransalatelabel="English"></option>
                <option value="es" nottransalatelabel="Spanish"></option>
                <option value="ru" nottransalatelabel="Russian"></option>            
                </combobox>
                </labelframe>
            }
            CreateWidgetsFromXml::AddAfterVariable $root "CreateAlwaysNewPoint" $checkbuttontoadd
            CreateWidgetsFromXml::AddBeforeName $root "interface" $labelframetoadd
            
            CreateWidgetsFromXml::TakeOutVariable $root "BackColorTop"
            
            CreateWidgetsFromXml::TakeOutName $root "generalother"
            
            CreateWidgetsFromXml::SetPreferencesXml $root
        }
        2 {
            #--- aux usually file already on disk
            set chan [open $xmlfile w]    
            puts $chan $contentxml
            close $chan
            #----
            
            #create preference window from zero
            set xml [tDOM::xmlReadFile $xmlfile]
            set document [dom parse $xml]
            set rootnode [$document documentElement]
            CreateWidgetsFromXml::SetPreferencesXml $rootnode
            
            
            #----
            file delete $xmlfile
        }
        3 {
            #--- aux usually file already on disk
            set chan [open $xmlfile w]    
            puts $chan $contentxml
            close $chan
            #----
            
            #Use CreateWidgetsFromXml for your own windows 
            #always file references from scripts folder            
            CreateWidgetsFromXml::ReadXmlFile "../$xmlfile" "" "" 0
            
            #----
            file delete $xmlfile            
        }
        4 {
            #open the window
            #always file references from scripts folder
            CreateWidgetsFromXml::IniWin $CreateWidgetsFromXml::fileread(../$xmlfile) "../$xmlfile" ""             
        }
        5 {
            #or
            #always file references from scripts folder
            CreateWidgetsFromXml::ReadXmlFile "../$xmlfile" "" "" 1
        }
    }
    
    
    
    
}

#---------end problemtypetest-------------------



#auxiliar functions out of namespace
proc CalculateMaxLength { list } {
    set maxlength 0
    foreach item $list {
        set length [string length $item]
        if {$length>$maxlength} {
            set maxlength $length
        }
    } 
    return $maxlength
}

proc SetValue { varname varvalue varmanager} {
    if { $varmanager=="" || $varmanager=="gid" } {
        #GiD_Set $varname $varvalue
        GiD_Process Mescape Utilities Variables $varname $varvalue escape escape
    } else {
        if { [info procs $varmanager] != "" } {
            $varmanager SetValue $varname $varvalue
        } else {
            WarnWinText "MUST IMPLEMENT $varmanager SetValue $varname $varvalue"
        }
    }
}

proc GetValue { varname varmanager} {
    if { $varmanager=="" || $varmanager=="gid" } {
        set result [GiD_Set $varname]
    } else {
        if { [info procs $varmanager] != "" } {
            set result [$varmanager GetValue $varname]
        } else {
            set result 0
            WarnWinText "MUST IMPLEMENT $varmanager GetValue $varname"
        }
    }
    return $result
}

proc GetDefaultValue { varname varmanager} {
    if { $varmanager=="" || $varmanager=="gid" } {
        set result [GiD_Set -default $varname]
    } else {
        if { [info procs $varmanager] != "" } {
            set result [$varmanager GetDefaultValue $varname]
        } else {
            set result 0
            WarnWinText "MUST IMPLEMENT $varmanager GetDefaultValue $varname"
        }        
    }  
    return $result    
}

#namespace CreateWidgetsFromXml

variable printactualvaluesasdefault 0

namespace eval CreateWidgetsFromXml {        
    variable fileread
    variable properties ;#array to store properties for each name like 
    #count, changedvariables, allvariables, listfunctions_tocall, content_frame, dynamicupdate, current_group
    #There are other variables on namespace, created dinamically when reading xml (more than 1 xml readed)
    variable aux ;#array for auxiliary variables for name and dom node (tmp or local)
}

#---- modules utilities --------
proc CreateWidgetsFromXml::GetXml { xmlfile } {
    variable fileread
    if { ![info exists fileread($xmlfile)] } {
        CreateWidgetsFromXml::ReadXmlFile $xmlfile "" "" 0
    }
    return $fileread($xmlfile)
}

proc CreateWidgetsFromXml::GetCurrentGroup { xmlfile } {
    variable fileread
    variable properties
    if { [info exists fileread($xmlfile)] } {
        set name $fileread($xmlfile,name)
        if { [info exists properties($name,current_group)] } {
            return $properties($name,current_group)
        }
    }
    return ""
}

proc CreateWidgetsFromXml::SetXml { xmlfile rootnode } {    
    variable fileread
    set itsopen 0    
    if { [info exists fileread($xmlfile)] } {
        set name $fileread($xmlfile,name)
        set w .gid.$name
        if { ![GidUtils::IsTkDisabled] &&[winfo exists $w] } {
            set itsopen 1
            set currentgroup [CreateWidgetsFromXml::GetCurrentGroup $xmlfile]
            destroy $w
        }
    }
    set fileread($xmlfile) $rootnode            
    #and reopen
    if { $itsopen } {
        CreateWidgetsFromXml::IniWin $fileread($xmlfile) $xmlfile $currentgroup $fileread($xmlfile,initfunction_on_gid_ini)
    }
}

proc CreateWidgetsFromXml::AddBeforeUniqueAttribute { uniqueattribute rootnode ref_attribbutevalue contenttoadd } {
    set findnode [$rootnode find $uniqueattribute $ref_attribbutevalue]    
    set parentnode [$findnode parentNode]
    $parentnode appendXML $contenttoadd
    set childadded [$parentnode lastChild]
    $parentnode removeChild $childadded
    $parentnode insertBefore $childadded $findnode    
}

proc CreateWidgetsFromXml::AddAfterUniqueAttribute {  uniqueattribute rootnode ref_attribbutevalue contenttoadd } {
    set findnode [$rootnode find $uniqueattribute $ref_attribbutevalue]
    
    set parentnode [$findnode parentNode]
    $parentnode appendXML $contenttoadd
    set childadded [$parentnode lastChild]
    set prevchild [$childadded previousSibling]
    if { $prevchild != $findnode } {
        set nextfind [$findnode nextSibling]
        $parentnode removeChild $childadded
        $parentnode insertBefore $childadded $nextfind   
    }    
}

proc CreateWidgetsFromXml::TakeOutUniqueAttribute { uniqueattribute rootnode ref_attribbutevalue } { 
    set findnode [$rootnode find $uniqueattribute $ref_attribbutevalue]
    if { $findnode != "" } {
        set parentnode [$findnode parentNode]
        $findnode delete
        while { ![$parentnode hasChildNodes] } {
            set findnode $parentnode
            set parentnode [$findnode parentNode]
            $findnode delete
        }
    } else {
        # WarnWinText "Not possible to take out $uniqueattribute = $ref_attribbutevalue from $rootnode"
    }    
}

#----- generic use -----

proc CreateWidgetsFromXml::AddBeforeName { rootnode refname contenttoadd } {
    CreateWidgetsFromXml::AddBeforeUniqueAttribute "name" $rootnode $refname $contenttoadd
}

proc CreateWidgetsFromXml::AddAfterName { rootnode refname contenttoadd } {
    CreateWidgetsFromXml::AddAfterUniqueAttribute "name" $rootnode $refname $contenttoadd
}

proc CreateWidgetsFromXml::AddBeforeVariable { rootnode refvar contenttoadd } {
    CreateWidgetsFromXml::AddBeforeUniqueAttribute "variable" $rootnode $refvar $contenttoadd
}

proc CreateWidgetsFromXml::AddAfterVariable { rootnode refvar contenttoadd } {
    CreateWidgetsFromXml::AddAfterUniqueAttribute "variable" $rootnode $refvar $contenttoadd
}

proc CreateWidgetsFromXml::TakeOutVariable { rootnode refvar } { 
    CreateWidgetsFromXml::TakeOutUniqueAttribute "variable" $rootnode $refvar   
}

proc CreateWidgetsFromXml::TakeOutName { rootnode refname } {    
    CreateWidgetsFromXml::TakeOutUniqueAttribute "name" $rootnode $refname   
}

#must do a similar procedure SetState to set the state to normal or disabled (of the frame and all child widgets)
#RecursiveChangeState $f $value 
#with value: normal disabled or restore to set again its previous state

proc CreateWidgetsFromXml::SetVisible { refname value {rootnode ""} } {
    if { $rootnode == "" } {
        set rootnode [CreateWidgetsFromXml::GetPreferencesXml]
    }
    set findnode [$rootnode find name $refname]
    if { $findnode != "" } {
        $findnode setAttribute visible $value
        if { [$findnode hasAttribute frame] } {
            set f [$findnode getAttribute frame]
            if { $value } {
                grid $f
            } else {
                grid remove $f
            }
        } else {
            
        }            
    } else {
        WarnWinText "CreateWidgetsFromXml::SetVisible. Item name $refname not found"
    } 
}

#---- for preference window

proc CreateWidgetsFromXml::GetPreferencesXml {} {
    return [CreateWidgetsFromXml::GetXml Preferences.xml]    
}

proc CreateWidgetsFromXml::SetPreferencesXml { rootnode } {
    CreateWidgetsFromXml::SetXml Preferences.xml $rootnode
}
#---- end modules utilities ------



if { $::printactualvaluesasdefault == "1" } {
    proc CreateWidgetsFromXml::PrintActualValuesAsDefault { name} {
        variable properties
        WarnWinText $name
        set listvarvalue ""
        set pos [string length $name,allvariables,]
        foreach {item varvalue} [array get properties $name,allvariables,*] {
            set varname [string range $item $pos end]
            if { [CreateWidgetsFromXml::IsVariableTmp $varname] } {
                set nodename [CreateWidgetsFromXml::GetNodenameFromVariableTmp $varname $name]
                set gidvarname [$nodename getAttribute variable]
                #valor actual set $varname [GetValue $gidvarname $varmanager]
                if { [$nodename hasAttribute whenwritevar] } {
                    set varvalue [::xmlprograms::[$nodename getAttribute whenwritevar] [list $varvalue]]
                }
                set varmanager ""
                if { [$nodename hasAttribute variablemanager] } {
                    set varmanager [$nodename getAttribute variablemanager]
                }
                #Sometimes you set something on GiD and Gid modify value before to set
                # its diferent what you set that what you get, we want "get values"
                SeValue $gidvarname $varvalue $varmanager
                set varvalue [GetValue $gidvarname $varmanager]
                #must be without process whenreadvar!!
                
                lappend listvarvalue [list $gidvarname $varvalue]
            }                        
        }                
        set listvarvalue [lsort -ascii -index 0 $listvarvalue]
        foreach item $listvarvalue {
            set text ""
            set varname [lindex $item 0]
            set varvalue [lindex $item 1]
            
            #remplace % for %% because is inside a sprintf, also not valid "" or ;
            #set varvalue [regsub -all "%" $varvalue "%%"]
            append text "  if(GIDstrncasecmp(variable,\"$varname\", len_variable)==0)\{\n"
            append text "    strcpy(retvalue,\"$varvalue\");\n"
            append text "    strcat(Match,\"$varname \");\n"
            append text "    NumMatch++;\n"
            append text "  \}"
            WarnWinText $text
        }    
    }
}

proc CreateWidgetsFromXml::ReadXml {} {
    #back compatibility
}

proc CreateWidgetsFromXml::ReadXmlFile { xmlfile {groupname ""} {initfunction_on_gid_ini ""} { initwindow 1} } {    
    variable fileread
    if { ![info exists fileread($xmlfile)] } {
        package require tdom
        if { [file pathtype $xmlfile] != "absolute" } {
            set xmlfile_absolute [file join $::GIDDEFAULTTCL $xmlfile]
        } else {
            set xmlfile_absolute $xmlfile    
            set pathlength [string length $::GIDDEFAULTTCL]
            if { ![string compare -length $pathlength $xmlfile $::GIDDEFAULTTCL] } {
                set xmlfile [string replace $xmlfile 0 $pathlength]
                if { [info exists fileread($xmlfile)] } {
                    if { $initwindow } {
                        CreateWidgetsFromXml::IniWin $fileread($xmlfile) $xmlfile $groupname $initfunction_on_gid_ini
                    }
                    return
                }            
            }
            #WarnWinText "Please don't use absolute path to reference $xmlfile_absolute file"
        }    
        if { [file pathtype $xmlfile_absolute] != "absolute" || ![file exists $xmlfile_absolute] } {
            WarnWinText "file $xmlfile_absolute don't exists"
            return
        }        
        set xml [tDOM::xmlReadFile $xmlfile_absolute]
        set document [dom parse $xml]
        set root [$document documentElement]
        
        if { [$root nodeName] != "xml_to_tcl" } {
            $document delete    
        }
        
        set domWindowNode [$root selectNodes window]
        set name [$domWindowNode getAttribute name]
        
        CreateWidgetsFromXml::ReadPrograms 
        
        set CreateWidgetsFromXml::fileread($xmlfile) $root
        set CreateWidgetsFromXml::fileread($xmlfile,name) $name      
        set CreateWidgetsFromXml::fileread($xmlfile,initfunction_on_gid_ini) $initfunction_on_gid_ini 
       
        #raise this GiD-Tcl event to allow plugins modify the preferences window
        PluginPreferencesProcs_Eval $root
    }
    if { $initwindow } {
        CreateWidgetsFromXml::IniWin $CreateWidgetsFromXml::fileread($xmlfile) $xmlfile $groupname $initfunction_on_gid_ini
    }
}

proc CreateWidgetsFromXml::ReadPrograms { } {
    #infoexist namespace xmlprograms
    SRC xmlprocs
    #set domProgramNode [$root selectNodes programs]
    #namespace eval ::xmlprograms [$domProgramNode text]
}

proc CreateWidgetsFromXml::IniWin { root xmlfile {groupname ""} {initfunction_on_gid_ini ""} } {    
    variable properties
    set domWindowNode [$root selectNodes window]
    set name [$domWindowNode getAttribute name]    
    
    set w .gid.$name
    if { [winfo exists $w] } {      
        destroy $w
        #destroy will destroy variables, be careful, don't put down
        update idletasks
    }
            
    #hacer algo con name para crear namespace o algo asi!!      
    
    set properties($name,count) 0
    #array containing leaveoftree listofvariables changed
    set properties($name,changedvariables) {}
    #array containing varname currentgidvalue
    set properties($name,allvariables) {}
    set properties($name,listfunctions_tocall) {}
        
        
    #if { ![info exists ::GidPriv(TopLevelsList)] || [lsearch $::GidPriv(TopLevelsList) $w] == -1 } {
    #    set assign_default_size 1
    #} else {
    #    set assign_default_size 0
    #}
    
    ::InitWindow $w [GetLabelTag $domWindowNode] "PrePostCreateWidgetsFromXml${name}WindowGeom" $initfunction_on_gid_ini 
    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0
    #if { $assign_default_size } {
    #    wm geometry $w 556x421
    #}    
    
    
    set SepFrame [ttk::frame $w.sepframe]
    set BottomFrame [ttk::frame $w.bottomframe -style BottomFrame.TFrame]
    
    set Apply [ttk::button $w.bottomframe.apply -text [_ "Apply"]  -style BottomFrame.TButton]
    #configuration command below
    set Close [ttk::button $w.bottomframe.close -text [_ "Close"] -command [list  destroy $w] -style BottomFrame.TButton]
    
    
    set dynupdate [$domWindowNode getAttribute dynamicupdate]
    set properties($name,dynamicupdate) 0
    if { $dynupdate == "show_on" || $dynupdate == "show_off" } {
        if { $dynupdate == "show_on" } {
            set properties($name,dynamicupdate) 1
        }
        set cb [ttk::checkbutton $w.bottomframe.combobox -variable CreateWidgetsFromXml::properties($name,dynamicupdate) -text [_ "Dynamic update:"] -style BottomFrame.TCheckbutton ]
        #every time user click checkbox we must activate-desactivate apply and we invoke the button to update variables 
        set function [list CreateWidgetsFromXml::ActivateDeactivate CreateWidgetsFromXml::properties($name,dynamicupdate) $Apply 0]
        trace add variable properties($name,dynamicupdate) write $function
        #execute 1 time 
        eval $function "args_not_used"
        bind $w <Destroy> [list +CreateWidgetsFromXml::DestroyTraceVar %W $w CreateWidgetsFromXml::properties($name,dynamicupdate) $function]  
        
        trace add variable CreateWidgetsFromXml::properties($name,dynamicupdate) write [list CreateWidgetsFromXml::ApplyInvoke $Apply]
        bind $w <Destroy> [list +CreateWidgetsFromXml::DestroyTraceVar %W $w CreateWidgetsFromXml::properties($name,dynamicupdate) [list CreateWidgetsFromXml::ApplyInvoke $Apply]]        
        
        grid $cb -column 0 -row 0 -sticky w -padx {5 0} -pady 6
        grid $Apply -column 1 -row 0 -sticky e -padx {0 5} -pady 6
        grid $Close -column 2 -row 0 -sticky w -padx 5 -pady 6
        grid columnconfigure $w.bottomframe 0 -weight 0 
        grid columnconfigure $w.bottomframe 1 -weight 1 -minsize 80
        grid columnconfigure $w.bottomframe 2 -weight 1 -minsize 80
    } else {
        grid $Apply $Close -sticky {} -padx 5 -pady 6
        grid anchor $w.bottomframe center
    }        
    
    set listchildnodes [$domWindowNode childNodes]
    
    if { [llength $listchildnodes]==1 } {
        #there is no tree, only 1 leave
        set f [ttk::frame $w.contentf]
        set properties($name,content_frame)  $f
        #grid $::MainL -sticky nsew -column 0 -row 0
        #grid $::ContentL -sticky nsew -column 0 -row 1
        grid $f -sticky nsew -column 0 -row 2
        grid $SepFrame -sticky nsew -column 0 -row 4 -columnspan 2 -pady 1       
        grid $BottomFrame -sticky nsew -column 0 -row 5 -columnspan 2
        grid columnconfigure $w 0 -weight 1
        grid rowconfigure $w 2 -weight 1
        
        $Apply configure -command [list CreateWidgetsFromXml::ApplyChanges "" $name "args_not_used"]
        bind $w <Destroy> [list +CreateWidgetsFromXml::DestroingCheckChanges %W $w "" $name]
        update idletasks
        
#         if { [catch {CreateWidgetsFromXml::CreateAndVisualizeTreeLeave [lindex $listchildnodes 0] $f "" "" $name} err] } {
#             WarnWinText "ERROR:\n$err\n" 
#             for { set i [info level]} { $i >0 } { incr i -1 } {
#                 WarnWinText [info level $i]
#             }
#         }
        CreateWidgetsFromXml::CreateAndVisualizeTreeLeave [lindex $listchildnodes 0] $f "" "" $name
    } else {
        
        set panedwin [ttk::panedwindow $w.p -orient horizontal]
        set f [ttk::frame $panedwin.contentf]
        set properties($name,content_frame) $f
        
        package require fulltktree
        
        # width name justify type is_editable
        set cols {
            { 25 Categories left item 1 }
            { 3 "" right image 1 }
            { 0 domdata left text 0 }
        }
        set T [fulltktree $panedwin.t -columns $cols -width 180]
        $T configure \
            -selecthandler "CreateWidgetsFromXml::LogData $name select" \
            -selecthandler2 "CreateWidgetsFromXml::LogData $name select" \
            -item_image "" \
            -folder_image "" \
            -arrow_image_expand 0 \
            -showlines 1 -indent 15 -showheader 0
            
            
#            -button1handler "CreateWidgetsFromXml::LogData $name button1handler"\
#-button1presshandler "CreateWidgetsFromXml::LogData $name button1presshandler"\
#-returnhandler "CreateWidgetsFromXml::LogData $name returnhandler"\
#-contextualhandler "CreateWidgetsFromXml::LogData $name contextualhandler"\
#-contextualhandler_menu "CreateWidgetsFromXml::LogData $name contextualhandler_menu"\
#-contextualhandler_fmenu "CreateWidgetsFromXml::LogData $name contextualhandler_fmenu"\
#-editbeginhandler "CreateWidgetsFromXml::LogData $name editbeginhandler"\
#-editaccepthandler "CreateWidgetsFromXml::LogData $name editaccepthandler"\
#-deletehandler "CreateWidgetsFromXml::LogData $name deletehandler"\
#-draghandler "CreateWidgetsFromXml::LogData $name draghandler"

        
        
        $T element create eWindow window
        $T element create eRect rect -fill ""
        #$T element create eText1 text -width 300
        $T element create eText2 text -wrap none
        
        set S [$T style create s2 -orient vertical]
        $T style elements $S {eText2 eRect eWindow}
        $T style layout $S eRect -union {eText2 eWindow} -ipadx 8 -ipady 8 -padx 4 -pady {0 4}
        $T style layout $S eText2 -pady {0 6} -squeeze x
        $T style layout $S eWindow -iexpand x -squeeze x
        
        
        $T notify bind MyTag <Expand-before> {
            %T item image %I 1 [gid_themes::GetImage blank.png small_icons]      
            %T item element configure %I 0 e_text_sel -fill ""
            #e_rect e_text_sel e_selmarker_up e_selmarker_down
        }
        $T notify bind MyTag <Collapse-before> {
            %T item image %I 1 [gid_themes::GetImage blank.png small_icons]
            %T item element configure %I 0 e_text_sel -fill ""
            set listchilds [%T item children %I]
            foreach itemid $listchilds {              
                if { [%T item image $itemid 1] == [gid_themes::GetImage ArrowLeft.png small_icons] } { 
                    %T item image %I 1 [gid_themes::GetImage ArrowLeft.png small_icons]
                    %T item element configure %I 0 e_text_sel -fill {red {focus} red {!focus} red {selected} red {!selected} red {active} red {!active} red {}}
                    break;
                }
            }
        }    
        $T dragimage configure -visible 0
        #------------------test cabeceras treectrl------------
        #No headers!!
        #gid_themes::ConfigureHeaders $T
        #-----------------part 1 fin----------------------
        
        bind $T <Button-$::gid_right_button> "[list CreateWidgetsFromXml::MenuContextual $name %W %x %y] ; break"
        
        
        #grid $T -sticky nsew -column 0 -row 0 -rowspan 3
        #grid $::MainL -sticky nsew -column 1 -row 0
        #grid $::ContentL -sticky nsew -column 1 -row 1
        #grid $f -sticky nsew -column 2 -row 2
        
        grid $panedwin -sticky nsew -column 0 -row 0
        
        grid $SepFrame -sticky nsew -column 0 -row 1 -pady 1
        
        # first pane, which would get widgets gridded into it:
        #ttk::labelframe .p.f1 -text Pane1 -width 100 -height 100
        #ttk::labelframe .p.f2 -text Pane2 -width 100 -height 100; # second pane
        $panedwin add $T
        $panedwin add $f
        
        grid $BottomFrame -sticky nsew -column 0 -row 2
        
        grid columnconfigure $w 0 -weight 1
        grid rowconfigure $w 0 -weight 1 
        grid rowconfigure $w 2 -weight 0
        
        #maximum number of levels 3
        #as we show it expanded group that have groups inside implies no image
        set selection ""
        foreach childNode $listchildnodes {
            if { [$childNode nodeName] == "group" } {
                set item [$T insert end [list [GetLabelTag $childNode] [gid_themes::GetImage blank.png small_icons] $childNode] root] 
                if { [$childNode getAttribute name] == $groupname  } {
                    set selection $item                         
                }
                
                $childNode setAttribute id $childNode
                set numgroupsinitem 0    
                foreach childNode_2 [$childNode childNodes] {
                    if { [$childNode_2 nodeName] == "group" } {
                        incr numgroupsinitem
                        set item_2 [$T insert end [list [GetLabelTag $childNode_2] [gid_themes::GetImage blank.png small_icons] $childNode_2] $item]
                        if { [$childNode_2 getAttribute name] == $groupname  } {
                            set selection $item_2                         
                        }
                        $childNode_2 setAttribute id $childNode_2
                        set numgroupsinitem_2 0
                        foreach childNode_3 [$childNode_2 childNodes] {
                            if { [$childNode_3 nodeName] == "group" } {
                                incr numgroupsinitem_2
                                set item_3 [$T insert end [list [GetLabelTag $childNode_3] [gid_themes::GetImage blank.png small_icons] $childNode_3] $item_2]
                                if { [$childNode_3 getAttribute name] == $groupname  } {
                                    set selection $item_3                         
                                }
                                $childNode_3 setAttribute id $childNode_3
                            }
                            if { $numgroupsinitem_2 != 0 } {
                                $T item image $item_2 1 [gid_themes::GetImage blank.png small_icons]
                                $T item element configure $item_2 0 e_text_sel -fill ""
                            }
                        }
                    }
                    if { $numgroupsinitem != 0 } {
                        $T item image $item 1 [gid_themes::GetImage blank.png small_icons]
                        $T item element configure $item 0 e_text_sel -fill ""
                    }
                }
            } else {
                #unexpected tag
                WarnWinText "Unexpected tag, expected group and is [$childNode nodeName]"                
            }
        }
        #------------------part 2 test cabeceras treectrl------------
        #  gid_themes::ApplyHeadersConfig $T
        #-----------------fin test----------------------
        
        #If deleted we can no acces to create the content
        #$document delete
        $Apply configure -command [list CreateWidgetsFromXml::ApplyChanges $T $name "args_not_used"]
        
        bind $w <Destroy> [list +CreateWidgetsFromXml::DestroingCheckChanges %W $w $T $name]
        
        bind $w <Escape> [list event generate $Close <<Invoke>>]
        #bind $w <Return>  [list event generate $Apply <<Invoke>>]
        
        update idletasks
        
        if { $::printactualvaluesasdefault == "1" } {
            ChangeWarnLineVar "Developer variable printactualvaluesasdefault=1 press p to print"
            bind $w <p> [list CreateWidgetsFromXml::PrintActualValuesAsDefault [set name] ]            
        }
        if { $selection != "" } {
            CreateWidgetsFromXml::LogData $name select $T.t $selection
        } else {
            set domWindowNode [$root selectNodes selected]
            
            if { $domWindowNode != "" } {
                set selection [$domWindowNode getAttribute selection]  
                
                CreateWidgetsFromXml::LogData $name select $T.t $selection
            } else {
                CreateWidgetsFromXml::LogData $name select $T.t 1
            }
        }
        
    }
}

proc CreateWidgetsFromXml::ApplyInvoke { apply_button args } {
    #invoke only works when button its !disabled
    # and we want to force invoke
    set command [$apply_button cget -command]
    eval $command
}

#--------------validate entryes functions------------
proc CreateWidgetsFromXml::IsFloatingPoint { item } {
    return [string is double -strict $item]    
}

proc CreateWidgetsFromXml::IsFloatingPointPositive { item } {
    if { [CreateWidgetsFromXml::IsFloatingPoint $item] } {
        return [expr $item>=0]
    } 
    return false  
}

proc CreateWidgetsFromXml::IsNatural { item } {
    if {$item == ""} {
        return false
    }
    return [regexp {^[+]?[1-9]+[0-9]*$} $item]
}

proc CreateWidgetsFromXml::IsFile { item } {
    if {$item == ""} {
        return false
    }
    return true
}

proc CreateWidgetsFromXml::IsRealFormat { item } {
    if {$item == ""} {  
        return false
    }
    if { [ regexp {^%[^eEfFgG]*[eEfFgG]$} $item] } {
        set err [ catch { format $item 1.2345678912345567890}]
        if { $err } { 
            return false 
        }
    } else {
        return false
    }
    return true
}

proc CreateWidgetsFromXml::IsNotEmpty { item } {
    if {$item == ""} {
        return false
    }
    return true
}

#--------------end validate entryes functions------------
#--------------utilities functions-------------------

proc CreateWidgetsFromXml::CalculateWidthSpinbox { from to increment } {
    set lengthunits 1
    set lengthdecimals 0
    foreach item {from to increment} {
        set list [split [set $item] .]
        switch [llength $list] {
            1 {   
                set lengthunits [::tcl::mathfunc::max $lengthunits [string length [lindex $list 0]]]
            } 
            2 {
                set lengthunits [::tcl::mathfunc::max $lengthunits [string length [lindex $list 0]]]
                set lengthdecimals [::tcl::mathfunc::max $lengthdecimals [string length [lindex $list 1]]]
            }
        }
    }
    if { $lengthdecimals != 0 } {
        #+1 for the point
        return [expr $lengthunits + $lengthdecimals + 1]
    } else {
        return $lengthunits
    }        
}

#return the list of options and values defined in the xml node that are widget options
proc CreateWidgetsFromXml::GetWidgetOptions { node widget } {
    set ignored_options {image label variable} ;#sorted list. ignore because are considered in a special way
    set options [list]
    foreach item [$widget configure] {
        set option [string range [lindex $item 0] 1 end]
        if { [lsearch -sorted -dictionary $ignored_options $option] == -1 } {        
            if { [$node hasAttribute $option] } {
                lappend options -$option [$node getAttribute $option]
            }
        }
    }
    return $options
}

proc CreateWidgetsFromXml::GetLabelTag { node } {
    #if no label tag, returns ""
    #also performs translation on label tag
    set text_return ""
    if { [$node hasAttribute label] } {
        set text_return [_ [$node getAttribute label]]
    } elseif { [$node hasAttribute nottransalatelabel] } {
        set text_return [$node getAttribute nottransalatelabel]
    }
    # performing a wrap length automatic on translated text if its too long
    set max_length 50
    if { [string length $text_return]>$max_length } {
        set text_before $text_return
        set text_return ""
        set line_letters 0
        # we don't cut words, each line will be $max_length+some letters (more robust than $max_length-some letters)        
        foreach word [lrange $text_before 0 end-1] {
            set text_return "$text_return$word"
            incr line_letters [string length $word]
            if { $line_letters < $max_length } {
                set text_return "$text_return "
                incr line_letters 1
            } else {
                set text_return "$text_return\n"
                set line_letters 0
            }
        }
        set text_return "$text_return[lindex $text_before end]"
    }
    return $text_return
}

#if no help tag, returns ""
#also performs translation on help tag and and . at the end   
proc CreateWidgetsFromXml::GetHelpTag { node } {   
    if { [$node hasAttribute help] } {
        return [subst -nocommands -novariables [_ [$node getAttribute help]].]
    } else {
        return ""
    }
}

#if no state tag, returns ""    
proc CreateWidgetsFromXml::GetStateTag { node } {    
    if { [$node hasAttribute state] } {
        return [subst -nocommands -novariables [$node getAttribute state]]
    } else {
        return ""
    }
}

#it can put one or more of this tags: -text -image -compound -style Toolbutton 
proc CreateWidgetsFromXml::GetVisualTags { node type_entity} {   
    set label [GetLabelTag $node]    
    set ret [list]
    if { $label != "" } {
        lappend ret -text $label
    }
    if { [$node hasAttribute image] } {
        lappend ret -image [gid_themes::GetImage [$node getAttribute image] small_icons] -compound left
        #if { $type_entity != "label" && $type_entity != "frame" && $type_entity != "labelframe" && $type_entity != "button" } {
        #    lappend ret -style Toolbutton
        #}
    }
    return $ret
}

proc CreateWidgetsFromXml::Add2PointsOnTextLabel { string } {
    set index [lsearch -exact $string "-text"]
    if { $index != -1 } {
        incr index
        lset string $index [lindex $string $index]:
    } else {
        #WarnWinText "CreateWidgetsFromXml::Add2PointsOnTextLabel. string=$string"
    }
    return $string
}
#proc CreateWidgetsFromXml::AddEndPointOnTextLabel { string } {
#    set index [lsearch -exact $string "-text"]
#    if { $index != -1 } {
#        incr index
#        lset string $index [lindex $string $index].
#    } else {
#    }
#    return $string
#}


proc CreateWidgetsFromXml::GetValidVariable { name node } {
    variable aux
    if { [$node hasAttribute variable] } {                
        return CreateWidgetsFromXml::aux(tmp,$name,$node)
    } else {                
        return CreateWidgetsFromXml::aux(local,$name,$node)
    }
}

proc CreateWidgetsFromXml::IsVariableTmp { varname } {
    if { [string range $varname 0 29] == "CreateWidgetsFromXml::aux(tmp," } {
        return 1
    } else {
        return 0
    }
}

proc CreateWidgetsFromXml::GetNodenameFromVariableTmp { varname name } {
    return [string range $varname [string length "CreateWidgetsFromXml::aux(tmp,$name,"] end-1]
}

proc CreateWidgetsFromXml::GetTmpVariableFromVariable { name var } {  
    variable properties
    set pos [string length $name,allvariables,]
    foreach {item varvalue} [array get properties $name,allvariables,*] {
        set varname [string range $item $pos end]
        if { [CreateWidgetsFromXml::IsVariableTmp $varname] } {
            set nodename [CreateWidgetsFromXml::GetNodenameFromVariableTmp $varname $name]
            set gidvarname [$nodename getAttribute variable]
            if { $var == $gidvarname } {
                return $varname
            }
        }                        
    }
    return "Variable not found"
}

proc CreateWidgetsFromXml::InitialitateAllVariablesOfChildNode { node name } {        
    variable properties
    variable aux
    array set managedvariables {}
    foreach childnode [$node childNodes] {
        if { [$childnode nodeName] == "checkbutton"
            || [$childnode nodeName] == "checkbuttonframe" 
            || [$childnode nodeName] == "combobox"
            || [$childnode nodeName] == "entry"
            || [$childnode nodeName] == "entrywithbutton"
            || [$childnode nodeName] == "radiobutton"
            || [$childnode nodeName] == "spinbox"
            || [$childnode nodeName] == "scale"
            || [$childnode nodeName] == "tablelist"
            || [$childnode nodeName] == "colorbutton" 
            || [$childnode nodeName] == "fontselection"} {
            #-------------------------------------------------------------------------------------------------------------------------
            #se lee de gid todas las que en en xml tienen "variable" , las q empiezan por tmp                
            set varname [GetValidVariable $name $childnode]
            if { [CreateWidgetsFromXml::IsVariableTmp $varname] } {                
                set varmanager ""
                if { [$childnode hasAttribute variablemanager] } {
                    set varmanager [$childnode getAttribute variablemanager]
                }                
                if { [$childnode hasAttribute whenreadvar] } {
                    set $varname [::xmlprograms::[$childnode getAttribute whenreadvar] [list [GetValue [$childnode getAttribute variable] $varmanager]]]        
                } else {
                    set $varname [GetValue [$childnode getAttribute variable] $varmanager]
                }
            } elseif {[$childnode hasAttribute onwhen] } {
                set $varname 0
                lappend properties($name,listfunctions_tocall) "set $varname \[expr [$childnode getAttribute onwhen]\]"               
            } else {
                set $varname 0
            }
            #-------------------------------------------------------------------------------------------------------------------------            
            array set managedvariables [list $varname [set $varname]]                                    
        }
        if { [$childnode hasChildNodes] } {
            array set local_managedvariables [InitialitateAllVariablesOfChildNode $childnode $name]
            array set managedvariables [array get local_managedvariables]
        }
    }
    return [array get managedvariables]
}

proc CreateWidgetsFromXml::GetDefaultValues { variables name } {
    variable properties
    foreach {varname varvalue} $variables {
        #-------------------------------------------------------------------------------------------------------------------------
        #se tendria que leer el valor por defecto de GiD=varmanager
        #set x [GetDefaultValue $varname $varmanager]
        #varname = CreateWidgetsFromXml::tmp_[set name]_gidvarname
        #set varname [lindex $varname_varvalue 0]
        if { [CreateWidgetsFromXml::IsVariableTmp $varname] } {
            set nodename [CreateWidgetsFromXml::GetNodenameFromVariableTmp $varname $name]
            set gidvarname [$nodename getAttribute variable]             
            set varmanager ""
            if { [$nodename hasAttribute variablemanager] } {
                set varmanager [$nodename getAttribute variablemanager]
            }
            if { [$nodename hasAttribute whenreadvar] } {
                set $varname [::xmlprograms::[$nodename getAttribute whenreadvar] [list [GetDefaultValue $gidvarname $varmanager]]]        
            } else {
                set $varname [GetDefaultValue $gidvarname $varmanager]
            }
        }
        #-------------------------------------------------------------------------------------------------------------------------        
    }
    foreach item $properties($name,listfunctions_tocall) {
        eval [subst -nocommands $item]
    }    
}

proc CreateWidgetsFromXml::ApplyChanges { fulltree name args} {
    variable properties
    
    set CommandsToSend ""
    
    set listofleaves ""
    set needsredraw 0   
    set pos [string length $name,changedvariables,]
    foreach {item listvariables} [array get properties $name,changedvariables,*] {
        set leaveid [string range $item $pos end]
        foreach varname $listvariables {
            #-------------------------------------------------------------------------------------------------------------------------
            #send to GiD or to variablemanager
            #-------------------------------------------------------------------------------------------------------------------------
            if { [CreateWidgetsFromXml::IsVariableTmp $varname] } {
                set nodename [CreateWidgetsFromXml::GetNodenameFromVariableTmp $varname $name]
                set gidvarname [$nodename getAttribute variable]               
                set varmanager ""
                if { [$nodename hasAttribute variablemanager] } {
                    set varmanager [$nodename getAttribute variablemanager]
                }
                if { [$nodename hasAttribute whenwritevar] } {
                    lappend CommandsToSend [list SetValue $gidvarname [::xmlprograms::[$nodename getAttribute whenwritevar] [list [set $varname]]] $varmanager]
                } else {
                    lappend CommandsToSend [list SetValue $gidvarname [set $varname] $varmanager]
                }
                if { [$nodename hasAttribute needsredraw] } {
                    if { [$nodename getAttribute needsredraw] == "1" } {
                        set needsredraw 1
                    }
                }
                #GiD_Process Mescape Utilities Variables $gidvarname [set $varname] escape
            }
            #GiD_Process Mescape utilities variables $varname $varvalue escape 
            #update values of varibles
            set properties($name,allvariables,$varname) [set $varname]
        }
        if { $needsredraw == "1" } {
            lappend CommandsToSend [list GiD_Process MEscape 'Redraw]
        }
        if { $fulltree != "" } {
            $fulltree item image $leaveid 1 [gid_themes::GetImage blank.png small_icons]
            $fulltree item element configure $leaveid 0 e_text_sel -fill ""
            set parentid [$fulltree item parent $leaveid] 
            if { ![$fulltree item state get $parentid open] } {
                while { $parentid != "0" } {            
                    set listchilds [$fulltree item children $parentid]
                    set change_on_one_child "0"
                    foreach itemid $listchilds {              
                        if { [$fulltree item image $itemid 1] == [gid_themes::GetImage ArrowLeft.png small_icons] } { 
                            set change_on_one_child "1"
                            break;
                        }
                    }
                    if { $change_on_one_child == "0" } {
                        $fulltree item image $parentid 1 [gid_themes::GetImage blank.png small_icons] 
                        $fulltree item element configure $parentid 0 e_text_sel -fill ""
                        set parentid [$fulltree item parent $parentid]
                    } else {
                        break;
                    }
                }
            }
        }
    }
    #be careful variable has traces associated
    array unset properties $name,changedvariables,*
    
    if { $properties($name,dynamicupdate) != "1" } {
        lappend CommandsToSend [list ChangeWarnLineVar [_ "Accepted new preferences"]]
    }
    foreach command $CommandsToSend {
        eval $command
    }    
}

proc CreateWidgetsFromXml::DestroingCheckChanges { W w fulltree name } {
    variable properties
    if { $W != $w } return
    set ThereAreChanges 0
    foreach {item listvariables} [array get properties $name,changedvariables,*] {        
        foreach varname $listvariables {            
            if { [CreateWidgetsFromXml::IsVariableTmp $varname] } {
                set ThereAreChanges 1
                break
            }
        }
        if { $ThereAreChanges==1 } {
            break;
        }
    }
    if { $ThereAreChanges==1 } {
     
        set text [_ "Do you want to save the changes you've made in the preference window?"]
        set retval [tk_messageBox -default no -icon question -message $text -type yesno]
        if { $retval=="yes" } {
            CreateWidgetsFromXml::ApplyChanges "" $name "args_not_used"
        }        
    }    
    array unset properties $name,*    
}

proc CreateWidgetsFromXml::AuxTraceFunc { fulltree leaveid name name1 name2 op } {
#WarnWinText "$fulltree $leaveid $name $name1 $name2 $op"
    variable properties
    set varname ${name1}($name2)
    #if actual value is different from gid_actual value
    if { [set $varname] != $properties($name,allvariables,$varname) } {        
        if { $fulltree != "" } { 
            $fulltree item image $leaveid 1 [gid_themes::GetImage ArrowLeft.png small_icons] 
            $fulltree item element configure $leaveid 0 e_text_sel -fill {red {focus} red {!focus} red {selected} red {!selected} red {active} red {!active} red {}}
            set itemid [$fulltree item parent $leaveid]
            if { ![$fulltree item state get $itemid open] } {
                while { $itemid } {
                    $fulltree item image $itemid 1 [gid_themes::GetImage ArrowLeft.png small_icons] 
                    $fulltree item element configure $itemid 0 e_text_sel -fill {red {focus} red {!focus} red {selected} red {!selected} red {active} red {!active} red {}}              
                    set itemid [$fulltree item parent $itemid]
                }
            }
        }
        
        if { ![info exists properties($name,changedvariables,$leaveid)] || $properties($name,changedvariables,$leaveid) == ""} {
            set properties($name,changedvariables,$leaveid) $varname
        } elseif { [lsearch -exact $properties($name,changedvariables,$leaveid) $varname] == -1 } {
            lappend properties($name,changedvariables,$leaveid) $varname
        }
        #else already inside
    } else {
        #we must extract variable from changed variables if it was contained
        
        #if there any variable has changed in this leave
        if { ![info exists properties($name,changedvariables,$leaveid)] || $properties($name,changedvariables,$leaveid) == "" } {
            #it must say already "no changes"
            #$fulltree item text $leaveid 1 "no changes"           
        } else {
            set varlist $properties($name,changedvariables,$leaveid)
            set searchresult [lsearch -exact $varlist $varname]
            if { $searchresult == -1} {
                #it must say already "has changes"
                #$fulltree item text $leaveid 1 "has changes"                                        
            } else {
                #takeout variable from list
                set varlist [lreplace $varlist $searchresult $searchresult]
                if { [llength $varlist] == 0 } {                    
                    unset properties($name,changedvariables,$leaveid)
                    if { $fulltree != "" } { 
                        $fulltree item image $leaveid 1 [gid_themes::GetImage blank.png small_icons]  
                        $fulltree item element configure $leaveid 0 e_text_sel -fill ""
                        set parentid [$fulltree item parent $leaveid] 
                        if { ![$fulltree item state get $parentid open] } {
                            while { $parentid != "0" } {            
                                set listchilds [$fulltree item children $parentid]
                                set change_on_one_child "0"
                                foreach itemid $listchilds {              
                                    if { [$fulltree item image $itemid 1] == [gid_themes::GetImage ArrowLeft.png small_icons] } { 
                                        set change_on_one_child "1"
                                        break;
                                    }
                                }
                                if { $change_on_one_child == "0" } {
                                    $fulltree item image $parentid 1 [gid_themes::GetImage blank.png small_icons] 
                                    $fulltree item element configure $parentid 0 e_text_sel -fill ""
                                    set parentid [$fulltree item parent $parentid]
                                } else {
                                    break;
                                }
                            }
                        }
                    }
                } else {
                    set properties($name,changedvariables,$leaveid) $varlist
                    #it must say already "has changes"
                    #$fulltree item text $leaveid 1 "has changes"                
                }                                
            }
        }
    }
    if { $properties($name,dynamicupdate) == 1 } {  
        ApplyChanges $fulltree $name "args_not_used"
    }
}

proc CreateWidgetsFromXml::AuxDestroyTraceFunc { var fulltree leaveid name } {
    trace remove variable $var write [list CreateWidgetsFromXml::AuxTraceFunc $fulltree $leaveid $name]
}

proc CreateWidgetsFromXml::CreateTracesAssociatedToALeave { f variables fulltree leaveid name } {
    #variables is an array
    foreach var $variables {   
        trace add variable $var write [list CreateWidgetsFromXml::AuxTraceFunc $fulltree $leaveid $name]
        bind $f <Destroy> [list +CreateWidgetsFromXml::AuxDestroyTraceFunc $var $fulltree $leaveid $name] 
    }
}

proc CreateWidgetsFromXml::ActivateDeactivate { var list onvalue args } {    
    variable properties
    if {[set $var] == $onvalue } {
        foreach item $list {
            if { ![winfo exists $item] } {
                WarnWinText "CreateWidgetsFromXml::ActivateDeactivate. $item not exists"
            } else {
                set classbutton [winfo class $item] 
                if { ( $classbutton != "Frame") && ( $classbutton != "Label") } {
                    if { $classbutton == "Button"} {
                        $item configure -state normal
                    } elseif { $classbutton == "TCombobox" } {
                        #$item configure -state readonly
                        $item state !disabled
                    } else {
                        $item state !disabled
                    }
                }
            }
        }
    } else {
        foreach item $list {
            if { ![winfo exists $item] } {
                WarnWinText "CreateWidgetsFromXml::ActivateDeactivate. $item not exists"
            } else {
                set classbutton [winfo class $item] 
                if { ( $classbutton != "Frame") && ( $classbutton != "Label") } {
                    if { $classbutton == "Button"} {
                        $item configure -state disabled
                    } elseif { $classbutton == "TCombobox" } {
                        # $item configure -state disabled
                        $item state disabled
                    } else {
                        $item state disabled
                    }
                }                
            }
        }
    }    
}

proc CreateWidgetsFromXml::DestroyTraceArray { W w var function} {
    if { $W != $w } return
    trace remove variable $var array $function
}

proc CreateWidgetsFromXml::DestroyTraceVar { W w var function} {
    if { $W != $w } return
    variable properties
    trace remove variable $var write $function
}

proc CreateWidgetsFromXml::ValidateCommand { P V W validatefunction varname name} {
    variable properties
    switch $V {
        "key" {
            #puts "CreateWidgetsFromXml::ValidateCommand key"
            #precaucion puede ser 1 tecla o 1 texto introducido con ctrl+v
            if { [eval [list $validatefunction $P]] } {
                $W state !invalid
            } else {
                $W state invalid                
            }
        }
        "focusout" - "forced" {
            #puts "CreateWidgetsFromXml::ValidateCommand focusout or forced: $V"            
            if { [eval [list $validatefunction $P]] } {
                $W state !invalid                
            } else {
                set $varname $properties($name,allvariables,$varname)
                #without after idle it remains red...
                after idle [list $W state !invalid]
            }
        }
        default {
            #puts "CreateWidgetsFromXml::ValidateCommand focusin or other: $V"
            
        }
    }
    return true
}

proc ApplyGidHelpToNoneHelpWidgets { listwidgets text } {
    foreach widget $listwidgets {
        if { ![HasGidHelp $widget] } {
            GidHelp $widget $text
        }
    }
}

#--------------end:utilities functions-------------------
#--------------widget functions-------------------
proc CreateWidgetsFromXml::ReadInsideLabelFrame { node lf name} {
    set listcreated ""
    foreach childnode [$node childNodes] {
        if { [$childnode nodeName] == "checkbutton" } {
            lappend listcreated {*}[CreateCheckbutton $childnode $lf $name]
        } elseif { [$childnode nodeName] == "checkbuttonframe" } {
            lappend listcreated {*}[CreateCheckbuttonFrame $childnode $lf $name]                      
        } elseif { [$childnode nodeName] == "combobox" } {
            lappend listcreated {*}[CreateCombobox $childnode $lf $name]
        } elseif { [$childnode nodeName] == "entry" } {
            lappend listcreated {*}[CreateEntry $childnode $lf $name]
        } elseif { [$childnode nodeName] == "entrywithbutton" } {
            lappend listcreated {*}[CreateEntryWithButton $childnode $lf $name]
        } elseif { [$childnode nodeName] == "radiobutton" } {
            lappend listcreated {*}[CreateRadiobutton $childnode $lf $name]
        } elseif { [$childnode nodeName] == "spinbox" } {
            lappend listcreated {*}[CreateSpinbox $childnode $lf $name]
        } elseif { [$childnode nodeName] == "label" } {
            lappend listcreated {*}[CreateLabel $childnode $lf $name]
        } elseif { [$childnode nodeName] == "button" } {
            lappend listcreated {*}[CreateButton $childnode $lf $name]
        } elseif { [$childnode nodeName] == "scale" } {
            lappend listcreated {*}[CreateScale $childnode $lf $name]
        } elseif { [$childnode nodeName] == "tablelist" } {
            lappend listcreated {*}[CreateTableList $childnode $lf $name]
        } elseif { [$childnode nodeName] == "colorbutton" } {
            lappend listcreated {*}[CreateColorButton $childnode $lf $name]
        } elseif { [$childnode nodeName] == "fontselection" } { 
            lappend listcreated {*}[CreateFontSelection $childnode $lf $name]
        } elseif { [$childnode nodeName] == "tcldraw" } {
            lappend listcreated {*}[CreateTclDraw $childnode $lf $name]
        } elseif { [$childnode nodeName] == "labelframe" } {
            lappend listcreated {*}[CreateLabelFrame $childnode $lf $name]
        } elseif {  [$childnode nodeName] == "#comment" } {
            #its a comment don't do anything
        } else {
            #unexpected tag
            WarnWinText "Unexpected tag, expected checkbutton,combobox,entry or label and is [$childnode nodeName]"
        }
    }
    return $listcreated
}

proc CreateWidgetsFromXml::IsOnBackUp { args } {
    WarnWinText $args
}

proc CreateWidgetsFromXml::CreateCheckbutton { node lf name} {
    variable properties
    variable aux
    #as item can have childs we must create a frame to work as a unique widget
    set f [ttk::frame $lf.$properties($name,count)] 
    set varname [GetValidVariable $name $node]
    #set cb [ttk::checkbutton $f.combobox -variable $varname {*}[AddEndPointOnTextLabel [GetVisualTags $node checkbutton]]]
    set cb [ttk::checkbutton $f.combobox -variable $varname {*}[GetVisualTags $node checkbutton]]
    
    if { [$node hasAttribute offvalue] } {
        $cb configure -offvalue [$node getAttribute offvalue]
    }
    if { [$node hasAttribute onvalue] } {
        $cb configure -onvalue [$node getAttribute onvalue]
    }
    
    
    if { [$node hasAttribute associatedto] } {
        set varnameassociated CreateWidgetsFromXml::tmp_[set name]_[$node getAttribute associatedto]
        set onwhen [$node getAttribute onwhen]
        lappend properties($name,listfunctions_tocall) [list $lf.$properties($name,count).colorbutton configure -background \$$varname]
        #trace add variable $varnameassociated write $onwhen
        #bind $w <Destroy> [list +CreateWidgetsFromXml::DestroyTraceVar %W $w $varnameassociated $onwhen]
        lappend properties($name,listfunctions_tocall)l [list $cb configure -background \$$varname]
    }
    
    set listcreated [list $f $cb]
    #it could have childs that only will be activate if checkbutton it's on
    if { [$node hasChildNodes] } { 
        #add 2 points to label
        $cb configure {*}[Add2PointsOnTextLabel [GetVisualTags $node checkbutton]]
        #end    
        set child_f [ttk::frame $f.frame]  
        lappend listcreated $child_f
        set f_listcreated [ReadInsideLabelFrame $node $child_f $name]
        
        grid $cb $child_f -sticky we
        grid columnconfigure $f 0 -weight 0
        grid columnconfigure $f 1 -weight 1
        if { $f_listcreated != "" } {
            #usually the grid is vertical, so that its done by default
            #but in this case we want a horizontal grid
            
            #we use grid slaves because only want 1st level childs
            
            foreach item [grid slaves $child_f] {
                array set info [grid info $item]
                grid configure $item -row $info(-column) -column $info(-row)
            }
            grid columnconfigure $child_f 0 -weight 1
            for { set i 1} { $i < [ llength [grid slaves $child_f]]} { incr i} {
                grid columnconfigure $child_f $i -weight 0
            }
            
            
            lappend listcreated {*}$f_listcreated
            
            set function [list CreateWidgetsFromXml::ActivateDeactivate $varname $f_listcreated 1]
            trace add variable $varname write $function
            #execute 1 time 
            eval $function "args_not_used"
            
            bind $lf <Destroy> [list +CreateWidgetsFromXml::DestroyTraceVar %W $lf $varname $function]            
        }
        
    } else {            
        grid $cb -sticky w
    }
    incr properties($name,count)
    
    grid $f -sticky we
    if { [$node hasAttribute visible] && ![$node getAttribute visible] } {
        grid remove $f
    }
    $node setAttribute frame $f ;#e.g. to allow change visibility
    GidHelp $listcreated [GetHelpTag $node] 
    #return created widgets
    return $listcreated
}

proc CreateWidgetsFromXml::TakeOutFocus { widget args } {
    $widget state !focus
}

proc CreateWidgetsFromXml::CreateCheckbuttonFrame { node lf name} {
    variable properties
    variable aux
    #as item can have childs we must create a frame to work as a unique widget
    set f [ttk::labelframe $lf.$properties($name,count)]
        
    set framewidget [ttk::frame $f.framewidget]
    set lb [ttk::label $framewidget.label -style TLabelframe.Label {*}[Add2PointsOnTextLabel [GetVisualTags $node label]]]    
    set varname [GetValidVariable $name $node]
    set cb [ttk::checkbutton $framewidget.combobox -variable $varname]
    grid $lb $cb
    $f configure -labelwidget $framewidget
    
    set function [list CreateWidgetsFromXml::TakeOutFocus $framewidget.combobox]
    trace add variable $varname write $function
    bind $lf <Destroy> [list +CreateWidgetsFromXml::DestroyTraceVar %W $lf $varname $function]   
    
    set listcreated [list $f $framewidget $lb $cb]
    
    #it have childs that only will be activate if checkbutton it's on
    if { [$node hasChildNodes] } {
        set f_listcreated [ReadInsideLabelFrame $node $f $name] 
        if { $f_listcreated != "" } {                
            lappend listcreated {*}$f_listcreated
            set onvalue 1
            if { [$node hasAttribute onvalue] } {
                set onvalue [$node getAttribute onvalue]
            }
            set function [list CreateWidgetsFromXml::ActivateDeactivate $varname $f_listcreated $onvalue]
            trace add variable $varname write $function
            #execute 1 time 
            eval $function "args_not_used"
            
            bind $lf <Destroy> [list +CreateWidgetsFromXml::DestroyTraceVar %W $lf $varname $function]            
        }
        
        
    } else {
        WarnWinText "error checkbuttonframe without childs"        
    }
    incr properties($name,count)
    
    grid $f -sticky ew        
    if { [$node hasAttribute visible] && ![$node getAttribute visible] } {
        grid remove $f
    }
    $node setAttribute frame $f ;#e.g. to allow change visibility
    # add help GidHelp
    ApplyGidHelpToNoneHelpWidgets $listcreated [GetHelpTag $node]
    #return created widgets
    return $listcreated
}

proc CreateWidgetsFromXml::ValidateCombobox { validatefunction varname varname_prev name1 name2 op } {
    set value [ set $varname]
    if { [eval [list $validatefunction $value]] || ( $value == "")} {
        # save last valid value
        if { $value != ""} {
            set $varname_prev $value
        }
    } else {
        # restore last valid value
        if { [ info exists $varname_prev]} {
            set $varname [set $varname_prev]
        }
    }
}

proc CreateWidgetsFromXml::CreateCombobox { node lf name } {
    variable properties
    variable aux
    #as item don't accept label we must create a frame to work as a unique widget
    set f [ttk::frame $lf.$properties($name,count)]    
    set l [ttk::label $f.label {*}[Add2PointsOnTextLabel [GetVisualTags $node label]]]
    
    set listvalues ""
    set listlabels ""
    set firstchildnode [lindex [$node childNodes] 0]
    if { [$firstchildnode nodeName] == "option" && [$firstchildnode hasAttribute fillfunction]} {
        set lists [::xmlprograms::[$firstchildnode getAttribute fillfunction]]
        set listvalues [lindex $lists 0]
        set listlabels [lindex $lists 1]
    } else {
        foreach childnode [$node childNodes] {
            if { [$childnode nodeName] == "option" } {
                lappend listvalues [$childnode getAttribute value]
                lappend listlabels [GetLabelTag $childnode]
            } else {        
                #unexpected tag
                WarnWinText "Unexpected tag, expected option and is [$childnode nodeName]"                
            }
        }
    }
    set cbstate [ GetStateTag $node]
    if { $cbstate == ""} {
        set cbstate readonly
    }
    
    set maxlength [expr {2+[CalculateMaxLength $listlabels]}]
    set varname [GetValidVariable $name $node]
    set cbox [TTKComboBox $f.cbox -labels $listlabels -values $listvalues \
            -width $maxlength -textvariable $varname -state $cbstate] 
    
    if { [ $node hasAttribute validation] } {
        set validatefunction [$node getAttribute validation]
        set varname_prev [GetValidVariable ${name}_prev $node]
        set function [list CreateWidgetsFromXml::ValidateCombobox $validatefunction $varname $varname_prev]
        trace add variable $varname write $function
        bind $lf <Destroy> [list +CreateWidgetsFromXml::DestroyTraceVar %W $lf $varname $function]                        
    }
    
    incr properties($name,count)
    grid $l $cbox -sticky we
    grid columnconfigure $f 0 -weight 0
    grid columnconfigure $f 1 -weight 1
    grid $f -sticky we
    if { [$node hasAttribute visible] && ![$node getAttribute visible] } {
        grid remove $f
    }
    $node setAttribute frame $f ;#e.g. to allow change visibility
    set listcreated [list $f $l $cbox]
    GidHelp $listcreated [GetHelpTag $node]    
    return $listcreated
}

proc CreateWidgetsFromXml::OffValueEntry { checkbutton entry } { 
    if { [set [$checkbutton cget -variable]] == [$checkbutton cget -onvalue] } {
        set [$entry cget -textvariable] [$checkbutton cget -onvalue]
    } else {
        set [$entry cget -textvariable] [$checkbutton cget -offvalue] 
    }
}

proc CreateWidgetsFromXml::OffValueEntryAutoCheckbutton { checkbutton name1 name2 op } {   
    set varname ${name1}($name2)
    if { [set $varname] == [$checkbutton cget -offvalue] } {
        #disabled only when variable == offvalue
        set [$checkbutton cget -variable] [$checkbutton cget -offvalue]
    } else {
        set [$checkbutton cget -variable] [$checkbutton cget -onvalue]
    }
}

proc CreateWidgetsFromXml::CreateEntry { node lf name} {
    variable properties
    variable aux
    #as item don't accept label we must create a frame to work as a unique widget
    set f [ttk::frame $lf.$properties($name,count)]    
    
    set varname [GetValidVariable $name $node]
    set e [ttk::entry $f.entry -textvariable $varname]
    if { [$node hasAttribute width] } {
        $e configure -width [$node getAttribute width]
    }
    if { [ $node hasAttribute validation] } {
        set validatefunction [$node getAttribute validation]
        $e configure -validate all -validatecommand [list CreateWidgetsFromXml::ValidateCommand %P %V %W $validatefunction $varname $name]
    }
    if { [$node hasAttribute offvalue] } {
        set ch_varname "[string range $varname 0 end-1],ch)"
        #save in the checkbutton the "offvalue" as offvalue and "ondefaultvalue" as onvalue
        set ch [ttk::checkbutton $f.ch {*}[Add2PointsOnTextLabel [GetVisualTags $node label]] \
                -variable $ch_varname \
                -offvalue [$node getAttribute offvalue] -onvalue [$node getAttribute ondefaultvalue]]

        
        $ch configure -command [list CreateWidgetsFromXml::OffValueEntry $ch $e]
        
        set [$ch cget -variable] [$ch cget -onvalue]
        if { [set $varname] == [$ch cget -offvalue] } {
            set [$ch cget -variable] [$ch cget -offvalue] 
        } 
        trace add variable $varname write [list CreateWidgetsFromXml::OffValueEntryAutoCheckbutton $ch]
        bind $lf <Destroy> [list +CreateWidgetsFromXml::DestroyTraceVar %W $lf $varname [list CreateWidgetsFromXml::OffValueEntryAutoCheckbutton $ch]]  
    } else {
        set l [ttk::label $f.label {*}[Add2PointsOnTextLabel [GetVisualTags $node label]]]
    }
    
    
    incr properties($name,count)
    
    if { [$node hasAttribute offvalue] } {
        grid $ch $e -sticky we
        set listcreated [list $f $ch $e] 
    } else {
        grid $l $e -sticky we
        set listcreated [list $f $l $e]
    }  
    grid $f -sticky we
    grid columnconfigure $f 0 -weight 0    
    grid columnconfigure $f 1 -weight 1
    if { [$node hasAttribute visible] && ![$node getAttribute visible] } {
        grid remove $f
    }
    $node setAttribute frame $f ;#e.g. to allow change visibility
    GidHelp $listcreated [GetHelpTag $node]
    #return created widgets
    return $listcreated
}

proc CreateWidgetsFromXml::CreateEntryWithButton { node lf name} {
    variable properties
    variable aux
    #as item don't accept label we must create a frame to work as a unique widget
    set f [ttk::frame $lf.$properties($name,count)]    
    
    set varname [GetValidVariable $name $node] 
    set e [ttk::entry $f.entry -textvariable $varname]
    if { [$node hasAttribute width] } {
        $e configure -width [$node getAttribute width]
    }
    if { [ $node hasAttribute validation] } {
        set validatefunction [$node getAttribute validation]
        $e configure -validate all -validatecommand [list CreateWidgetsFromXml::ValidateCommand %P %V %W $validatefunction $varname $name]
    }
    if { [$node hasAttribute offvalue] } {
        set ch_varname "[string range $varname 0 end-1],ch)"        
        #save in the checkbutton the "offvalue" as offvalue and "ondefaultvalue" as onvalue
        set ch [ttk::checkbutton $f.ch {*}[Add2PointsOnTextLabel [GetVisualTags $node label]] \
                -variable $ch_varname \
                -offvalue [$node getAttribute offvalue] -onvalue [$node getAttribute ondefaultvalue]]
        
        $ch configure -command [list CreateWidgetsFromXml::OffValueEntry $ch $e]
        
        set [$ch cget -variable] [$ch cget -onvalue]
        if { [set $varname] == [$ch cget -offvalue] } {
            set [$ch cget -variable] [$ch cget -offvalue] 
        } 
        trace add variable $varname write [list CreateWidgetsFromXml::OffValueEntryAutoCheckbutton $ch]
        bind $lf <Destroy> [list +CreateWidgetsFromXml::DestroyTraceVar %W $lf $varname [list CreateWidgetsFromXml::OffValueEntryAutoCheckbutton $ch]]  
    } else {
        set l [ttk::label $f.label {*}[Add2PointsOnTextLabel [GetVisualTags $node label]]]
    }
    if { [$node hasAttribute buttonfunction] } {
        set b [ttk::button $f.button -image [gid_themes::GetImage [$node getAttribute buttonimage] small_icons] \
                -command [list ::xmlprograms::[$node getAttribute buttonfunction] $lf $varname]]                        
    } else {
        set b [ttk::button $f.button -image [gid_themes::GetImage [$node getAttribute buttonimage] small_icons] \
                -command [$node getAttribute buttoncommand]]                
    }
    
    incr properties($name,count)
    if { [$node hasAttribute offvalue] } {
        grid $ch $e $b -sticky we    
        set listcreated [list $f $ch $e $b] 
        
        set function [list CreateWidgetsFromXml::ActivateDeactivate $ch_varname [list $e $b] [$node getAttribute ondefaultvalue]]
        trace add variable $ch_varname write $function
        #execute 1 time 
        eval $function "args_not_used"
            
        bind $lf <Destroy> [list +CreateWidgetsFromXml::DestroyTraceVar %W $lf $ch_varname $function]            
    } else {
        grid $l $e $b -sticky we
        set listcreated [list $f $l $e $b]
    }
    
    
    grid $f -sticky we
    grid columnconfigure $f 1 -weight 1
    $node setAttribute frame $f ;#e.g. to allow change visibility
    GidHelp $listcreated [GetHelpTag $node]
    #return created widgets
    return $listcreated
}

proc CreateWidgetsFromXml::CreateRadiobutton { node lf name } {
    variable properties
    variable aux
    set f [ttk::frame $lf.$properties($name,count)]
    set listcreated $f
    
    set numradiobutton 0
    array set rb {}
    set varname [GetValidVariable $name $node]
    foreach childnode [$node childNodes] {
        if { [$childnode nodeName] == "option" } {
            set rb($numradiobutton) [ttk::radiobutton $f.$numradiobutton \
                    -value [$childnode getAttribute value] {*}[GetVisualTags $childnode radiobutton] \
                    -variable $varname]
            set numradiobutton [expr $numradiobutton+1] 
            
        } else {
            #unexpected tag
            WarnWinText "Unexpected tag, expected option and is [$childnode nodeName]"                
        }
    }      
    set l [ttk::label $f.label {*}[Add2PointsOnTextLabel [GetVisualTags $node label]]]
    incr properties($name,count)
    
    grid $l -sticky w    
    for { set i 0 } { $i<$numradiobutton} { incr i } {
        grid $rb($i) -row 0 -column [expr $i+1] -sticky w
        lappend listcreated $rb($i)
    }
    grid $f -sticky w
    if { [$node hasAttribute visible] && ![$node getAttribute visible] } {
        grid remove $f
    }
    $node setAttribute frame $f ;#e.g. to allow change visibility
    GidHelp $listcreated [GetHelpTag $node]
    #return created widgets    
    return $listcreated
}

proc CreateWidgetsFromXml::CreateSpinbox { node lf name} {
    variable properties
    variable aux
    #as item don't accept label we must create a frame to work as a unique widget
    set f [ttk::frame $lf.$properties($name,count)]    
    set l [ttk::label $f.label {*}[Add2PointsOnTextLabel [GetVisualTags $node label]]]
    
    set varname [GetValidVariable $name $node]
    set from [$node getAttribute from]
    set to [$node getAttribute to]
    set increment [$node getAttribute increment]
    set maxlength [expr {1+[CalculateWidthSpinbox $from $to $increment]}]
    set sp [ttk::spinbox $f.spinbox -textvariable $varname -from $from -to $to -increment $increment -width $maxlength]
    if { [ $node hasAttribute validation] } {
        set validatefunction [$node getAttribute validation]
        $sp configure -validate all -validatecommand [list CreateWidgetsFromXml::ValidateCommand %P %V %W $validatefunction $varname $name]
    }
    
    incr properties($name,count)
    
    grid $l $sp -sticky we
    grid columnconfigure $f 0 -weight 0
    grid columnconfigure $f 1 -weight 1
    grid $f -sticky we
    if { [$node hasAttribute visible] && ![$node getAttribute visible] } {
        grid remove $f
    }
    $node setAttribute frame $f ;#e.g. to allow change visibility
    set listcreated [list $f $l $sp]
    GidHelp $listcreated [GetHelpTag $node]
    #return created widgets
    return $listcreated
}

proc CreateWidgetsFromXml::CreateLabel { node lf name} {
    variable properties
    set l [ttk::label $lf.$properties($name,count) {*}[Add2PointsOnTextLabel [GetVisualTags $node label]]]
    incr properties($name,count)
    grid $l -sticky w
    if { [$node hasAttribute visible] && ![$node getAttribute visible] } {
        grid remove $l
    }
    $node setAttribute frame $l ;#e.g. to allow change visibility
    GidHelp $l [GetHelpTag $node]
    #return created widgets
    return $l
}

proc CreateWidgetsFromXml::CreateButton { node lf name} {
    variable properties
    if { [$node hasAttribute commandfunction] } {
        set b [ttk::button $lf.$properties($name,count) -command [list ::xmlprograms::[$node getAttribute commandfunction] $lf] {*}[GetVisualTags $node button]]        
    } else {
        set b [ttk::button $lf.$properties($name,count) -command [$node getAttribute command] {*}[GetVisualTags $node button]]
    }
    incr properties($name,count)
    grid $b -sticky w
    if { [$node hasAttribute visible] && ![$node getAttribute visible] } {
        grid remove $b
    }
    $node setAttribute frame $b ;#e.g. to allow change visibility
    GidHelp $b [GetHelpTag $node]
    #return created widgets
    return $b
}

proc CreateWidgetsFromXml::CreateScale { node lf name} {
    variable properties
    variable aux
    set f [ttk::frame $lf.$properties($name,count)]  
    grid $f -sticky we  
    
    set scaletext [GetVisualTags $node label]
    set numcolumns 0
    set listcreated $f
    if { $scaletext!= "" } {
        set l [ttk::label $f.label {*}[Add2PointsOnTextLabel $scaletext]]
        grid $l -column $numcolumns -row 0 -sticky we
        incr numcolumns
        lappend listcreated $l
    }
    set varname [GetValidVariable $name $node]
    
    set childnodes [$node childNodes]
    if { [llength $childnodes] == 1} {
        if  { [$childnodes nodeName] == "entry" } {
            if { [$childnodes hasAttribute state] && [$childnodes getAttribute state]=="readonly" } {               
                    set e [ttk::label $f.entry -textvariable $varname]
                    if { [$childnodes hasAttribute width] } {               
                        $e configure -width [$childnodes getAttribute width]
                    }
                    grid $e -column $numcolumns -row 0 -sticky we
            } else {
                set validatefunction [$childnodes getAttribute validation]
                set e [ttk::entry $f.entry -textvariable $varname -validate all \
                    -validatecommand [list CreateWidgetsFromXml::ValidateCommand %P %V %W $validatefunction $varname $name]]
                foreach attribute {width state} {            
                    if { [$childnodes hasAttribute $attribute] } {               
                        $e configure -$attribute [$childnodes getAttribute $attribute]
                    }
                }
            }
            grid $e -column $numcolumns -row 0 -sticky we            
            incr numcolumns 
            lappend listcreated $e
        } else {
            #unexpected tag
            WarnWinText "Unexpected tag, expected only 1 entry and is [$childnodes nodeName]"                
            
        }  
    }
    
    set sc [gidscale $f.scale -from [$node getAttribute from] \
            -to [$node getAttribute to] -orient horizontal -resolution [$node getAttribute resolution] \
            -showvalue [$node getAttribute showvalue] -showbuttons [$node getAttribute showbuttons] \
            -variable $varname]
    
    incr properties($name,count)
    
    grid $sc -column $numcolumns  -row 0 -sticky we
    grid columnconfigure $f $numcolumns -weight 1
    incr numcolumns
    lappend listcreated $sc
    
      
    if { [$node hasAttribute visible] && ![$node getAttribute visible] } {
        grid remove $f
    }
    $node setAttribute frame $f ;#e.g. to allow change visibility
    GidHelp $listcreated [GetHelpTag $node]
    #return created widgets
    return $listcreated
}

#dark trick syncronizing with an auxiliary variables to not use use directly $varname as -listvariable for tablelist
proc CreateWidgetsFromXml::SetLinkedVariable { auxiliary_varname varname name1 name2 op } {   
    if { [set $varname] != [set $auxiliary_varname] } {
        #without the if will be an infinite closed loop set a -> set b -> set a ...
        set $varname [set $auxiliary_varname]
    }
}

proc CreateWidgetsFromXml::CreateTableList { node lf name} {
    package require tablelist_tile
    variable properties
    variable aux
    #as item don't accept label we must create a frame to work as a unique widget
    set f [ttk::frame $lf.$properties($name,count)]    
       
    set varname [GetValidVariable $name $node]    
    #not use directly $varname as -listvariable because tablelist to dark tricks
    #like upvar #0 $data(-listvariable) var (and maybe change other traces?)
    #then CreateWidgetsFromXml::AuxTraceFunc fail because it uses this name
    set auxiliary_varname "CreateWidgetsFromXml::aux(tmp,$name,$node,auxiliary)"

    #MUST try first a remove operation to delete this trace if previously was set to avoid repeated traces!!    
    trace add variable $auxiliary_varname write [list CreateWidgetsFromXml::SetLinkedVariable $auxiliary_varname $varname]    
    trace add variable $varname write [list CreateWidgetsFromXml::SetLinkedVariable $varname $auxiliary_varname]
    
    
    set tl [tablelist::tablelist $f.tablelist -listvariable $auxiliary_varname]    
    set options [CreateWidgetsFromXml::GetWidgetOptions $node $tl]
    if { [llength $options] } {
        $tl configure {*}$options
    }
    foreach childnode [$node childNodes] {
        set valid_commands {cellconfigure columnconfigure rowconfigure} ;#sorted list
        set command [$childnode nodeName]
        if { [lsearch -sorted -dictionary $valid_commands $command] == -1 } {            
            W "Unexpected tag [$childnode nodeName], expected $valid_commands"
        } else {
            if { [$childnode hasAttribute index] } {
                set index [$childnode getAttribute index]
            } else {
                set index -1
            }
            if { $index < 0 } {
                W "$command require an attribute 'index' of the column"
            } else {
                set ignored_options {index} ;#sorted list. ignore because are considered in a special way
                set options [list]
                foreach item [$tl $command $index] {
                    set option [string range [lindex $item 0] 1 end]
                    if { [lsearch -sorted -dictionary $ignored_options $option] == -1 } {        
                        if { [$childnode hasAttribute $option] } {
                            lappend options -$option [$childnode getAttribute $option]
                        }
                    }
                }
                if { [llength $options] } {
                    $tl $command $index {*}$options
                }
            }
        }
    }
    
    incr properties($name,count)
    
    grid $tl -sticky we
    grid columnconfigure $f 0 -weight 1   
    grid $f -sticky we
    if { [$node hasAttribute visible] && ![$node getAttribute visible] } {
        grid remove $f
    }
    $node setAttribute frame $f ;#e.g. to allow change visibility
    set listcreated [list $f $tl]
    GidHelp $listcreated [GetHelpTag $node]
    #return created widgets
    return $listcreated
}

proc CreateWidgetsFromXml::selectEntityColor { place var } {
    set retcol [GIDChooseColor $place.selectcolor -title [_ "Select color"] -color [set $var]]    
    # Set preference
    if { $retcol != "" } {
        set  $var $retcol
        $place configure -background [set $var]
        set res 1
    } else {
        set res 0
    }    
    return $res
}

#change from GiD color varible style rrr#ggg#bbb (xxx from 0 to 255)
#to Tk style #rrggbb hexadecimals
proc CreateWidgetsFromXml::GiDColorToTkColor { colorgid } {
    scan $colorgid  %03d#%03d#%03d r g b
    return #[format %02x%02x%02x $r $g $b]
}

#change from Tk style color #rrggbb hexadecimals
#to GiD color varible style rrr#ggg#bbb (xxx from 0 to 255)
proc CreateWidgetsFromXml::TkColorToGiDColor { tkcolor } {
    scan $tkcolor #%2x%2x%2x r g b
    return [format %03d#%03d#%03d $r $g $b]
}

proc CreateWidgetsFromXml::CreateColorButton { node lf name} {  
    variable properties
    variable aux

    set f [ttk::frame $lf.$properties($name,count)]    
    #set l [ttk::label $f.label {*}[AddEndPointOnTextLabel [GetVisualTags $node label]]]     
    set l [ttk::label $f.label {*}[GetVisualTags $node label]]     
    set varname [GetValidVariable $name $node]

    if { $::tcl_platform(os) != "Darwin"} {
	set b [tk::button $f.colorbutton -image [gid_themes::GetImage blank.png small_icons] -width 20 -height 8 -background [set $varname] \
		   -command [list CreateWidgetsFromXml::selectEntityColor $f.colorbutton $varname]]
    } else {
    # -width 20 -height 8
	set b [ LabelButton $f.colorbutton -width 3 -borderwidth 1 -background [set $varname] \
		   -command [ list CreateWidgetsFromXml::selectEntityColor $f.colorbutton $varname]]
    }
    
    lappend properties($name,listfunctions_tocall) [list $f.colorbutton configure -background \$$varname]
    incr properties($name,count)
    grid $b $l -sticky w -pady 1
    grid $f -sticky w
    if { [$node hasAttribute visible] && ![$node getAttribute visible] } {
        grid remove $f
    }
    $node setAttribute frame $f ;#e.g. to allow change visibility
    set listcreated [list $f $l $b]
    GidHelp $listcreated [GetHelpTag $node]
    #return created widgets
    return $listcreated
}

proc CreateWidgetsFromXml::ShowChooseFont { node name samplewidget {recalculatetype 0} } {

    variable aux
    set varname [GetValidVariable $name $node]
    set $varname [list [set ::$node.font] [set ::$node.type] [set ::$node.size]]        
    
    if { $recalculatetype } {
        set lst_ttf_fonts [ GiD_PGF_Fonts get]
        set types ""
        foreach ifont $lst_ttf_fonts {
            #WarnWinText [lindex [ lindex $ifont 2] 0]
            if { [lindex [ lindex $ifont 2] 0] == [set ::$node.font] } {
                lappend types [lindex [ lindex $ifont 2] 1]
            }
        }
        #WarnWinText $types
        if { $types == "" } {
            set types {Regular}
        }
        set maxlength [expr {2+[CalculateMaxLength $types]}]
        [set ::$node.typewidget] configure -labels $types -width $maxlength
        if { [lsearch -exact $types [set ::$node.type]]==-1 } {
            [set ::$node.typewidget] current 0
            set $varname [list [set ::$node.font] [set ::$node.type] [set ::$node.size]]    
        }
    }
    
    set result [list [set ::$node.font] [set ::$node.size]]
    if { [set ::$node.type] != "Regular" } {
        set typefont [set ::$node.type]
        set addons ""
        set boldindex [lsearch $typefont Bold]
        if { $boldindex != -1 } {
            lappend addons "bold"
            set typefont [lreplace $typefont $boldindex $boldindex]
        }
        set italicindex [lsearch $typefont Italic] 
        if { $italicindex != -1 } {
            lappend addons "italic"
            set typefont [lreplace $typefont $italicindex $italicindex]
        }
        if { $typefont != "" } {
            set index [lsearch $typefont Regular]
            set typefont [lreplace $typefont $index $index]
            set result [list [list [set ::$node.font] $typefont] [set ::$node.size] $addons]        
        } else {
            lappend result $addons
        }
    }
    
    #WarnWinText $result
    
    $samplewidget config -font $result
    
    #set $varname [list $result] 
    
    #Exemple 'PGF_DefaultFont'=({C:\Windows\Fonts\cour.ttf} 0 14 {{Courier New} {Regular} {Fixed width}})
    #set $varname [list {} 0 [set ::$node.size] [list [set ::$node.font] $name {}]] 
}

proc CreateWidgetsFromXml::FromGiDVarFontToThreeCombo { var node args } { 
    
    if { [info exists $var] } { 
        #WarnWinText "node: $node   var: $var    value: [set $var]"
        #each parameter its an index
        set ::$node.font [lindex [set $var] 0]
        set ::$node.type [lindex [set $var] 1]
        set ::$node.size [lindex [set $var] 2]  
        #} else {
        #  WarnWinText "node: $node   var: $var"
    }
}
proc CreateWidgetsFromXml::CreateFontSelection { node lf name} {
    variable properties
    variable aux
    # GiD give 'PGF_DefaultFont'=({C:\Windows\Fonts\cour.ttf} 0 14 {{Courier New} {Regular} {Fixed width}})
    # On preprocess variable its transform to: 
    # Input format {{Courier New} {Regular} 14}
    set varname [GetValidVariable $name $node]  
    
    
    set function "CreateWidgetsFromXml::FromGiDVarFontToThreeCombo $varname $node"
    trace add variable $varname write $function
    #execute 1 time 
    eval $function "args_not_used"
    
    bind $lf <Destroy> [list +CreateWidgetsFromXml::DestroyTraceVar %W $lf $varname $function]            
    
    # Output format {{Courier New} {Regular} 14}
    #'On postrocess variable its transform to: {C:\Windows\Fonts\cour.ttf} 0 14 (that its GiD input)
    
    set f [ttk::frame $lf.$properties($name,count)]
    set upf [ttk::frame $f.upf]
    set downf [ttk::frame $f.downf]
            
    set samplewidget [tk::label $upf.sample {*}[GetVisualTags $node label] -anchor center]        
    
    set firstchildnode [lindex [$node childNodes] 0]
    if { [$firstchildnode nodeName] == "option" && [$firstchildnode hasAttribute fillfunction]} {
        set lists [::xmlprograms::[$firstchildnode getAttribute fillfunction]]
        set listvalues [lindex $lists 0]
        set listlabels [lindex $lists 1]
    } else {
        foreach childnode [$node childNodes] {
            if { [$childnode nodeName] == "option" } {
                lappend listvalues [$childnode getAttribute value]
                lappend listlabels [GetLabelTag $childnode]
            } else {        
                #unexpected tag
                WarnWinText "Unexpected tag, expected option and is [$childnode nodeName]"                
            }
        }
    }                
    set maxlength [expr {2+[CalculateMaxLength $listlabels]}]
    
    set cbfonts [TTKComboBox $downf.cbfonts -labels $listlabels -values $listvalues \
            -width $maxlength -textvariable ::$node.font -state readonly]
    
    #set sizes {8 9 10 11 12 14 16 18 20 22 24 26 28 36 48 72}
    #  -labels $sizes
    #  -textvariable  ::$node.size
    #  -width 2
    #  -state readonly
    
    set cbsizes [ttk::spinbox $downf.cbsizes \
            -textvariable ::$node.size \
            -from 8 -to 72 -increment 1 -width 3 \
            -validate all \
            -validatecommand [list CreateWidgetsFromXml::ValidateCommand %P %V %W IsNotEmpty ::$node.size $name] \
            -command [list CreateWidgetsFromXml::ShowChooseFont $node $name $samplewidget]]
    
    
    set maxlength 7
    set cbtype [TTKComboBox $downf.cbtype -labels "" -textvariable ::$node.type -width $maxlength -state readonly]
    set ::$node.typewidget $cbtype
    
    CreateWidgetsFromXml::ShowChooseFont $node $name $samplewidget 1
    bind $cbfonts <<ComboboxSelected>> [list CreateWidgetsFromXml::ShowChooseFont $node $name $samplewidget 1]    
    bind $cbtype <<ComboboxSelected>> [list CreateWidgetsFromXml::ShowChooseFont $node $name $samplewidget]
    
    #WarnWinText [bind $cbfonts <<ComboboxSelected>>]
    
    incr properties($name,count)
    grid $samplewidget -column 0 -row 0 -sticky wens -padx {10 10} -pady {0 0}
    grid columnconfigure $upf 0 -weight 1  
    grid rowconfigure $upf 0 -minsize 40 -weight 1
    grid propagate $upf 0
    grid $cbfonts $cbtype $cbsizes -sticky we  
    grid columnconfigure $downf 0 -weight 1 
    grid $upf -sticky wens
    grid $downf -sticky we  
    grid columnconfigure $f 0 -weight 1
    grid rowconfigure $f 0 -minsize 40 -weight 1
    grid $f -sticky we
    if { [$node hasAttribute visible] && ![$node getAttribute visible] } {
        grid remove $f
    }
    $node setAttribute frame $f ;#e.g. to allow change visibility
    
    set listcreated [list $f $upf $downf $cbfonts $cbtype $cbsizes $samplewidget]
    GidHelp $listcreated [GetHelpTag $node]
    return $listcreated  
}

proc CreateWidgetsFromXml::AuxTclDrawTrace {a b c d args} {
    $a $b $c {*}[subst $d]
}

proc CreateWidgetsFromXml::CreateTclDraw { node lf name} {  
    variable properties
    set f [ttk::frame $lf.$properties($name,count)]    
    ::xmlprograms::[$node getAttribute function] $f $f
    if { [$node hasAttribute updatefunction] } {
        set listtoevaluate ""
        set listvariables ""
        foreach var [$node getAttribute updatevariables] {
            #add trace to the variable
            set var [GetTmpVariableFromVariable $name $var]
            lappend listtoevaluate "$$var"
            lappend listvariables $var
        }
        ::xmlprograms::[$node getAttribute updatefunction] $f $f {*}[subst $listtoevaluate]
        foreach var $listvariables {
            trace add variable $var write [list CreateWidgetsFromXml::AuxTclDrawTrace ::xmlprograms::[$node getAttribute updatefunction] $f $f $listtoevaluate]
            bind $lf <Destroy> [list +CreateWidgetsFromXml::AuxTclDrawTrace ::xmlprograms::[$node getAttribute updatefunction] $f $f $listtoevaluate]
        }
    }
    
    incr properties($name,count)
    
    grid $f -sticky w
    if { [$node hasAttribute visible] && ![$node getAttribute visible] } {
        grid remove $f
    }
    $node setAttribute frame $f ;#e.g. to allow change visibility
    return $f        
}

proc CreateWidgetsFromXml::CreateLabelFrame { node lf name} {
    variable properties
    #as item can have childs we must create a frame to work as a unique widget
    set f [ttk::labelframe $lf.$properties($name,count) {*}[GetVisualTags $node labelframe]]
    set listcreated $f
    
    #it have childs that only will be activate if checkbutton it's on
    if { [$node hasChildNodes] } {
        set f_listcreated [ReadInsideLabelFrame $node $f $name] 
        if { $f_listcreated != "" } {     
            lappend listcreated {*}$f_listcreated
        }
    } else {
        WarnWinText "error labelframe without childs"        
    }
    incr properties($name,count)
    
    grid $f -sticky ew
    grid columnconfigure $f 0 -weight 1
    grid rowconfigure $f 0 -weight 1
    if { [$node hasAttribute visible] && ![$node getAttribute visible] } {
        grid remove $f
    }     
    $node setAttribute frame $f ;#e.g. to allow change visibility
    # add help GidHelp
    ApplyGidHelpToNoneHelpWidgets $listcreated [GetHelpTag $node]  
    #return created widgets
    return $listcreated       
}

#--------------end:widget functions-------------------
#utils
proc CreateWidgetsFromXml::ShowAndSelectWidget { name } {
    variable properties
    set pos [string length $name,allvariables,]
    foreach {item varvalue} [array get properties $name,allvariables,*] {
        set varname [string range $item $pos end]
        if { [CreateWidgetsFromXml::IsVariableTmp $varname] } {
            set nodename [CreateWidgetsFromXml::GetNodenameFromVariableTmp $varname $name]
            set xmlalias [$nodename getAttribute variable]
        }
    }    
}

proc CreateWidgetsFromXml::Rebuild {} {
    #redoo tcldraw or update combobox list
}

proc CreateWidgetsFromXml::ChangeAllVariablesValues { {name "gidPreferences"} } {
    variable properties
    set pos [string length $name,allvariables,]
    foreach {item varvalue} [array get properties $name,allvariables,*] {
        set varname [string range $item $pos end]
        if { [CreateWidgetsFromXml::IsVariableTmp $varname] } {
            set nodename [CreateWidgetsFromXml::GetNodenameFromVariableTmp $varname $name]
            set gidvarname [$nodename getAttribute variable]
            
            set uservalue [set $varname]  
            
            set varmanager ""
            if { [$nodename hasAttribute variablemanager] } {
                set varmanager [$nodename getAttribute variablemanager]
            }                
            set value [GetValue $gidvarname $varmanager]
            
            if { [$nodename hasAttribute whenreadvar] } {
                set inputvalue [::xmlprograms::[$nodename getAttribute whenreadvar] [list $value]]
            } else {
                set inputvalue $value 
            }
            #varvalue its the value read from GiD when open the window (reference value to calculate it have changes)
            #uservalue its the user decision, if uservalue!=varvalue the there are changes without apply
            #inputvalue its the actual value of GiD (input value of this function)
            if { $varvalue != $uservalue } {
                #there are user changes
                set properties($name,allvariables,$varname) $inputvalue
                #its needed to recalculate if there are changes
                set $varname [set $varname]
            } else {
                set properties($name,allvariables,$varname) $inputvalue
                set $varname $inputvalue
            }
            
        }        
    }                        
    
}

proc CreateWidgetsFromXml::ChangeVariableValue { {name "gidPreferences"} {var 0} {value 0} } {
    variable properties
    set pos [string length $name,allvariables,]
    foreach {item varvalue} [array get properties $name,allvariables,*] {
        set varname [string range $item $pos end]    
        if { [CreateWidgetsFromXml::IsVariableTmp $varname] } {
            set nodename [CreateWidgetsFromXml::GetNodenameFromVariableTmp $varname $name]
            set gidvarname [$nodename getAttribute variable]
            if { $gidvarname == $var } {
                #variable found
                
                set uservalue [set $varname]        
                if { [$nodename hasAttribute whenreadvar] } {
                    set inputvalue [::xmlprograms::[$nodename getAttribute whenreadvar] [list $value]]
                } else {
                    set inputvalue $value 
                }
                #varvalue its the value read from GiD when open the window (reference value to calculate it have changes)
                #uservalue its the user decision, if uservalue!=varvalue the there are changes without apply
                #inputvalue its the actual value of GiD (input value of this function)
                if { $varvalue != $uservalue } {
                    #there are user changes
                    set properties($name,allvariables,$varname) $inputvalue
                    #its needed to recalculate if there are changes
                    set $varname [set $varname]
                } else {
                    set properties($name,allvariables,$varname) $inputvalue
                    set $varname $inputvalue
                }
                #ChangeWarnLineVar [_ "Some variables on preferences has been changed"]                
                return
            }        
        }                        
    }
} 


proc CreateWidgetsFromXml::CreateAndVisualizeTreeLeave { node w fulltree leaveid name} {
    variable properties
    variable aux
    set properties($name,current_group) [$node getAttribute name]
    
    #set f [ttk::frame $w.[$node getAttribute name]]
    variable printactualvaluesasdefault
    
    set listslaves [grid slaves $w]
    if { $listslaves != "" } {
        grid forget $listslaves
    }
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1
    if { [winfo exists $w.[$node getAttribute name]] } {
        grid $w.[$node getAttribute name] -sticky news
    } else {
        set l [ttk::label $w.l_[$node getAttribute name] -text [_ "Loading"]...]
        grid $l -sticky news
        update idletasks
        
        #set f [frame $w.[$node getAttribute name] -bg blue]
        set c [ CreateScrolledCanvas $w.[$node getAttribute name]]
        $w.[$node getAttribute name] configure -relief flat                
        set f [ ttk::frame $c.f]
        
        
        
        #grid $f -sticky news
        
        array set managedvariables [InitialitateAllVariablesOfChildNode $node $name]    
        
        foreach item $properties($name,listfunctions_tocall) {
            eval [subst $item]
        }
        
        foreach {item value} [array get managedvariables] {
            set properties($name,allvariables,$item) $value
        }        
        
        foreach childnode [$node childNodes] {
            if { [$childnode nodeName] != "group" } {
                if { [$childnode nodeName] == "labelframe" } {
                    set lf [ttk::labelframe $f.[$childnode getAttribute name] {*}[GetVisualTags $childnode labelframe]]
                    grid $lf -sticky ew
                    grid columnconfigure $lf 0 -weight 0            
                    grid rowconfigure $lf 0 -weight 1
                    ReadInsideLabelFrame $childnode $lf $name 
                    grid columnconfigure $lf 0 -minsize 340
                    
                    GidHelp $lf [GetHelpTag $childnode]
                    
                } elseif { [$childnode nodeName] == "checkbuttonframe" } {
                    set lf [ttk::labelframe $f.[$childnode getAttribute name] ]                    
                    set framewidget [ttk::frame $lf.framewidget]
                    set lb [ttk::label $lf.framewidget.label -style TLabelframe.Label {*}[Add2PointsOnTextLabel [GetVisualTags $childnode label]]]    
                    set varname [GetValidVariable $name $childnode]
                    set cb [ttk::checkbutton $lf.framewidget.combobox -variable $varname]   
                    grid $lb $cb
                    grid columnconfigure $lf 1 -weight 0            
                    grid rowconfigure $lf 0 -weight 0
                    $lf configure -labelwidget $framewidget
                    
                    #it have childs that only will be activate if checkbutton it's on
                    if { [$childnode hasChildNodes] } {
                        set f_listcreated [ReadInsideLabelFrame $childnode $lf $name] 
                        if { $f_listcreated != "" } {             
                            set function [list CreateWidgetsFromXml::ActivateDeactivate $varname $f_listcreated 1]
                            trace add variable $varname write $function
                            #execute 1 time 
                            eval $function "args_not_used"
                            bind $lf <Destroy> [list +CreateWidgetsFromXml::DestroyTraceVar %W $lf $varname $function]            
                        }
                    } else {
                        WarnWinText "error checkbuttonframe without childs"        
                    }
                    
                    grid $lf -sticky ew
                    grid columnconfigure $lf 0 -minsize 340
                    
                    GidHelp [list $lf $framewidget $lb $cb] [GetHelpTag $childnode]
                    
                } elseif { [$childnode nodeName] != "#comment" } {
                    #unexpected tag
                    WarnWinText "Unexpected tag, expected labelframe and is [$childnode nodeName]"                
                }
            }
        }
        set defaultvalues [ttk::button $f.setdefaultvalues -text [_ "Default values"] \
                -command [list CreateWidgetsFromXml::GetDefaultValues [array get managedvariables] $name ]]
        
        if { $fulltree != "" } {
            CreateTracesAssociatedToALeave $f [array names managedvariables] $fulltree $leaveid $name
        }
        
        grid $defaultvalues -sticky s  
        
        
        grid columnconfigure $f 0 -weight 1
        #last row:
        grid rowconfigure $f [expr [lindex [grid size $f] 1] -1] -weight 1
        
        AddToScrolledCanvas $w.[$node getAttribute name] $f
        
        grid forget $l
        
        
        grid $w.[$node getAttribute name] -sticky news
    }    
}

proc CreateWidgetsFromXml::ContextualDefaultValues { name T } {
    variable properties    
    #--------
    #execute w.[$node getAttribute name].f.setdefaultvalues
    
    set tree $T
    
    set listids [$tree selection get]
    foreach id $listids {
        
        set item [$tree item id $id]
        set mb [$tree item text $id]
        set domchild [lindex $mb 2]
        #$::ContentL configure -text "Namebytree:[lindex $mb 0], name tag xml:[$domchild getAttribute name]"   
        set root [$domchild root]
        
        
        while { [[winfo parent $tree] is_folder $id] } {
            set id [expr $id+1]
            set item [$tree item id $id]
            set mb [$tree item text $id]
            set domchild [lindex $mb 2]
        }
      
        CreateWidgetsFromXml::CreateAndVisualizeTreeLeave $domchild $properties($name,content_frame) [winfo parent $tree] $id $name
        $properties($name,content_frame).[$domchild getAttribute name].c.f.setdefaultvalues invoke
 
    }   
}
proc CreateWidgetsFromXml::ContextualUndoChanges { name T } {
    variable properties    
    #--------
    #execute w.[$node getAttribute name].f.setdefaultvalues
    
    set tree $T    
    set listids [$tree selection get]
    set workingids ""
    foreach id $listids {
        while { [[winfo parent $tree] is_folder $id] } {
            set id [expr $id+1]            
        }
        lappend workingids $id
    }
    set workingids [lsort -increasing -unique $workingids] 
    
    
    set pos [string length $name,changedvariables,]
    foreach {item listvariables} [array get properties $name,changedvariables,*] {
        set leaveid [string range $item $pos end]
        if { [lsearch $workingids $leaveid] != -1 } {
            foreach varname $listvariables {
                set $varname $properties($name,allvariables,$varname)
            }
        }
    }  
}


proc CreateWidgetsFromXml::MenuContextual { name T x y } {
    set w $T.menucontextual
    if { [winfo exists $w] } {
        destroy $w
    }

    menu $w

    $w add command -label [_ "Default values on selection"] -command [list CreateWidgetsFromXml::ContextualDefaultValues $name $T]
    $w add command -label [_ "Undo changes on selection"] -command [list CreateWidgetsFromXml::ContextualUndoChanges $name $T]
    
    
    set x [expr [winfo rootx $T]+$x+2]
    set y [expr [winfo rooty $T]+$y]
    GiD_PopupMenu $w $x $y
}

proc CreateWidgetsFromXml::LogData { name args } {
#WarnWinText "LogData: $name $args"
    variable properties
    #$::MainL configure -text "logdata: $args"
    if { [lindex $args 0] == "select" } {
        set tree [lindex $args 1]
        set id [lindex $args 2]
        if { $id == "" } {
            return
        } elseif { [llength $id] > 1 } {
            set id [lindex [lsort -integer $id] 0]
        }
        #$tree selection clear
        $tree selection modify $id {}
        
        #$tree activate $id
        
        set item [$tree item id $id]
        set mb [$tree item text $id]
        set domchild [lindex $mb 2]
        #$::ContentL configure -text "Namebytree:[lindex $mb 0], name tag xml:[$domchild getAttribute name]"   
        set root [$domchild root]
        #set rootnext [$root parentNode]
        #while { $rootnext != "" } {
            #  set root $rootnext
            #  set rootnext [$root parentNode]
            #}
        
        
        while { [[winfo parent $tree] is_folder $id] } {
            set id [expr $id+1]
            set item [$tree item id $id]
            set mb [$tree item text $id]
            set domchild [lindex $mb 2]
        }
        #we need name!!!
        
#         if { [catch {CreateWidgetsFromXml::CreateAndVisualizeTreeLeave $domchild $properties($name,content_frame) [winfo parent $tree] $id $name} err] } {
#             WarnWinText "ERROR:\n$err\n" 
#             for { set i [info level]} { $i >0 } { incr i -1 } {
#                 WarnWinText [info level $i]
#             }
#         }        
        CreateWidgetsFromXml::CreateAndVisualizeTreeLeave $domchild $properties($name,content_frame) [winfo parent $tree] $id $name
        
        set tree [lindex $args 1]
        set sel [$root selectNodes selected]
        if { $sel == "" } {
            $root appendXML {<selected></selected>}
            set sel [$root selectNodes selected]
        }
        $sel setAttribute selection [$tree selection get]
    }  
}
