
# Get default colors
set w [listbox .listbox]
set SystemHighlight [$w cget -selectbackground]
set SystemHighlightText [$w cget -selectforeground]
destroy $w
if { $::tcl_platform(platform) == "unix" } {
    # I hate that gray selection color
    set SystemHighlight #316ac5
    set SystemHighlightText White
}

proc EncodeName { x } {
    return [encrypter BASE64 encode [encoding convertto utf-8 $x]]
}

proc DecodeName { x } {
    return [encoding convertfrom utf-8 [encrypter BASE64 decode $x]]
}

proc SetLayerAsSelection { T layer } {
    if { ![winfo exists $T] } { return }
    $T selection clear
    set item [$T item id "tag [EncodeName $layer]"]
    if { $item != "" } {
	$T selection add $item
	$T see $item
    }
#     SetSelection $T
}

proc SelectLayersByRegexpPressOk { T e } {
    global wChangeLay LayerSelRegexp LayerSelAdd LayerSelCase
    if { $LayerSelAdd == "substitute" } {
	$T selection clear
    }
    foreach item [$T item range 1 end] {
	#set fullname [DecodeName [$T item tag names $item]]
	set i [$T item text $item 0]
	set change ""
	if { $LayerSelRegexp != "regex" } {
	    if { $LayerSelCase != "case" } {
		if { [string match [string tolower [$e get]] \
		        [string tolower $i]] } { set change yes }
	    } else {
		if { [string match [$e get] $i] } {
		    set change yes
		}
	    }
	} else {
	    if { $LayerSelCase != "case" } {
		if { [regexp -nocase -- [$e get] $i res] } {
		    if { $i == $res } { set change yes }
		}
	    } else {
		if { [regexp -- [$e get] $i res] } {
		    if { $i == $res } { set change yes }
		}
	    }
	}
	if { $change == "yes" } {
	    if { $LayerSelAdd != "substract" } {
		$T selection add $item
	    } else {
		$T selection clear $item
	    }
	}
    }
#     SetSelection $T
    destroy $wChangeLay.sel
    if { [winfo exists $wChangeLay.ent.e] } {
	focus $wChangeLay.ent.e
    }
}

proc SelectLayersByRegexp { tbl } {
    global wChangeLay LayerSelRegexp LayerSelAdd LayerSelCase

    if { [winfo exists $wChangeLay.sel] } {
	 destroy $wChangeLay.sel
    }
    ttk::frame $wChangeLay.sel -style ridge.TFrame -borderwidth 4
    ttk::frame $wChangeLay.sel.up
    set e [ttk::entry $wChangeLay.sel.up.e]
    ttk::button $wChangeLay.sel.up.ok -text [_ "OK"] \
	    -command [list SelectLayersByRegexpPressOk $tbl $e]
    grid $e $wChangeLay.sel.up.ok -sticky ew
    grid configure $wChangeLay.sel.up.ok -sticky w
    grid columnconfigure $wChangeLay.sel.up 0 -weight 1

    ttk::frame $wChangeLay.sel.down -borderwidth 2
    if { ![info exists LayerSelRegexp] } { set LayerSelRegexp 0 }
    ttk::checkbutton $wChangeLay.sel.down.reg -text [_ "Regexp"] -onvalue regex \
	    -variable LayerSelRegexp
	if { ![info exists LayerSelCase] } { set LayerSelCase 0 }
    ttk::checkbutton $wChangeLay.sel.down.case -text [_ "Case"] -onvalue case \
	    -variable LayerSelCase

    ttk::frame $wChangeLay.sel.rad -borderwidth 2
    ttk::radiobutton $wChangeLay.sel.rad.substitute -text [_ "Substitute"] -value substitute \
	    -variable LayerSelAdd
    ttk::radiobutton $wChangeLay.sel.rad.add -text [_ "Add"] -value add \
	    -variable LayerSelAdd
    ttk::radiobutton $wChangeLay.sel.rad.substract -text [_ "Substract"] -value substract \
	    -variable LayerSelAdd

    set LayerSelRegexp ""
    set LayerSelAdd substitute
    set LayerSelCase ""

    grid $wChangeLay.sel.down.reg $wChangeLay.sel.down.case -sticky ew

    grid $wChangeLay.sel.rad.substitute $wChangeLay.sel.rad.add \
	    $wChangeLay.sel.rad.substract -sticky ew

    grid $wChangeLay.sel.up -sticky ew
    grid $wChangeLay.sel.down -sticky ew
    grid $wChangeLay.sel.rad -sticky ew

    grid $wChangeLay.sel -row 1 -sticky new
    grid columnconfigure $wChangeLay.sel 0 -weight 1

    focus $e
    bind $e <Return> [list $wChangeLay.sel.up.ok invoke]
}

proc FillInfoLayers { } {
    global wChangeLay GidPriv
    if { [info exists wChangeLay] } {
	FillTableInfoLayers $GidPriv(Layers,table)
	if { [winfo exists $wChangeLay.touse.l] } {
	    $wChangeLay.touse.l configure -text [GiD_Info Project LayerToUse]
	}
    }
    if { [info exists GidPriv(ComboLayers,table)] } {
	FillTableInfoLayers $GidPriv(ComboLayers,table)
	#update size
	set_layers_menu_size $GidPriv(ComboLayers,table) [winfo toplevel $GidPriv(ComboLayers,table)]
    }
    if { [info exists GidPriv(ComboLayers,entry)] && [winfo exists $GidPriv(ComboLayers,entry)] } {
	$GidPriv(ComboLayers,entry) delete 0 end
	$GidPriv(ComboLayers,entry) insert end [GiD_Info Project LayerToUse]
    }
}

proc SetLayersToDelete { tbl } {
    set parent [winfo parent $tbl]
    set LayersSelection [SetSelection $tbl]
    if { $LayersSelection == "" } {
	tk_dialogRAM  $parent.modal [_ "error"] \
		[_ "before applying operation delete it is necessary to select layers"] \
		error 0 [_ "OK"]
	return
    }

    GiD_Process 'Layers
    set warnmessage 1
    foreach i $LayersSelection {
	GiD_Process Delete $i
	
	if { $warnmessage && [catch { GiD_Info layers $i } ] == 0 } {
	    set ret [tk_dialogRAM  $parent.modal [_ "error"] \
		         [_ "Layer '%s' can't be erased.\n(see the message bar for details)" $i] \
		         error 0 [_ "OK"] [_ "OK to all"]]
	    if { $ret == 1 } { set warnmessage 0 }
	}
    }

    GiD_Process escape
}

proc SetLayersToColor { tbl } {
    set parent [winfo parent $tbl]
    set LayersSelection [SetSelection $tbl]
    if { $LayersSelection == "" } {
	tk_dialogRAM  $parent.modal [_ "error"] \
	[_ "before applying operation color it is necessary to select layers"] \
	error 0 [_ "OK"]
	return
    }
    set sel $LayersSelection ;#trick because GiD_Process 'Layers can change LayersSelection

    GiD_Process 'Layers
    set LayersSelection $sel

    set LayersProp [lindex [GiD_Info layers [lindex $LayersSelection 0]] 0]

    set cur_col [ format "#%02x%02x%02x%02x" [lindex $LayersProp 2] \
	    [lindex $LayersProp 3] [lindex $LayersProp 4] [lindex $LayersProp 5]]

    #set retcol [tk_chooseColor -parent $parent -title [_ "Select color"] -initialcolor $cur_col]
    set retcol [GIDChooseColor $parent.selectcolor -title [_ "Select color"] -color $cur_col]

    if { $retcol != "" } {
	set retcol [string range $retcol 1 end]
	if { [string length $retcol] == 8 } {
	    scan $retcol %2x%2x%2x%2x col1 col2 col3 alpha
	    set col [format %03i%03i%03i $col1 $col2 $col3]
	    foreach i $LayersSelection {
		GiD_Process Color $i $col Transparent $i $alpha
	    }
	} else {
	    scan $retcol %2x%2x%2x col1 col2 col3
	    set col [format %03i%03i%03i $col1 $col2 $col3]
	    foreach i $LayersSelection {
		GiD_Process Color $i $col
	    }
	}
    }
    GiD_Process escape
}

proc SetLayersTo { type tbl {newname ""} } {
    set LayersSelection [SetSelection $tbl]
    if { $LayersSelection == "" } {
	WarnWin [_ "before applying operation %s it is necessary to select layers" $type]
	return
    }

    set VarToSend 'Layers    
    foreach i $LayersSelection {
	switch $type {
	    ON {
		lappend VarToSend On $i
	    }
	    OFF {
		lappend VarToSend Off $i
	    }
	    FREEZE {
		lappend VarToSend Freeze $i
	    }
	    UNFREEZE {
		lappend VarToSend UnFreeze $i
	    }
	    TRANSPARENT {
		lappend VarToSend Transparent $i 127
	    }
	    OPAQUE {
		lappend VarToSend Opaque $i
	    }
	    TOUSE {
		lappend VarToSend ToUse $i
	    }
	    RENAME {
		lappend VarToSend ChangeName $i $newname
	    }
	    BRINGTOFRONT {
		lappend VarToSend BringToFront $i
	    }
	}
    }
    lappend VarToSend escape   

    GiD_Process {*}$VarToSend
    
    #FillInfoLayers ;#automatically called
    if { $type == "RENAME" } {
	#Trick to have selected the new name after FillInfoLayers will be automatically called
	SetLayerAsSelection $tbl $newname
    }
}

proc SendLayerEntitiesAA { T what } {
    global wChangeLay GidPriv

    set LayersSelection [SetSelection $T]
    if { [llength $LayersSelection] !=1 } {
	WarnWin [_ "before applying 'Entities' it is necessary to select 1 layer"]
	return
    }
    set layer [lindex $LayersSelection 0]

    if { $GidPriv(LayersAlsoLower) == 0 } {
	GiD_Process 'Layers Entities $layer $what
    } else {
	GiD_Process 'Layers Entities $layer LowerEntities $what
    }
    set SmallWinSelecting [GiD_Set SmallWinSelecting]
    # $w is same parent as $::wChangeLayButtonFrame
    set w [ winfo parent $::wChangeLayFrame]
    FinishButton $w $::wChangeLayButtonFrame \
	[_ "Press 'Finish' to end selection"] \
	"" disableall  $SmallWinSelecting
}

proc SendLayerEntities { m T } {
    set geomlistNames [GetGeometryLabels]
    lappend geomlistNames {*}[list [_ "Dimensions"] [_ "All"]]
    set geomlist [GetGeometryCommands]
    lappend geomlist {*}{Dimensions All}

    AddEntitiesToMenuAfter 0 $m [list SendLayerEntitiesAA $T] $geomlistNames $geomlist \
	    [list [_ "Nodes"] [_ "Elements"] [_ "Dimensions"] [_ "All"]] \
	    {Nodes Elements Dimensions All}
}

proc LayerToUseEntitiesAA { what } {
    GidUtils::SetWarnLine [_ "Pick one entity to set its layer as current layer to use"]
    GidUtils::DisableWarnLine
    set nnum [GidUtils::PickEntities $what single]
    GidUtils::EnableWarnLine
    if { $nnum != "" } {
	set retval [GiD_Info list_entities $what $nnum]
	set layername ""
	regexp {LAYER:([^\n\r]+)} $retval {} layername
	set layername [string trim $layername]
	if { $layername != "" } {
	    GiD_Process 'Layers ToUse $layername
	    FillInfoLayers
	    SetLayerAsSelection $::GidPriv(Layers,table) $layername
	} else {
	    WarnWin [_ "Selected entity does not belong to one layer"]
	}
    }
}

proc LayerToUseEntities { menu } {
    if { $::GidPriv(HideVolumeLevel) ==0} {
	set geomlistNames [list [_ "Point"] [_ "Line"] [_ "Surface"] [_ "Volume"]]
    } else {
	set geomlistNames [list [_ "Point"] [_ "Line"] [_ "Surface"]]
    }
    set geomlist [GetGeometryCommands]
    set meshlistNames [list [_ "Node"] [_ "Element"]]
    set meshlist {Nodes Elements}
    AddEntitiesToMenu $menu "LayerToUseEntitiesAA " $geomlistNames $geomlist $meshlistNames $meshlist
}

proc SendLayerEntitiesToBackAA { what } {
    global wChangeLay GidPriv

    if { $GidPriv(LayersBackOpposite) } {
	if { $GidPriv(LayersAlsoLower) == 0 } {
	    GiD_Process 'Layers SendToBackOpposite $what
	} else {
	    GiD_Process 'Layers SendToBackOpposite LowerEntities $what
	}
    } else {
	if { $GidPriv(LayersAlsoLower) == 0 } {
	    GiD_Process 'Layers SendToBack $what
	} else {
	    GiD_Process 'Layers SendToBack LowerEntities $what
	}
    }
    set SmallWinSelecting [GiD_Set SmallWinSelecting]
    # $w is same parent as $::wChangeLayButtonFrame
    set w [ winfo parent $::wChangeLayFrame]
    FinishButton $w $::wChangeLayButtonFrame \
	[_ "Press 'Finish' to end selection"] \
	"FillInfoLayers" disableall $SmallWinSelecting
}


proc SendLayerEntitiesToBack { m tbl } {    
    AddEntitiesToMenuAfter 1 $m "SendLayerEntitiesToBackAA" [GetGeometryLabels 1] [GetGeometryCommands 1] \
	[GetMeshLabels 1] [GetMeshCommands 1]

    $m add separator
    if { [GiD_Info layers -hasbacklayers] } {
	$m add command -label [_ "Bring to front"] \
	    -command [list SetLayersTo BRINGTOFRONT $tbl] \
	    -image [gid_themes::GetImage bring_to_front.png small_icons] -compound left
	$m add command -label [_ "Bring all to front"] \
	    -command {GiD_Process 'Layers BringToFrontAll escape ; FillInfoLayers} \
	    -image [gid_themes::GetImage bring_to_front.png small_icons] -compound left
    } else {
	$m add command -label [_ "Bring to front"] -state disabled \
	    -image [gid_themes::GetImage bring_to_front.png small_icons] -compound left
	$m add command -label [_ "Bring all to front"] -state disabled \
	    -image [gid_themes::GetImage bring_to_front.png small_icons] -compound left
    }
}

proc CreateNewLayerFolder { {T ""} {name ""} } {
    set prefix ""
    if { $T != "" && [$T selection count] == 1 } {
	set item [$T selection get 0]
	set prefix [DecodeName [$T item tag names $item]]        
    } else {
	WarnWin [_ "A parent layer must be selected before"]
	return ""
    }
    if { $name == "" } {
	set name [GetAutomaticLayerName $prefix]
    } else {
	if { ![IsValidLayername $name] } {
	    WarnWin [_ "Bad layer name, start or end by '//' is not allowed"]
	    return ""
	}
    }
    GiD_Process 'Layers New $name escape
    update ;#to fill in again the treectrl to create the item
    set item [$T item id "tag [EncodeName $name]"]
    if { $item != "" } {
	$T activate $item
	BeginEditLayers $T
    }
    return $name
}

proc GetAutomaticLayerName { {prefix ""} } {
    set LayerNames [GiD_Info layers]
    set name ""
    for {set i 0} {$i<10000} {incr i} {
	if { $prefix != "" } {
	    set name ${prefix}//Layer${i}
	} else {
	    set name Layer${i}
	}
	if { [lsearch -exact $LayerNames $name] == -1 } { break }
    }
    return $name
}

proc CreateNewLayer { {T ""} {name ""} } {
    set prefix ""
    if { $name == "" } {
	set name [GetAutomaticLayerName $prefix]
    } else {
	if { ![IsValidLayername $name] } {
	    WarnWin [_ "Bad layer name, start or end by '//' is not allowed"]
	    return ""
	}
    }
    GiD_Process 'Layers New $name escape
    update ;#to fill in again the treectrl to create the item
    if { $T != "" } {
	set item [$T item id "tag [EncodeName $name]"]
	if { $item != "" } {
	    $T activate $item
	    BeginEditLayers $T
	}
    }
    return $name
}


proc NewLayerFromEntry { T w } {
    set name [string trim [$w.ent.e get]]
    #regsub -all {[ ]+} $name {_} name

    if { $name != "" } {
	set LayerNames [GiD_Info layers]
	set ipos [lsearch -exact $LayerNames $name]
	if { $ipos != -1 } {
	    tk_dialogRAM  $w.modal [_ "error"] [_ "Layer %s already exists" $name] error 0 [_ "OK"]
	    return
	}
    }
    set name [CreateNewLayer $T $name]

    #FillInfoLayers ;#automatically called
    $w.ent.e delete 0 end
    if { $name != "" } {
	# seleccionamos la que acabamos de crear
	SetLayerAsSelection $T $name
    }
}

proc ConfigureListScrollbars { listbox sx sy } {
    foreach i "x y" {
	if { ![info exists s${i}] || ![winfo exists [set s${i}]] } { continue }
	foreach "${i}1 ${i}2" [$listbox ${i}view] break
	if { [set ${i}1] == 0 && [set ${i}2] == 1 } {
	    after idle grid remove [set s${i}]
	} else {
	    after idle grid [set s${i}]
	}
    }
}

proc MotionTableList { x y w tbl } {
    set tablelist::x [expr {$x + [winfo x $w]}]
    set tablelist::y [expr {$y + [winfo y $w]}]
    set cell [$tbl nearestcell $tablelist::x $tablelist::y]
    foreach {row col} [ split $cell ,] break
    if { $col != 1 } {
	return -code break
    }
}

proc ClickTableList { x y T } {
#     focus $T
	
    set info [$T identify $x $y]
    if { [lindex $info 0] == "item" && [llength $info] >= 4 } {
	set item [lindex $info 1]
	set col [lindex $info 3]        
    } elseif { [lindex $info 0] == "header" && [lindex $info 1] == "0" } {
	if { [$T column cget C0 -arrow] == "up" } {
	    $T column configure C0 -arrow down
	    $T item sort 0 -dictionary -increasing
	} else {
	    $T column configure C0 -arrow up
	    $T item sort 0 -dictionary -decreasing
	}
	return ""
    } else {
	return ""
    }

    if { $col != 0 } {
	if { ![$T selection includes $item] } {
	    $T selection clear
	    $T selection add $item
	}
    }
    if { $col == 0 } {
	#SetLayersTo TOUSE $T
    } elseif { $col == 1 } {
	SetLayersTo TOUSE $T
    } elseif { $col == 2 } {
	SetLayersToColor $T
    } elseif { $col == 3 } {
	if { [$T item element cget $item C$col elemImgAny -image] == [gid_themes::GetImage layer_off.png small_icons] } {
	    SetLayersTo ON $T
	} else {
	    SetLayersTo OFF $T
	}
    } elseif { $col == 4 } {
	if { [$T item element cget $item C$col elemImgAny -image] == [gid_themes::GetImage lock_on.png small_icons] } {
	    SetLayersTo UNFREEZE $T
	} else {
	    SetLayersTo FREEZE $T
	}
    } elseif { $col == 5 } {
	if { [$T item element cget $item C$col elemImgAny -image] == [gid_themes::GetImage transparent_off.png small_icons] } {
	    SetLayersTo TRANSPARENT $T
	} else {
	    SetLayersTo OPAQUE $T
	}
    } elseif { $col == 6 } {
	if { [$T item element cget $item C$col elemImgAny -image] == [gid_themes::GetImage bring_to_front.png small_icons] } {
	    SetLayersTo BRINGTOFRONT $T
	} else {
	}
    }

    if { $col != 0 } {
	return -code break
    }
    return ""
}

proc InvertSelectionTableList { T } {
    set newselection {}
    foreach item [$T item range 0 end] {
	if { ![$T selection includes $item] } {
	    lappend newselection $item
	}
    }
    $T selection clear
    foreach item $newselection {
	$T selection add $item
    }
}

proc DoubleClickTableList { x y T } {
    set info [$T identify $x $y]
    if { [lindex $info 0] == "item" && [llength $info] >= 4 } {
	set item [lindex $info 1]
	set col [lindex $info 3]
    } else {
	return
    }

    if { $col == 0 } {
	$T selection clear
	$T selection add $item
	SetLayersTo TOUSE $T        
    }
}


#separator can be a multicharacter, like //
proc split2 { x separator } {
    set splitchar \uFFFF ;#forbidden unicode character, x must never contain it
    return [split [string map "$separator $splitchar" $x] $splitchar]
}

#hierarchy of layers by its name (separated by //)
#e.g. parent//chidren//subchildren
#return the list of chidren layers
#now not return items with subchilds, because parent not must be a physical layer
proc GetChildLayerNames { {parent ""} } {
    global GidPriv
    set LayerNames0 [GiD_Info layers]
    set len [string length $parent]
    set LayerNames {}
    foreach layer $LayerNames0 {
	if { $len } {
	    if { [string equal -length $len $parent $layer] } {
		if { [string range $layer $len $len+1] == "//" } {
		    if { [string first // $layer $len+2] == -1 } {
		        lappend LayerNames [string range $layer $len+2 end]
		    } else {
		        #only append childs of one level
		    }
		}
	    }
	} else {
	    if { [string first // $layer] == -1 } {
		lappend LayerNames $layer
	    } else {
		#only append childs of one level
	    }            
	}
    }
    if { [info exists GidPriv(LayersAlphabetic)] && $GidPriv(LayersAlphabetic) } {
	set LayerNames1 [lsort -unique -dictionary $LayerNames]
    } else {
	#must remove repetitions
	set LayerNames1 {}
	foreach layer $LayerNames {
	    if { [info exists name($layer)] } continue
	    lappend LayerNames1 $layer
	    set name($layer) 1
	}
    }
    return $LayerNames1
}

proc HasChilds { {parent ""} } {
    set LayerNames0 [GiD_Info layers]
    set len [string length $parent]
    foreach layer $LayerNames0 {
	if { $len } {
	    if { [string equal -length $len $parent $layer] } {
		if { [string range $layer $len $len+1] == "//" } {
		    return 1
		} 
	    }
	} else {
	    return 1
	}
    }
    return 0
}

proc  IsItemFolderOfLayers { T item } {
    return [$T item numchildren $item]
}

proc IsValidLayername { layername } {
    if { [string range $layername end-1 end] == "//" } {
	return 0
    } elseif { [string range $layername 0 1] == "//" } {
	return 0
    } elseif { $layername == "" } {
	return 0
    }
    return 1
}

proc FillTableInfoLayers { T {parent ""} {parentitem root} } {
    if { [winfo exists $T] } {
	
	array unset stateitems
	array unset selectionitems
	#try to mantain the previous state: open, selected
	foreach item [$T item range 0 end] {
	    set fullnameencoded [$T item tag names $item]
	    set stateitems($fullnameencoded) [$T item isopen $item]
	    set selectionitems($fullnameencoded) [$T selection includes $item]
	}
	$T item delete all ;#coult try not to delete and rebuild all...
	FillTableInfoLayers_New $T $parent $parentitem
	foreach item [$T item range 0 end] {
	    set fullnameencoded [$T item tag names $item]
	    if { [info exists stateitems($fullnameencoded)] &&
		!$stateitems($fullnameencoded) } {
		$T item collapse $item
	    }
	    if { [info exists selectionitems($fullnameencoded)] &&
		$selectionitems($fullnameencoded) } {
		$T selection add $item
	    }
	}
	set id [$T item id "tag [EncodeName [GiD_Info Project LayerToUse]]"]
	if { $id != "" } {
	    $T activate $id
	}
	if { [$T selection count] == 1 } {
	    $T see [$T selection get]
	}
	set ::TreeCtrl::Priv(DirCnt,$T) [$T item count] ;#to use drag file-list bindings
    }
}

proc SetSelection { T } {
    if { ![winfo exists $T] } { return }
    set LayersSelection ""
    set sel [$T selection get]
    foreach item $sel {
	lappend LayersSelection [DecodeName [$T item tag names $item]]
    }
    set LayersSelection [lsort -unique -dictionary $LayersSelection]
    return $LayersSelection
}

proc FillTableInfoLayers_New { T {parent ""} {parentitem root} } {

#     if { ![winfo exists $T] } { return }

    set LayerNames [GetChildLayerNames $parent]
    set LayerToUse [GiD_Info Project LayerToUse]

    set BackLayerNames [GiD_Info back_layers]

    if { $BackLayerNames != "" } {
	regsub -all {_\*back\*} $BackLayerNames {} pp
	set BackLayerNames $pp
    }

    foreach i $LayerNames {
	if { $parent != "" } {
	    set fullname $parent//$i
	} else {
	    set fullname $i
	}
	set has_childs [HasChilds $fullname]
	if { $has_childs } {
	    set isbutton yes
	} else {
	    set isbutton no
	}
	

	set item [$T item create -button $isbutton -tags [EncodeName $fullname]]
	$T item lastchild $parentitem $item
	$T item style set $item C0 styAnyRead
	$T item element configure $item C0 elemTxtRead -text $i
	
	$T item style set $item C1 styAnyImage
	if { $fullname == $LayerToUse } {
	    #                 $T item element configure $item C0 elemTxtRead -text $i + elemImgAny -image [gid_themes::GetImage in_use.png small_icons]
	    $T item element configure $item C1 elemImgAny -image [gid_themes::GetImage in_use.png small_icons]
	    #                 $T item state set $item touse
	} else {
	    #                 $T item element configure $item C0 elemTxtRead -text $i
	    $T item element configure $item C1 elemImgAny -image [gid_themes::GetImage blank.png small_icons];#void image
	    #                 $T item state set $item !touse
	}
	
	set LayersProp [lindex [GiD_Info layers $fullname] 0]
	set cur_col [format "#%02x%02x%02x" [lindex $LayersProp 2] \
		[lindex $LayersProp 3] [lindex $LayersProp 4]]
	$T item style set $item C2 styRectColor
	$T item element configure $item C2 elemRectColor -fill $cur_col
	
	$T item style set $item C3 styAnyImage
	if { [lindex $LayersProp 0] } {
	    $T item element configure $item C3 elemImgAny -image [gid_themes::GetImage layer_on.png small_icons]
	    #                 $T item state set $item on
	} else {
	    $T item element configure $item C3 elemImgAny -image [gid_themes::GetImage layer_off.png small_icons]
	    #                 $T item state set $item !on
	}
	
	
	$T item style set $item C4 styAnyImage
	if { [lindex $LayersProp 1] } {
	    $T item element configure $item C4 elemImgAny -image [gid_themes::GetImage lock_on.png small_icons]
	    #                 $T item state set $item freeze
	} else {
	    $T item element configure $item C4 elemImgAny -image [gid_themes::GetImage lock_off.png small_icons]
	    #                 $T item state set $item !freeze
	}
	
	$T item style set $item C5 styAnyImage
	if { [lindex $LayersProp 5] == 255 } {
	    $T item element configure $item C5 elemImgAny -image [gid_themes::GetImage transparent_off.png small_icons]
	    #                 $T item state set $item transparent
	} else {
	    $T item element configure $item C5 elemImgAny -image [gid_themes::GetImage transparent_on.png small_icons]
	    #                 $T item state set $item !transparent
	}
	
	$T item style set $item C6 styAnyImage
	if { [ lsearch $BackLayerNames $fullname] != -1 } {
	    $T item element configure $item C6 elemImgAny -image [gid_themes::GetImage bring_to_front.png small_icons]
	    #                 $T item state set $item back
	} else {
	    $T item element configure $item C6 elemImgAny -image [gid_themes::GetImage blank.png small_icons] ;#void image
	    #                 $T item state set $item !back
	}                
	
	
	if { $has_childs } {
	    FillTableInfoLayers_New $T $fullname $item
	}
    }
    if { [$T column cget C0 -arrow] == "up" } {       
	$T item sort 0 -dictionary -decreasing
    } else {
	$T item sort 0 -dictionary -increasing
    }
}

proc ChangeParentLayers { T dragged_list new_parent_name } {
    set cmd ""
    foreach item $dragged_list {
	set fullname [DecodeName [$T item tag names $item]]
	lappend cmd ChangeParent $fullname $new_parent_name        
    }
    return $cmd
}

proc ReceiveDragLayers { state T dragged_list dst } {
    if { $state == "receive" } {        
	set new_parent_name [DecodeName [$T item tag names $dst]]
	set cmd [ChangeParentLayers $T $dragged_list $new_parent_name]
	GiD_Process 'Layers {*}$cmd escape
    } elseif { $state == "end" } {
	if { $::ReceiveDragLayers_prev_state != "receive" } {
	    #trick, if previous to "end" was not "receive" then was dropped 
	    #outside any parent, then set as a first level layer without parent
	    set dragged_list [$T selection get]
	    set new_parent_name ""
	    set cmd [ChangeParentLayers $T $dragged_list $new_parent_name]
	    GiD_Process 'Layers {*}$cmd escape            
	}
    }
    set ::ReceiveDragLayers_prev_state $state
}

proc SetLayersToRename { T item newtext } {
    if { $newtext == ""} return
    if { $item == 0 } {
	WarnWin [_ "Root folder can't be edited"]
	return
    }
    set fullname [DecodeName [$T item tag names $item]]
    set parts [split2 $fullname //]
    lset parts end $newtext
    set newname [join $parts //]
    if { $fullname != $newname } {
	GiD_Process 'Layers ChangeName $fullname $newname escape
    }
}

proc BeginEditLayers { T } {
    set I [$T item id active]
    set C 0
    set E elemTxtRead
    ::TreeCtrl::FileListEdit $T $I $C $E
}

proc LayersCreateListWin { w } {
    package require treectrl
    package require base64


    
    set vsb $w.vsb1
    set hsb $w.hsb1
    #pseudottk::
    set T [treectrl $w.t -xscrollcommand [list $hsb set] -yscrollcommand [list $vsb set]]
    ttk::scrollbar $vsb -orient vertical   -command [list $T yview]
    ttk::scrollbar $hsb -orient horizontal -command [list $T xview]
    

    set height [font metrics [$T cget -font] -linespace]
    if {$height < 18} {
	set height 18
    }

    $T configure -indent 15 -itemheight $height -selectmode extended \
	-showroot 0 -showrootbutton 0 -showbuttons 1 -showlines 1 \
	-highlightthickness 0 -borderwidth 0 \
	-xscrollincrement 20 -yscrollincrement 20 
	#-scrollmargin 16 -xscrolldelay "500 50" -yscrolldelay "500 50"
	
    
#------------------test cabeceras treectrl------------
    gid_themes::ConfigureHeaders $T
#-----------------part 1 fin----------------------
    
    set coltxt [list [_ "Name"] { } [_ "C#C#Color"] [_ "I/O#C#On/Off"] \
	    [_ "F/U#C#Freeze/Unfreeze"] [_ "Tr#C#transparent"] [_ "B#C#back"]]

#     $T state define touse
#     $T state define on
#     $T state define freeze
#     $T state define transparent
#     $T state define back
	
    set i 0
    foreach txt $coltxt {
	$T column create -text $txt -tags C$i -weight 0 
	#$T column create -text $txt -tags C$i -weight 0 -justify center \
	#    -gridrightcolor gray90 -itembackground {linen {} #00ff00 {}}
	incr i
    }
    $T column configure C0 -weight 1 -arrow down
    #     $T column configure tail -width 0
    #     $T column configure tail -visible 0
    $T configure -treecolumn C0

#------------------part 2 test cabeceras treectrl------------
    gid_themes::ApplyHeadersConfig $T
#-----------------fin test----------------------



    #create images
    #     image create photo layer_folder -file {C:\gid project\bitmaps\folder.png}

    # Create elements
    $T element create elemImgAny image
    $T element create elemTxtRead text -fill [list $::SystemHighlightText {selected focus}] \
	-lines 1
    $T element create elemRectSel rect -fill [list $::SystemHighlight {selected focus} gray {selected !focus}] \
		-showfocus yes
    $T element create elemRectColor rect -width 30
    # Create styles using the elements
    set S [$T style create styAnyRead]
    $T style elements $S {elemRectSel elemTxtRead elemImgAny}
    $T style layout $S elemImgAny -expand ns
    $T style layout $S elemTxtRead -padx 4 -expand ns -squeeze x
    $T style layout $S elemRectSel -union [list elemTxtRead] -iexpand ns -ipadx 2

    set S [$T style create styRectColor]
    $T style elements $S {elemRectColor elemTxtRead}
    $T style layout $S elemRectColor -union [list elemTxtRead] -iexpand ns -ipadx 2

    set S [$T style create styAnyImage]
    $T style elements $S {elemImgAny}
    $T style layout $S elemImgAny -expand ns
	

    set item root
    $T item configure $item -button yes
    $T item style set $item C0 styAnyRead
    $T item element configure $item C0 elemTxtRead -text [_ "Root"]
    #$T item element configure $item C0 elemImgAny -image [gid_themes::GetImage layer.png small_icons]


    # List of lists: {column style element ...} specifying text elements
    # the user can edit
    TreeCtrl::SetEditable $T {
	{C0 styAnyRead elemTxtRead}
    }

    # List of lists: {column style element ...} specifying elements
    # the user can click on or select with the selection rectangle
    TreeCtrl::SetSensitive $T {
	{C0 styAnyRead elemTxtRead}
    }


    # List of lists: {column style element ...} specifying elements
    # added to the drag image when dragging selected items
    TreeCtrl::SetDragImage $T {
		{C0 styAnyRead elemTxtRead}
    }

    #$T notify install <Drag-begin>
    $T notify install <Drag-end>    
    $T notify install <Drag-receive>    
#     $T notify install <Edit-begin>
#     $T notify install <Edit-end>
    $T notify install <Edit-accept>

    #$T notify bind DragTag <Drag-begin> { ReceiveDragLayers begin %T %l %I}
    $T notify bind DragTag <Drag-end> { ReceiveDragLayers end %T %l %I}
    $T notify bind DragTag <Drag-receive> { ReceiveDragLayers receive %T %l %I}

    $T notify bind EditTag <Edit-accept> { SetLayersToRename %T %I %t }

    bind $T <Button-1> [list ClickTableList %x %y $T]
    bind $T <Double-Button-1> [list DoubleClickTableList %x %y $T]
    #bind $T <Return> [list SetLayersTo TOUSE $T]
    bind $T <Return> [list CreateNewLayer $T]
    bind $T <Key-Delete> [list SetLayersToDelete $T]
    bind $T <Alt_L> [list InvertSelectionTableList $T]
    bind $T <Alt_R> [list InvertSelectionTableList $T]
    bind $T <Meta_L> [list InvertSelectionTableList $T]
    bind $T <Meta_R> [list InvertSelectionTableList $T]
    bind $T <F2> [list BeginEditLayers $T]

    bind $T <Button-$::gid_right_button> "[list MenuContextualLayer %W %x %y] ; break"

    grid $T $vsb -sticky nsew
    grid configure $vsb -sticky ns
    grid $hsb -sticky ew
    grid remove $vsb $hsb
    bind $T <Configure> [list ConfigureListScrollbars $T $hsb $vsb]    

    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1

#     bindtags $T [list $T DemoExplorer TreeCtrlFileList TreeCtrl [winfo toplevel $T] all]
    bindtags $T [list $T TreeCtrlFileList TreeCtrl [winfo toplevel $T] all]

    #to avoid mousewheel of layers TreeCtrl propagated to .gid.central.s
   
    #bind $T <MouseWheel> "[bind [winfo class $T] <MouseWheel>] ; break"    
    #fulltktree is removing the binding of TreeCtrl agressively with bind TreeCtrl <MouseWheel> {} !!!
    #then can't ask here its binding, use a copy of the original code!!
    bind $T <MouseWheel> {%W yview scroll [expr {- (%D / 120) * 4}] units ; break}

    return $T
}

proc ChangeLayers { {w .gid.central.wlay} } {
    global busy
    if { [info exists busy] && $busy } {
	#avoid Tcl error when pressing continuously a key that invoke a procedure
	return
    }
    set busy 1
    if { ![info exists ::GidPriv(LayerLayout)] } {
	set ::GidPriv(LayerLayout) OUTSIDE ;
	#not use INSIDERIGHT because sharing gid.ini cause problems for previous GiD's in ReadDefaultValues
    }
    if { $::GidPriv(LayerLayout) == "RIGHT" } {
	set inside 1
    } else {
	set inside 0
    }
    #problems with panedwindow: without after idle have bad size when restarting
    #gid with Layers window inside    
    if { [catch { ChangeLayers_New $w $inside } err] } {
	WarnWinText $err
    }
    set busy 0
}


proc WriteLayerGeomToVar { w what geomname {InitComm ""}} {
    global GidPriv
    set trans 1
    update idletasks
    if { $::GidPriv(LayerLayout) == "RIGHT" } {
	#maintain all dimensions except the new width
	set prevgeom [lindex $GidPriv($geomname) 1]
	foreach {width height x y} [split $prevgeom x+] break
	set width [winfo width $w]
    } else {
	#"OUTSIDE"
	set width [winfo width $w]
	set height [winfo height $w]
	set x [winfo x $w]
	set y [winfo y $w]
    }
    set geom ${width}x${height}+${x}+${y}
    set GidPriv($geomname) "$what $geom $trans $InitComm"
#     WarnWinText $GidPriv($geomname)
}

########################
#   Some changes to take account GidPriv(pwCentral), a panedwindow to manage several windows
########################
proc ChangeLayers_New { w {inside 1} } {
    global GidPriv
    
    if { $inside } {
	open_window_inside $w
	if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0
	if { [winfo class $w] == "Toplevel" } return 

	#######
	# pwCentral is a panedwindow to manage $grWindow with other windows in GiD
	if {![info exists GidPriv(pwCentral)]} return
	if {![winfo exists $GidPriv(pwCentral)]} return
	#######
	

	ttk::frame $w.caption -relief solid -borderwidth 1
	GidHelp $w.caption [_ "Double-click toggle window or inline visualization"]
	ttk::label $w.caption.title -state disabled -anchor center -text [_ "Double click here to tear off the window"]
	tk::button $w.caption.close -image [gid_themes::GetImage close17.png] \
	    -command {GidUtils::CloseWindow LAYER} -borderwidth 0 -relief flat
    
	grid $w.caption.title $w.caption.close -sticky ew
	grid configure $w.caption.close -sticky w      
	grid columnconfigure $w.caption 0 -weight 1
	bind $w.caption.title <Double-Button-1> [list switch_window_inside_outside]
		
	ttk::frame $w.body
	grid $w.caption -sticky ew        
	grid $w.body -sticky nsew
	grid rowconfigure $w 1 -weight 1
	grid columnconfigure $w 0 -weight 1
		
	if { [info exists ::GID_INTERNAL_GROUPS] && $::GidPriv(ShowGroupsTab) } {
	    grid rowconfigure $w.body 0 -weight 1
	    grid columnconfigure $w.body 0 -weight 1
	    set nb [ttk::notebook $w.body.nb]
	    ttk::notebook::enableTraversal $nb
	    set f_layers [ttk::frame $nb.f_layers]
	    $nb add $f_layers -text [_ "Layers"] -underline 0
	    ChangeLayers_New2 $f_layers
	    set f_groups [ttk::frame $nb.f_groups]
	    $nb add $f_groups -text [_ "Groups"] -underline 0
	    Groups::Win $f_groups            
	    grid $nb -sticky nsew
	    if { [info exists ::GidPriv(LayersOrGroupsCurrentTab)] } {
		$nb select $::GidPriv(LayersOrGroupsCurrentTab)                
	    }
	} else {
	    ChangeLayers_New2 $w.body
	}
	
	update idletasks
	if { [info exists ::GidPriv(LayersWindowGeom)] } {
	    set w1 [lindex [split [lindex $::GidPriv(LayersWindowGeom) 1] x] 0]
	} else {
	    set w1 [winfo reqwidth $w.body]
	}
	set sepw 4
	set w0 [expr [winfo width $GidPriv(pwCentral)]-$w1-$sepw]
	if { $w0<=0 || $w1<=0 } {
	    set w0 750
	    set w1 250
	}

	set wsash [expr [lsearch [$GidPriv(pwCentral) panes] $w]-1]
	if {$wsash < 0} {
	    set wsash 0
	}
	
	#$w add $p ;#raise reshape_cb of togl with unexpected width!!
	if { [winfo class $GidPriv(pwCentral)] == "Panedwindow" } {
	    #set w0 [expr {$w0-[$w cget -sashwidth]}]
	    #problems with panedwindow: without update idletasks have bad size when
	    #reopening again the layers window inside
	    update idletasks
	    $GidPriv(pwCentral) sash place $wsash $w0 0 ;#raise reshape_cb of togl
	    #$GidPriv(pwCentral) paneconfigure $p -width $w1
	} elseif { [winfo class $GidPriv(pwCentral)] == "TPanedwindow" } {
	    #ttk::pane
	    #$GidPriv(pwCentral) pane 0 -weight $w0
	    #$GidPriv(pwCentral) pane 1 -weight $w1
	    $GidPriv(pwCentral) sashpos $wsash $w0
	}

	set ::GidPriv(LayerLayout) "RIGHT"
	#WriteLayerGeomToVar $p OPEN LayersWindowGeom ChangeLayers
	bind $w.body <Configure> [list WriteLayerGeomToVar $w.body OPEN LayersWindowGeom ChangeLayers]
	# internal window --> no finishbutton
	set ::wChangeLayFrame $w.body
	set ::wChangeLayButtonFrame ""
    } else {
	open_window_outside $w
	if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0
	set ::GidPriv(LayerLayout) "OUTSIDE"
	
	ttk::frame $w.caption -relief solid -borderwidth 1
	GidHelp $w.caption [_ "Double-click toggle window or inline visualization"]
	
	ttk::label $w.caption.title -state disabled -anchor center -text [_ "Double click here to integrate the window"]                
	
	grid $w.caption.title -sticky ew
	grid columnconfigure $w.caption 0 -weight 1
	bind $w.caption.title <Double-Button-1> [list switch_window_inside_outside]
	
	ttk::frame $w.body
	grid $w.caption -sticky ew
	grid $w.body -sticky nsew
	grid rowconfigure $w 1 -weight 1
	grid columnconfigure $w 0 -weight 1
	
	if { [info exists ::GID_INTERNAL_GROUPS] && $::GidPriv(ShowGroupsTab) } {
	    grid rowconfigure $w.body 0 -weight 1
	    grid columnconfigure $w.body 0 -weight 1
	    set nb [ttk::notebook $w.body.nb]
	    ttk::notebook::enableTraversal $nb
	    set f_layers [ttk::frame $nb.f_layers]
	    $nb add $f_layers -text [_ "Layers"] -underline 0
	    ChangeLayers_New2 $f_layers
	    set f_groups [ttk::frame $nb.f_groups]
	    $nb add $f_groups -text [_ "Groups"] -underline 0
	    Groups::Win $f_groups
		     
	    grid $nb -sticky nsew         
	} else {
	    ChangeLayers_New2 $w.body
	}            
		
	#add lower buttons
	ttk::frame $w.buts -style BottomFrame.TFrame
	ttk::button $w.buts.can -text [_ "Close"] -command [list destroy $w] -style BottomFrame.TButton
	grid $w.buts.can -sticky ew -padx 5 -pady 6
	grid $w.buts -sticky ews -pady {2 0}
	grid anchor $w.buts center
	bind $w <Alt-c> "destroy $w"
	bind $w <Escape> "destroy $w"
	set ::wChangeLayFrame $w.body
	set ::wChangeLayButtonFrame $w.buts
    }
}

proc ChangeLayers_New2 { w } {
    global wChangeLay GidPriv
    set wChangeLay $w        

    if { ![info exists GidPriv(LayersAlsoLower)] } {
	set GidPriv(LayersAlsoLower) 1
    }
    if { ![info exists GidPriv(LayersBackOpposite)] } {
	set GidPriv(LayersBackOpposite) 0
    }
    
    ttk::frame $w.middle
    set T [LayersCreateListWin $w.middle]
    set GidPriv(Layers,table) $T

    ttk::frame $w.top -style ForcedFrame
    #new.png small_icons
    ttk::button $w.top.newlayer -image [gid_themes::GetImage newtopic.png small_icons]  -command [list CreateNewLayer $T] -style IconButton
    GidHelp $w.top.newlayer [_ "Create a new layer at root level."]
    ttk::button $w.top.newfolder -image [gid_themes::GetImage newfolder.png small_icons]  -command [list CreateNewLayerFolder $T] -style IconButton
    GidHelp $w.top.newfolder [_ "Create a child layer with the selected parent."]
    #ttk::button $w.top.rename -image [gid_themes::GetImage rename.png small_icons]  -command [list BeginEditLayers $T]
    #GidHelp $w.top.rename [_ "Rename a layer."]
    ttk::button $w.top.delete -image [gid_themes::GetImage erase-cross.png small_icons]  -command [list SetLayersToDelete $T] -style IconButton
    GidHelp $w.top.delete [_ "Delete a layer."]
    ttk::menubutton $w.top.sendto -image [gid_themes::GetImage sendto.png small_icons] -menu $w.top.sendto.m \
	-takefocus 1 -style IconButton
    GidHelp $w.top.sendto \
	[_ "Send the selected entities to a layer.\
	   If 'Also lower entities' is checked, the subentities are also send to the layer"]
    CreateMenuSendTo $w.top.sendto.m $T
    ttk::menubutton $w.top.toback -image [gid_themes::GetImage send_to_back.png small_icons] -text [_ "To back"] -menu $w.top.toback.m \
	-takefocus 1 -style IconButton
    GidHelp $w.top.toback \
	[_ "Send entities to back/front of a layer (entities in back are not drawn and frozen)"]
    CreateMenuBack $w.top.toback.m $T

    ttk::menubutton $w.top.entitylayer -menu $w.top.entitylayer.m -width 0 -image [gid_themes::GetImage layerofentity.png small_icons] -style IconButton
    GidHelp $w.top.entitylayer [_ "This option permmits to select one entity and set its layer as layer to use."]
    menu $w.top.entitylayer.m -postcommand [list LayerToUseEntities $w.top.entitylayer.m]

    ttk::button $w.top.filter -image [gid_themes::GetImage filter.png small_icons]  -command [list SelectLayersByRegexp $T] -style IconButton
    GidHelp $w.top.filter \
	[_ "Select layers with filters (for example: 'b*' select all the layers with name beginning by 'b')"]

    #grid $w.top.newlayer $w.top.newfolder $w.top.rename  $w.top.delete ...
    grid $w.top.newlayer $w.top.newfolder $w.top.delete \
	 $w.top.sendto $w.top.toback $w.top.entitylayer $w.top.filter -sticky w
    
    grid $w.top -sticky ew
    grid $w.middle -sticky wens

    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 1 -weight 1

    FillInfoLayers
    
    focus $T
}

proc CreateMenuBack { m T } {
    global GidPriv
    menu $m -postcommand [list SendLayerEntitiesToBack $m $T]
    $m add check -label [_ "Also lower entities"] -variable GidPriv(LayersAlsoLower)
    $m add check -label [_ "Opposite"] -variable GidPriv(LayersBackOpposite)
    $m configure -activeborderwidth 1
}

proc CreateMenuSendTo { m T } {
    global GidPriv
    menu $m -postcommand [list SendLayerEntities $m $T]
    $m add check -label [_ "Also lower entities"] -variable GidPriv(LayersAlsoLower)
    $m configure -activeborderwidth 1
}

proc MenuContextualLayer { T x y } {
    set w $T.menucontextuallayer
    if { [winfo exists $w] } {
	destroy $w
    }

    menu $w

    $w add command -label [_ "New layer#C#layer"] -command [list CreateNewLayer $T] \
	-image [gid_themes::GetImage newtopic.png small_icons] -compound left
    $w add command -label [_ "New child#C#layer"] -command [list CreateNewLayerFolder $T] \
	-image [gid_themes::GetImage newfolder.png small_icons] -compound left        

    set nfolder 0
    set nsingle 0
    foreach item [$T item range 0 end] {
	if { [$T selection includes $item] } {
	    if { [IsItemFolderOfLayers $T $item] } {
		incr nfolder
	    } else {
		incr nsingle
	    }
	}
    }
    if { [expr {$nsingle+$nfolder}] == 1 } {
	$w add command -label [_ "Rename"] -command [list BeginEditLayers $T] -state normal \
	    -image [gid_themes::GetImage rename.png small_icons] -compound left
    } else {
	$w add command -label [_ "Rename"] -state disabled \
	    -image [gid_themes::GetImage rename.png small_icons] -compound left
    }
    if { [expr {$nsingle+$nfolder}] > 0 } {
	$w add command -label [_ "Delete"] -command [list SetLayersToDelete $T] -state normal \
	    -image [gid_themes::GetImage erase-cross.png small_icons] -compound left
    } else {
	$w add command -label [_ "Delete"] -state disabled \
	    -image [gid_themes::GetImage erase-cross.png small_icons] -compound left
    }
    if { [expr {$nsingle+$nfolder}] == 1 } {
	$w add command -label [_ "To use"] -command [list SetLayersTo TOUSE $T] -state normal \
	    -image [gid_themes::GetImage in_use.png small_icons] -compound left        
    }
    $w add separator

    if { [expr {$nsingle+$nfolder}] > 0 } {
	set state normal
    } else {
	set state disabled
    }
    $w add command -label [_ "Color"]... -command [list SetLayersToColor $T] -state $state \
	-image [gid_themes::GetImage color.png small_icons] -compound left
    if { [expr {$nsingle+$nfolder}] ==1 } {        
	set item [$T selection get 0]
	set fullname [DecodeName [$T item tag names $item]]
	set LayersProp [lindex [GiD_Info layers $fullname] 0]
	if { [lindex $LayersProp 0] } {
	    $w add command -label [_ "Off#C#verb"] -command [list SetLayersTo OFF $T] -state $state \
		-image [gid_themes::GetImage layer_off.png small_icons] -compound left
	} else {
	    $w add command -label [_ "On#C#verb"] -command [list SetLayersTo ON $T] -state $state \
		-image [gid_themes::GetImage layer_on.png small_icons] -compound left
	}
	if { [lindex $LayersProp 1] } {
	    $w add command -label [_ "Unfreeze#C#verb"] -command [list SetLayersTo UNFREEZE $T] -state $state \
		-image [gid_themes::GetImage lock_off.png small_icons] -compound left
	} else {
	    $w add command -label [_ "Freeze#C#verb"] -command [list SetLayersTo FREEZE $T] -state $state \
		-image [gid_themes::GetImage lock_on.png small_icons] -compound left
	}        
	if { [lindex $LayersProp 5] == 255 } {
	    $w add command -label [_ "Transparent#C#verb"] -command [list SetLayersTo TRANSPARENT $T] -state $state \
		-image [gid_themes::GetImage transparent_on.png small_icons] -compound left
	} else {
	    $w add command -label [_ "Opaque#C#verb"] -command [list SetLayersTo OPAQUE $T] -state $state \
		-image [gid_themes::GetImage transparent_off.png small_icons] -compound left
	}
    } else {
	$w add command -label [_ "On#C#verb"] -command [list SetLayersTo ON $T] -state $state \
	    -image [gid_themes::GetImage layer_on.png small_icons] -compound left
	$w add command -label [_ "Off#C#verb"] -command [list SetLayersTo OFF $T] -state $state \
	    -image [gid_themes::GetImage layer_off.png small_icons] -compound left
	$w add command -label [_ "Freeze#C#verb"] -command [list SetLayersTo FREEZE $T] -state $state \
	    -image [gid_themes::GetImage lock_on.png small_icons] -compound left
	$w add command -label [_ "Unfreeze#C#verb"] -command [list SetLayersTo UNFREEZE $T] -state $state \
	    -image [gid_themes::GetImage lock_off.png small_icons] -compound left
	$w add command -label [_ "Opaque#C#verb"] -command [list SetLayersTo OPAQUE $T] -state $state \
	    -image [gid_themes::GetImage transparent_off.png small_icons] -compound left
	$w add command -label [_ "Transparent#C#verb"] -command [list SetLayersTo TRANSPARENT $T] -state $state \
	    -image [gid_themes::GetImage transparent_on.png small_icons] -compound left
    }
    $w add separator
    $w add command -label [_ "Select all"] -command [list $T selection add all]
    $w add separator
    if { $nsingle == 1 && $nfolder == 0 } {
	$w add cascade -label [_ "Send to"] -menu $w.sendto -state normal \
	    -image [gid_themes::GetImage sendto.png small_icons] -compound left
    } else {
	$w add cascade -label [_ "Send to"] -menu $w.sendto -state disabled \
	    -image [gid_themes::GetImage sendto.png small_icons] -compound left
    }
    CreateMenuSendTo $w.sendto $T
    $w add cascade -label [_ "To back"] -menu $w.back \
	-image [gid_themes::GetImage send_to_back.png small_icons] -compound left
    CreateMenuBack $w.back $T
    $w add separator
    $w add command -label [_ "Filter"] -command [list SelectLayersByRegexp $T] \
	-image [gid_themes::GetImage filter.png small_icons] -compound left

    set x [expr [winfo rootx $T]+$x+2]
    set y [expr [winfo rooty $T]+$y]
    GiD_PopupMenu $w $x $y
}

proc switch_window_inside_outside {  { w .gid.central.wlay } } {
    if { ![winfo exists $w] } {
	ChangeLayers_New $w 1
    } elseif { [winfo class $w] eq "Toplevel" } {
	close_window_outside $w
	ChangeLayers_New $w 1
    } else {
	close_window_inside $w
	ChangeLayers_New $w 0
    }
}

proc open_window_outside { w } {
    if { [winfo exists $w] } {
	close_window_outside $w
    }
    if { [info exists ::GID_INTERNAL_GROUPS] && $::GidPriv(ShowGroupsTab) } {
	InitWindow $w [_ "Layers and groups"] LayersWindowGeom ChangeLayers
    } else {
	InitWindow $w [_ "Layers"] LayersWindowGeom ChangeLayers
    }
    return $w
}

proc close_window_outside { w } {
    destroy $w
}


########################
#   This procedure take account new panedwindow GidPriv(pwCentral) to manage several windows
########################
proc close_window_inside { p } {
    global GidPriv
    
    if {![winfo exists $p]} return

    #######
    # pwCentral is a panedwindow to manage $grWindow with other windows in GiD
    if {![info exists GidPriv(pwCentral)]} {
	destroy $p
	return
    }
    #######
	
    if {[winfo exists $p.gg]} {
	bind $p.gg <Configure> ""
	WriteLayerGeomToVar $p.gg NONE LayersWindowGeom ChangeLayers
    }

    if { [winfo exists $GidPriv(pwCentral)] } {
	if { [winfo class $GidPriv(pwCentral)] in "Panedwindow TPanedwindow" } {
	    set panes [$GidPriv(pwCentral) panes]
	    set _pos [lsearch $panes $p]
	    if {$_pos >= 0} {
	    if { [llength $panes] == 2 } {
		    set oldwin [lreplace $panes $_pos $_pos]
		    set mg [winfo manager $GidPriv(pwCentral)]
		    set mginfo [$mg info $GidPriv(pwCentral)]        
		    $GidPriv(pwCentral) forget $p
		    $mg $oldwin {*}$mginfo
		    update idletask
		    destroy $GidPriv(pwCentral)
		    unset GidPriv(pwCentral)
		}
	    }
	    }
	} else {
	unset GidPriv(pwCentral)
    }
    destroy $p
}

#create a new panedwindw named $p and a new frame wich name is returned
proc open_window_inside_prev { p } {
    if { [winfo exists $p] } {
	close_window_inside $p
    }

#     if { [lsearch -exact [list GEOMETRYUSE MESHUSE] [GiD_Info Project ViewMode]] == -1 } {
#         return
#     }
    panedwindow $p -borderwidth 0 -showhandle 0 -sashpad 1 -sashrelief sunken -opaqueresize 0 -background #00ff00
    #ttk::panedwindow $p -orient horizontal

    set focus [focus]

    if { [winfo exists .gid.central.wins] } {
	set grWindow .gid.central.wins
    } else {
	set grWindow .gid.central.s
    }
    if { [grid slaves .gid.central] ne "" } {
	eval grid $p [grid info $grWindow]
    } elseif { [pack slaves .gid.central] ne "" } {
	pack $p -in .gid.central -before $grWindow \
	    -anchor center -expand 1 -fill both -side left
    } else {
	destroy $p
	return ""
    }

    $p add $grWindow

    raise $grWindow $p
    raise .gid.central.s

    set f [ttk::frame $p.gg]
    $p add $f

    #tkTabToWindow $p.gg
    if { $focus ne "" } {
	update
	catch { focus -force $focus }
    }
    return $f
}


########################
#   Procedure to open window inside, include creation of panedwindow to manage several windows
########################
proc open_window_inside { w } {
    global GidPriv
    
    if { [winfo exists $w] } {
	close_window_inside $w
    }
    
    ttk::frame $w

    set focus [focus]

    if { [winfo exists .gid.central.wins] } {
	set grWindow .gid.central.wins
    } else {
	set grWindow .gid.central.s
    }

    #######
    # pwCentral is a panedwindow to manage $grWindow with other windows in GiD
    if {![info exists GidPriv(pwCentral)]} {
	set GidPriv(pwCentral) .gid.central.pwCentral
    }
    #######

    if {![winfo exists $GidPriv(pwCentral)]} {
	set mg [winfo manager $grWindow]
	set mginfo [$mg info $grWindow]        
	$mg forget $grWindow

	panedwindow $GidPriv(pwCentral) -borderwidth 0 -showhandle 0 -sashpad 1 -sashrelief sunken -opaqueresize 0
	#-background #880000
	#ttk::panedwindow $GidPriv(pwCentral) -orient horizontal

	$mg $GidPriv(pwCentral) {*}$mginfo
	$GidPriv(pwCentral) add $grWindow $w -minsize 60
	raise $grWindow
	if {$grWindow == ".gid.central.wins"} {
	    raise .gid.central.s
	}

	$GidPriv(pwCentral) paneconfigure $grWindow -stretch always 

	raise $w
    } else {
	set panes  [$GidPriv(pwCentral) panes]
	set maxpane [expr [llength $panes]-1]
	# Are there any window after grWindow?
	if {[lsearch $panes $grWindow] < $maxpane} {
	    close_window_inside $w
	    ChangeLayers_New $w 0
	} else {
	    # Save sash position
	    for {set i 0} {$i < $maxpane} {incr i} {
		set _wsash($i) [lindex [$GidPriv(pwCentral) sash coord $i] 0]
	    }
	    $GidPriv(pwCentral) add $w -after $grWindow -minsize 60
	    # Restore sash position
	    foreach i [array names _wsash]  {
		update idletask
		$GidPriv(pwCentral) sash place $i $_wsash($i) 0
	    }
	}
    }

    if { $focus ne "" } {
	update
	catch { focus -force $focus }
    }
    return $w
}
