proc GetResultQuality { criteria from to num_divisions } {
    #criteria is a list of result name and component (0 for scalar)
    lassign $criteria result_name result_component  
    set result_analysis [GiD_Info postprocess get cur_analysis]
    set result_step [GiD_Info postprocess get cur_step]    
    set data [GiD_Result get [list $result_name $result_analysis $result_step]]    
    set header [lindex $data 0]
    set over [lindex $header 5] ;# "OnNodes" 
    set type [lindex $header 4] ;# "Scalar" "Vector" 
    
    set component_names [lindex $data 1]
    set values [lrange $data 2 end]    
    set span [expr {$to-$from}]
    if { $span == 0 } {
        #calculate min and max
        set min_value 1e300
        set max_value -1e300
        foreach item $values {
            lassign $item id value
            set value [lindex $value $result_component]            
            if { $min_value > $value } {
                set min_value $value
            }
            if { $max_value < $value } {
                set max_value $value
            }            
        }        
    } else {
        set min_value $from
        set max_value $to       
    }
    set span [expr {$max_value-$min_value}]
    if { $span == 0 } {
        set res [list]
        set num_items [llength $values]
        set v [expr $num_items/$num_divisions]
        for {set i 0} {$i < $num_divisions } {incr i} {
            lappend res $v
        }
        lappend res $min_value $max_value
    } else {
        for {set i 0} {$i < $num_divisions } {incr i} {
            set num_items($i) 0
        }
        foreach item $values {
            lassign $item id value
            set value [lindex $value $result_component]
            if { $value >= $min_value && $value <= $max_value } {
                set pos [expr {int($num_divisions*($value-$min_value)/$span)}]
                incr num_items($pos)
            }
        }
        set res [list]
        for {set i 0} {$i < $num_divisions } {incr i} {
            lappend res $num_items($i)
        }
        lappend res $min_value $max_value
    }
    return $res
}

proc GetSelectionResultQuality { criteria filter_value } {
    lassign $criteria result_name result_component  
    set result_analysis [GiD_Info postprocess get cur_analysis]
    set result_step [GiD_Info postprocess get cur_step]    
    set data [GiD_Result get [list $result_name $result_analysis $result_step]]    
    set header [lindex $data 0]
    set over [lindex $header 5] ;# "OnNodes" 
    set type [lindex $header 4] ;# "Scalar" "Vector" 
    
    set values [lrange $data 2 end]    
    
    if { $over == "OnNodes" } {
        set selection Nodes
    } else {
        set selection Elements
    }    
    foreach item $values {
        lassign $item id value
        set value [lindex $value $result_component]            
        if { $value <= $filter_value } {
            lappend selection $id
        }
    }        
    return $selection
}

proc ResultQualitySelectEntities { w x y } {
    upvar #0 [winfo name $w] data
    global ResultQualityPriv

    set c $w.f.c
    set fr $w.f2
            
    $c delete lineselect
    set from [$fr.f1.e1 get]
    set to [$fr.f1.e2 get]

    set xmax [winfo width $c]
    set ymax [winfo height $c]
    set xm $ResultQualityPriv(xmargin)
    set ym $ResultQualityPriv(ymargin)
    set value [expr ($x-$xm)/($xmax-2.0*$xm)*($to-$from)+$from]
    if { $value < $from } {
        set value $from
        set x [expr ($value-$from)/($to-$from)*($xmax-2.0*$xm)+$xm]
    } elseif { $value > $to } {
        set value $to
        set x [expr ($value-$from)/($to-$from)*($xmax-2.0*$xm)+$xm]
    }    
    $c create line $x $ym $x [expr $ymax-$ym] -tags lineselect -fill red        
    
    if { [GiD_Info Project ViewMode] == "GRAPHUSE" } {
        GiD_Process Mescape Results Graphs OptionsGraph ShowGraphs No escape
    }
    
    set criteria $::ResultQualityPriv(qualitytype)
    set selection [GetSelectionResultQuality $criteria $value]

    GiD_Redraw
    GiD_Process 'SelectEntities {*}$selection escape
}

proc OnChangeQualityResult { w } {
    #reset from and to to have all range
    set fr $w.f2
    $fr.f1.e1 delete 0 end
    $fr.f1.e1 insert end 0.0
    $fr.f1.e2 delete 0 end
    $fr.f1.e2 insert end 0.0
    
    DrawResultQualityCurrentValues $w
}

proc DrawResultQualityCurrentValues { w } {        
    global ResultQualityPriv   
    
    set c $w.f.c
    set fr $w.f2
    
    set from [$fr.f1.e1 get]
    set to [$fr.f1.e2 get]
    set numdivisions [$fr.f2.e1 get]
    
    set criteria $::ResultQualityPriv(qualitytype)
    
    set listb [GetResultQuality $criteria $from $to $numdivisions]
    
    set from [lindex $listb end-1]
    set to [lindex $listb end]    
    
    $fr.f1.e1 delete 0 end
    $fr.f1.e1 insert end [format "%3.3g" $from]
    $fr.f1.e2 delete 0 end
    $fr.f1.e2 insert end [format "%3.3g" $to]   
    
    set distributiontype $::ResultQualityPriv(distributiontype)
    
    lassign $criteria result_name result_component  
    set result_analysis [GiD_Info postprocess get cur_analysis]
    set result_step [GiD_Info postprocess get cur_step]    
    set data [GiD_Result get -info [list $result_name $result_analysis $result_step]]    
    set header [lindex $data 0]
    set over [lindex $header 5] ;# "OnNodes" 
    set type [lindex $header 4] ;# "Scalar" "Vector" 
    
    set labelx [_ $result_name]
    if { $over == "OnNodes" } {
        set labely [_ "Number of nodes"]
    } else {
        set labely [_ "Number of elements"]
    }
    
    set margins [MQDrawGraphCanvas $c $from $to $numdivisions $listb $distributiontype $labelx $labely]
    lassign $margins ::ResultQualityPriv(xmargin) ::ResultQualityPriv(ymargin)
}

proc PostResultQuality { { w .gid.resq } } {
    global ResultQualityPriv
    if { [GetCurrentPrePostMode] != "POST" } {
        WarnWin [_ "Result Quality: This function is only valid in postprocess"]
        return 1
    }    
    InitWindow $w [_ "Result distribution"] PostResultQualityWindowGeom PostResultQuality
    if { ![winfo exists $w] } { return 1 };# windows disabled || usemorewindows == 0

    set all_results [GiD_Info postprocess get cur_results_list contour_fill]    
    if { ![llength $all_results] } {
        WarnWin [_ "Result Quality: There are no results"]
        DestroyResultQuality $w $w
        destroy $w
        return 1
    }
    
    set def_back [$w cget -background]
    ttk::frame $w.buts  -style BottomFrame.TFrame
    ttk::button $w.buts.cn -text [_ "Close"] -style BottomFrame.TButton -command [list destroy $w]

    grid $w.buts.cn -row 1 -column 1 -padx 5 -pady 6

    ttk::frame $w.f    
    canvas $w.f.c -background white -relief ridge -borderwidth 2
    grid $w.f.c -sticky nsew    
    
    set fr [ttk::frame $w.f2]
    ttk::frame $fr.f1
    ttk::label $fr.f1.l1 -text [_ "From"]:
    ttk::entry $fr.f1.e1
    $fr.f1.e1 insert end 0.0
    ttk::label $fr.f1.l2 -text [_ "To"]:
    ttk::entry $fr.f1.e2
    $fr.f1.e2 insert end 0.0
    #ttk::label $fr.f1.l3 -text "units"
    grid $fr.f1.l1 $fr.f1.e1 $fr.f1.l2 $fr.f1.e2 -sticky w
    grid configure $fr.f1.e1 $fr.f1.e2 -sticky ew
    grid columnconfigure $fr.f1 "1 3" -weight 1

    ttk::frame $fr.f2
    ttk::label $fr.f2.l1 -text [_ "Num divisions"]:
    ttk::entry $fr.f2.e1
    $fr.f2.e1 insert end 100
    ttk::label $fr.f2.l2 -text [_ "Distribution"]:
    if { ![info exists ::ResultQualityPriv(distributiontype) ] } {
        set ::ResultQualityPriv(distributiontype) Accumulated
    }
    [TTKMB_CreateOrConfigure $fr.f2.m] $fr.f2.m -textvariable ::ResultQualityPriv(distributiontype) \
        -labels [list [_ "Accumulated"] [_  "Frequency"]] \
        -values {Accumulated Frequency}                            
    grid $fr.f2.l1 $fr.f2.e1 $fr.f2.l2 $fr.f2.m -sticky w
    grid configure $fr.f2.e1 -sticky ew
    grid columnconf $fr.f2 "1 3" -weight 1
    ttk::frame $fr.type
    ttk::label $fr.type.l -anchor w -text [_ "Result"]:
    
    set values [list]
    set labels [list]
    foreach result_name [GiD_Info postprocess get cur_results_list contour_fill] {        
        set result_analysis [GiD_Info postprocess get cur_analysis]
        set result_step [GiD_Info postprocess get cur_step]
        set data [GiD_Result get -info [list $result_name $result_analysis $result_step]]
        set header [lindex $data 0]
        set over [lindex $header 5] ;# "OnNodes" 
        set type [lindex $header 4] ;# "Scalar" "Vector" 
        if { $type == "Scalar" } {
            lappend values [list $result_name 0]
            lappend labels [_ $result_name]    
        } elseif { $type == "Vector" } {
            foreach axe {x y z} i {0 1 2} {
                lappend values [list $result_name $i]
                lappend labels [_ $result_name]-$axe
            }
            lappend values [list $result_name 3]
            lappend labels |[_ $result_name]|
        } elseif { $type == "Matrix" } {
            foreach axe {xx yy zz xy yz xz} i {0 1 2 3 4 5} {
                lappend values [list $result_name $i]
                lappend labels [_ $result_name]-$axe
            }
        } else {
            lappend values [list $result_name 0]
            lappend labels [_ $result_name]    
        }                
    }

    set ::ResultQualityPriv(qualitytype) [lindex $values 0]
    
    [TTKMB_CreateOrConfigure $fr.type.m] $fr.type.m -textvariable ::ResultQualityPriv(qualitytype) \
        -labels $labels -values $values
    if { [GiD_Info postprocess get cur_result] != "" } {
        set ::ResultQualityPriv(qualitytype) [list [GiD_Info postprocess get cur_result] 0]
    }

    grid $fr.type.l $fr.type.m
    grid $fr.f1 -sticky ew
    grid $fr.f2 -sticky ew
    grid $fr.type -sticky w
    grid columnconfigure $fr 0 -weight 1
    grid rowconfigure $fr 0 -weight 1
    
    bind $fr.f1.e1 <Return> [list DrawResultQualityCurrentValues $w]
    bind $fr.f1.e2 <Return> [list DrawResultQualityCurrentValues $w]
    bind $fr.f2.e1 <Return> [list DrawResultQualityCurrentValues $w]
    bind $w.f.c <Double-Button-1> [list ResultQualitySelectEntities $w %x %y]
    bind $w.f.c <Configure> [list DrawResultQualityCurrentValues $w]    

    grid $w.f -sticky nsew
    grid $fr -sticky sew
    
    grid $w.buts -sticky ews -columnspan 7
    if { $::tcl_version >= 8.5 } { grid anchor $w.buts center }
    grid rowconfigure $w.f 0 -weight 1
    grid columnconfigure $w.f 0 -weight 1
    grid rowconfigure $w 0 -weight 1
    grid columnconfigure $w 0 -weight 1
            

    bind $w <Escape> "$w.buts.cn invoke"
    bind $w <Destroy> [list +DestroyResultQuality %W $w]
    
        
    trace add variable ResultQualityPriv(qualitytype) write "OnChangeQualityResult $w ;#"
    trace add variable ResultQualityPriv(distributiontype) write "DrawResultQualityCurrentValues $w ;#"
}

proc DestroyResultQuality { W w } {
    global ResultQualityPriv
    if { $W != $w } return
    trace remove variable ResultQualityPriv(qualitytype) write "OnChangeQualityResult $w ;#"
    trace remove variable ResultQualityPriv(distributiontype) write "DrawResultQualityCurrentValues $w ;#"
}
