

namespace eval RetrievePT {
    variable w
    variable progressbar
    variable progresslabel
    variable ftpid ""
    variable FileSize
    variable LastTimeMiliSec 0
    variable num_modules
    variable modulesarray
    variable images
    variable lists
    variable currenttab

    variable server www.gidhome.com
    variable relativedir [file join pub gidmodules]    
    variable blocksize 4096 ;#default value 4096
    variable contcallbacks 0
    variable show_content   
    array set show_content {Program 1 Interface 1 Plugin 0 Example 0 Document 0}    
}

proc RetrievePT::Close {} {
    variable w
    variable ftpid
    variable progresslabel

    if { [info exists ftpid] && $ftpid != "" } {
        set progresslabel [_ "Closing"]...
        update
        if { [catch { ftp::Close $ftpid } msg] } {
            GidUtils::SetWarnLine "RetrievePT::Close: $msg"
        }
        set ftpid ""
    }
    destroy $w
}


proc RetrievePT::FtpProgress { bytes } {
    variable FileSize
    variable progressbar
    variable progresslabel
    variable LastTimeMiliSec
    variable blocksize
    variable contcallbacks
    if { $contcallbacks == [expr int($FileSize/$blocksize/15)] } {
        set contcallbacks 0
        set percent [expr 10+90*$bytes/double($FileSize)]
        set progressbar $percent
        
        set TimeMiliSec [clock clicks -milliseconds]
        set speed [expr $bytes/1024.0/($TimeMiliSec-$LastTimeMiliSec)*1000.0]
        
        set progresslabel [format "$bytes/$FileSize (%.3g Kb/s)" $speed]
        update idletasks
    } else {
        incr contcallbacks
    }
}

#procedure to avoid ftp::DisplayMsg error when printing to stderr that is closed in GiD
proc RetrievePT::FtpDisplayMsg {s msg {state ""}} {
    WarnWinText "ftp: '$msg' state:$state"
}

proc RetrievePT::Connect {} {
    variable w
    variable ftpid
    variable server
    variable progressbar
    variable progresslabel
    variable relativedir
    variable InitialPercentage
    variable blocksize    
    
    package require ftp

    if { [info exists w] && [winfo exists $w] } {
        set with_window 1
    } else {
        set with_window 0
    }

    if { $with_window } {
        set progressbar 0
        $w configure -cursor watch        
    }
    update

    if { ![info exists ftpid] } {
        set ftpid ""
    }
    if { $ftpid != "" && [catch { ftp::Pwd $ftpid }] } {
        set ftpid ""
    }

    if { $ftpid == "" } {
        set ftpid [ftp::Open $server ftp gidretriever@  -mode passive \
                        -progress ::RetrievePT::FtpProgress -timeout 30 -blocksize $blocksize -output ::RetrievePT::FtpDisplayMsg]

        if { $ftpid == -1 } {
	    set txt1 [_ "Problems while getting problem type list."]
            set txt2 [_ "Could not connect to server"]
            WarnWin "$txt1 \n$txt2"
            if { $with_window } {
                set progressbar 0
                $w configure -cursor ""
            }
            set progresslabel ""
            return -1
        }
        ftp::Cd $ftpid $relativedir
    }
    set progresslabel [_ "Connected"]
    update
    return 0
}

proc RetrievePT::Retrievelist {} {
    variable w
    variable ftpid
    variable progressbar
    variable progresslabel
    variable FileSize

    variable modulesarray
    variable num_modules

    set num_modules 0      
    
    if { [Connect] } { return }    
    
    if { [info exists w] && [winfo exists $w] } {
        set with_window 1
    } else {
        set with_window 0
    }

    set filename_info modulesinfo
    if { $with_window } {
        set FileSize [ftp::FileSize $ftpid $filename_info]        
        set progresslabel 0/$FileSize
        update
    }
    ftp::Type $ftpid ascii
    ftp::Get $ftpid $filename_info -variable modulesinfo
    if { ![info exists modulesinfo] } {
        if { $with_window } {
            set progresslabel [_ "Error retrieving list"]
            $w configure -cursor ""
            update
            after 1000 {set ::RetrievePT::progresslabel ""}
        }
        return
    }

    regsub -lineanchor -all {^#[^\n]*} $modulesinfo {} modulesinfo
    unset -nocomplain modulesarray
      
    catch {
        foreach {name value} $modulesinfo {
            if { $name == "Name" } {
                #new style, ignore Name and use instead ProgramName, 
                #but not replace Name for back compatibility with previous GiD's
                continue                             
            } elseif { $name == "ProgramName" } {
                set name Name
                incr num_modules                
            }            
            set modulesarray($num_modules,$name) $value       
        }
    }
    catch {
        # if Date field does not exists, get ModTime from file
        for { set imod 0} { $imod < $num_modules} { incr imod} {
            if { ![ info exists modulesarray($imod,Date)] && \
                     [ info exists modulesarray($imod,DownloadFile)] } {
                set modtime [ ftp::ModTime $ftpid $modulesarray($imod,DownloadFile)]
                set modulesarray($imod,Date) [ clock format $modtime -format %d.%m.%Y]
            }
        }
    }
    if { $with_window } {
        CheckModulesInstalled
                
        set progressbar 100
        set progresslabel ""    
        $w configure -cursor ""
        update
        after 1000 set RetrievePT::progressbar 0
    }
}

proc RetrievePT::FillTable { list i } {
    variable num_modules
    variable modulesarray
    variable images
    variable show_content

    if { ![winfo exists $list] } { return }

    lassign [GetCurrentPlatform] currentos currentbits

    $list delete 0 end
    for {set i_module 1} {$i_module<=$num_modules} {incr i_module} {
        set content $modulesarray($i_module,Content)
        if { [info exists show_content($content)] &&  $show_content($content) } {
            set name $modulesarray($i_module,Name)
            set platform $modulesarray($i_module,Platform)
            lassign $platform os bits
            if { ($os == "All" || $os == $currentos) && ($bits == "All" || $bits == $currentbits ) } {
                if { $i == 1 } continue
            } else {
                if { $i == 0 } continue
            }
            if { [info exists modulesarray($i_module,Version)] } {
                set version $modulesarray($i_module,Version)
            } else {
                set version ""
            }
            if { [info exists modulesarray($i_module,Date)] } {
                set date [string trim $modulesarray($i_module,Date)]
            } else {
                set date ""
            }
            if { [info exists modulesarray($i_module,ShortDescription)] } {
                set shortdesc [string trim $modulesarray($i_module,ShortDescription)]
            } else {
                set shortdesc ""
            }
            $list insert end [list $name $version $platform "" $date $shortdesc]
        }
    }

    set row 0
    for {set i_module 1} {$i_module<=$num_modules} {incr i_module} {
        set content $modulesarray($i_module,Content)
        if { [info exists show_content($content)] && $show_content($content) } {
            lassign $modulesarray($i_module,Platform) os bits
            if { ($os == "All" || $os == $currentos) &&
                ($bits == "All" || $bits == $currentbits ) } {
                if { $i == 1 } continue
            } else {
                if { $i == 0 } continue
            }
            if { ![info exists modulesarray($i_module,IsInstalled)] } {
                RetrievePT::CheckModuleInstalled $i_module
            }
            if { $modulesarray($i_module,IsInstalled) == 1 } {
                $list cellconfigure $row,3 -image $images(checked-on)
            } elseif { $modulesarray($i_module,IsInstalled) == -1 } {
                #GiD and the module are not compatible
                $list cellconfigure $row,3 -image $images(cross)
            } else {
                $list cellconfigure $row,3 -image $images(checked-off)
            }
            incr row
        }
    }
}

proc RetrievePT::GetModuleIndex { name version platform } {
    variable num_modules
    variable modulesarray
    lassign $platform os bits   
    set same_platform_other_versions {}
    set other_platforms {}
    for {set i_module 1} {$i_module<=$num_modules} {incr i_module} {
        if { ![string compare -nocase $modulesarray($i_module,Name) $name] } {
            set repository_module_version $modulesarray($i_module,Version)
            set repository_module_platform $modulesarray($i_module,Platform)
            lassign $repository_module_platform repository_module_os repository_module_bits
            if { ($repository_module_os == $os || $repository_module_os == "All") && 
                ($repository_module_bits == $bits || $repository_module_bits == "All") } {
                if { $repository_module_version == $version || $version == "" } {
                    #exact version found
                    return [list $i_module 0]
                } else {
                    lappend same_platform_other_versions $i_module
                }                
            } else {
                lappend other_platforms $i_module
            }
        }        
    }
    set i_best 0
    if { [llength $same_platform_other_versions] } {
        set i_best [lindex $same_platform_other_versions 0] 
        foreach i_module [lrange $same_platform_other_versions 1 end] {
            if { [GidUtils::TwoVersionsCmp $modulesarray($i_best,Version) $modulesarray($i_module,Version)] == -1 } {
                #second version is greater
                set i_best $i_module
            }          
        }
    } else {
        if { [llength $other_platforms] } {
            set i_best [lindex $other_platforms 0] 
            foreach i_module [lrange $other_platforms 1 end] {
                if { [GidUtils::TwoVersionsCmp $modulesarray($i_best,Version) $modulesarray($i_module,Version)] == -1 } {
                    #second version is greater
                    set i_best $i_module
                }          
            }
        }
    }        
    return [list 0 $i_best]
}

proc RetrievePT::IsVersionOKForCurrentGiD { i_module } {
    variable modulesarray
    if { [info exists modulesarray($i_module,MinimumGiDVersion)] } {
        set module_required_gid_version $modulesarray($i_module,MinimumGiDVersion)
    } else {
        return 1
    }
    set comp [GidUtils::VersionCmp $module_required_gid_version]
    if { $comp < 0 } {
        #GiD is Old
        return 0
    } else {
        #GiD is OK
        return 1
    }
}

proc RetrievePT::GetLocalFolder { content } {
    if { $content == "Program" || $content == "Interface" } {
        set folder [file join $::GIDDEFAULT problemtypes] 
    } elseif { $content == "Plugin" } {
        set folder [file join $::GIDDEFAULT plugins]
    } elseif { $content == "Example" } {
        set folder [file join $::GIDDEFAULT examples]
    } elseif { $content == "Document" } {
        set folder [file join $::GIDDEFAULT info]
    } else {
        #unexpected
        set folder ""
    }
    return $folder
}

proc RetrievePT::GetXmlRootName { content } {
    if { $content == "Program" || $content == "Interface" } {
        set xml_root_name Infoproblemtype
    } elseif { $content == "Plugin" } {
        set xml_root_name InfoPlugin
    } elseif { $content == "Example" } {
        set xml_root_name InfoExample
    } elseif { $content == "Document" } {
        set xml_root_name InfoDocument
    } else {
        #unexpected
        set xml_root_name ""
    }
    return $xml_root_name
}

proc RetrievePT::GetCallbackAfterUnzip { content module_xml_file } {
    if { $content == "Program" || $content == "Interface" } {
        set callback_after_unzip RetrievePT::UpdateMenuProblemtypes
    } elseif { $content == "Plugin" } {
        set callback_after_unzip [list RetrievePT::LoadPlugin [file dirname $module_xml_file]]
    } elseif { $content == "Example" } {
        set callback_after_unzip ""
    } elseif { $content == "Document" } {
        set callback_after_unzip ""
    } else {
        #unexpected
        set callback_after_unzip ""
    }
    return $callback_after_unzip
}


proc RetrievePT::CheckModuleInstalled { i_module } {
    variable modulesarray 
    set name $modulesarray($i_module,Name)
    set version $modulesarray($i_module,Version)
    set platform $modulesarray($i_module,Platform)  
    set content $modulesarray($i_module,Content)
    set localdir [RetrievePT::GetLocalFolder $content]
    set xml_root_name [RetrievePT::GetXmlRootName $content]    
    set problemtype_exists 0                
    set module_xml_file [file join $localdir $modulesarray($i_module,XmlFile)]
    set problemtype_exists [RetrievePT::GetModuleExists $module_xml_file $xml_root_name $name $version $platform]        
    if { $problemtype_exists } {
        set modulesarray($i_module,IsInstalled) 1
    } elseif { ![IsVersionOKForCurrentGiD $i_module] } {
        set modulesarray($i_module,IsInstalled) -1
    } else {
        set modulesarray($i_module,IsInstalled) 0
    }     
}

proc RetrievePT::CheckModulesInstalled {} {
    variable num_modules
    variable modulesarray
    variable show_content
    
    for {set i_module 1} {$i_module<=$num_modules} {incr i_module} {  
        set content $modulesarray($i_module,Content)
        if { [info exists show_content($content)] && $show_content($content) } {
            RetrievePT::CheckModuleInstalled $i_module
        }
    }
}

proc RetrievePT::RetrieveSelectedModules { } {
    variable lists
    variable currenttab

    set widget $lists($currenttab)
    set selection [$widget curselection]
    if { [llength $selection] == 0 } {
	    WarnWin [_ "Select first one Problem type"] $widget
        return
    }
    set selected_modules {}
    foreach item $selection {
        set values [$widget get $item]
        set name [lindex $values 0]
        set version [lindex $values 1]
        set platform [lindex $values 2]
        lappend selected_modules [list $name $version $platform]
    }

    RetrievePT::RetrieveModules $selected_modules "" ""
}

proc RetrievePT::GetModuleExists { module_xml_file xml_root_name name version platform} {
    set problemtype_exists 0        
    if { ![file exists $module_xml_file] } {
        #some part of the path could be a link
        set real_name [GiveRealFilenameDereferencingLinks $module_xml_file 1]
        if { $real_name != "" } {
            set module_xml_file $real_name
        }
    }
    set data [ReadProblemtypeXml $module_xml_file $xml_root_name {Name Version Platform}]
    if { $data != "" } {
        array set problemtype_local $data
        if { ![string compare -nocase $name $problemtype_local(Name)] } {
            # $version == $problemtype_local(Version)
            # if installed version is equal or greather than version in list/download.xml
            if { [::GidUtils::TwoVersionsCmp $problemtype_local(Version) $version] >= 0 || $version == "" } {
                if { $problemtype_local(Platform) == "" } {
                    # not defined in XML
                    set problemtype_local(Platform) [RetrievePT::GetCurrentPlatform]
                }
                lassign $problemtype_local(Platform) local_os local_bits
                lassign $platform listed_os listed_bits
                # considering as installed the listed_32 bits version on a 64bits OS will also give
                # a false positive when the installed version is the 64bits one !
                # i.e. PT-OS-64 is installed, then both listed PT will be checked: PT-OS-64 AND PT-OS-32 !!!!
                if { $local_os == $listed_os || $local_os == "All" || $listed_os == "All"} {
                    if { $local_bits == $listed_bits ||
                         $local_bits == "All" || $listed_bits == "All" ||
                         (($local_bits == 64) && ($listed_bits == 32))} {
                        #current problemtype theorically match all requirements
                        # or the 32 bits version is installed in a 64 bits OS
                        return 1
                    }
                }
            }
        }
    }
    return 0
}

#this procedure try to create localdir or an auxiliary file
proc RetrievePT::CanWrite { localdir } {
    if { ![file exists $localdir] } {
        file mkdir $localdir
        if { [file exists $localdir] && [file isdirectory $localdir] } {
            file delete $localdir
            set can_write 1
        } else {
            set can_write 0
        }
    } else {
        set tmpname ""
        for { set i 0 } { $i < 1000 } {incr i} {
            set tmpname [file join $localdir gid$i].txt
            if { ![file exists $tmpname] } break
        }
        set fp ""
        catch {set fp [open $tmpname w]} msg
        if { $fp != "" } {
            close $fp
            file delete $tmpname
            set can_write 1
        } else {
            set can_write 0
        }
    }
    return $can_write
}

#modules is a list of modules to get, where each item specify {name version platform}
#callback could be set to invoke our own procedure after finish uncompression
#the callback could have arguments like %f that will be replaced by the temporary filename (to allow delete this file)
proc RetrievePT::RetrieveModules { modules {callback_unzipping ""} {callback_after_unzip ""}} {
    variable w
    variable modulesarray
    variable ftpid
    variable progressbar
    variable progresslabel
    variable FileSize
    variable LastTimeMiliSec       

    if { [Connect] } { return 1 }
    
    if { [info exists w] && [winfo exists $w] } {
        set with_window 1
    } else {
        set with_window 0
    }
    
    if { ![llength [array names RetrievePT::modulesarray]] } {      
        #if list was not retrieved try to get it now
        RetrievePT::Retrievelist
    }

    set GidVersion [GiD_Info GiDVersion]

    foreach item $modules {
        set name [lindex $item 0]
        set version [lindex $item 1]
        set platform [lindex $item 2] ;#a list with two words like "Windows 32"
        lassign $platform os bits
        
        lassign [RetrievePT::GetModuleIndex $name $version $platform] i_module i_best
        if { $i_module == 0 && $i_best != 0 } {
            # if 64 bits not found, try 32 bits...
            if { $bits == 64} {
                set platform32 [ list [ lindex $platform 0] 32]
                lassign [ RetrievePT::GetModuleIndex $name $version $platform32] i_module32 i_best32
                if { $i_module32 != 0} {
                    set i_best $i_module32
                } elseif { $i_best32 != 0} {
                    # only set new best if previous best platform is different than demanded
                    if { ( $i_best == 0) || ( [ lindex $modulesarray($i_best,Platform)] != [ lindex $platform]) } {
                        set i_best $i_best32
                    }
                }
            }
	        set text [_ "Warning: Required problem type '%s' version %s for %s and found %s version for %s. Continue?" \
                    $name $version $platform $modulesarray($i_best,Version) $modulesarray($i_best,Platform)]
                           
            set retval [tk_messageBox -default ok -icon question -message $text -type okcancel]
            if { $retval == "cancel" } {
                set progresslabel ""
                if { $with_window } { 
                    set progressbar 0
                    $w configure -cursor ""                
                }
                return 1
            } else {
                set i_module $i_best
            }
        }
        if { $i_module == 0 } {
	        WarnWinText [_ "Warning: Required problem type '%s' not found" $name]
            return 1
        }
        if { ![IsVersionOKForCurrentGiD $i_module] } {
            set module_required_gid_version $modulesarray($i_module,MinimumGiDVersion)
     	    set text [_ "Warning: Problem type '%s' requires %s version %s and current version is %s. There can be incompatibilities. Proceed?" \
                        $name $::GidPriv(ProgName) $module_required_gid_version $GidVersion]
            
            set retval [tk_messageBox -default ok -icon question -message $text -type okcancel]
            if { $retval == "cancel" } {
                set progresslabel ""
                if { $with_window } {        
                    set progressbar 0
                    $w configure -cursor ""                    
                }
                return 1
            }
        }
        
        set content $modulesarray($i_module,Content)
        set localdir [RetrievePT::GetLocalFolder $content]
        set xml_root_name [RetrievePT::GetXmlRootName $content]
        set problemtype_exists 0        
        #find inside the xml file if match all requirements
        set module_xml_file [file join $localdir $modulesarray($i_module,XmlFile)]
        set problemtype_exists [GetModuleExists $module_xml_file $xml_root_name $name $version $platform]   
        if { $problemtype_exists  } {
            #already downloaded
	        set text [_ "Warning: Problem type '%s' is already installed in the local system. Are you sure to proceed?" $name]
            set retval [tk_messageBox -default ok -icon question -message $text -type okcancel]
            if { $retval == "cancel" } {
                set progresslabel ""
                if { $with_window } {
                    set progressbar 0
                    $w configure -cursor ""
                }
                return 1
            }
        }

        if { 0 } {
            #without administrative rights the folder maybe cannot be created
            if { ![file exists $localdir] } {
                file mkdir $localdir
            }
            
            if { ![file exists $localdir] || ![file isdirectory $localdir] } {
                WarnWin [_ "Directory %s does not exist" $localdir]
                set progresslabel ""
                if { $with_window } {
                    set progressbar 0
                    catch { $w configure -cursor "" }
                }
                return 1
            }
        }

        unset -nocomplain modulesarray($i_module,IsInstalled) ;#to refresh it
        set DownloadFile $modulesarray($i_module,DownloadFile)
        ftp::Type $ftpid binary ;#must set binary before asking file size
        set FileSize [ftp::FileSize $ftpid $DownloadFile]

        set LastTimeMiliSec [clock clicks -milliseconds]
        set tmpfilename [GidUtils::GetTmpFilename [file extension $DownloadFile]]

        ftp::Get $ftpid $DownloadFile $tmpfilename
        if { ![file exists $tmpfilename] } {
	        WarnWin [_ "Problems retrieving the Problem type"]
            set progresslabel ""
            if { $with_window } {
                set progressbar 0
                catch { $w configure -cursor "" }
            }
            return 1
        }
        set progresslabel "$FileSize/$FileSize"
        if { $with_window } {
            $w configure -cursor watch
            update
        }
        if { [file size $tmpfilename] == $FileSize } {
            set pwd [pwd]
            cd $localdir
            #set can_write [GiD_IsRunAsAdministrator]
            set can_write [RetrievePT::CanWrite $localdir]
            #arranco un proceso sin bloqueo con callback        
            if { $callback_unzipping != "" } {
                 regsub -all -- {%f} $callback_unzipping $tmpfilename callback_unzipping
            }
            if { $callback_after_unzip == "" } {
                set content $modulesarray($i_module,Content)  
                set callback_after_unzip [RetrievePT::GetCallbackAfterUnzip $content $module_xml_file]         
            }
            if { $::tcl_platform(platform) == "windows"} {
                if { $can_write } {
                    if { $callback_unzipping == "" } {
                        set callback_unzipping [list RetrievePT::UnzipProgress %m $tmpfilename $callback_after_unzip]                    
                    }
                    set exe "\"[file join $::GIDDEFAULT unzip.exe]\" -x -o -qq -U \"$tmpfilename\""
                    runExe run $exe -blocking false -callback $callback_unzipping -timeout 3600
                } else {
                    if { $callback_unzipping == "" } {
                        set callback_unzipping [list RetrievePT::UnzipProgress finished $tmpfilename $callback_after_unzip]
                    }
                    GiD_RunAsAdministrator [file join $::GIDDEFAULT unzip.exe] -x -o -qq -U $tmpfilename                                       
                    # retrieving is asynchronous, so we'll wait until all processes named unzip.exe                   
                    package require twapi
                    # maximum of 300 seconds (e.g. kratos expend a lot of time, 60 seconds are not enought)
                    set timeout 300
                    for { set i 0 } { $i < $timeout } { incr i } {
                        if { ![llength [twapi::get_process_ids -name unzip.exe]] } {
                            break
                        }
                        after 1000
                    }
                    #raise the callback
                    uplevel \#0 $callback_unzipping
                }
            } else {
                if { $callback_unzipping == "" } {
                    set callback_unzipping [list RetrievePT::UnzipProgress %m $tmpfilename $callback_after_unzip]
                }
                set sudo_cmd ""
                if { !$can_write} {
                    set sudo_cmd "[ properSudoCmd] "
                }
                if { [string tolower [file extension $DownloadFile]] == ".tgz" } {
                    # set exe "gunzip -c $tmpfilename | tar -xf -"
                    set exe "${sudo_cmd}tar -zxf $tmpfilename"
                } else {
                    # flags para ficheros ASCII (CR LF => LF)
                    set exe "${sudo_cmd}unzip -a -o $tmpfilename"              
                }
                runExe run $exe -blocking false -callback $callback_unzipping -timeout 3600
            }
            cd $pwd
        } elseif { [winfo exists $w] } {
	        WarnWin [_ "Error retrieving the Problem type"] $w
        }
    }
    return 0
}

proc RetrievePT::DeleteFile { filename } {   
    if { [ catch { file delete $filename  } err_txt] } {
        # # may be it's because of permissions
        # set can_write [RetrievePT::CanWrite [file dirname $Dfilename]]
        # if { !$can_write} {
        #     # it is !        
        if { $::tcl_platform(platform) == "windows"} {
            set gid gid.exe
        } else {
            set gid gid
        }
        runAsAdministrator [file join $::GIDDEFAULT $gid] -tclsh [file join $::GIDDEFAULT scripts file_delete.tcl] $filename
        # } else {
        #     WarnWin "Error deleting file $DownloadFile : \n$err_txt"
        # }
    }
}

proc RetrievePT::UnzipProgress { mode args } {    
    switch -exact -- $mode {
        prepare  {
            set ::RetrievePT::progresslabel [_ "Unzipping"]...
            update
        }
        output {
            #set ::RetrievePT::progresslabel [lindex $args 0]
        }
        finished {           
            set download_file [lindex $args 0]
            set callback_after_unzip [lindex $args 1]
            RetrievePT::DeleteFile $download_file 
            RetrievePT::UpdateDownloadModuleWindow
            if { $callback_after_unzip != "" } {
                uplevel \#0 $callback_after_unzip
            }
        }
    }
}

proc RetrievePT::UpdateDownloadModuleWindow { } {    
    variable w
    variable lists
    variable currenttab
        
    set ::RetrievePT::progresslabel [_ "Problemtype downloaded"]
    if { [info exists w] && [winfo exists $w] } {
        set with_window 1
    } else {
        set with_window 0
    }
    if { $with_window } {
        #::RetrievePT::CheckModulesInstalled
        $w configure -cursor ""
        set ::RetrievePT::progressbar 0            
        set list $lists($currenttab)
        RetrievePT::FillTable $list $currenttab
        update
        after 2000 { set ::RetrievePT::progresslabel "" }
    }
}

proc RetrievePT::UpdateMenuProblemtypes { } {    
    variable w
    variable lists
    variable currenttab
    GidUtils::SetWarnLine [_ "Problemtype downloaded"]
    ClearCacheFormProblemtypeMenuDir
    #if the problemtypes menu must be deleted to use FastCacheFromProblemtypeMenu
    #FastCacheFromProblemtypeMenu

}

proc RetrievePT::LoadProblemtype { projecttype } {    
    #already downloaded and unzipped (or timeout exceeded)   
    RetrievePT::Close ;#close because have used RetrievePT::OpenGUI to show progress downloading (but not unzipping)
    GidUtils::SetWarnLine [_ "Problemtype downloaded"]
    #try to load now this downloaded problemtype, when uncompression is finished
    SetProblemTypeChoose $projecttype 0
}

proc RetrievePT::LoadPlugin { plugin_folder } {    
    #already downloaded and unzipped (or timeout exceeded)
    GidUtils::SetWarnLine [_ "Plugin downloaded"]
    #try to load now this downloaded plugin, when uncompression is finished
    SourcePlugins $plugin_folder
}

proc RetrievePT::SetProblemTypeIfNotExitsTryDownload { projecttype } {  
    set fail 0
    set download_filename [file join [file dirname [GiveFileInsideProblemType $projecttype "" 0]] download.xml]
    set data [ReadProblemtypeXml $download_filename DownloadProblemtype {Name Version ShortDescription}]
    if { $data != "" } {
        array set module_download $data
        set current_platform [RetrievePT::GetCurrentPlatform]
        set module_download(Platform) [RetrievePT::GetCurrentPlatform]

        set module_xml_file [GiveFileInsideProblemType $projecttype .xml 1]
        set xml_root_name Infoproblemtype ;#[RetrievePT::GetXmlRootName Program]
        set problemtype_exists [RetrievePT::GetModuleExists $module_xml_file $xml_root_name $module_download(Name) $module_download(Version) $module_download(Platform)]
        if { $problemtype_exists } {
            #ok, already downloaded
            SetProblemTypeChoose $projecttype 0 ;#invoke it again to continue loading
        } else {
	        set text [_ "Problem type '%s %s' will be download from Internet. Continue?" $module_download(Name) $module_download(Version)]
            set text [concat $text \n\n\n [_ "Description"]: $module_download(ShortDescription)]
            set retval [tk_messageBox -default ok -icon question -message $text -type okcancel]
            if { $retval == "cancel" } {
                set fail 1
            } else {
                set modules [list [list $module_download(Name) $module_download(Version) $module_download(Platform)]]
                RetrievePT::OpenGUI
                #the callback could have arguments like %f that will be replaced by the temporary filename
                set callback_unzipping "" ;#if replace it must raise in the procedure the callback_after_unzip when finised!!!
                set callback_after_unzip [list RetrievePT::LoadProblemtype $projecttype]
                set fail [RetrievePT::RetrieveModules $modules $callback_unzipping $callback_after_unzip]                
                if { $fail } {
		            GidUtils::SetWarnLine [_ "Error downloading problem type '%s %s'" $module_download(Name) $module_download(Version)]
                }
            }
        }         
    } else {
        SetProblemTypeChoose $projecttype 0 ;#invoke it again to continue loading
    }
    return $fail
}


proc RetrievePT::DisplayText { parent textinfo } {

    set w $parent.retrievepttext

    InitWindow $w [_ "Display info"] PreRetrievePTDisplayHelpWindowGeom
    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0

    set sw [ScrolledWindow $w.lf \
            -auto both -relief sunken -borderwidth 2]
    set text [text $sw.text -width 60 -height 20]
    bind $text <1> "focus $text"
    $sw setwidget $text

    ttk::button $w.close -text [_ "Close"] -width 10 -command "destroy $w" -style BottomFrame.TButton
    #SetWidgetsWidthFromText $w.close
    grid $sw -sticky nsew
    grid $w.close

    grid columnconf $w 0 -weight 1
    grid rowconf $w 0 -weight 1

    $text insert end $textinfo
    $text configure -state disabled
}

proc RetrievePT::ModuleInfo {} {
    variable modulesarray    
    variable lists
    variable currenttab

    set widget $lists($currenttab)
    set item [$widget curselection]    

    if { [llength $item] == 0 } {
    	WarnWin [_ "Select first one Problem type"] $widget
        return
    }
    if { [llength $item] > 1 } {
    	WarnWin [_ "Select ONLY one Problem type"] $widget
        return
    }
    
    set values [$widget get $item]
    set name [lindex $values 0]
    set version [lindex $values 1]
    set platform [lindex $values 2]
    lassign [RetrievePT::GetModuleIndex $name $version $platform] i_module i_best    
    if { $i_module == 0 } {
	    WarnWinText [_ "Warning: Required problem type '%s' not found" $name]
        return 1
    }
    set desc ""
    if { ![IsVersionOKForCurrentGiD $i_module] } {
        set module_required_gid_version $modulesarray($i_module,MinimumGiDVersion)
        set GidVersion [GiD_Info GiDVersion]
    	set desc [_ "Warning: This Problem type requires %s version %s and current version is %s. There can be incompatibilities" \                
                $::GidPriv(ProgName) $module_required_gid_version $GidVersion]
    }
    if { [info exists modulesarray($i_module,Description)] } {
        append desc $modulesarray($i_module,Description)        
    } elseif { [info exists modulesarray($i_module,ShortDescription)] } {
        append desc $modulesarray($i_module,ShortDescription)
    } else {
    	WarnWin [_ "This Problem type has no description"] $widget
        return
    }
    RetrievePT::DisplayText $widget $desc
}

proc RetrievePT::ModuleNews {} {
    variable modulesarray
    variable lists
    variable currenttab

    set widget $lists($currenttab)
    set item [$widget curselection]    

    if { [llength $item] == 0 } {
    	WarnWin [_ "Select first one Problem type"] $widget
        return
    }
    if { [llength $item] > 1 } {
    	WarnWin [_ "Select ONLY one Problem type"] $widget
        return
    }
    
    set values [$widget get $item]
    set name [lindex $values 0]
    set version [lindex $values 1]
    set platform [lindex $values 2]
    lassign [RetrievePT::GetModuleIndex $name $version $platform] i_module i_best    
    if { $i_module == 0 } {
    	WarnWinText [_ "Warning: Required problem type '%s' not found" $name]
        return 1
    }    

    set desc ""
    if { ![IsVersionOKForCurrentGiD $i_module] } {
        set GidVersion [GiD_Info GiDVersion]
        set module_required_gid_version $modulesarray($i_module,MinimumGiDVersion)
    	set desc [_ "Warning: This Problem type requires %s version %s and current version is %s. There can be incompatibilities" $::GidPriv(ProgName) $module_required_gid_version $GidVersion]
    }
    if { [info exists modulesarray($i_module,NewsInVersion)] } {
        append desc $modulesarray($i_module,NewsInVersion)
    } else {
    	WarnWin [_ "This Problem type has no news information"] $widget
        return
    }
    RetrievePT::DisplayText $widget $desc
}

proc RetrievePT::GetCurrentPlatform {} {
    set is64bits 0
    if { $::tcl_platform(pointerSize) == 8 } { 
        set is64bits 1
    }
    if { $::tcl_platform(platform) == "windows" } {
        if { $is64bits } {
            set myplatform "Windows 64"
        } else {
            set myplatform "Windows 32"
        }
    } elseif { $::tcl_platform(os)        == "Linux" } {
        if { $is64bits } {
            set myplatform "Linux 64"
        } else {
            set myplatform "Linux 32"
        }
    } elseif { $::tcl_platform(os) == "Darwin" } {
        if { $is64bits } {
            set myplatform "MacOSX 64"
        } else {
            set myplatform "MacOSX 32"
        }
    } else {
        #AIX , HP-UX, IRIX, SunOS, DragonFly, FreeBSD, NetBSD, OpenBSD, OSF1
        set myplatform "$::tcl_platform(os) ?"
    }
    return $myplatform
}

proc RetrievePT::OpenGUI { { parent .gid } } {
    variable w
    variable progressbar
    variable progresslabel
    variable images
    variable lists
    variable currenttab

    set images(checked-on) [image create photo -data {
        R0lGODlhEAAQAIQAAIKCgrxdXe4AANnZ2fePj+0BAdVOWdl6fugGBuSgpeimpu0AANvW1up8
        fOwAANympv///+ttbe0fH+0UFNyrq+TAwOa1tf//////////////////////////////////
        /yH+Dk1hZGUgd2l0aCBHSU1QACH5BAEKAB8ALAAAAAAQABAAAAVL4CeOZGmeJqCuLFC2bSCQ
        6mDfA1EYNIDbB0Ti5VMsGIOG4wEhDhbHiEMBafYGAoFkQhlUndlFxfa9WhaWW3mkqrrfr7fc
        iqrb7/cQADs=
    }]
    set images(checked-off) [image create photo -data {
        R0lGODlhEAAQAKEAAP///4KCgv///////yH+Dk1hZGUgd2l0aCBHSU1QACH5BAEKAAIALAAA
        AAAQABAAAAImlI+py50AowRvylBXwAhqfnjKlo2gISZkR5mlegrQRtdPjcfO3hQAOw==
    }]
    set images(cross) [image create photo -data {
        R0lGODlhEAAQAIQAAIKCgrxdXe4AANnZ2fePj+0BAdVOWdl6fugGBuSgpeimpu0AANvW1up8
        fOwAANympv///+ttbe0fH+0UFNyrq+TAwOa1tf//////////////////////////////////
        /yH+Dk1hZGUgd2l0aCBHSU1QACH5BAEKAB8ALAAAAAAQABAAAAVJ4CeOZGmeJqCuLFC2bSCQ
        qiAMOG4bNDDYOVvi5VMIGD/BA0IcLBY2gQLC7CWjA2ozWrFprTaLTlAd1chUqu0FQafVqLh8
    }]

    package require ftp
    package require tablelist_tile
    package require progressbar

    set w $parent.retrievept

    catch { destroy $parent.retrievept }

    InitWindow $w [_ "Retrieve analysis Problem types"] PreRetrievePTWindowGeom RetrievePT::OpenGUI
    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0

    wm protocol $w WM_DELETE_WINDOW RetrievePT::Close


    ttk::label $w.l1 -text \
        [_ "Note: In order to use this function, it is necessary to be connected to internet"] \
            -font BoldFont
    set nb [ttk::notebook $w.nb]
    ttk::notebook::enableTraversal $nb

    set f1 [ttk::frame $nb.f1]
    $nb add $f1 -text [GetCurrentPlatform] -underline 0
    set lists(0) [RetrievePT::CreateTable $f1]

    set f2 [ttk::frame $nb.f2]
    $nb add $f2 -text [_ "Other OS"] -underline 0
    set lists(1) [RetrievePT::CreateTable $f2]

    set wp [::progressbar::progressbar $w.f -variable ::RetrievePT::progressbar -width 300 \
            -background [ [winfo toplevel $w] cget -background]]
    ttk::label $w.l2 -textvariable RetrievePT::progresslabel
    set progressbar 0
    set progresslabel ""

    grid $w.l1 -sticky ew    
    grid $nb -sticky nsew
    grid $w.l2 -sticky ew
    grid $wp -sticky ew
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 1 -weight 1

    set def_back [$w cget -background]
    ttk::frame $w.buts -style BottomFrame.TFrame
    # -bg [CCColorActivo $def_back]

    ttk::button $w.buts.b2 -text [_ "Retrieve Problem type"] -command RetrievePT::RetrieveSelectedModules \
      -style BottomFrame.TButton
    ttk::button $w.buts.b3 -text [_ "Problem type info"] -command RetrievePT::ModuleInfo \
      -style BottomFrame.TButton
    ttk::button $w.buts.b4 -text [_ "Problem type news"] -command RetrievePT::ModuleNews \
      -style BottomFrame.TButton
    ttk::button $w.buts.b5 -text [_ "Close"] -command RetrievePT::Close \
      -style BottomFrame.TButton
    #SetWidgetsWidthFromText $w.buts.b2 $w.buts.b3 $w.buts.b4 $w.buts.b5
    grid $w.buts.b2 $w.buts.b3 $w.buts.b4 $w.buts.b5 -padx 2 -pady 3

    grid $w.buts -sticky ew
    grid columnconf $w.buts "0 1 2 3" -weight 1
    if { $::tcl_version >= 8.5 } { grid anchor $w.buts center }    

    update idletasks

    set progresslabel [_ "Retrieving Problem type list"]...
    update
    RetrievePT::Retrievelist

    RetrievePT::FillTable $lists(0) 0
    RetrievePT::FillTable $lists(1) 1

    if { ![info exists currenttab] } {
        set currenttab 0
    }
    $nb select $currenttab

    set progresslabel ""
    update
    bind $w <<NotebookTabChanged>> [list RetrievePT::TabChanged $nb]
    bind $w <Destroy> [list +RetrievePT::OnDestroyWindow %W $w] ;# + to add to previous script
    set contents {Program Interface Plugin}
    foreach content $contents {
        trace add variable RetrievePT::show_content($content) write [list RetrievePT::OnChangeShowContent]
    }       
}

proc RetrievePT::OnDestroyWindow { W w } {
    variable show_content
    if { $W != $w } return   
    set contents {Program Interface Plugin}
    foreach content $contents {
        trace remove variable RetrievePT::show_content($content) write [list RetrievePT::OnChangeShowContent]
    }   
}

proc RetrievePT::OnChangeShowContent { args } {
    variable lists
    RetrievePT::FillTable $lists(0) 0
    RetrievePT::FillTable $lists(1) 1
}

proc RetrievePT::TabChanged { nb } {
    variable currenttab
    set currenttab [$nb index current]
}

proc RetrievePT::CreateTable { w } {
    set sw [ScrolledWindow $w.lf \
            -auto both -relief sunken -borderwidth 2]
    
    package require tablelist_tile
    set list [tablelist::tablelist $sw.lb \
            -exportselection 0 \
            -columns [list \
	    	18 [_ "Problem type"]        left \
                8 [_ "Version"]        left \
                12 [_ "Platform"]        left \
                8 [_ "Installed"] center \
                10 [_ "Publish date"] center \
                20 [_ "Characteristics"]        left \
                ] \
            -labelcommand tablelist::sortByColumn \
            -stretch end -selectmode extended]

    $sw setwidget $list

    bind [$list bodytag] <Button-$::gid_right_button> "[list RetrievePT::MenuContextualRetrievePT %W %x %y] ; break"
    grid $sw -sticky nsew
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1
    return $list
}

proc RetrievePT::MenuContextualRetrievePT { T x y } {
    if { [winfo class [winfo parent $T]] != "Tablelist" } {        
        return
    }
    set txt [[winfo parent $T] cellcget active -text]
    if { $txt == "" } return
    set w $T.menucontextualretrievept
    if { [winfo exists $w] } {
        destroy $w
    }
    menu $w
    $w add command -label [_ "Retrieve %s" $txt] -command RetrievePT::RetrieveSelectedModules
    $w add command -label [_ "%s info" $txt] -command RetrievePT::ModuleInfo
    $w add command -label [_ "%s news" $txt] -command RetrievePT::ModuleNews
    set x [expr [winfo rootx $T]+$x+2]
    set y [expr [winfo rooty $T]+$y]
    GiD_PopupMenu $w $x $y
}

