proc test_pkgs {} {
    interp create test_intp
    test_intp eval lappend auto_path $::auto_path

    if { [ info exists ::env(LD_LIBRARY_PATH)] } { 
        WarnWinText "LD_LIBRARY_PATH = $::env(LD_LIBRARY_PATH)"
    } else {
        WarnWinText "::env(LD_LIBRARY_PATH) does not exist"
    }
    WarnWinText "--> Starting some BASIC TESTS"
    set lst_pkgs [ list Tcl Tk treectrl gdal objarray hdf5 \
		       compass_utils sqlite3 pdfwriter \
		       Lognoter lognoter_db xml2pdf drawformulas \
		       tktablet pdfwriter \
		       wordwidget_snit pdf4tcl \
		       tdom tkdnd Tkhtml gid_groups_conds fulltktable vtk vtk_objarray verifp \
		       Itcl Thread tdbc tdbc::odbc tdbc::mysql tdbc::postgres]
    lappend lst_pkgs Lognoter pdfwriter tktablet xml2pdf
    if { $::tcl_platform(platform) == "windows"} {
	# former 'reg' now in Tcl 8.6.1 is registry
	lappend lst_pkgs dde registry twapi
	lappend lst_pkgs gdi printer
    }
    foreach pkg $lst_pkgs {
	set err 0
	set err_txt ""
	# no new-line: final 1
	WarnWinText "package require $pkg ... " "basic test" 0 1
	set err [ catch {
	    test_intp eval package require $pkg
	} err_txt]
	WarnWinText " $err_txt"
    }
    WarnWinText "--> versions...."
    foreach pkg $lst_pkgs {
	set err 0
	set err_txt ""
	# no new-line: final 1
	WarnWinText "package versions $pkg ... " "basic test" 0 1
	set err [ catch {
	    test_intp eval package versions $pkg
	} err_txt]
	WarnWinText " $err_txt"
    }
    WarnWinText "--> Done with some BASIC TESTS"
    if { [ info exists ::env(LD_LIBRARY_PATH)] } {
        WarnWinText "LD_LIBRARY_PATH = $::env(LD_LIBRARY_PATH)"
    } else {
        WarnWinText "::env(LD_LIBRARY_PATH) does not exist"
    }
    interp delete test_intp
}

proc test_packages {} {
    test_pkgs
}

proc TP {} {
    test_pkgs
}

proc test_fps { { num_redraws 0} } {
    # set cmd [ list GiD_Process escape escape escape escape escape redraw]
    set cmd [ list GiD_Process 'redraw]
    if { $num_redraws == 0 } {
	# try to guess number of redraws in a second
	# minimum of 30 redraws
	# time returns: XXX microseconds per iteration
	# to 3 redraws to force and count, at least, one swap buffers
	set try_t [ lindex [ time $cmd 2] 0]
	set num_redraws [ expr { round( 1 / ( $try_t * 1e-6))}]
	if { $num_redraws < 30} {
	    set num_redraws 30
	}
	if { $num_redraws > 300} {
	    set num_redraws 300
	}
    }
    # WarnWinText text title translation no_new_line
    WarnWinText "Redrawing $num_redraws times ..." "Warning" 0 1
    # for { set i 0} { $i < 30} { incr i} {
    # 	WarnWinText "Redrawing $i"
    # 	GiD_Process escape escape escape escape escape redraw
    # }
    # time already returns the average time per iteration
    set total_t [ lindex [ time $cmd $num_redraws] 0]
    set fps [ expr round( 1.0 / ( $total_t * 1e-9)) / 1000.0]
    WarnWinText " done: [ format %.3g $fps] fps."
    return $fps
}

proc fps { { num_redraws 0} } {
    test_fps $num_redraws
}

proc FPS { { num_redraws 0} } {
    test_fps $num_redraws
}
