

#  New menu funcions
#  Now menus are identified by UNTRANSLATED name (English is considered as reference untranslated string)
#  (positions of options in their menus start at 0)

namespace eval GiDMenu {
    variable do_UpdateMenus 1
}

proc GiDMenu::EnableUpdateMenus {} {
    variable do_UpdateMenus
    set do_UpdateMenus 1
}

proc GiDMenu::DisableUpdateMenus {} {
    variable do_UpdateMenus
    set do_UpdateMenus 0
}

# Updates changes
proc GiDMenu::UpdateMenus {} {
    variable do_UpdateMenus
    if { $do_UpdateMenus} {
	CreateTopMenus
    }
}

# -----------------------------------------------------------------------------
#  GiDMenu::Delete
#
#  Delete a toplevel menu
#  menu_name_untranslated (untranslated english menu name, for example "Files#C#menu"
#  prepost: PRE POST or PREPOST
#  translationfunc can be _ (the default value, for GiD standard strings) or = (for module owned strings)
#  for example:
#    GiDMenu::Delete "Mesh#C#menu" PRE
#    it's also valid the old sintax, without #C#menu , but is more expensive to find, e.g:
#    GiDMenu::Delete "Mesh" PRE
#
#  Remember call GiDMenu::UpdateMenus to apply all changes.
# -----------------------------------------------------------------------------
proc GiDMenu::Delete { menu_name_untranslated prepost {translationfunc _} } {
    if {$prepost!="PRE" && $prepost!="POST" && $prepost!="PREPOST"} {
        return -code error [_ "Wrong arg. Must be PRE, POST or PREPOST."]
    }
    if { $prepost eq "PRE" } {
        set where [list "" "PRE"]
    } elseif { $prepost eq "POST" } {
        set where [list "P" "POST"]
    } else {
        set where [list "" "PRE" "P" "POST"]
    }
    set variable_names { MenuEntries MenuCommands MenuAcceler MenuIcons }
    foreach { c desc } $where {
        set pos [GiDMenu::_FindIndex $menu_name_untranslated $desc $translationfunc]
        if { $pos == "-1" } {
            WarnWin [_ "ERROR: Menu %s doesn't exist in $desc." $menu_name_untranslated]
            return
        }
        upvar \#0 MenuNames$c MenuNames
        foreach variable_name $variable_names {
            upvar \#0 ${variable_name}$c $variable_name
        }        
        set MenuNames [lreplace $MenuNames $pos $pos]
        foreach variable_name $variable_names {
            array unset $variable_name $pos,*
            array unset $variable_name $pos
        }        
        set len [llength $MenuNames]        
        for {set i [expr $pos+1]} {$i <= $len} {incr i} {
            set i_prev [expr {$i-1}]
            foreach variable_name $variable_names {
                foreach item [concat [array names $variable_name $i,*] [array names $variable_name $i]] {
                    set indexes [split $item ,]
                    lset indexes 0 $i_prev
                    set new_item [join $indexes ,]
                    set ${variable_name}($new_item) [set ${variable_name}($item)]
                    unset ${variable_name}($item)
                }
            }            
        }
        foreach variable_name $variable_names {
            array unset $variable_name $len,*
            array unset $variable_name $len
        }
    }
}

# -----------------------------------------------------------------------------
#  GiDMenu::Create
#
#  Creates a new menu. New menus are inserted between the 'Calculate' and 'Help' menu
#  menu_name_untranslated (untranslated english menu name, for example "Files#C#menu"
#  prepost: PRE POST or PREPOST
#  translationfunc can be _ (the default value,for GiD standard strings) or = (for module strings)
#  pos: index where the new menu will be inserted, begin from 0, "end" can be used
#       if set to -1 (default) then is inserted before the 'Help' menu
#  Remember call GiDMenu::UpdateMenus to apply all changes.
# -----------------------------------------------------------------------------
proc GiDMenu::Create { menu_name_untranslated prepost {pos -1} {translationfunc _}} {
    if { $prepost != "PRE" && $prepost != "POST" && $prepost != "PREPOST" } {
        WarnWin [_ "Wrong arg. Must be PRE, POST or PREPOST."]
        return 1
    }
    if {$prepost eq "PRE"} {
        set where [list ""]
    } elseif {$prepost eq "POST"} {
        set where [list "P"]
    } else {
        set where [list "" "P"]
    }
    set variable_names { MenuEntries MenuCommands MenuAcceler MenuIcons }
    foreach c $where {
        upvar \#0 MenuNames$c MenuNames
        foreach variable_name $variable_names {
            upvar \#0 ${variable_name}$c $variable_name
        } 
        
        set length [llength $MenuNames]
        if { $pos == "end" } {
            set pos $length
        }
        if { $pos == -1 } {
            #find the position of the help menu
            set pos [GiDMenu::_FindIndex "Help#C#menu" PRE _]
            if { $pos == -1 } {
                set pos [expr [llength $MenuNames]-1]
            }
        }
        if { $pos < 0 } {
            set pos 0
        } elseif { $pos > $length } {
            set pos $length
        }
        set MenuNames [linsert $MenuNames $pos [$translationfunc $menu_name_untranslated]]
        for { set i $length} { $i > $pos } { incr i -1 } {
            set i_prev [expr {$i-1}]
            foreach variable_name $variable_names {           
                foreach item [concat [array names $variable_name $i_prev,*] [array names $variable_name $i_prev]] {
                    set indexes [split $item ,]
                    lset indexes 0 $i
                    set new_item [join $indexes ,]
                    set ${variable_name}($new_item) [set ${variable_name}($item)]            
                }
            }
        }
        foreach variable_name $variable_names {            
            foreach item [array names $variable_name $pos,*] {
                unset ${variable_name}($item)
            }
            set ${variable_name}($pos) [list ""]
        }
    }
    return 0
}


# -----------------------------------------------------------------------------
#  GiDMenu::InsertOption
#
#  Creates a new option for a given menu in a given  position.
#  menu_name_untranslated: the toplevel menu name untranslated
#  option_name_untranslated: the option name, must be a list, allowing sublevels
#  e.g [list "Create#C#menu" "Arc#C#menu"]
#  it is also accepted the old name without comments: [list "Create" "Arc"]
#  position: the new menu option is inserted after the given position
#  (positions start at 0), "end" can be used
#  prepost: PRE POST or PREPOST
#  Arg 'command' is the command called when the menu option is selected.
#  Arg acceler is an optional key accelerator, like "Control-s"
#  Arg icon is the name of an small icon (16x16 pixels)
#  ins_repl: "replace" (default): the new menu option will replace an old if exists in the given position,
#            "insert" : will be inserted before the position
#            "insertafter" : will be inserted after the position
#  translationfunc can be _ (the default value,for GiD standard strings) or = (for module strings)
#
#  Remember call GiDMenu::UpdateMenus to apply all changes.
# -----------------------------------------------------------------------------

proc GiDMenu::InsertOption { menu_name_untranslated option_name_untranslated position prepost \
    command {acceler ""} {icon ""} {ins_repl "replace"} {translationfunc _} } {

    if {$prepost!="PRE" && $prepost!="POST" && $prepost!="PREPOST"} {
        WarnWin [_ "Wrong arg. Must be PRE, POST or PREPOST."]
        return 1
    }

    if {$prepost eq "PRE"} {
        set where [list "" "PRE"]
    } elseif {$prepost eq "POST"} {
        set where [list "P" "POST"]
    } else {
        set where [list "" "PRE" "P" "POST"]
    }
    set variable_names { MenuEntries MenuCommands MenuAcceler MenuIcons }
    foreach {c desc} $where {
        upvar \#0 MenuNames$c MenuNames
        foreach variable_name $variable_names {
            upvar \#0 ${variable_name}$c $variable_name
        } 
        
        set i [GiDMenu::_FindIndex $menu_name_untranslated $desc $translationfunc]
        if { $i == "-1" } {
            WarnWin [_ "ERROR: Menu %s doesn't exist in $desc." $menu_name_untranslated]
            return 1
        }
                
        set i [GiDMenu::_FindOptionIndex $i [lrange $option_name_untranslated 0 end-1] $desc]
        if { $i == -1 } {
            return 1
        }
        if { ![info exists MenuEntries($i)] } {
            set MenuEntries($i) ""
        }
        set tail_opt_name [$translationfunc [lindex $option_name_untranslated end]]
        if { $position == "end" } {
            set position [expr {[llength $MenuEntries($i)]-1}]
        }
        if { $ins_repl == "insertafter" } {
            incr position
        }
        set is_runtime_menu 0
        if { [lindex $command 0] == "-np-" } {
            set command [lrange $command 1 end]       
            set is_runtime_menu 1
        }        
        if {[llength $MenuEntries($i)]<= $position} {
            lappend MenuEntries($i) $tail_opt_name
            lappend MenuCommands($i) "-np- $command"
            if {[info exists MenuAcceler($i)]} {
                lappend MenuAcceler($i) $acceler
            }
            if {[info exists MenuIcons($i)]} {
                lappend MenuIcons($i) $icon
            }
            if { $is_runtime_menu } {
                #trick used by CreateCascadedMenu to add to submenu
                set position [expr {[llength MenuCommands($i)]-1}]
                set MenuEntries($i,$position) ""
                #set MenuCommands($i,$position) ""
            }
        } else {
            if { $ins_repl == "replace" } {
                del_submenus $i $position $desc
                set MenuEntries($i) [lreplace $MenuEntries($i) $position $position $tail_opt_name]
                set MenuCommands($i) [lreplace $MenuCommands($i) $position $position "-np- $command"]
                if {[info exists MenuAcceler($i)] && [llength $MenuAcceler($i)] > $position} {
                    set MenuAcceler($i) [lreplace $MenuAcceler($i) $position $position $acceler]
                }
                if {[info exists MenuIcons($i)] && [llength $MenuIcons($i)] > $position } {
                    set MenuIcons($i) [lreplace $MenuIcons($i) $position $position $icon]
                }
            } else {
                # "insert"
                set MenuEntries($i) [linsert $MenuEntries($i) $position $tail_opt_name]
                
                set MenuCommands($i) [linsert $MenuCommands($i) $position "-np- $command"]
                if {[info exists MenuAcceler($i)]} {
                    set MenuAcceler($i) [linsert $MenuAcceler($i) $position $acceler]
                }
                if {[info exists MenuIcons($i)]} {
                    set MenuIcons($i) [linsert $MenuIcons($i) $position $icon]
                }
                move_submenus $i $position $desc "down"                              
            }
            if { $is_runtime_menu } {
                #trick used by CreateCascadedMenu to add to submenu
                set MenuEntries($i,$position) ""
                #set MenuCommands($i,$position) ""
            }
        }
    }
    return 0
}


# -----------------------------------------------------------------------------
#  GiDMenu::RemoveOption
#
#  Removes an option of a given menu.
#  menu_name_untranslated: the toplevel menu name untranslated
#  option_name_untranslated: the option name, can be a list for sublevels
#  e.g [list "Create#C#menu" "Arc#C#menu"]
#  it is also accepted the old name without comments: [list "Create" "Arc"]
#  prepost: PRE POST or PREPOST
#  translationfunc can be _ (the default value,for GiD standard strings) or = (for module strings)
#
#  Remember call GiDMenu::UpdateMenus to apply all changes.
# -----------------------------------------------------------------------------
proc GiDMenu::RemoveOption {menu_name_untranslated option_name_untranslated prepost {translationfunc _}} {
    if {$prepost!="PRE" && $prepost!="POST" && $prepost!="PREPOST"} {
        WarnWin [_ "Wrong arg. Must be PRE, POST or PREPOST."]
        return 1
    }

    if {$prepost eq "PRE"} {
        set where [list "" "PRE"]
    } elseif {$prepost eq "POST"} {
        set where [list "P" "POST"]
    } else {
        set where [list "" "PRE" "P" "POST"]
    }
    set variable_names { MenuEntries MenuCommands MenuAcceler MenuIcons }
    foreach {c desc} $where {
        upvar \#0 MenuNames$c MenuNames
        foreach variable_name $variable_names {
            upvar \#0 ${variable_name}$c $variable_name
        } 
        
        set i [GiDMenu::_FindIndex $menu_name_untranslated $desc $translationfunc]
        if { $i == "-1" } {
            WarnWin [_ "ERROR: Menu %s doesn't exist in $desc." $menu_name_untranslated]
            return 1
        }
        set index [GiDMenu::_FindOptionIndex $i $option_name_untranslated $desc]
        if { $index == -1 } {
            return 1
        }
        set i [join [lrange [split $index ,] 0 end-1] ,]
        set position [lindex [split $index ,] end]
        del_submenus $i $position $desc
        move_submenus $i $position $desc "up"
        set MenuEntries($i)  [lreplace $MenuEntries($i) $position $position ]
        set MenuCommands($i) [lreplace $MenuCommands($i) $position $position]
        if {[info exists MenuAcceler($i)] && [llength $MenuAcceler($i)] > $position} {
            set MenuAcceler($i) [lreplace $MenuAcceler($i) $position $position ]
        }
        if {[info exists MenuIcons($i)] && [llength $MenuIcons($i)] > $position} {
            set MenuIcons($i) [lreplace $MenuIcons($i) $position $position]
        }
    }
    return 0
}

# -----------------------------------------------------------------------------
#  GiDMenu::ModifyOption
#
#  Modify a previous menu identified by menu_name_untranslated option_name_untranslated and prepost
#  menu_name_untranslated: the toplevel menu name untranslated
#  option_name_untranslated: the option name, can be a list for sublevels
#  e.g [list "Create#C#menu" "Arc#C#menu"]
#  it is also accepted the old name without comments: [list "Create" "Arc"]
#  Arg prepost: PRE POST or PREPOST
#  Arg new_option_name new option name (-default- to keep the current)
#  Arg new_command is an optional new command (-default- to keep the current)
#  Arg new_acceler is an optional key accelerator, like "Control-s" (-default- to keep the current)
#  Arg new_icon is the name of an small icon (16x16 pixels) (-default- to keep the current)
#  Arg translationfunc can be _ (the default value,for GiD standard strings) or = (for module strings)
#
#  Remember call GiDMenu::UpdateMenus to apply all changes.
# -----------------------------------------------------------------------------
proc GiDMenu::ModifyOption { menu_name_untranslated option_name_untranslated prepost new_option_name \
    {new_command -default-} {new_acceler -default-} {new_icon -default-} {translationfunc _} } {

    if {$prepost!="PRE" && $prepost!="POST" && $prepost!="PREPOST"} {
        return -code error [_ "Wrong arg. Must be PRE, POST or PREPOST."]
    }
        
    if {$prepost eq "PRE"} {
        set where [list "" "PRE"]
    } elseif {$prepost eq "POST"} {
        set where [list "P" "POST"]
    } else {
        set where [list "" "PRE" "P" "POST"]
    }
    set variable_names { MenuEntries MenuCommands MenuAcceler MenuIcons }
    foreach {c desc} $where {
        upvar \#0 MenuNames$c MenuNames
        foreach variable_name $variable_names {
            upvar \#0 ${variable_name}$c $variable_name
        } 
        
        set i [GiDMenu::_FindIndex $menu_name_untranslated $desc $translationfunc]
        if { $i == "-1" } {
            ::GidUtils::SetWarnLine [_ "ERROR: Menu %s doesn't exist in $desc." $menu_name_untranslated]
            return 1
        }
        
        set index [GiDMenu::_FindOptionIndex $i $option_name_untranslated $desc]
        if { $index == -1 } {
            return 1
        }
        set i [join [lrange [split $index ,] 0 end-1] ,]
        set position [lindex [split $index ,] end]

        if { $new_option_name != "-default-" } {
            set MenuEntries($i) [lreplace $MenuEntries($i) $position $position $new_option_name]
        }
        if { $new_command != "-default-" } {
            set MenuCommands($i) [lreplace $MenuCommands($i) $position $position "-np- $new_command"]
        }
        if {[info exists MenuAcceler($i)] && [llength $MenuAcceler($i)] > $position} {
            if { $new_acceler != "-default-" } {
                set MenuAcceler($i) [lreplace $MenuAcceler($i) $position $position $new_acceler]
            }
        }
        if {[info exists MenuIcons($i)] && [llength $MenuIcons($i)] > $position } {
            if { $new_icon != "-default-" } {
                set MenuIcons($i) [lreplace $MenuIcons($i) $position $position $new_icon]
            }
        }
    }
    return 0
}

# -----------------------------------------------------------------------------
#  GiDMenu::_FindIndex
#
#  return the menu index
#  menu_name_untranslated (untranslated english menu name, for example "Files#C#menu"
#  prepost: PRE or POST (where to find)
#  translationfunc can be _ (for GiD standard strings) or = (for additional module strings)
# -----------------------------------------------------------------------------
proc GiDMenu::_FindIndex { menu_name_untranslated prepost {translationfunc _} } {
    global MenuNames MenuNamesP
    if {$prepost eq "PRE" } {
	upvar \#0 MenuNames TranslatedNames
    } else { ;#POST
	upvar \#0 MenuNamesP TranslatedNames
    }
    set i [lsearch $TranslatedNames [$translationfunc $menu_name_untranslated]]
    if { $i!= -1 } {
	return $i
    }
    if {$prepost eq "PRE" } {
	#back compatibility: Meshing menu renamed to Mesh in 7.5.9b
	if { $menu_name_untranslated == "Meshing" } {
	    set i [lsearch $TranslatedNames [$translationfunc "Mesh#C#menu"]]
	    if { $i != "-1" } {
		return $i
	    }
	}
    }
    #back compatibility: menus renamed from oldname to oldname#C#menu in 8.0
    set postfix "#C#menu"
    set i [lsearch $TranslatedNames [$translationfunc $menu_name_untranslated$postfix]]
    if { $i != "-1" } {
	return $i
    }
    return -1
}


#to find a option position by its name. returns -1 if not found
proc GiDMenu::GetOptionIndex { menu_name_untranslated option_name_untranslated prepost {translationfunc _} } {
    if { $prepost != "PRE" && $prepost != "POST" } {
	return -code error [_ "Wrong arg. Must be PRE or POST."]
    }
    set i [GiDMenu::_FindIndex $menu_name_untranslated $prepost $translationfunc]
    if { $i == -1 } {
	return -1 
    }
    
    set index [GiDMenu::_FindOptionIndex $i $option_name_untranslated $prepost]
    if { $index != -1 } {
	set position [lindex [split $index ,] end]
    } else {
	set position -1
    }
    return $position
}


# -----------------------------------------------------------------------------
#  return the submenu option index
#  parentindex: the index (from 0) of the parent menu
#  option_name_untranslated (untranslated english menu name, for example "Export#C#menu>IGES#C#menu")
#  prepost: PRE or POST (where to find)
#  insert:
#  translationfunc can be _ (for GiD standard strings) or = (for additional module strings)
# -----------------------------------------------------------------------------
proc GiDMenu::_FindOptionIndex { parentindex nameslist prepost {translationfunc _}} {
    global MenuEntries MenuEntriesP
    if {$prepost eq "PRE" } {
	upvar \#0 MenuEntries TranslatedEntries
    } else { ;#POST
	upvar \#0 MenuEntriesP TranslatedEntries
    }

    set postfix "#C#menu"
    set index $parentindex
    foreach suboption $nameslist {
	set found 0
	set cont 0
	if { ![info exists TranslatedEntries($index)] } {
	    return -1
	}
	if { [string range $suboption 0 2] == "---" } {
	    set nseparator [string range $suboption 3 end]
	    set suboption "---"
	    set iseparator 0
	}
	if { $suboption == "---" } {
	    foreach item $TranslatedEntries($index) {
		if { $item == "---" } {        
		    if { $iseparator == $nseparator} {
		        append index ,$cont
		        set found 1
		        break
		    }
		    incr iseparator                    
		}
		incr cont
	    }
	} else {
	    #is not a separator
	    #find first translating the original name
	    set cont 0
	    foreach item $TranslatedEntries($index) {
		if { [cmpoption $item [$translationfunc $suboption]] == 0 } {
		    if { $suboption == "---" } {
		        if { $iseparator == $nseparator} {
		            append index ,$cont
		            set found 1
		            break
		        }
		        incr iseparator
		    } else {
		        append index ,$cont
		        set found 1
		        break
		    }
		}
		incr cont
	    }
	    
	    if { !$found } {
		#find translating the name adding #C#menu
		set cont 0
		foreach item $TranslatedEntries($index) {
		    if { [cmpoption $item [$translationfunc $suboption$postfix]] == 0 } {                    
		        append index ,$cont
		        set found 1
		        break                    
		    }
		    incr cont
		}            
	    }    
	    if { !$found } {
		#find translating the name adding #C#menu,verb or noun or nothing
		set cont 0
		foreach item $TranslatedEntries($index) {
		    foreach comment {verb noun nothing} {
		        if { [cmpoption $item [$translationfunc $suboption$postfix,$comment]] == 0 } {                    
		            append index ,$cont
		            set found 1
		            break                    
		        }
		    }
		    if { $found } {
		        break
		    }
		    incr cont
		}            
	    }
	    if { !$found } {
		set pos [string first "->" $suboption]
		if { $pos != -1 } {
		    #find translating the name splitting a->b 
		    set suboption_part0 [string range $suboption 0 $pos-1]
		    set suboption_part1 [string range $suboption $pos+2 end]
		    set cont 0
		    foreach item $TranslatedEntries($index) {
		        if { [cmpoption $item [$translationfunc $suboption_part0]->[$translationfunc $suboption_part1]] == 0 } {                    
		            append index ,$cont
		            set found 1
		            break                    
		        }
		        if { $found } {
		            break
		        }
		        incr cont
		    }            
		}
	    }
	}    
	
	if { !$found } {
	    return -1
	}
    }
    return $index
}

proc GiDMenu::cmpoption { a b } {
    if { [string match *... $a] } {
	set a [string range $a 0 end-3]
    }
    if { [string match *... $b] } {
	set b [string range $b 0 end-3]
    }
    return [string compare $a $b]
}


proc GiDMenu::del_submenus {i position pre} {
    global MenuNames MenuEntries MenuCommands MenuAcceler MenuIcons
    global MenuNamesP MenuEntriesP MenuCommandsP MenuAccelerP MenuIconsP

    if {$pre=="PRE"} {
	if {[info exists MenuEntries($i,$position)]} {
	    append i ",$position"
	    rec_del_menu $i MenuEntries MenuCommands MenuAcceler MenuIcons
	}
    } elseif {$pre=="POST"} {
	if {[info exists MenuEntriesP($i,$position)]} {
	    append i ",$position"
	    rec_del_menu $i MenuEntriesP MenuCommandsP MenuAccelerP MenuIconsP
	}
    }
}

proc GiDMenu::rec_del_menu {param ent com acc icons} {
    upvar $ent Entries
    upvar $com Commands
    upvar $acc Acceler
    upvar $icons Icons

    set m_length [llength $Entries($param)]

    set pos 0

    unset Entries($param)
    if {[info exists Commands($param)]} {unset Commands($param) }
    if {[info exists Acceler($param)]}  {unset Acceler($param) }
    if {[info exists Icons($param)]}  {unset Icons($param) }
	
    set base $param
    while {$pos<$m_length} {
	set param "$base,$pos"
	if {[info exists Entries($param)]} {
	    rec_del_menu $param Entries Commands Acceler Icons
	}
	set pos [expr $pos +1]
    }
}

proc GiDMenu::move_submenus {i position pre inc} {
    global MenuNames MenuEntries MenuCommands MenuAcceler MenuIcons
    global MenuNamesP MenuEntriesP MenuCommandsP MenuAccelerP MenuIconsP
	
    set last_menu [llength [split $i ,]]
    incr last_menu

    if {$pre=="PRE"} {
	set j 1
	set m_length [llength $MenuEntries($i)]
	set base $i
	while {$position<$m_length} {
	    # read from bottom to top when insert, and opposite when removing
	    set param1 "$base,$position"
	    set param2 "$base,[expr $m_length - $j]"
	    if {[info exists MenuEntries($param1)]} {
		if {$inc=="up"} {
		    rec_move_menu $param1 $last_menu MenuEntries MenuCommands MenuAcceler MenuIcons $inc
		}
	    }
	    if {[info exists MenuEntries($param2)]} {
		if {$inc=="down"} {
		    rec_move_menu $param2 $last_menu MenuEntries MenuCommands MenuAcceler MenuIcons $inc
		}
	    }
	    set position [expr $position +1]
	    set j [expr $j +1]
	}
    } elseif {$pre=="POST"} {
	set j 1
	set m_length [llength $MenuEntriesP($i)]
	set base $i
	while {$position<$m_length} {
	    # read from bottom to top when insert, and opposite when removing
	    set param1 "$base,$position"
	    set param2 "$base,[expr $m_length - $j]"
	    if {[info exists MenuEntriesP($param1)]} {
		if {$inc=="up"} {
		    rec_move_menu $param1 $last_menu MenuEntriesP MenuCommandsP MenuAccelerP MenuIconsP $inc
		}
	    }       
	    if {[info exists MenuEntriesP($param2)]} {
		if {$inc=="down"} {
		    rec_move_menu $param2 $last_menu MenuEntriesP MenuCommandsP MenuAccelerP MenuIconsP $inc
		}
	    }
	    set position [expr $position +1]
	    set j [expr $j +1]
	}
    }
}


proc GiDMenu::dec_inc_menu {param last_menu inc} {
    set up_menus   ""
    set down_menus ""
    set menu ""
    set j 0
    set num 0

    while {$num < [expr $last_menu-1] && $j<[string length $param]} {
	set char [string index $param $j]
	append up_menus $char
	if {$char == ","} {set num [expr $num +1]}
	set j [expr $j +1]
    }

    set char [string index $param $j]
    while {$char != "," && $j<[string length $param]}  {
	append menu $char
	set j [expr $j +1]
	set char [string index $param $j]
    }

    while {$j <= [string length $param]} {
	append down_menus $char
	set j [expr $j +1]
	set char [string index $param $j]
    }
    append down_menus $char

    if {$inc=="down"} {
	set menu [expr $menu + 1]
    } else {
	set menu [expr $menu - 1]
    }
    set new_menu $up_menus$menu$down_menus
    return $new_menu
}


proc GiDMenu::rec_move_menu {param last_menu ent com acc icons inc} {
    upvar $ent Entries
    upvar $com Commands
    upvar $acc Acceler
    upvar $icons Icons

    set m_length [llength $Entries($param)]
    set pos 0

    set new_menu [dec_inc_menu $param $last_menu $inc]

    set Entries($new_menu) $Entries($param)
    unset Entries($param)

    if {[info exists Commands($param)]} {
	set Commands($new_menu) $Commands($param)
	unset Commands($param)
    }
    if {[info exists Acceler($param)]}  {
	set Acceler($new_menu)  $Acceler($param)
	unset Acceler($param)
    }

    if {[info exists Icons($param)]}  {
	set Icons($new_menu)  $Icons($param)
	unset Icons($param)
    }
    set base $param
    while {$pos<$m_length} {
	set param "$base,$pos"
	if {[info exists Entries($param)]} {rec_move_menu $param $last_menu Entries Commands Acceler Icons $inc}
	set pos [expr $pos +1]
    }
}


#utilities not interacting with GiD

namespace eval MathUtils {
    variable Epsilon 1e-10
    variable Pi [ expr 2.0 * acos(0.0)]
}

#pi is a list with x y z? coordinates, return the diff vector p2-p1 as a list
proc MathUtils::VectorDiff { p1 p2 } {    
    foreach i $p1 j $p2 {
	lappend res [expr {$j-$i}]
    }
    return $res
}

proc MathUtils::VectorSum { p1 p2 } {    
    foreach i $p1 j $p2 {
	lappend res [expr {$j+$i}]
    }
    return $res
}

#pi is a list with x y z? coordinates, return the dot prod value
proc MathUtils::VectorDotProd { p1 p2 } {
    set res 0
    foreach i $p1 j $p2 {
	set res [expr $res+($i*$j)]
    }
    return $res
}

#pi is a list with x y z coordinates, return the vectorial prod vector as list
proc MathUtils::VectorVectorialProd { p1 p2 } {
    set ax [lindex $p1 0]
    set ay [lindex $p1 1]
    set az [lindex $p1 2]
    set bx [lindex $p2 0]
    set by [lindex $p2 1]
    set bz [lindex $p2 2]
    set vectorprod [expr $ay*$bz-$az*$by]
    lappend vectorprod [expr $az*$bx-$ax*$bz]
    lappend vectorprod [expr $ax*$by-$ay*$bx]
    return $vectorprod
}

#multiplication of a scalar 'a' by a vector 'p'
proc MathUtils::ScalarByVectorProd { a p } {
    foreach i $p {
	lappend res [expr {$a*$i}]
    }
    return $res
}

#typical operation a*x+y
proc MathUtils::AXPY { a x y } {
    foreach i $x j $y {
    lappend res [expr {$a*$i+$j}]
    }
    return $res
}

#p is a list with x y z? coordinates, return a unitary vector
proc MathUtils::VectorNormalized { p } {
    variable Epsilon
    set m [::MathUtils::VectorModulus $p]
    if { [expr abs($m)] <  $Epsilon } {
	return $p
    }
    return [::MathUtils::ScalarByVectorProd [expr 1.0/$m] $p]
}

#p is a list with x y z? coordinates, return the modulus value
proc MathUtils::VectorModulus { p } {
    return [expr sqrt([MathUtils::VectorDotProd $p $p])]
}

#pi is a list with x y z? coordinates, return the distance value
proc MathUtils::VectorDistance { p1 p2 } {
    return [MathUtils::VectorModulus [MathUtils::VectorDiff $p1 $p2]]
}

#pi is a list with x y z? coordinates, return the distance^2 value
#(for some uses must be faster to compare dist^2 avoiding calculate the square root)
proc MathUtils::VectorDistance2 { p1 p2 } {
    set p [MathUtils::VectorDiff $p1 $p2]
    return [MathUtils::VectorDotProd $p $p]
}

#pi is a list with x y z? coordinates, return the angle value in radians
proc MathUtils::VectorAngle { pvertex p2 p3} {
    set v1 [MathUtils::VectorDiff $p2 $pvertex]
    set modul1 [MathUtils::VectorModulus $v1]
    if { $modul1<$MathUtils::Epsilon } return ""

    set v2 [MathUtils::VectorDiff $p3 $pvertex]
    set modul2 [MathUtils::VectorModulus $v2]
    if { $modul2<$MathUtils::Epsilon } return ""

    set cosangle [expr [MathUtils::VectorDotProd $v1 $v2]/($modul1*$modul2)]
    set angle [expr acos($cosangle)]
    return $angle
}

proc MathUtils::FromRadToDegree { x } {
    return [expr $x*180.0/$MathUtils::Pi]
}

proc MathUtils::FromDegreeToRad { x } {
    return [expr $x*$MathUtils::Pi/180.0]
}

#integrate the xy curve defined by xs and ys. xs must be increasing
#method could be "linear" (piecewise linear segments) 
#             or "romberg" Romberg's intgration method of a interpolating cubic spline
proc MathUtils::IntegrateXY { xs ys {method linear} } {
    set value ""
    if { $method == "linear" } {
        set value 0.0
        set x_prev [lindex $xs 0]
        set y_prev [lindex $ys 0]
        foreach x [lrange $xs 1 end] y [lrange $ys 1 end] {
	    set dx [expr {$x-$x_prev}]
	    set y_mid [expr {($y+$y_prev)*0.5}]
	    set value [expr {$value+$dx*$y_mid}]
	    set x_prev $x
	    set y_prev $y
        }
    } elseif { $method == "romberg" } {
        package require math::interpolate
        package require math::calculus
        set ::f_cubic_splines_coeffs [math::interpolate::prepare-cubic-splines $xs $ys]
        proc f_cubic_splines { x } {
	    math::interpolate::interp-cubic-splines $::f_cubic_splines_coeffs $x
        }
        set value [lindex [::math::calculus::romberg f_cubic_splines [lindex $xs 0] [lindex $xs end]] 0]
        unset ::f_cubic_splines_coeffs
        rename f_cubic_splines ""   
    } else {
	set value ""
    }
    return $value
}


#utilities interacting with GiD
namespace eval GidUtils {
}

#register a tcl command for the GiD Undo/Batch, and eval this command
proc GidUtils::EvalAndEnterInBatch { args } {
    GiD_EnterInBatch "\"-tcl- $args\"\n"
    eval $args
}

# returns the distance (or null) between two selected coordinates
proc GidUtils::GetDistance { } {
    set p1 [GidUtils::GetCoordinates [_ "Enter first point (ESC to leave)"]]
    if { $p1=="" } return ""
    set p2 [GidUtils::GetCoordinates [_ "Enter second point (ESC to leave)"]]
    if { $p2=="" } return ""

    set d [MathUtils::VectorDistance $p1 $p2]

    GidUtils::SetWarnLine [_ "Distance=%s" $d]
    return $d
}

proc GidUtils::GetVector { } {
    set p1 [GidUtils::GetCoordinates [_ "Enter origin point (ESC to leave)"]]
    if { $p1=="" } return ""
    set p2 [GidUtils::GetCoordinates [_ "Enter destination point (ESC to leave)"]]
    if { $p2=="" } return ""

    set p [MathUtils::VectorDiff $p1 $p2]

    GidUtils::SetWarnLine [_ "Vector=%s" $p]
    return $p
}

#return the angle in degrees
proc GidUtils::GetAngle { } {
    set p1 [GidUtils::GetCoordinates [_ "Enter vertex point (ESC to leave)"]]
    if { $p1=="" } return ""
    set p2 [GidUtils::GetCoordinates [_ "Enter second point (ESC to leave)"]]
    if { $p2=="" } return ""
    set p3 [GidUtils::GetCoordinates [_ "Enter third point (ESC to leave)"]]
    if { $p3=="" } return ""

    set angle [MathUtils::FromRadToDegree [MathUtils::VectorAngle $p1 $p2 $p3]]
    GidUtils::SetWarnLine [_ "Angle=%s" $angle]
    return $angle
}

#return the curvature o the curve with id=num relative to the parameter t (from 0.0 to1.0)
proc GidUtils::GetCurveCurvature { num t } {
    set dert [GiD_Info parametric line $num deriv_t $t]
    set dertt [GiD_Info parametric line $num deriv_tt $t]
    set a [::MathUtils::VectorModulus [MathUtils::VectorVectorialProd $dert $dertt]]
    set b [::MathUtils::VectorModulus $dert]
    return [expr $a/($b*$b*$b)]
}


# It returns the enties identifier numbers (for example 1 3 5:8 represents 1 3 5 6 7 8)
# type can be "Points" "Lines" "Surfaces" "Volumes" "Dimensions" "AllTypes" "Nodes" "Elements" or "Graphs"
# selectmode can be "single" or "multiple"
# for AllTypes selection returns some as
# Points 19 23 Lines 28:29 32:33 37 Surfaces 1 6 21:22 Volumes 1 Dimension 1:2
# or for mesh entities some as Nodes 236 244 Elements 4122 4130 4152 4165
proc GidUtils::PickEntities { type selectmode } {
    #gidmakecurrent
    set wait 1
    switch $selectmode {
      single {
	set args_sel "justone $type"
      }
      multiple {
	set args_sel "nojustone $type"
      }
      default {
	set args_sel "$type $selectmode escape"
	set wait 0
      }
    }
    GiD_Process Mescape 'SelectEntities {*}$args_sel
    if { $wait } {
      vwait ::GidPriv(selection)
    }
    if { $selectmode == "single" } {
	set selection [lindex {*}$::GidPriv(selection) 1]
    } else {
	set selection [lrange {*}$::GidPriv(selection) 1 end]
    }
    return $selection
}


# returns the coordinates x y z Or null selecting from the screen
# GidPriv(selcoord,id) store the point id (-1 nothing selected, 0 new point, >0 point id)
# to get the number of a existent point, cal also use PickEntities
# PickMode can be "Join" (default)    : to pick existing points
#                 "NoJoin"            : to pick a new point generic
#                 "PointInLine"       : to pick a new point on a curve
#                 "PointInSurface"    : to pick a new point on a curve
#                 "TangentInLine"     : to pick a new unitary curve tangent vector
#                 "NormalInSurface"   : to pick a new unitary surface normal vector
#                 "ArcCenter"         : to pick a new point center of an arc
#                 "LineParameter"     : to pick a new point in a line, specifying the t parameter
#                 "SurfaceParameters" : to pick a new point in a surface, specifying the u,v parameters
#                 "MapPointSurface"   : to pick a new point in a surface, specifying a near initial point
#                 "PointInElement"    : to pick a new point on a mesh element
#                 "NormalInElement"    : to pick a new point on a mesh element
#Use can be GEOMETRYUSE or MESHUSE (e.g. to force pick nodes instead geometric points)
#         
#Note: just after the selection, for PointInElement or NormalInElement the element number is stored 
#      in the variable $::GidPriv(selection)

proc GidUtils::GetCoordinates { { WarnLineMsg "" } { PickMode  "Join" } { Use "" } } {
    if { $WarnLineMsg  == "" } { set WarnLineMsg [_ "Enter point (ESC to leave)"] }
    #gidmakecurrent
    GidUtils::SetWarnLine $WarnLineMsg
    GidUtils::DisableWarnLine    
    if { $Use == "" } {
	set Use [GiD_Info Project ViewMode]
    }
    if { $PickMode != "Join" && $PickMode != "NoJoin" } {
	#Join and NoJoin could be for both, geometry or mesh
	if { $PickMode == "PointInElement" || $PickMode == "NormalInElement" } {
	    set Use MESHUSE    
	} else {
	    set Use GEOMETRYUSE
	}
    }
    set viewmode [GiD_Info Project ViewMode]
    if { $Use != $viewmode } {
	if { $Use== "MESHUSE" } {
	    GiD_Process MEscape Meshing MeshView            
	} else {
	    GiD_Process MEscape Geometry            
	}
    }
    
    GiD_Process Mescape Utilities Id
    update idletasks
    if { [lsearch -exact [lrange $::ActionsMenuVar 0 end-1] $PickMode] != -1 } {
	GiD_Process $PickMode
    }
    GidUtils::EnableWarnLine

    vwait ::GidPriv(selcoord,id)
    if { $::GidPriv(selcoord,id) != -1 } {
      set xyz "$::GidPriv(selcoord,x) $::GidPriv(selcoord,y) $::GidPriv(selcoord,z)"
    } else {
      set xyz ""
    }
    GidUtils::DisableWarnLine
    GiD_Process escape escape
    GidUtils::EnableWarnLine
    return $xyz
}


#to compare if a GiD version is more recent of another
proc GidUtils::VersionNumber { VersionString } {
    set patron {([0-9]+)[.]([0-9]+)[.]?([0-9]*)([a-z]?)(?:-beta([0-9]+))?}
    regexp -nocase $patron $VersionString total v1 v2 v3 letra betanumber
    set res 0
    if { [info exists v1] } {
	if { $v1 != "" } { set res [expr $res+$v1*100000] }
    }
    if { [info exists v2] } {
	if { $v2 != "" } { set res [expr $res+$v2*1000] }
    }
    if { [info exists v3] } {
	if { $v3 != "" } { set res [expr $res+$v3] }
    }
    return $res
}

proc GidUtils::VersionSubchar { VersionString } {
    set patron {([0-9]+)[.]([0-9]+)[.]?([0-9]*)([a-z]?)(?:-beta([0-9]+))?}
    regexp -nocase $patron $VersionString total v1 v2 v3 letra betanumber
    set res ""
    if { [info exists letra] } { set res $letra }
    return $res
}

proc GidUtils::VersionBetaNumber { VersionString } {
    set patron {([0-9]+)[.]([0-9]+)[.]?([0-9]*)([a-z]?)(?:-beta([0-9]+))?}
    regexp -nocase $patron $VersionString total v1 v2 v3 letra betanumber
    set res ""
    if { [info exists betanumber] } { set res $betanumber }
    return $res
}

proc GidUtils::IsDeveloperVersion { } {
    set letter [GidUtils::VersionSubchar [GiD_Info GiDVersion]]
    if { $letter == "d" || $letter == "b" } {
    set is_developer 1
    } else {
    set is_developer 0
    }
    return $is_developer
}

#return -1 0 or 1 if Version1 is less than, equal to, or greater than Version1
proc GidUtils::TwoVersionsCmp { Version1  Version2 } {    
    set Number1 [GidUtils::VersionNumber $Version1]
    set Number2 [GidUtils::VersionNumber $Version2]
    if { $Number1 < $Number2 } { return -1 }
    if { $Number1 > $Number2 } { return 1 }
    set c1 [GidUtils::VersionSubchar $Version1]
    set c2 [GidUtils::VersionSubchar $Version2]
    if { $c1 < $c2 } { return -1 }
    if { $c1 > $c2 } { return 1 }
    #assumed more recent the non-beta version
    set BetaNumber1 [GidUtils::VersionBetaNumber $Version1]
    set BetaNumber2 [GidUtils::VersionBetaNumber $Version2]
    if { $BetaNumber1 != "" } {
	if { $BetaNumber2 != "" } {
	    if { $BetaNumber1 < $BetaNumber2 } { return -1 }
	    if { $BetaNumber1 > $BetaNumber2 } { return 1 }
	} else {
	    return -1
	}
    } else {
	if { $BetaNumber2 != "" } { return 1 }
    }
    return 0
}

# this function always returns -1 if GiD is before to the
# 6.2.0 version, first including info GiDVersion
# return -1 if GidPriv(Version) < Version
# return  0 if GidPriv(Version) == Version
# return  1 if GidPriv(Version) > Version

#sample use for a probletype that need GiD7.4b or later new features:
#    set VersionRequired "7.4b"
#    set comp -1
#    catch {
#      set comp [::GidUtils::VersionCmp $VersionRequired]
#    }
#    if { $comp < 0 } {
#        WarnWin "Error: This interface requires GiD $VersionRequired or later."
#    }

proc GidUtils::VersionCmp { Version } {
    if { [catch { set GidVersion [GiD_Info GiDVersion] }] }  {
	return -1
    }
    if { $GidVersion == "Unknown" } {
	return -1
    }
    return [GidUtils::TwoVersionsCmp $GidVersion $Version]        
}

#check if current version satisfy the minimum required version
proc GidUtils::VersionSatisfy { MinimumOfficialVersion MinimumDeveloperVersion } {
    if { [GidUtils::IsDeveloperVersion] } {
    set cmp [GidUtils::VersionCmp $MinimumDeveloperVersion] 
    } else {
    set cmp [GidUtils::VersionCmp $MinimumOfficialVersion]
    }    
    return [expr {$cmp>=0}]
}

#to access to values by fieldname instead index (more easy to maintain code)
proc GidUtils::GetProblemDataValue { fieldname } {

    foreach "foundname foundvalue" [list "" ""] break
    foreach "name value" [lrange [GiD_Info gendata] 1 end] {
	if { [string equal $fieldname $name] } { return $value }
	if { [string match $fieldname* $name] } {
	    if { $foundname != "" } {
		error "error: '$fieldname' has more than one names: $foundname and $name"
	    }
	    foreach "foundname foundvalue" [list $name $value] break
	}
    }
    if { $foundname == "" } {
	error "error: '$fieldname' is not found in Problem data"
    }
    return $foundvalue
}

#to access to values by fieldname instead index (more easy to maintain code)
proc GidUtils::GetCurrentIntervalDataValue { fieldname } {
    foreach "foundname foundvalue" [list "" ""] break
    foreach "name value" [lrange [GiD_Info intvdata] 1 end] {
	if { [string equal $fieldname $name] } { return $value }
	if { [string match $fieldname* $name] } {
	    if { $foundname != "" } {
		error "error: '$fieldname' has more than one names: $foundname and $name"
	    }
	    foreach "foundname foundvalue" [list $name $value] break
	}
    }
    if { $foundname == "" } {
	error "error: '$fieldname' is not found in Problem data"
    }
    return $foundvalue
}

#change status of redraws, etc to speed or enhance scripts

#after use a pair: GidUtils::DisableGraphics - GidUtils::EnableGraphics
#the previous stated is automatically restored (using a stack)
proc GidUtils::DisableGraphics { } {
    if { [GidUtils::IsTkDisabled] } {  
    #e.g. batch mode without windows
    return
    }
    variable state
    foreach i [list windows graphinput graphics warnline] {
	lappend state($i) [.central.s disable $i]
	.central.s disable $i 1
    }
}

proc GidUtils::EnableGraphics { } {
    if { [GidUtils::IsTkDisabled] } {  
    #e.g. batch mode without windows
    return
    }
    variable state
    foreach i [list windows graphinput graphics warnline] {
	if { [info exists state($i)] && [llength $state($i)] } {
	    .central.s disable $i [lindex $state($i) end]
	    set state($i) [lrange $state($i) 0 end-1]
	} else {
	    .central.s disable $i 0
	}
    }
    return
}

# i must be: windows graphinput graphics warnline writebatch
proc GidUtils::Disable { i } {
    variable state
    if { [lsearch {windows graphinput graphics warnline writebatch} $i] == -1 } {
	return 1
    }
    lappend state($i) [.central.s disable $i]
    .central.s disable $i 1
    return 0
}

# i must be: windows graphinput graphics warnline writebatch
proc GidUtils::Enable { i } {
    variable state
    if { [lsearch {windows graphinput graphics warnline writebatch} $i] == -1 } {
	return 1
    }
    if { [info exists state($i)] && [llength $state($i)] } {
	set prevstate [lindex $state($i) end]
	.central.s disable $i [lindex $state($i) end]
	set state($i) [lrange $state($i) 0 end-1]
    } else {
	.central.s disable $i 1
    }
    return 0
}

proc GidUtils::DisableWarnLine {  } {
    return [GidUtils::Disable warnline]
}

proc GidUtils::EnableWarnLine {  } {
    return [GidUtils::Enable warnline]
}


proc GidUtils::WaitState { w } {
    if { [GidUtils::IsTkDisabled] } {    
    return
    }
    $w configure -cursor watch
    .central.s waitstate 1
    update
}

proc GidUtils::EndWaitState { w } {
    if { [GidUtils::IsTkDisabled] } {    
    return
    }
    .central.s waitstate 0
    $w configure -cursor ""
    update
}

proc GidUtils::SetWarnLine { text } {
    if { [GidUtils::IsTkDisabled] || [.central.s disable warnline] } {
	return
    }
    ChangeWarnLineVar $text
}

#Utility to show a progress bar.
#varname must be a global variable name with the current value to be showed automatically
#stopproc is an optional callback procedure name to be called when the user press stop
#when the current value is equal to max the advance bar is automatically deleted
#Note: only is possible to use one progress bar
#example:
#  global MyVarName
#  set MyVarName 0
#  ::GidUtils::CreateAdvanceBar [= "Loading file"] [= "Percentage"] ::MyVarName 100  [list WarnWin [= "User stop"]]
#  # ...
#  set MyVarName 20

proc GidUtils::CreateAdvanceBar { title text varname { max 100 } { stopproc "" } } {
    if { [GidUtils::IsTkDisabled] } {
    return
    }
    variable ab_maximum
    variable ab_varname
    variable ab_stopproc
    set ab_maximum $max
    set ab_varname $varname
    set ab_stopproc $stopproc
    set ::GidPriv(StopLoading) 0
    AdvanceBar 0 $ab_maximum 0 $title $text
    if { $ab_varname != "" } {
	trace add variable $ab_varname write "::GidUtils::UpdateAdvanceBar ;#"
    }
    if { $ab_stopproc != "" } {
	trace add variable ::GidPriv(StopLoading) write "::GidUtils::CallStopProc ;#"
    }
}

proc GidUtils::UpdateAdvanceBar { } {
    variable ab_maximum
    variable ab_varname
    if { ![info exists $ab_varname] } return
    if { [set $ab_varname] >= $ab_maximum } {
	::GidUtils::DeleteAdvanceBar
    } else {
	AdvanceBar [set $ab_varname] $ab_maximum [set $ab_varname]
    }
}

proc GidUtils::CallStopProc { } {
    variable ab_stopproc
    if { $::GidPriv(StopLoading) == "0" } return
    ::GidUtils::DeleteAdvanceBar
    if { $ab_stopproc != "" } {
	eval $ab_stopproc
    }
}

# proc GidUtils::UpdateAdvanceBar { current } {
#     variable ab_maximum
#     AdvanceBar $current $ab_maximum $current
# }

proc GidUtils::DeleteAdvanceBar { } {
    if { [GidUtils::IsTkDisabled] } {
    return
    }
    variable ab_maximum
    variable ab_varname
    variable ab_stopproc
    if { $ab_varname != "" } {
	trace remove variable $ab_varname write "GidUtils::UpdateAdvanceBar ;#"
    }
    if { $ab_stopproc != "" } {
	trace remove variable ::GidPriv(StopLoading) write "::GidUtils::CallStopProc ;#"
    }
    AdvanceBar $ab_maximum $ab_maximum $ab_maximum
}

#to show a splash image, for example:
#global GIDDEFAULT
#GidUtils::Splash [gid_themes::GetImage about.png]
#VersionAndPos: list with three items: version_label posx posy
proc GidUtils::Splash { imagefile {w .splash} {selfclose 1} {VersionAndPos ""} } {
    if { [GidUtils::AreWindowsDisabled] } return
    #if selfclose==0 ignore the SplashWindow GiD variable and show the window
    #tipically selfclose==1 for splash initial windows (then use SplashWindow)
    #          selfclose==0 for help about windows (then show avoiding the SplashWindow state)
    if { ![GiD_Set SplashWindow] && $selfclose==1 } return

    if { ![file exists $imagefile] } {
	WarnWinText [_ "File '%s' does not exist" $imagefile]
	return
    }
    if { [ winfo exists $w]} {
	destroy $w
	update
    }
    toplevel $w
    if { $::tcl_platform(platform) == "windows" } {        
	wm attributes $w -toolwindow 1
    }
    set im [image create photo -file $imagefile]
    set x [expr [winfo screenwidth $w]/2-[image width $im]/2]
    set y [expr [winfo screenheight $w]/2-[image height $im]/2]
    wm geom $w +$x+$y
    wm transient $w .gid
    wm overrideredirect $w 1
    pack [label $w.l -image $im -relief ridge -borderwidth 2]

    if { $VersionAndPos != "" } {
	if { [llength $VersionAndPos] == 3 } {
	    set VersionNumber [lindex $VersionAndPos 0]
	    set x [lindex $VersionAndPos 1]
	    set y [lindex $VersionAndPos 2]
	} else {
	    set VersionNumber [lindex $VersionAndPos 0]
	    set x 62
	    set y 62
	}
    set fnt "Sans 10"
    set fnt_small "Sans 8"
    if { $::tcl_platform(platform) == "windows" } {
	set fnt "verdana 10"
	set fnt_small "verdana 8"
    }
	label .splash.lv -text $VersionNumber -font $fnt \
	-background white -foreground black \
	-relief solid -borderwidth 1 -padx 12 -pady 3
    # -foreground white -relief raised -borderwidth 3 -background blue4
    set label_width_and_place [ expr $x + [ winfo reqwidth .splash.lv]]
    # - 4 = oborderwidth
    set image_width [ expr [ image width $im] + 4]
    set toplevel_width [ expr [ winfo reqwidth $w] - 4]
    set max_width $toplevel_width
    if { $image_width > $max_width} {
	set max_width $image_width
    }
    if { $max_width - $label_width_and_place < 4} {
	# not centered or outside window
	.splash.lv configure -font $fnt_small
    }
	place .splash.lv -x $x -y $y
    }

    bind $w <1> "destroy $w"
    bind $w <KeyPress> "destroy $w"
    raise $w .gid
    grab $w
    focus $w
    update
    if { $selfclose } {
	after 2000 "if { [ winfo exists $w] } { destroy $w }"
    }
}

proc GidUtils::GetCurrentLanguage { } {
    #instead for example "es_es_modern", return only "es"
    if { [::msgcat::mclocale] == "" } {
	set lan en
    } else {
	set lan [string range [::msgcat::mclocale] 0 1]
    }
    return $lan
}

proc GidUtils::SetCurrentLanguage { newlan {updatewidgets 1 } } {
    global GidPriv
    if { ![info exists GidPriv(Language)] || $GidPriv(Language) != $newlan } {
	set GidPriv(Language) $newlan
    }
    if { [::GidUtils::GetCurrentLanguage] != $newlan } {
	MsgcatSetLocale $newlan
    }

    if { $updatewidgets } {
	UpdateWidgetsLanguage
    }    
    RaiseEvent_GenericProc ::ChangedLanguage $newlan
}

proc GidUtils::GetAllLanguages { } {
    return [MsgcatGetLocales]
}


#GiD selection accept and returns entities idetifiers (integers > 0) as
#a "compacted" list, the sintax "a:a+n" is interpreted as "a a+1 a+2 ... a+n"

#instead "4 8 9 10 11 15" compact list --> "4 8:11 15"
#items must be positive integer numbers (can be unordered)
proc GidUtils::CompactNumberList { a } {
    set ret ""
    set istart 0
    if { [string match "*:*" $a] } {
	#imput list is compacted
	return $a
    }
    foreach i [lsort -integer $a] {
	if { !$istart} {
	    set istart $i
	} else {
	    if {$i != [expr $iprev+1]} {
		if { $istart != $iprev } {
		    if { [expr $istart +1] == $iprev } {
		        append ret "$istart $iprev "
		    } else {
		        append ret "$istart:$iprev "
		    }
		} else {
		    append ret "$istart "
		}
		set istart $i
	    }
	}
	set iprev $i
    }
    if { $istart } {
	if { $istart != $iprev } {
	    if { [expr $istart +1] == $iprev } {
		append ret "$istart $iprev"
	    } else {
		append ret "$istart:$iprev"
	    }
	} else {
	    append ret "$istart"
	}
    }
    return $ret
}

#instead "4 8:11 15"  uncompact list --> "4 8 9 10 11 15"
#items must be positive increasing integer numbers
proc GidUtils::UnCompactNumberList { a } {
    set ret ""
    foreach item $a {
	if { [regexp   {([0-9]+)[ ]*:[ ]*([0-9]+)} $item dummy istart iend] } {
	    for { set i $istart } { $i <= $iend } { incr i } {
		lappend ret $i
	    }
	} else {
	    lappend ret $item
	}
    }
    return $ret
}

#function to close some window if it exists, window can be:
# CALCULATE COORDINATES COPY HELP LAYER LIST MESHQUALITY MOVE PAGESETUP
# PREFERENCES SELECTION TOOLBARS WARNING NOTES ...
# ALL (Close all window)
proc GidUtils::CloseWindow { window } {
    if { [GidUtils::IsTkDisabled] } {
        return 1
    }
    #GROUPS for window outside
    set keyProg {
	CALCULATE { set w .gid.wrun }
	CALCULATOR { set w .gid.wcalc }
	COORDINATES { set w .gid.wSetCoord }
	COPY { set w .gid.wcopymove }
	#HELP { set w .gid.helpref }
	GRAPHS { set w .gid.graphs }
	GROUPS { set w .gid.central.wgroup }
	LAYER { set w .gid.central.wlay }	
	LIST { set w .gid.listent }
	MESHERRORS { set w .gid.mesherrors }
	MESHQUALITY { set w .gid.meshq }
	MESHPROGRESS { set w .gid.wprogress }
	MOVE { set w .gid.wcopymove }
	NOTES { set w .gid.texteditor }
	PAGESETUP { set w .gid.wPageSetup }
	POSTDISPLAY { set w .gid.wPostDisplay}
	PREFERENCES { set w .gid.gidPreferences }
	SELECTION { set w .gid.filter_win }
	TOOLBARS { set w .gid.toolbars }
	UNDO { set w .gid.wundocom }
	WARNING { set w .gid.warning }	
    }
    if { $window == "HELP" } {
	if { [interp exists lognoter_intp] } {
	    foreach w [lognoter_intp eval winfo children .] {
		if { [string range $w 0 8] == ".lognoter" } {
		    lognoter_intp eval destroy $w
		}
	    }
	}
    } elseif {  $window == "ALL" || $window == "LAYER" } {
	eval [dict get $keyProg "LAYER"]
	if { [winfo exists $w] } {
	    if { [winfo class $w] == "Panedwindow" } {
		close_window_inside $w
	    } else {
		if { [info exists ::GID_INTERNAL_GROUPS] } {
		    #save the state of the current tab
		    set nb $w.body.nb
		    if { [winfo exists $nb] } {
		        set ::GidPriv(LayersOrGroupsCurrentTab) [$nb index [$nb select]]
		    }
		}
		destroy $w
	    }
	}
	if { $window == "ALL" } {
	    #do the all other windows 
	    foreach {key prog} $keyProg {
		if { $key != "LAYER" } {
		    eval $prog
		    if { [winfo exists $w] } {
		        destroy $w
		    }
		}
	    }
	}    
    } elseif { [dict exists $keyProg $window] } { #$window != "LAYER" (previous if)
	eval [dict get $keyProg $window]
	if { [winfo exists $w] } {
	    destroy $w
	}
    } else {
	WarnWinText [_ "Unexpected window %s" $window]                    
    }   
}

#Added window option ALL
proc GidUtils::OpenWindow { window } {   
    if { [GidUtils::IsTkDisabled] } {
        return 1
    }
    #WARNING empty because not interesting
    set keyProg {
	CALCULATE { OpenProcWin }
	CALCULATOR { CalculatorV }
	COORDINATES { SetCoord }
	COPY { CopyMove copy }
	HELP { GiDHelpWindowLognoter }
	LAYER { ChangeLayers }
	GRAPHS { PostGraphs::Create }
	GROUPS { Groups::OpenOutside }
	LIST { ListEntities status }
	MESHERRORS { CreateWindowMeshErrors }
	MESHQUALITY { MeshQuality }
	MOVE { CopyMove move }
	PAGESETUP { PageSetup }
	PREFERENCES { ChangeVariables }
	SELECTION { make_win_filter }
	TOOLBARS { TOBegin }
	UNDO { UndoCom }
	WARNING { }
	NOTES { OpenNotes }
    }
    
    if { [dict exists $keyProg $window] } {
	eval [dict get $keyProg $window]
    } elseif { $window == "ALL" } {
	foreach {key prog} $keyProg { 
	    eval $prog
	}
    } else {
	WarnWinText [_ "Unexpected window %s" $window]                    
    }
}

proc GidUtils::UpdateWindow { window } {  
    if { [GidUtils::IsTkDisabled] } {
        return 1
    }
    set keyProg {
	CLIPPLANES { FillClipPlanesInfo }
	COORDINATES { UpdateSetCoord }
	COPY { UpdateCopyMove }  
	LAYER { FillInfoLayers }
	GROUPS { Groups::FillInfo }
	MOVE { UpdateCopyMove }
	PERSPECTIVE { FillClipPlanesInfo }
	PREFERENCES { CreateWidgetsFromXml::ChangeAllVariablesValues } 
    }
    if { [dict exists $keyProg $window] } {
	eval [dict get $keyProg $window]
    } elseif { $window == "ALL" } {
	foreach {key prog} $keyProg { 
	    eval $prog
	}
    } else {
	WarnWinText [_ "Unexpected window %s" $window]                    
    }    
}


#return the temporary folder
proc GidUtils::GetTmp { } {
    global tcl_platform env
    set tmpdir ""
    if { $tcl_platform(platform) == "unix" } {
	set tmpdir /tmp
    } else {
	if { [info exists env(TMP)] } {
	    set tmpdir $env(TMP)
	} elseif { [info exists env(TEMP)] } {
	     set tmpdir $env(TEMP)
	} else {
	    set tmpdir "$env(windir)/Temp"
	}
    }
    return $tmpdir   
}

#returns a new folder inside the temporary directory
proc GidUtils::CreateTmpFolderName { } {
    set tmpdir [GidUtils::GetTmp]
    set tmpname ""
    for { set i 0 } { $i < 10000 } {incr i} {
	set tmpname [file join $tmpdir gidtmp$i]
	if { ![file exists $tmpname] } {
	    file mkdir $tmpname
	    break
	}
    }
    return $tmpname
}

#returns a non existent filename inside the temporary directory
proc GidUtils::GetTmpFilename { {extension .tmp} } {
    set tmpdir [GidUtils::GetTmp]
    set tmpname ""
    for { set i 0 } { $i < 10000 } {incr i} {
	set tmpname [file join $tmpdir gid$i]$extension
	if { ![file exists $tmpname] } break
    }
    return $tmpname
}

#return 0 if is not a link,  1 if the file is a link or  2 if it is Windows shell link
proc GidUtils::IsLink { lnk } {
    set islink 0
    if { ![file exists $lnk] } {
	return 0
    }
    if { [string match "windows" $::tcl_platform(platform)] } {
	set type [file type $lnk]
	if { $type == "file" && [string tolower [file extension $lnk]] == ".lnk" } {
	    set fp [open $lnk]
	    fconfigure $fp -encoding binary -translation binary -eofchar {}
	    binary scan [read $fp 4] i tmp
	    if { $tmp == "76" } {
		binary scan [read $fp 16] h32 tmp
		if { $tmp == "10412000000000000c00000000000064" } {
		    set islink 2 ;#windows shell link
		}
	    }
	    close $fp
	}
	if { !$islink } {
	    if { ![catch  {set res [file readlink $lnk]}] } {
		set islink 1
	    }
	}
    } else {
	if { ![catch  {set res [file readlink $lnk]}] } {
	    set islink 1
	}
    }
    return $islink
}

#return the name pointed by a link or Windows shell link
proc GidUtils::ReadLink { lnk show_ui } {
    set res ""
    set islink [IsLink $lnk]
    if { $islink == 1 } {
	catch {set res [file readlink $lnk]} err
    } elseif { $islink == 2 } {
#         set fp [open $lnk]
#         fconfigure $fp -encoding binary -translation binary -eofchar {}
#         foreach snip [split [read $fp] \x00] {
#             if {[regexp {^[A-Z]:\\} $snip] && [file exists $snip]} {
#                 lappend res $snip
#             }
#         }
#         close $fp
#         set res [join $res]
	catch { set res [GiD_GetInfoLnkFile $lnk $show_ui] }
    }
    return $res
}

#return the relative (to basedir) path of target
#useful to have the relative path of a problemtype (relative to the \problemtypes GiD directory)
#e.g: GidUtils::MakeRelativePath {C:\gid\problemtypes} {C:\gid\problemtypes\ansys55\ansys55_3d}
#will return "ansys55\ansys55_3d"
proc GidUtils::MakeRelativePath { basedir target } {
    set bparts [file split [file normalize $basedir]]
    set tparts [file split [file normalize $target]]

    if {[lindex $bparts 0] eq [lindex $tparts 0]} {
	# If the first part doesn't match - there is no good relative path
	set blen [llength $bparts]
	set tlen [llength $tparts]
	for {set i 1} {$i < $blen && $i < $tlen} {incr i} {
	    if {[lindex $bparts $i] ne [lindex $tparts $i]} { break }
	}
	set path [lrange $tparts $i end]
	for {} {$i < $blen} {incr i} {
	    set path [linsert $path 0 ..]
	}
	set target [join $path [file separator]]
    }
    #file join to change \ by / also for windows
    return [file join $target]
}

proc GiveFileInsideProblemType { ptname ext {checkifexists 0} } {
    if { [string tolower [file extension $ptname]] == ".gid" } {
	set ptname [file rootname $ptname]
    }
    set tailname [file tail $ptname]
    set ptname $ptname.gid
    if { [file pathtype $ptname] == "relative" } {
	set fullptname [file join $::GIDDEFAULT problemtypes]        
    } else {
	set fullptname $ptname
    }
    foreach item [file split $ptname] {
	set fullptname [file join $fullptname $item]
	if { [GidUtils::IsLink $fullptname.lnk] == 2 } {
	    set fullptname [GidUtils::ReadLink $fullptname.lnk 1]
	}
    }

    set filename [file join $fullptname $tailname$ext]
    if { $checkifexists } {
	if { ![file exists $filename] } {
	    set filename ""
	}
    }
    return $filename
}

#filename could have links in some parts of the path, this procedure dereference them
proc GiveRealFilenameDereferencingLinks { filename {checkifexists 0} } {
    set outfilename ""
    foreach item [file split $filename] {
	set outfilename [file join $outfilename $item]
	if { [GidUtils::IsLink $outfilename.lnk] == 2 } {
	    set outfilename [GidUtils::ReadLink $outfilename.lnk 1]
	}
    }
    if { $checkifexists } {
	if { ![file exists $outfilename] } {
	    set outfilename ""
	}
    }
    return $outfilename    
}

#return the full path to a relative or absolute problemtype name, e.g.
#return "" if is not a valid GiD problemtype (must have some problemtype file .bas, .cnd, etc)
#in:  nastran3/nastran            out: C:/GiD project/problemtypes/nastran3/nastran
#in:  C:/tmp/nastran3/nastran     out: C:/tmp/nastran3/nastran
proc GiveProblemTypeFullname { ptname } {
    set checkifexists 1
    if { [GiveFileInsideProblemType $ptname ".geo" $checkifexists] != "" } {
	#it is a model, not a problemtype
	return ""
    }
    set fullfilename ""
    foreach ext {.bas .cnd .prb .mat .tcl .tbe .bat} {
	set fullfilename [GiveFileInsideProblemType $ptname $ext $checkifexists]
	if { $fullfilename != "" } {
	    break
	}
    }
    if { $fullfilename != "" } {
	set fullpath [file root [file dirname $fullfilename]]
    } else {
	set fullpath ""
    }
    return $fullpath
}

#create a raster image from a widget c with Img
# image create photo -format window -data $c

proc GidUtils::CaptureWindow { win } {
    package require img::window
    # package require Img
    regexp {([0-9]*)x([0-9]*)\+([0-9]*)\+([0-9]*)} [winfo geometry $win] - w h x y
    # Make the base image based on the window
    set image [image create photo -format window -data $win]
    foreach child [winfo children $win] {
	CaptureWindowSub $child $image 0 0
    }
    #save image to disk
    #$image write $filename -format GIF
    return $image
}

proc GidUtils::CaptureWindowSub { win image px py } {
    if {![winfo ismapped $win]} {
	return
    }
    regexp {([0-9]*)x([0-9]*)\+([0-9]*)\+([0-9]*)} [winfo geometry $win] - w h x y
    incr px $x
    incr py $y
    # Make an image from this widget
    set tempImage [image create photo -format window -data $win]
    # Copy this image into place on the main image
    $image copy $tempImage -to $px $py
    image delete $tempImage
    foreach child [winfo children $win] {
	CaptureWindowSub $child $image $px $py
    }
}

# Copy photo image "phImg" into Windows clipboard.
proc GidUtils::Img2Clipboard { phImg } {
    package require twapi
    package require img::bmp
    package require base64

    # First 14 bytes are bitmapfileheader - get rid of this
    set data [string range [base64::decode [$phImg data -format bmp]] 14 end]
    twapi::open_clipboard
    twapi::empty_clipboard
    twapi::write_clipboard 8 $data
    twapi::close_clipboard
}


# Copy the contents of the Windows clipboard into a photo image.
# Return the photo image identifier.
proc GidUtils::Clipboard2Img {} {
    package require twapi
    package require img::bmp

    twapi::open_clipboard    
    # Assume clipboard content is in format 8 (CF_DIB)
    set retVal [catch {twapi::read_clipboard 8} clipData]
    if { $retVal != 0 } {
    error "Invalid or no content in clipboard"
    }    
    # First parse the bitmap data to collect header information
    binary scan $clipData "iiissiiiiii" \
    size width height planes bitcount compression sizeimage \
    xpelspermeter ypelspermeter clrused clrimportant    
    # We only handle BITMAPINFOHEADER right now (size must be 40)
    if {$size != 40} {
    error "Unsupported bitmap format. Header size=$size"
    }    
    # We need to figure out the offset to the actual bitmap data
    # from the start of the file header. For this we need to know the
    # size of the color table which directly follows the BITMAPINFOHEADER
    if {$bitcount == 0} {
    error "Unsupported format: implicit JPEG or PNG"
    } elseif {$bitcount == 1} {
    set color_table_size 2
    } elseif {$bitcount == 4} {
    # TBD - Not sure if this is the size or the max size
    set color_table_size 16
    } elseif {$bitcount == 8} {
    # TBD - Not sure if this is the size or the max size
    set color_table_size 256
    } elseif {$bitcount == 16 || $bitcount == 32} {
    if {$compression == 0} {
	# BI_RGB
	set color_table_size $clrused
    } elseif {$compression == 3} {
	# BI_BITFIELDS
	set color_table_size 3
    } else {
	error "Unsupported compression type '$compression' for bitcount value $bitcount"
    }
    } elseif {$bitcount == 24} {
    set color_table_size $clrused
    } else {
    error "Unsupported value '$bitcount' in bitmap bitcount field"
    }    
    set phImg [image create photo]
    set filehdr_size 14                 ; # sizeof(BITMAPFILEHEADER)
    set bitmap_file_offset [expr {$filehdr_size+$size+($color_table_size*4)}]
    set filehdr [binary format "a2 i x2 x2 i" \
	     "BM" [expr {$filehdr_size + [string length $clipData]}] \
	     $bitmap_file_offset]    
    append filehdr $clipData
    $phImg put $filehdr -format bmp    
    twapi::close_clipboard
    return $phImg
}

proc colorize {image color} {
    # get image sizes
    set width  [image width  $image]
    set height [image height $image]
    # get color's R G B
    foreach {rr gg bb} [winfo rgb . $color] break
    # compute new colors
    set colors {}
    for {set y 0} {$y < $height} {incr y} \
    {
      set row {}
      for {set x 0} {$x < $width} {incr x} \
      {
	# save transparency
	lappend trans $x $y [$image transparency get $x $y]
	# compute the new color
	foreach {r g b} [$image get $x $y] break
	set r [expr {round((256 - $r) * $rr / 256)}]
	set g [expr {round((256 - $g) * $gg / 256)}]
	set b [expr {round((256 - $b) * $bb / 256)}]
	# append to the current row
	lappend row [format #%4.4x%4.4x%4.4x $r $g $b]
      }
      # append the row
      lappend colors $row
    }
    # set new image
    $image put $colors
    # restore transparency
    foreach {x y t} $trans { $image transparency set $x $y $t }
  }


#to print a integer number over an image (create a new image if dst == "")
proc GidUtils::PrintNumberOverImage { number {dst ""} {xorig 0} {yorig 0} } {
    if { ![string is integer $number] } {
	return $dst
    }
    #gif black digits 0 to 9 with transparent background
    set src [image create photo -data {
	    R0lGODlhRgAIAJEAANnZ2QAAAP///////yH5BAEAAAAALAAAAABGAAgAAALM
	    RBIEO84igmBExF0QjIiIC0oAQLEB4AAISAQABMExiDCjABAAAQQEACCCYMRZ
	    XBDsuAABChAAEEAIAAIgCCF8C7Q4CgIAQfDj7uIoEARAAAEBCACgAABB8C0w
	    AikIAEQEBYCICIIdZxFBcAzCwoACAATBh8ALowBB8CEsLghGHMQFwYkICwMK
	    ABAEP+4sjGIT/LC4OAphx8VdEBCDMAuCHWdxQbDj4sIoAAQABMGOC4ujABAR
	    BDsOKDYAHMGPg4ggGBFxEAQ/Ig6CEgAUADs=
	}]
	
	colorize $src #919191
	
    set w 7
    set h 8
    set n [string length $number]
    if { $dst == "" } {
	set dst [image create photo -width [expr $n*$w] -height $h]
	$dst blank
    }
    for {set i 0} {$i < $n} {incr i} {
	set digit [string index $number $i]
	set x1 [expr $digit*$w]
	set x2 [expr $i*$w+$xorig]
	set y2 $yorig
	$dst copy $src -from $x1 0 [expr $x1+$w] $h -to $x2 $y2  [expr $x2+$w] [expr $y2+$h]
    }
    image delete $src
    return $dst
}

#hide the toolbar and avoid to be showed again from the Toolbar window
#after use a pair: GidUtils::DisableToolbar - GidUtils::EnableToolbar
#the previous stated is automatically restored (using a stack)
#name is the toolbar name: "Right buttons" "Command line" "Up menu"
# "Geometry & View bar" "Standard bar" "View results bar" "MacrosToolbar"
# Added option to disable all toolbars, name=AllToolbars
proc GidUtils::DisableToolbar { name } {
    variable state
    global ToolbarsPriv GidPriv
    set item ""
    foreach i $ToolbarsPriv(toolbars) {
	if { $name=="AllToolbars" || $name == [lindex $i 0] } {
	    set item $i
	    if { ![info exists GidPriv([lindex $item 1])] } {
		set GidPriv([lindex $item 1]) [list NONE {} [lindex $item 2]]
	    }       
	    lappend state(toolbar,$name) $GidPriv([lindex $item 1])
	    [lindex $item 2] NONE
	    set ToolbarsPriv(Hide,$name) 1
	    if { $name!="AllToolbars" } { break }
	}
    }
}

proc GidUtils::EnableToolbar { name } {
    variable state
    global ToolbarsPriv GidPriv
    set item ""
    foreach i $ToolbarsPriv(toolbars) {
	if { $name=="AllToolbars" || $name == [lindex $i 0] } {
	    set item $i
	    if { [info exists state(toolbar,$name)] && [llength $state(toolbar,$name)] } {
		set GidPriv([lindex $item 1]) [lindex $state(toolbar,$name) end]
		set state(toolbar,$name) [lrange $state(toolbar,$name) 0 end-1]
	    } else {
		set GidPriv([lindex $item 1]) [list NONE {} [lindex $item 2]]
	    }
	    [lindex $item 2] [lindex $GidPriv([lindex $item 1]) 0]
	    set ToolbarsPriv(Hide,$name) 0
	    if { $name!="AllToolbars" } { break }
	}
    }
}


#to visually show thousands separated by a character like ,
#e.g. change 12345 by 12,345
proc GidUtils::AddThousandsSeparator { str { sep ,}} {
    # let's hide whats right at the decimal point
    while { [ regsub {(.*\d\.(\d\d\d)+)(\d+.*)$} $str "\\1|!|!|!|\\3" str]} {
    }
    while { [ regsub {^(.*[-+]?\d+)(\d\d\d)} $str "\\1$sep\\2" str]} {
    }
    regsub -all {\|\!\|\!\|\!\|} $str {} str
    return $str
}

#to save a georeference file when saving an image
#if read this image as background real size, will be set again in its location.
#This procedure can de automatically invoked when saving an image by
#defining the AfterSaveImage event procedure like this:
#
#proc AfterSaveImage { filename type } {
#    GidUtils:SaveGeoreferenceFile $filename $type
#}
#
proc GidUtils:SaveGeoreferenceFile { filename type } {
    set fail 0
    switch $type {
	tif {
	    set newext tfw
	}
	jpg {
	    set newext jgw
	}
	png {
	    set newext pgw
	}
	gif {
	    set newext gfw
	}
	bmp {
	    set newext bpw
	}
	default {
	    set fail 1
	}
    }
    if { !$fail } {
	foreach item [GiD_Info view] {
	    #check that the current view is in XY plane, else don't save the georeference file
	    if { [lindex $item 0] == "m" } {
		for { set i 1 } { $i <= 12 } { incr i } {
		    set v [lindex $item $i]
		    switch $i {
		        1 -
		        6 -
		        11 {
		            if { [expr abs($v-1.0)] > 1e-6 } {
		                set fail 1
		                break
		            }
		        }
		        default {
		            if { [expr abs($v)] > 1e-6 } {
		                set fail 1
		                break
		            }
		        }
		    }
		}
		break
	    }
	}
	if { $fail } {
	    GidUtils::SetWarnLine [_ "Georeference file not saved because current view is not XY"]
	}
    }
    if { !$fail } {
	set newfilename [file rootname $filename].$newext
	set fp [open $newfilename w]
	if { $fp == "" } {
	    set fail 1
	} else {
	    foreach {L R B T N F E} [GiD_Info ortholimits] break
	    set Mxx [expr {double($R-$L)/[winfo width .gid.central.s]}]
	    set Myx 0
	    set Mxy 0
	    set Myy [expr {double($B-$T)/[winfo height .gid.central.s]}]
	    set Dx $L
	    set Dy $T
	    puts $fp $Mxx
	    puts $fp $Myx
	    puts $fp $Mxy
	    puts $fp $Myy
	    puts $fp $Dx
	    puts $fp $Dy
	    close $fp
	    GidUtils::SetWarnLine [_ "Georeference file saved"]
	}
    }
    return $fail
}


#to be used by TKWIDGET to easily add a select file button
#e.g.
#QUESTION: your_question
#VALUE: your_filename
#TKWIDGET: GidUtils::TkwidgetGetFilenameButton

proc GidUtils::TkwidgetGetFilenameButton { event args } {
    global tkwidgedprivfilenames tkwidgedprivfilebuttons
    switch $event {
	INIT {
	    set PARENT [lindex $args 0]
	    upvar [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	    set STRUCT [lindex $args 3]
	    set QUESTION [lindex $args 4]
	    #initialize variable to current field value
	    set tkwidgedprivfilenames($QUESTION) [DWLocalGetValue $GDN $STRUCT $QUESTION]
	    #set entry $PARENT.e$ROW
	    set entry ""
	    foreach item [grid slaves $PARENT -row [expr $ROW-1]] {
		if { [winfo class $item] == "Entry"  || [winfo class $item] == "TEntry" } {
		    #assumed that it is the only entry of this row
		    set entry $item
		    break
		}
	    }           
	    if { $entry != "" } {        
		set tkwidgedprivfilebuttons($QUESTION) [ttk::button $PARENT.bfolder$QUESTION \
		        -image [gid_themes::GetImage "folder.png"] \
		        -command [list GidUtils::_GetFilenameCmd tkwidgedprivfilenames($QUESTION) $entry 0]]
		grid $tkwidgedprivfilebuttons($QUESTION) -row [expr $ROW-1] -column 2 -sticky w                
		grid configure $entry -sticky ew
		grid columnconfigure [winfo parent $entry] 1 -weight 1
	    }
	}
	SYNC {
	    #set GDN [lindex $args 0]
	    #set STRUCT [lindex $args 1]
	    #set QUESTION [lindex $args 2]
	    #DWLocalSetValue $GDN $STRUCT $QUESTION $tkwidgedprivfilenames($QUESTION)
	}
	DEPEND {
	    #set GDN [lindex $args 0]
	    #set STRUCT [lindex $args 1]
	    set QUESTION [lindex $args 2]
	    set ACTION [lindex $args 3]
	    #set value [lindex $args 4]
	    if { [info exists tkwidgedprivfilebuttons($QUESTION)] && \
		    [winfo exists $tkwidgedprivfilebuttons($QUESTION)] } {
		if { $ACTION == "HIDE" } {
		    grid remove $tkwidgedprivfilebuttons($QUESTION)
		} else {
		    #RESTORE
		    grid $tkwidgedprivfilebuttons($QUESTION)
		}
	    } else {

	    }
	}
	CLOSE {
	    array unset tkwidgedprivfilenames
	    array unset tkwidgedprivfilebuttons
	}
	default {
	    return [list ERROR [_ "Unexpected tkwidget event"]]
	}
    }
    return ""
}


#axiliary procedure used by GidUtils::TkwidgetGetFilenameButton
proc GidUtils::_GetFilenameCmd { varname entry {tail 0}} {    
    set types [list [list [_ "All files"] ".*"]]
    set defaultextension ""
    set title [_ "Select file"]
    set ::$varname [Browser-ramR file read .gid $title [set ::$varname] $types $defaultextension 0]
    if { [set ::$varname] != "" && $entry != "" && [winfo exists $entry] } {
	$entry delete 0 end
	if { $tail } {
	    $entry insert end [file tail [set ::$varname]]
	} else {
	    $entry insert end [set ::$varname]
	}
    }
    return [set ::$varname]
}

#to be used by TKWIDGET to easily add a pick node button
#e.g.
#QUESTION: your_question
#VALUE: your_node_id
#TKWIDGET: GidUtils::TkwidgetPickNode

proc GidUtils::TkwidgetPickNode { event args } {
    global tkwidgedprivpicknodebuttons
    switch $event {
	INIT {
	    set PARENT [lindex $args 0]
	    upvar [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	    set STRUCT [lindex $args 3]
	    set QUESTION [lindex $args 4]
	    set entry ""
	    foreach item [grid slaves $PARENT -row [expr $ROW-1]] {
		if { [winfo class $item] == "Entry"  || [winfo class $item] == "TEntry" } {
		    #assumed that it is the only entry of this row
		    set entry $item
		    break
		}
	    }            
	    set tkwidgedprivpicknodebuttons($QUESTION) [ttk::button $PARENT.bpicknode$QUESTION \
		    -image [gid_themes::GetImage "point.png" small_icons] -command [list GidUtils::_PickNodeCmd $entry]]
	    grid $tkwidgedprivpicknodebuttons($QUESTION) -row [expr $ROW-1] -column 2 -sticky w
	    grid configure $entry -sticky ew
	    return ""
	}
	SYNC {
	    #set GDN [lindex $args 0]
	    #set STRUCT [lindex $args 1]
	    #set QUESTION [lindex $args 2]
	    #DWLocalSetValue $GDN $STRUCT $QUESTION $value
	}
	DEPEND {
	    #set GDN [lindex $args 0]
	    #set STRUCT [lindex $args 1]
	    set QUESTION [lindex $args 2]
	    set ACTION [lindex $args 3]
	    #set value [lindex $args 4]
	    if { [info exists tkwidgedprivpicknodebuttons($QUESTION)] && \
		    [winfo exists $tkwidgedprivpicknodebuttons($QUESTION)] } {
		if { $ACTION == "HIDE" } {
		    grid remove $tkwidgedprivpicknodebuttons($QUESTION)
		} else {
		    #RESTORE
		    grid $tkwidgedprivpicknodebuttons($QUESTION)
		}
	    }
	}
	CLOSE {
	    array unset tkwidgedprivpicknodebuttons
	}
	default {
	    return [list ERROR [_ "Unexpected tkwidget event"]]
	}
    }
}

#axiliary procedure used by GidUtils::TkwidgetPickNode
proc GidUtils::_PickNodeCmd { entry } {
    set id [GidUtils::PickEntities Nodes single]
    if { $id != "" } {
       $entry delete 0 end
       $entry insert end $id
    }
}


#to be used by TKWIDGET to easily add a pick node button
#e.g.
#QUESTION: your_question
#VALUE: your_node_id
#TKWIDGET: GidUtils::TkwidgetPickPointOrNode

proc GidUtils::TkwidgetPickPointOrNode { event args } {
    global tkwidgedprivpicknodebuttons
    switch $event {
	INIT {
	    set PARENT [lindex $args 0]
	    upvar [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	    set STRUCT [lindex $args 3]
	    set QUESTION [lindex $args 4]
	    set entry ""
	    foreach item [grid slaves $PARENT -row [expr $ROW-1]] {
		if { [winfo class $item] == "Entry"  || [winfo class $item] == "TEntry" } {
		    #assumed that it is the only entry of this row
		    set entry $item
		    break
		}
	    }      
	    set tkwidgedprivpicknodebuttons($QUESTION) [ttk::button $PARENT.bpicknode$QUESTION \
		    -image [gid_themes::GetImage "point.png" small_icons] \
		    -command [list GidUtils::_PickPointOrNodeCmd $entry]]
	    grid $tkwidgedprivpicknodebuttons($QUESTION) -row [expr $ROW-1] -column 2 -sticky w
	    grid configure $entry -sticky ew
	    return ""
	}
	SYNC {
	    #set GDN [lindex $args 0]
	    #set STRUCT [lindex $args 1]
	    #set QUESTION [lindex $args 2]
	    #DWLocalSetValue $GDN $STRUCT $QUESTION $value            
	}
	DEPEND {
	    #set GDN [lindex $args 0]
	    #set STRUCT [lindex $args 1]
	    set QUESTION [lindex $args 2]
	    set ACTION [lindex $args 3]
	    #set value [lindex $args 4]
	    if { [info exists tkwidgedprivpicknodebuttons($QUESTION)] && \
		     [winfo exists $tkwidgedprivpicknodebuttons($QUESTION)] } {
		if { $ACTION == "HIDE" } {
		    grid remove $tkwidgedprivpicknodebuttons($QUESTION)
		} else {
		    #RESTORE
		    grid $tkwidgedprivpicknodebuttons($QUESTION)
		}
	    }
	}
	CLOSE {
	    array unset tkwidgedprivpicknodebuttons
	}
	default {
	    return [list ERROR [_ "Unexpected tkwidget event"]]
	}
    }
}

#axiliary procedure used by GidUtils::TkwidgetPickPoint
proc GidUtils::_PickPointCmd { entry } {
    set id [GidUtils::PickEntities Points single]
    if { $id != "" } {
       $entry delete 0 end
       $entry insert end $id
    }
}

#axiliary procedure used by GidUtils::TkwidgetPickPointOrNode
proc GidUtils::_PickPointOrNodeCmd { entry } {
    set viewmode [GiD_Info Project ViewMode]
    if { $viewmode == "GEOMETRYUSE" } {
	return [GidUtils::_PickPointCmd $entry]     
    } elseif { $viewmode == "MESHUSE" } {  
	return [GidUtils::_PickNodeCmd $entry]   
    } else {
	return ""
    }
}

#to be used by TKWIDGET to easily add a layer combobox
#e.g.
#QUESTION: your_question
#VALUE: your_layername
#TKWIDGET: GidUtils::TkwidgetGetLayername

proc GidUtils::TkwidgetGetLayername { event args } {
    global tkwidgedprivlayer
    switch $event {
	INIT {
	    set PARENT [lindex $args 0]
	    upvar [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	    set STRUCT [lindex $args 3]
	    set QUESTION [lindex $args 4]
	    #initialize variable to current field value
	    if { [DWLocalGetValue $GDN $STRUCT $QUESTION] != "" } {
		set tkwidgedprivlayer($QUESTION) [DWLocalGetValue $GDN $STRUCT $QUESTION]
	    } else {
		set tkwidgedprivlayer($QUESTION) [lindex [GiD_Info layers] 0]
	    }
	
	    #set entry $PARENT.e$ROW
	    set entry ""
	    foreach item [grid slaves $PARENT -row [expr $ROW-1]] {
		if { [winfo class $item] == "Entry"  || [winfo class $item] == "TEntry" } {
		    #assumed that it is the only entry of this row
		    set entry $item
		    break
		}
	    }      
	    if { $entry != "" } {                
		#set width [$entry cget -width]
		set width 9        
		set tkwidgedprivlayer($QUESTION) [ttk::combobox $PARENT.clayer$QUESTION \
		        -textvariable tkwidgedprivlayer($QUESTION,layername) \
		        -values [GiD_Info Layers] \
		        -width $width]

		grid remove $entry
		grid $tkwidgedprivlayer($QUESTION) -row [expr $ROW-1] -column 1 -sticky ew -columnspan 2
	    }
	}
	SYNC {         
	    lassign $args GDN STRUCT QUESTION            
	    if { [info exists tkwidgedprivlayer($QUESTION,layername)] } {        
		DWLocalSetValue $GDN $STRUCT $QUESTION $tkwidgedprivlayer($QUESTION,layername)
	    }
	}
	DEPEND {            
	    #set GDN [lindex $args 0]
	    #set STRUCT [lindex $args 1]
	    set QUESTION [lindex $args 2]
	    set ACTION [lindex $args 3]
	    #set value [lindex $args 4]
	    if { [info exists tkwidgedprivlayer($QUESTION)] && \
		    [winfo exists $tkwidgedprivlayer($QUESTION)] } {
		if { $ACTION == "HIDE" } {
		    grid remove $tkwidgedprivlayer($QUESTION)
		} else {
		    #RESTORE
		    grid $tkwidgedprivlayer($QUESTION)
		}
	    } else {

	    }
	}
	CLOSE {
	    #array unset tkwidgedprivlayer
	}
	default {
	    return [list ERROR [_ "Unexpected tkwidget event"]]
	}
    }
    return ""
}

#to be used by TKWIDGET to easily add a material combobox
#e.g.
#QUESTION: your_question
#VALUE: your_materialname
#TKWIDGET: GidUtils::TkwidgetGetMaterialname

proc GidUtils::TkwidgetGetMaterialname { event args } {
    global tkwidgedprivmaterial
    switch $event {
	INIT {
	    set PARENT [lindex $args 0]
	    upvar [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	    set STRUCT [lindex $args 3]
	    set QUESTION [lindex $args 4]
	    #initialize variable to current field value
	    if { [DWLocalGetValue $GDN $STRUCT $QUESTION] != "" } {
		set tkwidgedprivmaterial($QUESTION) [DWLocalGetValue $GDN $STRUCT $QUESTION]
	    } else {
		set tkwidgedprivmaterial($QUESTION) [lindex [GiD_Info Materials] 0]
	    }
	
	    #set entry $PARENT.e$ROW
	    set entry ""
	    foreach item [grid slaves $PARENT -row [expr $ROW-1]] {
		if { [winfo class $item] == "Entry"  || [winfo class $item] == "TEntry" } {
		    #assumed that it is the only entry of this row
		    set entry $item
		    break
		}
	    }      
	    if { $entry != "" } {                
		#set width [$entry cget -width]
		set width 9        
		set tkwidgedprivmaterial($QUESTION) [ttk::combobox $PARENT.cmaterial$QUESTION \
		        -textvariable tkwidgedprivmaterial($QUESTION,materialname) \
		        -values [GiD_Info Materials] \
		        -width $width]

		grid remove $entry
		grid $tkwidgedprivmaterial($QUESTION) -row [expr $ROW-1] -column 1 -sticky ew -columnspan 2
	    }
	}
	SYNC {         
	    lassign $args GDN STRUCT QUESTION            
	    if { [info exists tkwidgedprivmaterial($QUESTION,materialname)] } {        
		DWLocalSetValue $GDN $STRUCT $QUESTION $tkwidgedprivmaterial($QUESTION,materialname)
	    }
	}
	DEPEND {            
	    #set GDN [lindex $args 0]
	    #set STRUCT [lindex $args 1]
	    set QUESTION [lindex $args 2]
	    set ACTION [lindex $args 3]
	    #set value [lindex $args 4]
	    if { [info exists tkwidgedprivmaterial($QUESTION)] && \
		    [winfo exists $tkwidgedprivmaterial($QUESTION)] } {
		if { $ACTION == "HIDE" } {
		    grid remove $tkwidgedprivmaterial($QUESTION)
		} else {
		    #RESTORE
		    grid $tkwidgedprivmaterial($QUESTION)
		}
	    } else {

	    }
	}
	CLOSE {
	    #array unset tkwidgedprivmaterial
	}
	default {
	    return [list ERROR [_ "Unexpected tkwidget event"]]
	}
    }
    return ""
}

proc GidUtils::ValidateEntryDouble { value entry } {    
    set res [string is double $value]
    if { !$res } {        
	GidUtils::SetWarnLine [_ "Invalid value: %s, must be a real number" $value]
	if { 0 && [winfo exists $entry] } {
	    $entry delete 0 end
	    $entry insert end 0
	    focus $entry
	}
    }
    return $res
}

#to be used by TKWIDGET to easily add a vector field with all coordinates in a line
#e.g.
#QUESTION: your_question
#VALUE: vx vy vz
#TKWIDGET: GidUtils::TkwidgetGetVector3D

proc GidUtils::TkwidgetGetVector3D { event args } {
    global tkwidgedprivvector3d
    switch $event {
	INIT {            
	    lassign $args PARENT current_row_variable GDN STRUCT QUESTION    
	    upvar $current_row_variable ROW      
	    #initialize variable to current field value
	    set default_value [DWLocalGetValue $GDN $STRUCT $QUESTION]
	    lassign $default_value tkwidgedprivvector3d($QUESTION,x) tkwidgedprivvector3d($QUESTION,y) tkwidgedprivvector3d($QUESTION,z)                    
	    #set entry $PARENT.e$ROW
	    set entry ""
	    foreach item [grid slaves $PARENT -row [expr $ROW-1]] {
		if { [winfo class $item] == "Entry"  || [winfo class $item] == "TEntry" } {
		    #assumed that it is the only entry of this row
		    set entry $item
		    break
		}
	    }      
	    #trick to fill in the values pressing transfer from an applied condition
	    if { [lindex [info level 2] 0] == "DWUpdateConds" } {
		set values [lrange [lindex [info level 2] 2] 3 end]
		set index_field [LabelField $GDN $STRUCT $QUESTION]
		set value [lindex $values $index_field-1]
		lassign $value tkwidgedprivvector3d($QUESTION,x) tkwidgedprivvector3d($QUESTION,y) tkwidgedprivvector3d($QUESTION,z)                    
	    }
	    set w [ttk::frame $PARENT.cvector3d$QUESTION]
	    foreach axe {x y z} {
		ttk::entry $w.$axe -textvariable tkwidgedprivvector3d($QUESTION,$axe) \
		    -validate focusout -validatecommand [list GidUtils::ValidateEntryDouble %P %W] -width 6
	    }
	    set tkwidgedprivvector3d($QUESTION,widget) $w
	    
	    grid $w.x $w.y $w.z -sticky ew
	    grid columnconfigure $w {0 1 2} -weight 1
	    grid $tkwidgedprivvector3d($QUESTION,widget) -row [expr $ROW-1] -column 1 -sticky ew -columnspan 2    
	    if { $entry != "" } {
		grid remove $entry
	    } else {
		#assumed that entry is hidden
		grid remove $tkwidgedprivvector3d($QUESTION,widget)
	    }
	}
	SYNC {
	    lassign $args GDN STRUCT QUESTION                      
	    if { [info exists tkwidgedprivvector3d($QUESTION,x)] && \
		     [info exists tkwidgedprivvector3d($QUESTION,y)] && \
		     [info exists tkwidgedprivvector3d($QUESTION,z)] } {                
		set value [list $tkwidgedprivvector3d($QUESTION,x) $tkwidgedprivvector3d($QUESTION,y) $tkwidgedprivvector3d($QUESTION,z)]
		DWLocalSetValue $GDN $STRUCT $QUESTION $value
	    }
	}
	DEPEND {
	    lassign $args GDN STRUCT QUESTION ACTION VALUE           
	    if { [info exists tkwidgedprivvector3d($QUESTION,widget)] && [winfo exists $tkwidgedprivvector3d($QUESTION,widget)] } {
		if { $ACTION == "HIDE" } {
		    grid remove $tkwidgedprivvector3d($QUESTION,widget)
		} else {
		    #RESTORE
		    grid $tkwidgedprivvector3d($QUESTION,widget)
		}
	    } else {
 
	    }
	}
	CLOSE {
	    array unset tkwidgedprivvector3d
	}
	default {
	    return [list ERROR [_ "Unexpected tkwidget event"]]
	}
    }
    return ""
}

proc GidUtils::IsTkDisabled { } {
    if { [info commands wm] == "" } {
	return 1
    } else {
	return 0
    }
}

proc GidUtils::AreWindowsDisabled { } {
    if { [GidUtils::IsTkDisabled] } {
	return 1
    }
    if { [winfo exists .gid.central.s] && [.gid.central.s disable windows] } {
	return 1
    }
    return 0
}

# change the color of a widget asking the user
proc GidUtils::ChangeWidgetColor { w colvar} {
    upvar $colvar UserColor

    set w_par $w
    if { $w == "" } {
	set w_par .gid
    }

    set color_old $UserColor
    set color_new [ tk_chooseColor -parent $w_par -initialcolor $color_old]
    if { $color_new != ""} {
	set n [ scan $color_new #%2x%2x%2x r g b]
	if { $n == 3} {
	    if { $w != ""} {
		$w configure -background $color_new
	    }
	    set UserColor $color_new
	}
    }
}

# given a pair of screen coordinates
# returns a list of coord_x coord_y coord_z norm_x norm_y norm_z
# of the intersecion of the plane paralel to screen through rotation centre
# and the perpendicular line defined by the mouse cursor
proc GidUtils::TransformScreenToWorld { scr_x scr_y} {
    set ::GidUtils(WorldCoord) [ GiD_GetWorldCoord $scr_x $scr_y]
    return $::GidUtils(WorldCoord)
}

# waits for the user to pick a world point
# this procedure is unneeded !!! can get basically the same with GidUtils::GetCoordinates $WarnLineMsg NoJoin
proc GidUtils::PickWorldPoint { { WarnLineMsg ""}} {
    if { $WarnLineMsg  == "" } { set WarnLineMsg [_ "Click a point (ESC to leave)"] }
    #gidmakecurrent
    GidUtils::SetWarnLine $WarnLineMsg

    set w .gid.central.s
    set ::GidUtils(WorldCoord) {}
    set ::GidUtils(B1_script) [ bind $w <1>]
    $w configure -cursor cross
    bind $w <1> "GidUtils::TransformScreenToWorld %x %y"
    tkwait variable ::GidUtils(WorldCoord)
    $w configure -cursor ""
    bind $w <1> $::GidUtils(B1_script)
    return $::GidUtils(WorldCoord)
}

#get list increasing from start to end
proc GidUtils::GetListIncreasingFromTo { start end } {  
    set xs {}
    for {set i $start} {$i<=$end} {incr i} {
	lappend xs $i
    }
    return $xs
}

#root_name is a name to check if not empty if it is an appropiated xml file: Infoproblemtype , DownloadProblemtype
#fields are the fieldnames we are interested in, if empty all Program fields are returned
#it is returned a list of field value
proc ReadProblemtypeXml { xmlfile root_name fields } {
    set res {}
    if { [file exists $xmlfile] && [file isfile $xmlfile]} {
	package require tdom    
	set xml [tDOM::xmlReadFile $xmlfile]
	set doc [dom parse $xml]
	set root [$doc documentElement]  
	if { $root_name == "" || [$root nodeName] == $root_name} {
	    #set file_version [$root getAttribute version]
	    if { $fields == "" } {
		set fields ""
		foreach node [$root selectNodes Program/*] {
		    lappend fields [$node nodeName]
		}
	    } 
	    foreach field $fields {
		set node [$root select Program/$field]
		if {$node ne ""} {
		    set value [$node text]
		} else {
		    set value ""
		}     
		lappend res $field $value
	    }
	}
	$doc delete
    }
    return $res
}


#integrate the xy curve graph specified by its numeric index
#method could be "linear" (piecewise linear segments) 
#             or "romberg" Romberg's intgration method of a interpolating cubic spline
proc GidUtils::IntegrateGraph { graphset_index graph_index {method linear} } {
    set integral 0
    set graphset_name [lindex [GiD_GraphSet list] $graphset_index]
    if { $graphset_name  != "" } {
	set graph_name [lindex [GiD_Graph list $graphset_name] $graph_index]
	if { $graph_name  != "" } {
	    set graph_data [GiD_Graph get $graph_name $graphset_name]
	    lassign $graph_data lx ly xs ys
	    set integral [MathUtils::IntegrateXY $xs $ys $method]
	    if { $integral == "" } {
		set integral 0
	    }
	}
    }  
    return $integral
}

#window dialog examples
#
#get a filename
#to write
#   set w .gid ;#parent
#   set types [list [list [= "IGES file"] ".igs .iges"] [list [_ "All files"] ".*"]]
#   set defaultextension .igs
#   set title [= "Write IGES file"]
#   set filename [Browser-ramR file write .gid $title "" $types $defaultextension 0]
#to read
#   set title [= "Read IGES file"]
#   set filename [Browser-ramR file read .gid $title $filename $types $defaultextension 0]



#user procedures that modify menu variables could be registered to be automatically called
#procname is a procedure without any argument
#GiD_RegisterPluginAddedMenuProc <procname>
#GiD_UnRegisterPluginAddedMenuProc <procname> 

#user procedures to handle some file extension when dropping files on GiD 
#prepost could be: PRE POST or PREPOST
#procname is a procedure with an argument, the name of the dropped file that match the extension
#GiD_RegisterExtensionProc <list of extensions> <procname> <prepost>
#GiD_UnRegisterExtensionProc <list of extensions> <prepost>
#proc MyUnvRead { file } { ... }
#GiD_RegisterExtensionProc ".unv .uff" PRE MyUnvRead

