
set GIDDEFAULTTCL [file dirname [info script]]
set GIDDEFAULT [file dirname $GIDDEFAULTTCL]
set ::env(TK_LIBRARY) [file join $GIDDEFAULTTCL tk]
lappend auto_path $GIDDEFAULTTCL $GIDDEFAULTTCL/tcl $GIDDEFAULTTCL/tk
# in linux & co. we'll use gidx, to ensure gid.exe will launch due to library dependencies
if { $::tcl_platform(platform) eq "windows" } {
    set gidexe [file join $GIDDEFAULTTCL .. gid]
} else {
    if { $::tcl_platform(os) eq "Darwin" } {
	set gidexe [file join $GIDDEFAULTTCL .. gid]
    } else {
	set gidexe [file join $GIDDEFAULTTCL .. gidx]
    }
}

#use gid instead of gid.exe
#  for Windows gid.exe will be used
#  but for Linux must use gid to set before environment variables

package require Tk
package require dialogwin
package require msgcat

set SoftwareOpenGL 0
set Language en

set ret [exec "$gidexe" -outputinfo defaults]
regexp -line {GidDefaults=(.*)} $ret {} inifile

set inifile0 [file join $GIDDEFAULTTCL gidDefaults]

foreach file [list $inifile0 $inifile] {
    if { [file exists $file] } {
	set fin [open $file r]
	set data [read $fin]
	close $fin
	regexp -line {^SoftwareOpenGL\s+(\d+)} $data {} SoftwareOpenGL
	regexp -line {^Language\s+(\S+)} $data {} Language
    }
}

msgcat::mclocale $Language
msgcat::mcload [file join $GIDDEFAULT resources msgs]
interp alias "" _ "" msgcat::mc

set Languages [list en]
foreach file [glob -nocomplain -dir [file join $GIDDEFAULT resources msgs] *.msg] {
    lappend Languages [file root [file tail $file]]
}
set Languages [lsort -dictionary -unique $Languages]

wm withdraw .

set w [dialogwin_snit ._ask -title [_ "Safe mode"] -cancelname [_ Exit]]
set f [$w giveframe]

set f1 [ttk::labelframe $f.f1 -text OpenGL]

if { ![info exists SoftwareOpenGL] } { set SoftwareOpenGL 0 }
ttk::checkbutton $f1.cb -text [_ "Activate Software OpenGL"] -variable SoftwareOpenGL

label $f1.l1 -text [_ "Note: If OpenGL by software is activated, graphics will be slower.\
	Only activate it if you are experiencing problems with graphics visualization"] \
    -wraplength 250 -justify left

grid $f1.cb -sticky w -padx 3 -pady 3
grid $f1.l1 -sticky w -padx 3 -pady 3

set f2 [ttk::labelframe $f.f2 -text [_ "Preferences"]]

set reset_prefs 0
ttk::checkbutton $f2.cb -text [_ "Reset preferences"] -variable reset_prefs


label $f2.l1 -text [_ "Note: If preferences are reset. Program will start as if it was\
	installed for the first time. All preferences data will be lost"] \
    -wraplength 250 -justify left

grid $f2.cb -sticky w -padx 3 -pady 3
grid $f2.l1 -sticky w -padx 3 -pady 3

set f3 [ttk::labelframe $f.f3 -text [_ "Languages"]]

ttk::label $f3.l1 -text [_ "Language"]:

array set lans [list en English es Spanish ja Japanese fr French ca Catalan \
	de German ko Korean ru Russian zh Chinese]
set Language_p $Language
set Languages_p ""
foreach i $Languages {
    if { [info exists lans($i)] } {
	set i_p $lans($i)
    } else {
	set i_p $i
    }
    lappend Languages_p $i_p
    if { $i eq $Language } {
	set Language_p $i_p
    }
}

ttk::combobox $f3.cb -textvariable Language_p -state readonly -values $Languages_p

grid $f3.l1 $f3.cb -padx 3 -pady 3

set f4 [ttk::labelframe $f.f4 -text [_ "GiD Theme"]]
ttk::label $f4.l1 -text [_ "Theme"]:
set Theme_p [_ "Classic GiD Theme"]
ttk::combobox $f4.cb -textvariable Theme_p -state readonly -values [list [_ "Classic GiD Theme"] [_ "Dark GiD Theme"]]

grid $f4.l1 $f4.cb -padx 3 -pady 3

grid $f1 -sticky nwe
grid $f2 -sticky nwe
grid $f3 -sticky nwe
grid $f4 -sticky nwe

set action [$w createwindow]

if { $action < 1 } { exit }

set Language $Language_p
foreach "n v" [array get lans] {
    if { $Language_p eq $v } {
	set Language $n
	break
    }
}

if { $Theme_p == [_ "Dark GiD Theme"] } {
    set Theme GiD_black
} else { ;#[_ "Classic GiD Theme"]
    set Theme GiD_classic
}
    

if { $reset_prefs } {
    if { $::tcl_platform(platform) eq "windows" } {
        # WARNING: all problemtype init files are deleted
        set inidir [file dirname $inifile]
        if { [file exists $inidir~] } {
            file delete -force $inidir~
            update idletasks
        }
        if { [file exists $inidir] } {
            file rename $inidir $inidir~
        }
        file mkdir $inidir
    } else {        
        file delete $inifile        
        set postfile [file join [file dirname $inifile] .GiDPostPreferences.ini]
        file delete $postfile
    }   
}

set fout [open $inifile a]
puts $fout "SoftwareOpenGL $SoftwareOpenGL"
puts $fout "OGL_configured 1"
puts $fout "Theme_configured 1"
puts $fout "Theme(Current) $Theme"
puts $fout "Language $Language"
close $fout


eval [list exec $gidexe] [lrange $argv 2 end] &

exit




















