package provide gid_themes 1.0

namespace eval gid_themes {
    variable IconCategories {"generic" "small_icons" "large_icons" "menu" "toolbar"}
    variable Themes
}

if { ![info exists ::GIDDEFAULT] || ( [ info exists ::DO_NOT_USE_GID_THEMES] && $::DO_NOT_USE_GID_THEMES)} {
    proc gid_themes::SetFontsDefault { disp_height } { } 
    proc gid_themes::SetOptions { } { }
    proc gid_themes::ReadThemes_test { } { }
    proc gid_themes::ReadThemes { } { }
    proc gid_themes::SetNextRestartTheme { name } { }
    proc gid_themes::GetNextRestartTheme { } { }
    proc gid_themes::SetCurrentTheme { name } { }
    proc gid_themes::GetCurrentTheme { name } { }
    proc gid_themes::SetCurrentThemeAndApplyBackground { name } { }
    proc gid_themes::SetCurrentThemeSize { size } { }
    proc gid_themes::UpdateCurrentThemeSize { } { }
    proc gid_themes::CheckThemes {} { }
    proc gid_themes::GetImage { filename {IconCategory "generic"} } {        
        if { [info exists ::GidPrivImages($IconCategory,$filename)] } {
            set img $::GidPrivImages($IconCategory,$filename)
        } else {        
            if { [catch {set ::GidPrivImages($IconCategory,$filename) [image create photo -file $filename] } err] } {
                #WarnWinText $err
                set img ""
            } else {
                set img $ ::GidPrivImages($IconCategory,$filename)
            }
        }
        return $img    
    }  
    proc gid_themes::GetOnlyDefineColorsOnMainGiDFrame { } { }
    proc gid_themes::GetTtkTheme { } { }  
    proc gid_themes::GetTkFromTheme { } { }
} else {
    #for old compatibility define also GetImage
    proc GetImage { filename } {
        return [gid_themes::GetImage $filename]
    }
    #for old compatibility (ramseries 5.9.5) define also imagefrom
    proc imagefrom { filename } {
        return [gid_themes::GetImage $filename]
    }

    proc gid_themes::GetOnlyDefineColorsOnMainGiDFrame { } {
        #clasic=true, new=false, test=false
        variable Themes
        return $Themes([gid_themes::GetCurrentTheme],OnlyDefineColorsOnMainGiDFrame)                
    }
    proc gid_themes::GetTtkTheme { } {
        #clasic=classicgid, new=newgid, test=testcolors
        variable Themes
        return $Themes([gid_themes::GetCurrentTheme],TtkTheme)                
    }
    proc gid_themes::GetTkFromTheme { } {
        #clasic=false, new=true, test=true
        variable Themes
        return $Themes([gid_themes::GetCurrentTheme],TkFromTheme)                
    }
    
    proc gid_themes::ConfigureHeaders { T } {
        set style header5
        
        upvar ::ttk::theme::[GetTtkTheme]::colors colors
        upvar ::ttk::theme::[GetTtkTheme]::fonts fonts
        
       
        $T element create header.text text -statedomain header \
            -lines 1 -fill black -font $fonts(-default)
        
        $T element create header.sort image -statedomain header \
        -image [list [gid_themes::GetImage ArrowDown.png small_icons] down [gid_themes::GetImage ArrowUp.png small_icons] up]
       
        $T gradient create Gnormal -orient vertical \
        -stops [list [list 0.0 $colors(-text_bg)] [list 0.5 $colors(-frame_bg)] [list 1.0 $colors(-toolbars_bg)]] \
        -steps 6
        
        $T gradient create Gactive -orient vertical \
        -stops [list [list 0.0 $colors(-text_bg)] [list 0.5 $colors(-highlight)] [list 1.0 $colors(-toolbars_bg)]] \
        -steps 6
        
        $T gradient create Gpressed -orient vertical \
        -stops [list [list 0.0 $colors(-toolbars_bg)] [list 0.5 $colors(-frame_bg)] [list 1.0 $colors(-text_bg)]] \
        -steps 6
        
        
        $T gradient create Gsorted -orient vertical \
        -stops [list [list 0.0 $colors(-text_bg)] [list 0.5 $colors(-frame_bg)] [list 1.0 $colors(-toolbars_bg)]] \
        -steps 6
        
        $T gradient create Gactive_sorted -orient vertical \
        -stops [list [list 0.0 $colors(-text_bg)] [list 0.5 $colors(-highlight)] [list 1.0 $colors(-toolbars_bg)]] \
        -steps 6
        
        $T gradient create Gpressed_sorted -orient vertical \
        -stops [list [list 0.0 $colors(-toolbars_bg)] [list 0.5 $colors(-frame_bg)] [list 1.0 $colors(-text_bg)]] \
        -steps 6
        
        $T element create header.rect5 rect -statedomain header \
        -fill [list \
            Gactive_sorted {active up} \
            Gpressed_sorted {pressed up} \
            Gactive_sorted {active down} \
            Gpressed_sorted {pressed down} \
            Gsorted up \
            Gsorted down \
            Gactive active \
            Gpressed pressed \
                Gnormal {} ] \
        -outline [list \
            $colors(-highlight) up \
            $colors(-highlight) down \
            $colors(-frame_bg) {} ] \
         -outlinewidth 1 -open { nw !pressed }
    
        set S [$T style create $style -orient horizontal -statedomain header]
        $T style elements $S {header.rect5 header.text header.sort}
        $T style layout $S header.rect5 -detach yes -iexpand xy
        $T style layout $S header.text -center xy -padx 6 -pady 2 -squeeze x
        $T style layout $S header.sort -expand nws -padx {0 6} -visible {no {!down !up}}
    
        set textColor $colors(-colortext_fg)
        $T element configure header.text -fill $textColor
    }
    
    proc gid_themes::ApplyHeadersConfig { T } {
    
        set style header5
        
        foreach H [$T header id !header4] {
            $T header style set $H all $style
            $T header configure all all -textpadx 6
            foreach C [$T column list] {
                $T header text $H $C [$T header cget $H $C -text]
            }        
        }
    }

    
    
    
    proc gid_themes::SetFontsDefault { disp_height } {
        global tcl_platform        
        #change for panoramic screens: 768 --> 800
        if { $disp_height > 1024} {
            set fontsize 10
            set bigfontsize 14
            set smallfontsize 8
        } elseif { $disp_height > 800 && $disp_height <= 1024 } {
            set fontsize 10
            set bigfontsize 14
            set smallfontsize 8
        } elseif { $disp_height > 600 && $disp_height <= 800 } {
            # for linux (before it was 8)
            set fontsize 10
            set bigfontsize 14
            set smallfontsize 8
        } elseif { $disp_height > 480 && $disp_height <= 600 } {
            set fontsize 8
            set bigfontsize 11
            set smallfontsize 6
        } else {
            set fontsize 6
            set bigfontsize 10
            set smallfontsize 5
        }
        
        if { $::tcl_platform(os) == "IRIX" } {
            font create NormalFont -family helvetica -size 10 -slant roman
            option add *font -*-helvetica-medium-r-*-*-17-*-*-*-*-*-*-*
            font create BoldFont -weight bold -size $fontsize -family helvetica
        } elseif { $::tcl_platform(platform) == "windows"} {
            set fontsize 8
            catch {
                #too big font
                #set size [expr [ GiD_Info menuheight]-4]
                set size [expr [ GiD_Info menuheight]-6]
                font create NormalFont -family "MS Sans Serif" -size -$size
                set fontsize [font actual NormalFont -size]
                font delete NormalFont
            }
            set bigfontsize [ expr $fontsize + 4]
            set smallfontsize [ expr $fontsize - 2]
            font create NormalFont -family "MS Sans Serif" -size $fontsize
            font create BoldFont -size $fontsize -family "MS Sans Serif" -weight bold
            option add *font NormalFont
        } elseif { $::tcl_platform(os) == "Darwin"} {
            set fontsize 12
            catch {
                font create NormalFont -family "Geneva" -size -$fontsize
                set fontsize [font actual NormalFont -size]
                font delete NormalFont
            }
            set bigfontsize [ expr $fontsize + 4]
            set smallfontsize [ expr $fontsize - 2]
            font create NormalFont -family "Geneva" -size $fontsize
            font create BoldFont -size $fontsize -family "Geneva" -weight bold
            option add *font NormalFont
        } else {
            # porque hay un menos en el tamanno?
            #font create NormalFont -family Helvetica -size [expr -$fontsize]
            font create NormalFont -family Helvetica -size [expr $fontsize]
            font create BoldFont -weight bold -size $fontsize -family helvetica
            option add *font NormalFont
        }
        
        set family [font conf NormalFont -family]
        font create FixedFont -size $fontsize -family Courier
        font create BigFont -family $family -size $bigfontsize
        font create SmallFont -family $family -size $smallfontsize
        # [expr int($fontsize*1.5)]
        
        option add *font TkDefaultFont
    }
    
    
    proc gid_themes::SetOptions { } {                
        global tcl_platform GidPriv        
        #other gidpriv variables done down
        # WarnWinText "gid_themes::SetOptions"
        #ya se ha pasado por ReadDefaultValues
        
        uplevel #0 [list source [file join $::GIDDEFAULT scripts gid_themes ttk_themes [gid_themes::GetTtkTheme].tcl] ]
        
                
        option add *Entry*BorderWidth 1
        option add *Button*relief "raised"
        option add *Menubutton*relief "raised"        
        
        if { $::tcl_platform(platform) != "windows" } {
            option add *Scrollbar*Width 10            
            option add *Scrollbar*BorderWidth 1
            option add *Button*BorderWidth 1
            
            #option add *Menu*BorderWidth 1
            #option add *Menu*activeBorderWidth 0
            #option add *Menu*Relief flat
            
        }
        option add *Menu*TearOff 0
               
        # if { [ tk windowingsystem] == "x11"} {
        #     ttk::style theme use clam
        # }
               
        if { [gid_themes::GetTkFromTheme] } {
            upvar ::ttk::theme::[GetTtkTheme]::colors colors
            upvar ::ttk::theme::[GetTtkTheme]::fonts fonts            
                
            #option add *TopLevel.background $colors(-frame_bg)
            #option add *Frame.background $colors(-frame_bg)
            
            
            
            option add *background $colors(-frame_bg)
            option add *selectBackground $colors(-highlight)
            option add *activeBackground $colors(-highlight) 
            option add *highlightBackground $colors(-frame_bg) 
            option add *insertBackground $colors(-highlight)
 
            
            option add *foreground $colors(-colortext_fg)
            option add *activeForeground $colors(-colortext_fg)
            option add *selectForeground $colors(-select_fg)
            option add *disabledForeground $colors(-disabled_fg)
             
                  
            option add *font $fonts(-default)            
            option add *troughColor $colors(-text_bg)
            
            #CAUTION DON'T UNCOMMNENT
            # styles configured with borderwidth, become wrong, example ComboboxPopdownFrame
            # option add *borderWidth 0
            # option add *relief flat 
            
            option add *selectBorderWidth 0
            option add *activeBorderWidth 0    
            option add *insertBorderWidth 0
            
            option add *highlightThickness 0
            option add *highlightColor $colors(-highlight)
            
            option add *insertWidth 2
                  
             
                    
            #??exist??
            option add *fieldBackground $colors(-text_bg)
            option add *focusthickness 0
            option add *focuscolor $colors(-highlight) 
            option add *indicatorcolor $colors(-frame_bg)   
            
            #other possible: anchor compound insertOffTime insertOnTime justify orient padX padY relief repeatDelay repeatInterval
            # setGrid takeFocus underline wrapLength
            
            #specific for some widgets differ from general
            option add *Button*borderWidth 2
            #-background $GidPriv(Color,ToolbarBg) -borderwidth 1 -relief raised -activeborderwidth 0
            option add *Menu*background $colors(-frame_bg)            
            option add *Menu*borderWidth 1
            option add *Menu*disabledForeground $colors(-disabled_fg)
            option add *Menu*activeBorderWidth 0
            option add *Menu*activeForeground $colors(-colortext_fg)
            option add *Menu*relief solid                
            
            option add *Entry*background $colors(-text_bg)            
            option add *Listbox*background $colors(-text_bg)            
            option add *Text*background $colors(-text_bg)
            option add *TreeCtrl*background $colors(-text_bg)
            option add *Fulltktree*background $colors(-text_bg)
            
            #package require tablelist_tile
            #tablelist::setTheme newgid
            
            option add *Tablelist*background $colors(-text_bg)            
            option add *Tablelist*labelRelief flat
            option add *Tablelist*labelBackground $colors(-frame_bg)
            option add *Tablelist*labelActiveBackground $colors(-highlight) 
            
           
            #foreground font borderwidth highlightthickness  highlightcolor headerfont headerforeground 
            set ::GidPriv(Color,ToolbarBg) $colors(-toolbars_bg)
            set ::GidPriv(Color,CommandLineBg) $colors(-text_bg)
            set ::GidPriv(ToolbarBg) $::GidPriv(Color,ToolbarBg) ;#defined only for back compatibility (GiD-CEM3.0,etc)
            
            set ::GidPriv(Color,BackgroundListbox) $colors(-text_bg)
            set ::GidPriv(Color,DisabledBackgroundEntry) $colors(-text_bg)
            set ::GidPriv(Color,DisabledForegroundEntry) $colors(-disabled_fg)
            set ::GidPriv(Color,DisabledForegroundMenu) $colors(-disabled_fg)
            set ::GidPriv(Color,BackgroundFileDialog) $colors(-text_bg)
            set ::GidPriv(ToolbarBg) $::GidPriv(Color,ToolbarBg) ;#defined only for back compatibility (GiD-CEM3.0,etc)
            
            set ::GidPriv(Color,BotonActivo) $colors(-highlight)
            set ::GidPriv(Color,BotonMarcoActivo) $colors(-highlight)           
            
            set ::GidPriv(Original,Color,Background) $colors(-toolbars_bg)
            #set GidPriv(Original,Color,Foreground) [ .gid cget -foreground]
            
            set ::GidPriv(CustomColor,ToolbarBg) $::GidPriv(Color,ToolbarBg)
            set ::GidPriv(MenuType) "generic"
            
            #foreach col_idx {BotonActivo BotonMarcoActivo DisabledBackgroundEntry\
                #                     DisabledForegroundEntry BackgroundListbox\
                #                     DisabledForegroundMenu CommandLineBg ToolbarBg BackgroundFileDialog} {
                #    set GidPriv(Original,Color,$col_idx) $GidPriv(Color,$col_idx)
                #}
                
            
            DynamicHelp::configure -background $colors(-frame_bg)
            DynamicHelp::configure -borderwidth 1
            #DynamicHelp -border color forced to be black

            package require Plotchart
            ::Plotchart::plotstyle configure default xyplot margin bottom 40    
            ::Plotchart::plotstyle configure default xyplot leftaxis   color      green
            ::Plotchart::plotstyle configure default xyplot leftaxis   textcolor  green
            ::Plotchart::plotstyle configure default xyplot bottomaxis color      green
            ::Plotchart::plotstyle configure default xyplot bottomaxis textcolor  green
            ::Plotchart::plotstyle configure default xyplot background outercolor $colors(-toolbars_bg)
            ::Plotchart::plotstyle configure default xyplot background innercolor $colors(-toolbars_bg) ;#not implemented yet
            ::Plotchart::plotstyle configure default xyplot title      background $colors(-toolbars_bg)
            ::Plotchart::plotstyle configure default xyplot title      textcolor  $colors(-text_bg)

            ::Plotchart::plotstyle load default
            
            
        } else {
            DynamicHelp::configure -background #f0f0f0
            DynamicHelp::configure -borderwidth 1
            #DynamicHelp -border color forced to be black
            
            #package require Plotchart
            #::Plotchart::plotstyle configure default xyplot margin bottom 40    
            #::Plotchart::plotstyle configure default xyplot leftaxis   color      green
            #::Plotchart::plotstyle configure default xyplot leftaxis   textcolor  green
            #::Plotchart::plotstyle configure default xyplot bottomaxis color      green
            #::Plotchart::plotstyle configure default xyplot bottomaxis textcolor  green
            #::Plotchart::plotstyle configure default xyplot background outercolor white
            #::Plotchart::plotstyle configure default xyplot background innercolor white ;#not implemented yet
            #::Plotchart::plotstyle configure default xyplot title      background white
            #::Plotchart::plotstyle configure default xyplot title      textcolor  black

            #::Plotchart::plotstyle load default
           
            
            
            
            option add *activeBackground [ CCColorActivo orange 129]            
            option add *activeForeground Black         
                
            set ::GidPriv(Color,BotonActivo) [ CCColorActivo orange 129]            
            set ::GidPriv(Color,BotonMarcoActivo) [ CCColorActivo orange 193]                
           
            set ::GidPriv(Color,ToolbarBg) [ CCGetRGB .gid [.gid cget -background]]                 
            set ::GidPriv(Color,CommandLineBg) #FFFFFF
           
            set ::GidPriv(ToolbarBg) $::GidPriv(Color,ToolbarBg) ;#defined only for back compatibility (GiD-CEM3.0,etc)
            
            set ::GidPriv(Original,Color,Background) [ .gid cget -background]
            #set GidPriv(Original,Color,Foreground) [ .gid cget -foreground]
            #foreach col_idx {BotonActivo BotonMarcoActivo DisabledBackgroundEntry\
                #                     DisabledForegroundEntry BackgroundListbox\
                #                     DisabledForegroundMenu CommandLineBg ToolbarBg BackgroundFileDialog} {
                #    set GidPriv(Original,Color,$col_idx) $GidPriv(Color,$col_idx)
            #}
            if { ( "$::GidPriv(Configuration)" == "debug") || ( $::tcl_platform(platform) != "windows")} {
                set ::GidPriv(CustomColor,ToolbarBg) \#e9d0d0
            } else {
                set ::GidPriv(CustomColor,ToolbarBg) $::GidPriv(Color,ToolbarBg)
            }
            
            
            set ::GidPriv(Color,BackgroundListbox) #f5f5f5
            set ::GidPriv(Color,DisabledBackgroundEntry) #f5f5f5
            set ::GidPriv(Color,DisabledForegroundEntry) #000000
            set ::GidPriv(Color,DisabledForegroundMenu) #999999
            set ::GidPriv(Color,BackgroundFileDialog) #ffffff
            
            
            if { ( $tcl_platform(platform) == "windows") } {            
                set ::GidPriv(MenuType) "native"
            } else {
                set ::GidPriv(MenuType) "generic"
            }
        }
        
        
        
        ########################
        #change from old look to new look
        ###############################
        #set newlook ![gid_themes::GetOnlyDefineColorsOnMainGiDFrame]
        
        
        set BottomFrameInRed 0
        #set testcolors 1
        #testcolors == 1 => ttk::style theme use classicblue
        
        
        
        
        set newgidbuttonlayout ""
        set newgidiconbuttonlayout ""
        set newgidhorizontaliconbuttonlayout ""
        set newgidverticaliconbuttonlayout ""
        set newgidframelayout ""
        set newgidcomboboxlayout ""
        set newgidseparatorlayout ""
        set optionscolors ""
        if { [gid_themes::GetOnlyDefineColorsOnMainGiDFrame] } {                
            #from current theme
            set newgidbuttonlayout [ttk::style layout TButton]
            ttk::style theme settings [gid_themes::GetTtkTheme] {
                set newgidiconbuttonlayout [ttk::style layout IconButton]
                set newgidhorizontaliconbuttonlayout [ttk::style layout Horizontal.IconButton]
                set newgidverticaliconbuttonlayout [ttk::style layout Vertical.IconButton]
                set newgidiconbuttonwithmenulayout [ttk::style layout IconButtonWithMenu]
                set newgidhorizontaliconbuttonwithmenulayout [ttk::style layout Horizontal.IconButtonWithMenu]
                set newgidverticaliconbuttonwithmenulayout [ttk::style layout Vertical.IconButtonWithMenu]
            }
            set newgidframelayout [ttk::style layout TFrame]
            set newgidcomboboxlayout [ttk::style layout TCombobox]
            set newgidlabellayout [ttk::style layout TLabel]
            set newgidentrylayout [ttk::style layout TEntry]
            set newgidhorizontalscrolllayout [ttk::style layout Horizontal.TScrollbar]
            set newgidverticalscrolllayout [ttk::style layout Vertical.TScrollbar]
            set newgidhorizontalseparatorlayout [ttk::style layout Horizontal.TSeparator]
            set newgidverticalseparatorlayout [ttk::style layout Vertical.TSeparator]
            set optionscolors [ttk::style configure .]
            #set optionscolors [dict replace $optionscolors -font SmallFont] 
            #set optionscolors [dict replace $optionscolors -background #eeeeee]    
        } else {
            ttk::style theme settings [gid_themes::GetTtkTheme] {
                set newgidbuttonlayout [ttk::style layout TButton]
                set newgidiconbuttonlayout [ttk::style layout IconButton]
                set newgidhorizontaliconbuttonlayout [ttk::style layout Horizontal.IconButton]
                set newgidverticaliconbuttonlayout [ttk::style layout Vertical.IconButton]
                set newgidiconbuttonwithmenulayout [ttk::style layout IconButtonWithMenu]
                set newgidhorizontaliconbuttonwithmenulayout [ttk::style layout Horizontal.IconButtonWithMenu]
                set newgidverticaliconbuttonwithmenulayout [ttk::style layout Vertical.IconButtonWithMenu]
                set newgidframelayout [ttk::style layout TFrame]
                set newgidcomboboxlayout [ttk::style layout TCombobox]
                set newgidlabellayout [ttk::style layout TLabel]
                set newgidentrylayout [ttk::style layout TEntry]
                set newgidhorizontalscrolllayout [ttk::style layout Horizontal.TScrollbar]
                set newgidverticalscrolllayout [ttk::style layout Vertical.TScrollbar]  
                set newgidhorizontalseparatorlayout [ttk::style layout Horizontal.TSeparator]
                set newgidverticalseparatorlayout [ttk::style layout Vertical.TSeparator]
                #options colors to apply to toolbars:
                set optionscolors [ttk::style configure IconButton]
            }
        }                
        #if { $testcolors == 1 } {
            #    set optionscolors { \
                #                            -background #00ff00 \
                    #                            -selectbackground #0fff00 \
                    #                            -fieldbackground #00ff0f \
                    #                            -foreground #0f0fff \
                    #                            -selectforeground #30ffff \
                    #                            -font TkDefaultFont \
                    #                            -troughcolor #00ff30 \
                    #                            -borderwidth 0 \
                    #                            -selectborderwidth 0 \
                    #                            -focusthickness 0 \
                    #                            -focuscolor #30ff30 \
                    #                            -highlightthickness 0 \
                    #                            -highlightcolor #3fff00 \
                    #                            -indicatorcolor #00ff3f  \
                    #                            -insertwidth 2 \
                    #                        }
            #}
        
        set listdefaultttkthemes {winnative clam alt default classic vista xpnative aqua}        
        foreach th [ttk::style theme names] {
            if { $BottomFrameInRed == 1 } {
                set framebuttoncolor #ff0000
            } else {
                #nicer color depending on theme its calculated on 
                #tclfile-opengl.tcl :  bind . <<ThemeChanged>>
                #caution don't change following value 
                set framebuttoncolor #fdfae9            
            }
            
            
            if { $BottomFrameInRed == 1 } {
                set buttonoptions {-font "Helvetica 14" -foreground red -padding 10 -background $framebuttoncolor}
                #-width 6
            } else {
                set buttonoptions {-font TkDefaultFont -padding {4 2} -background $framebuttoncolor}
                #-width 6
            }
            
            if { [gid_themes::GetOnlyDefineColorsOnMainGiDFrame] } {
                if { [lsearch $listdefaultttkthemes $th] != "-1" } {
                    ttk::style theme settings $th {        
                        ttk::style element create newgid.button from [gid_themes::GetTtkTheme]
                        ttk::style element create iconbutton.button from [gid_themes::GetTtkTheme]
                        ttk::style element create horizontaliconbutton.button from [gid_themes::GetTtkTheme]
                        ttk::style element create verticaliconbutton.button from [gid_themes::GetTtkTheme]
                        ttk::style element create iconbuttonwithmenu.button from [gid_themes::GetTtkTheme]
                        ttk::style element create horizontaliconbuttonwithmenu.button from [gid_themes::GetTtkTheme]
                        ttk::style element create verticaliconbuttonwithmenu.button from [gid_themes::GetTtkTheme]
                    }
                    
                    
                    #cutre apao para definir los elementos que solo se utilizan en el gidclassico, motivo:
                    #Frames working different because put background image in a frame
                    #delays a bit hover options, so if its not really needed better don't put
                    if { 1 } {
                    #[gid_themes::GetTtkTheme] == "classicgid"
                        ttk::style theme settings $th {        
                            ttk::style element create FrameIconButton.border from [gid_themes::GetTtkTheme]
                            ttk::style element create HorizontalFrameIconButton.border from [gid_themes::GetTtkTheme]
                            ttk::style element create VerticalFrameIconButton.border from [gid_themes::GetTtkTheme]
                        }
                        ttk::style theme settings $th [subst {
                                ttk::style layout TFrame.IconButton { 
                                    FrameIconButton.border -sticky nswe
                                }                                
                                ttk::style layout Horizontal.TFrame.IconButton { 
                                    HorizontalFrameIconButton.border -sticky nswe
                                }
                                ttk::style layout Vertical.TFrame.IconButton { 
                                    VerticalFrameIconButton.border -sticky nswe
                                }                                                
                            }   ]
                    } else {                
                        ttk::style theme settings $th [subst {
                                ttk::style layout TFrame.IconButton { 
                                    $newgidframelayout
                                }                                
                                ttk::style layout Horizontal.TFrame.IconButton { 
                                    $newgidframelayout
                                }
                                ttk::style layout Vertical.TFrame.IconButton { 
                                    $newgidframelayout
                                }        
                                ttk::style configure "TFrame.IconButton" $optionscolors
                                ttk::style configure "Horizontal.TFrame.IconButton" $optionscolors
                                ttk::style configure "Vertical.TFrame.IconButton" $optionscolors
                            }   ]
                    }
                }                        
                
            } else {
                if { 1 } {
                        ttk::style theme settings $th [subst {
                                ttk::style layout TFrame.IconButton { 
                                    FrameIconButton.border -sticky nswe
                                }                                
                                ttk::style layout Horizontal.TFrame.IconButton { 
                                    HorizontalFrameIconButton.border -sticky nswe
                                }
                                ttk::style layout Vertical.TFrame.IconButton { 
                                    VerticalFrameIconButton.border -sticky nswe
                                }                                                
                            }   ]
                } else {            
                ttk::style theme settings $th [subst {
                        ttk::style layout TFrame.IconButton { 
                            $newgidframelayout
                        }                                
                        ttk::style layout Horizontal.TFrame.IconButton { 
                            $newgidframelayout
                        }
                        ttk::style layout Vertical.TFrame.IconButton { 
                            $newgidframelayout
                        }        
                        ttk::style configure "TFrame.IconButton" $optionscolors
                        ttk::style configure "Horizontal.TFrame.IconButton" $optionscolors
                        ttk::style configure "Vertical.TFrame.IconButton" $optionscolors
                    }   ]
                }
            }
            #done outside next ttk::style them since problems with subst 
            ttk::style theme settings $th {
                if { [ttk::style map "." -foreground] != "" } {
                    ttk::style map "." -foreground [list {*}[ttk::style map "." -foreground] invalid "#ff0000"]
                } else {
                        #this destroy previous map
                    ttk::style map "." -foreground {invalid "#ff0000"}
                }
            }            
            
            ttk::style theme settings $th [subst {
                    #independent of the theme
                    ttk::style layout ForcedButton { eval $newgidbuttonlayout }
                    #ttk::style layout Horizontal.ForcedButton { eval $newgidbuttonlayout  }
                    #ttk::style layout Vertical.ForcedButton { eval $newgidbuttonlayout }
                    ttk::style layout ForcedFrame { eval $newgidframelayout }
                    #ttk::style layout Horizontal.ForcedFrame { eval $newgidframelayout }
                    #ttk::style layout Vertical.ForcedFrame { eval $newgidframelayout }
                    ttk::style layout ForcedCombobox { eval $newgidcomboboxlayout }
                    #ttk::style layout Horizontal.ForcedCombobox { eval $newgidcomboboxlayout }
                    #ttk::style layout Vertical.ForcedCombobox { eval $newgidcomboboxlayout }
                    ttk::style layout ForcedLabel { eval $newgidlabellayout }
                    #ttk::style layout Horizontal.ForcedLabel { eval $newgidlabellayout }
                    #ttk::style layout Vertical.ForcedLabel { eval $newgidlabellayout }
                    ttk::style layout ForcedEntry { eval $newgidentrylayout }
                    #ttk::style layout Horizontal.ForcedEntry { eval $newgidentrylayout }
                    #ttk::style layout Vertical.ForcedEntry { eval $newgidentrylayout }                    
                    
                    ttk::style layout Horizontal.ForcedScrollbar { eval $newgidhorizontalscrolllayout }
                    ttk::style layout Vertical.ForcedScrollbar { eval $newgidverticalscrolllayout }                                
                    
                    ttk::style layout Horizontal.ForcedSeparator { eval $newgidhorizontalseparatorlayout }
                    ttk::style layout Vertical.ForcedSeparator { eval $newgidverticalseparatorlayout }                    
                    
                    ttk::style layout IconButton { eval $newgidiconbuttonlayout }
                    ttk::style layout Horizontal.IconButton { eval $newgidhorizontaliconbuttonlayout }
                    ttk::style layout Vertical.IconButton { eval $newgidverticaliconbuttonlayout }
                    
                    ttk::style layout IconButtonWithMenu { eval $newgidiconbuttonwithmenulayout }
                    ttk::style layout Horizontal.IconButtonWithMenu { eval $newgidhorizontaliconbuttonwithmenulayout }
                    ttk::style layout Vertical.IconButtonWithMenu { eval $newgidverticaliconbuttonwithmenulayout }
                    
                    
                    ttk::style configure TopMenu.IconButton -font TkDefaultFont -padding {6 0}
                    ttk::style configure TopMenu.Horizontal.IconButton -font TkDefaultFont -padding {6 0}
                    ttk::style configure TopMenu.Vertical.IconButton -font TkDefaultFont -padding {0 6}
                    ttk::style configure RightButtons.IconButton -anchor w -padding {4 2}            
                    
                    ttk::style configure ForcedButton -anchor left $optionscolors                        
                    ttk::style configure ForcedFrame $optionscolors   
                    ttk::style configure ForcedCombobox [dict replace $optionscolors -foreground #000000]                    
                    ttk::style configure ForcedLabel $optionscolors
                    ttk::style configure ForcedEntry [ttk::style configure TEntry]
                    ttk::style configure ForcedScrollbar $optionscolors
                    ttk::style configure ForcedSeparator $optionscolors
                    ttk::style configure IconButton -anchor center $optionscolors
                    ttk::style configure IconButtonWithMenu -anchor center
                                          
                                     
                    #dependent of the theme (derived classes example pp.TButton)                    
                    ttk::style configure "BottomFrame.TButton" $buttonoptions -width -10  
                    ttk::style configure "BottomFrame.TFrame" -background $framebuttoncolor -padding {8 12}
                    ttk::style configure "BottomFrame.TLabel" -background $framebuttoncolor
                    ttk::style configure "BottomFrame.TEntry" -background $framebuttoncolor 
                    ttk::style configure "BottomFrame.TCheckbutton" -background $framebuttoncolor
                    ttk::style configure "BottomFrame.TCombobox" -background $framebuttoncolor
                    ttk::style configure "BottomFrame.TLabelframe" -background $framebuttoncolor
                    ttk::style configure "BottomFrame.TMenubutton" -background $framebuttoncolor
                    ttk::style configure "BottomFrame.TNotebook" -background $framebuttoncolor
                    ttk::style configure "BottomFrame.TPanedwindow" -background $framebuttoncolor
                    ttk::style configure "BottomFrame.Horizontal.TProgressbar" -background $framebuttoncolor
                    ttk::style configure "BottomFrame.Vertical.TProgressbar" -background $framebuttoncolor
                    ttk::style configure "BottomFrame.TRadiobutton" -background $framebuttoncolor
                    ttk::style configure "BottomFrame.Horizontal.TScrollbar" -background $framebuttoncolor
                    ttk::style configure "BottomFrame.Vertical.TScrollbar" -background $framebuttoncolor
                    ttk::style configure "BottomFrame.TSeparator" -background $framebuttoncolor
                    ttk::style configure "BottomFrame.TSizegrip" -background $framebuttoncolor
                    ttk::style configure "BottomFrame.Treeview" -background $framebuttoncolor
                                       
                    #don't work well :ttk::style configure "BigFont.TLabel" -font BigFont
                    # use instate -font BigFont
                    #diferent definition depending on theme
                    if { "[gid_themes::GetTtkTheme]" == "newgid" } {
                        ttk::style configure "flat.TFrame" -relief flat -borderwidth 0
                        ttk::style configure "groove.TFrame" -relief flat -borderwidth 0 
                        ttk::style configure "raised.TFrame" -relief flat -borderwidth 0 
                        ttk::style configure "ridge.TFrame" -relief flat -borderwidth 0 
                        ttk::style configure "solid.TFrame" -relief flat -borderwidth 0
                        ttk::style configure "sunken.TFrame" -relief flat -borderwidth 0 
                    } else {
                        
                        ttk::style configure TLabelframe -labelanchor nw
                        
                        ttk::style configure "flat.TFrame" -relief flat -borderwidth 0
                        ttk::style configure "groove.TFrame" -relief groove -borderwidth 2 
                        ttk::style configure "raised.TFrame" -relief raised -borderwidth 2 
                        ttk::style configure "ridge.TFrame" -relief ridge -borderwidth 2 
                        ttk::style configure "solid.TFrame" -relief solid -borderwidth 2
                        ttk::style configure "sunken.TFrame" -relief sunken -borderwidth 2                         
                    }
                    
                }   ]             
        }
        
        #if { $testcolors == 1 } {
            #    ttk::style theme use classicblue
            #} else
        if { [gid_themes::GetOnlyDefineColorsOnMainGiDFrame] } {
            if { "clam" in [ttk::style theme names] } {
                ttk::style theme settings clam {
                    ttk::style configure TButton -padding 1
                    ttk::style configure TMenubutton -padding 1
                    # ttk::style map Toolbutton -background "focus grey [ttk::style map Toolbutton -background]"
                }
            }
            # if { "xpnative" in [ttk::style theme names] } {
                #         ttk::style theme settings xpnative {
                    #             ttk::style map TMenubutton -background "focus grey [ttk::style map Toolbutton -background]"
                    #         }
                # }
        } else {
            ttk::style theme use [gid_themes::GetTtkTheme]
        }
        
        #USEFULL THINKS
        #web resources:
        #        http://www.tkdocs.com/tutorial/styles.html
        #        http://tktable.sourceforge.net/tile/doc/converting.txt
        
        #ttk::style layout TButton
        #ttk::style layout TCheckbutton
        #ttk::style layout TCombobox
        #ttk::style layout TEntry
        #ttk::style layout TFrame
        #ttk::style layout TLabel
        #ttk::style layout TLabelframe
        #ttk::style layout TMenubutton
        #ttk::style layout TNotebook
        #ttk::style layout TPanedwindow
        #ttk::style layout Horizontal.TProgressbar
        #ttk::style layout Vertical.TProgressbar
        #ttk::style layout TRadiobutton
        #ttk::style layout Horizontal.TScrollbar
        #ttk::style layout Vertical.TScrollbar
        #ttk::style layout TSeparator
        #ttk::style layout TSizegrip
        #ttk::style layout Treeview
        #to inspect elements inside layout
        
        #ttk::style layout TButton
        #result in windows: Button.button -sticky nswe -children {Button.focus -sticky nswe -children {Button.padding -sticky nswe -children {Button.label -sticky nswe}}}
        #ttk::style element options Button.label
        #result:-compound -space -text -font -foreground -underline -width -anchor -justify -wraplength -embossed -image -stipple -background
        #ttk::style element options Button.padding
        #result:-padding -relief -shiftrelief
        #ttk::style element options Button.button
        #result: none
        #ttk::style element options Button.focus
        #result: none
        #in each theme the layout could be diferent and each element have diferent options
        #also in some themes some options could not have effect
        
        #STATES: active disabled focus pressed selected background readonly alternate invalid hover
        #-border padding: padding is a list of up to four integers, specifying the left, top, right, and bottom borders, respectively. See IMAGE STRETCHING, below. 
        #-height height: Specifies a minimum height for the element. If less than zero, the base image's height is used as a default. 
        #-padding padding: Specifies the element's interior padding. Defaults to -border if not specified. 
        #-sticky spec: Specifies how the image is placed within the final parcel. spec contains zero or more characters n, s, w, or e. 
        #-width width: Specifies a minimum width for the element. If less than zero, the base image's width is used as a default. 
        
        #    End configurating look
    }
    
    
    
    
    #really this information must be read from xml files of themes folder...
    proc gid_themes::ReadThemes_test { } {    
        variable Themes
        array unset Themes
        
        set Themes(names) {GiD_black GiD_classic}
        
        set name GiD_classic
        set Themes($name,folder) GiD_classic
        set Themes($name,alternative_theme) GiD_black
        set Themes($name,SizeDefinitions) {small_size large_size}
        set SizeDefinition small_size
        set Themes($name,SizeDefinition,$SizeDefinition,label) [_ "small size"]   
        set Themes($name,SizeDefinition,$SizeDefinition,folder) small_size(16andmore)
        set Themes($name,SizeDefinition,$SizeDefinition,alternative_folder) GiD_black/images/size16
        set Themes($name,SizeDefinition,$SizeDefinition,font) {{MS Sans Serif} 8 normal roman}
        set SizeDefinition large_size
        set Themes($name,SizeDefinition,$SizeDefinition,label) [_ "large size"]   
        set Themes($name,SizeDefinition,$SizeDefinition,folder) large_size(24andmore)
        set Themes($name,SizeDefinition,$SizeDefinition,alternative_folder) GiD_black/images/size24
        set Themes($name,SizeDefinition,$SizeDefinition,font) {{MS Sans Serif} 14 normal roman}
        
        set Themes($name,ThemeSizes) 0
        
        set Themes($name,IconCategories,0,menu) small_size
        set Themes($name,IconCategories,0,toolbar) large_size
        set Themes($name,IconCategories,0,small_icons) small_size
        set Themes($name,IconCategories,0,large_icons) large_size
        set Themes($name,IconCategories,0,generic) ""
        
        set name GiD_black
        set Themes($name,folder) GiD_black
        set Themes($name,alternative_theme) GiD_classic
        #... similar to GiD_classic theme
        
    }
    
    #read themes information from themes folders
    proc gid_themes::ReadThemes { } {
        variable Themes
        if {  [GidUtils::IsTkDisabled] } {
            return
        }
        
        package require tdom
        
        array unset Themes
        set dir_themes [file join $::GIDDEFAULT themes]
        foreach dir_theme [glob -directory $dir_themes -types d *] {
            set xmlfile [file join $dir_theme configuration.xml]
            if { ![file exists $xmlfile] } {
                continue
            }
            set xml [tDOM::xmlReadFile $xmlfile]
            set document [dom parse $xml]
            set root [$document documentElement]
            
            if { [$root nodeName] != "GID_THEME" } {
                $document delete
                continue
            }
            set name [$root getAttribute name]
            lappend Themes(names) $name
            
            set Themes($name,folder) [file tail $dir_theme]
            foreach key {label version alternative_theme OnlyDefineColorsOnMainGiDFrame TtkTheme TkFromTheme
                        BackgroundColor BackColorType BackColorTop BackColorBottom BackColorSense ColorPoints
                        ColorNodes ColorLines ColorPolyLines ColorSurfaces ColorSurfsIsoparametric ColorVolumes
                        ColorElements} {
                set domNode [$root selectNodes $key]
                if { $domNode != "" } {
                    set Themes($name,$key) [$domNode text]
                }
            }
            
            set domGroupNodes [$root selectNodes SizeDefinitions]
            if {$domGroupNodes != ""} {
                foreach childNode [$domGroupNodes childNodes] {
                    set SizeDefinition [$childNode getAttribute name]
                    foreach key {label folder alternative_folder font} {
                        set domNode [$childNode selectNodes $key]
                        if { $domNode != "" } {
                            set Themes($name,SizeDefinition,$SizeDefinition,$key) [$domNode text]           
                        }                
                    }
                    
                }
            }
            set domGroupNodes [$root selectNodes ThemeSizes]
            set Themes($name,ThemeSizes) ""
            if {$domGroupNodes != ""} {           
                foreach childNode [$domGroupNodes childNodes] {
                    set ThemeSize [$childNode getAttribute size]
                    lappend  Themes($name,ThemeSizes) $ThemeSize
                    foreach childchildNode [$childNode childNodes] {
                        set Themes($name,IconCategories,$ThemeSize,[$childchildNode getAttribute name]) [$childchildNode text] 
                    }
                    foreach IconCategory {small_icons large_icons menu toolbar generic} {
                        if { ![info exists Themes($name,IconCategories,$ThemeSize,$IconCategory)] } {
                            set Themes($name,IconCategories,$ThemeSize,$IconCategory) ""
                        }
                    }                    
                }
            }
            $document delete
        }
        
        
    }
    
    proc ::BeforeInitGIDProject { dir } {
        gid_themes::ReadModuleThemes $dir
    }
    
    #read themes information from module themes folders
    proc gid_themes::ReadModuleThemes { ModuleDir } {
        variable Themes
        variable IconCategories
        
        if {  [GidUtils::IsTkDisabled] } {
            return
        }
        array unset ::ModulePriv
        array unset ::ModulePrivImages
        
        set ::ModulePriv(path) $ModuleDir
        
        if { $ModuleDir != "" } {
            set ::ModulePriv(valid) 1
            
            set aux_module_themes(names) ""
            set dir_themes [file join $ModuleDir themes]
            
            if { [file exists $dir_themes] } {
                package require tdom
                foreach dir_theme [glob -directory $dir_themes -types d *] {
                    set xmlfile [file join $dir_theme configuration.xml]
                    if { ![file exists $xmlfile] } {
                        continue
                    }
                    set xml [tDOM::xmlReadFile $xmlfile]
                    set document [dom parse $xml]
                    set root [$document documentElement]
                    
                    if { !( [$root nodeName] == "GID_THEME" || [$root nodeName] == "MODULE_THEME" ) } {
                        $document delete
                        continue
                    }
                    set name [$root getAttribute name]
                    lappend aux_module_themes(names) $name
                    
                    set aux_module_themes($name,folder) [file tail $dir_theme]
                    foreach key {label version alternative_theme} {
                        set domNode [$root selectNodes $key]
                        if { $domNode != "" } {
                            set aux_module_themes($name,$key) [$domNode text]
                        }
                    }
                    
                    set domGroupNodes [$root selectNodes SizeDefinitions]
                    if {$domGroupNodes != ""} {
                        foreach childNode [$domGroupNodes childNodes] {
                            set SizeDefinition [$childNode getAttribute name]
                            foreach key {label folder alternative_folder font} {
                                set domNode [$childNode selectNodes $key]
                                if { $domNode != "" } {
                                    set aux_module_themes($name,SizeDefinition,$SizeDefinition,$key) [$domNode text]           
                                }                
                            }
                            
                        }
                    }
                    set domGroupNodes [$root selectNodes ThemeSizes]
                    set aux_module_themes($name,ThemeSizes) ""
                    if {$domGroupNodes != ""} {
                        foreach childNode [$domGroupNodes childNodes] {
                            set ThemeSize [$childNode getAttribute size]
                            lappend aux_module_themes($name,ThemeSizes) $ThemeSize
                            foreach childchildNode [$childNode childNodes] {
                                set aux_module_themes($name,IconCategories,$ThemeSize,[$childchildNode getAttribute name]) [$childchildNode text]
                            }
                            foreach IconCategory {small_icons large_icons menu toolbar generic} {
                                if { ![info exists aux_module_themes($name,IconCategories,$ThemeSize,$IconCategory)] } {
                                    set aux_module_themes($name,IconCategories,$ThemeSize,$IconCategory) ""
                                }
                            }
                        }
                    }                    
                    $document delete        
                }
            }
            foreach name $Themes(names) {
                if { [lsearch $aux_module_themes(names) $name] != -1 } {
                    set ::ModulePriv($name,valid) 1
                    set theme_folder $aux_module_themes($name,folder)    
                    foreach IconCategory $IconCategories {
                        foreach ThemeSize $aux_module_themes($name,ThemeSizes) {
                            set SizeDefinition $aux_module_themes($name,IconCategories,$ThemeSize,$IconCategory)
                            set size_folder [file join $theme_folder images]
                            if { [info exists aux_module_themes($name,SizeDefinition,$SizeDefinition,folder)] } {           
                                set size_folder [file join $theme_folder images $aux_module_themes($name,SizeDefinition,$SizeDefinition,folder)]
                            }
                        
                            set ::ModulePriv($name,path,$ThemeSize,$IconCategory) [file join $ModuleDir themes $size_folder]        
                        
                        
                            set size_folder_alternative ""
                            if { [info exists aux_module_themes($name,alternative_theme)] } {
                                set size_folder_alternative [file join $aux_module_themes($name,alternative_theme) images]
                            }
                            if { [info exists aux_module_themes($name,SizeDefinition,$SizeDefinition,alternative_folder)] } {
                                set size_folder_alternative $aux_module_themes($name,SizeDefinition,$SizeDefinition,alternative_folder)            
                            }
                            if { $size_folder_alternative != "" } {
                                set ::ModulesPriv($name,path_alternative,$ThemeSize,$IconCategory) [file join $ModuleDir themes $size_folder_alternative]
                            }
                        }
                    }
                } else {
                    set ::ModulePriv($name,valid) 0
                }
            }
        } else {
            foreach name $Themes(names) {
                set ::ModulePriv($name,valid) 0 
            }
        }
    }
    
    
    
    proc gid_themes::GetThemes { } {
        variable Themes
        if { ![info exists Themes(names)] } {
            return ""
        }
        return $Themes(names)
    }
    proc gid_themes::SetNextRestartTheme { name } {
        if { $name == [gid_themes::GetCurrentTheme] } {
            if { [info exists ::GidPriv(NextRestartTheme)] } {
                unset ::GidPriv(NextRestartTheme)                
            }
        }
        set ::GidPriv(NextRestartTheme) $name        
    }
    proc gid_themes::GetNextRestartTheme { } {
        if { ![info exists ::GidPriv(NextRestartTheme)] } {
            return [gid_themes::GetCurrentTheme]
        }
        return $::GidPriv(NextRestartTheme)    
    }
   
    proc gid_themes::GetCurrentTheme { } {
        if { ![info exists ::GidPriv(CurrentTheme)] } {
            return ""
        }
        return $::GidPriv(CurrentTheme)
    }
        
    #set ::GidPriv(CurrentTheme) name, and path and possible path_alternative for each icon category
    proc gid_themes::SetCurrentTheme { name } {
        variable IconCategories
        variable Themes
                
        # WarnWinText "gid_themes::SetCurrentTheme"
        if { [GidUtils::IsTkDisabled] } {       
            return
        }                
        
        global GidPriv        
        if { [gid_themes::GetCurrentTheme] == $name } {
            return
        }
        
        if { [lsearch [gid_themes::GetThemes] $name] == -1 } {
            WarnWinText [_ "Error, theme %s not found. Valid names are %s" $name [gid_themes::GetThemes]]
            return
        }
        
        # WarnWinText "gid_themes::SetCurrentTheme doing things"
        
        set ::GidPriv(CurrentTheme) $name
        set theme_folder $Themes($name,folder)    
        
        foreach IconCategory $IconCategories {
            foreach ThemeSize $Themes($name,ThemeSizes) {
                set SizeDefinition $Themes($name,IconCategories,$ThemeSize,$IconCategory)
                set size_folder [file join $theme_folder images]
                if { [info exists Themes($name,SizeDefinition,$SizeDefinition,folder)] } {           
                    set size_folder [file join $theme_folder images $Themes($name,SizeDefinition,$SizeDefinition,folder)]
                }
                set ::GidPriv(CurrentTheme,path,$ThemeSize,$IconCategory) [file join $::GIDDEFAULT themes $size_folder]        
            
                set size_folder_alternative ""
                if { [info exists Themes($name,alternative_theme)] } {
                    set size_folder_alternative [file join $Themes($name,alternative_theme) images]
                }
                if { [info exists Themes($name,SizeDefinition,$SizeDefinition,alternative_folder)] } {
                    set size_folder_alternative $Themes($name,SizeDefinition,$SizeDefinition,alternative_folder)            
                }
                if { $size_folder_alternative != "" } {
                    set ::GidPriv(CurrentTheme,path_alternative,$ThemeSize,$IconCategory) [file join $::GIDDEFAULT themes $size_folder_alternative]
                }                
            }
        }
        
        set ::GidPriv(ThemeSize) 0
        #WarnWinText "info exists ::GidPriv(ThemeSize,$name) [info exists ::GidPriv(ThemeSize,$name)]"
        if { [info exists ::GidPriv(ThemeSize,$name)] } {
            set ::GidPriv(ThemeSize) $::GidPriv(ThemeSize,$name)
            #WarnWinText "::GidPriv(ThemeSize,$name) $::GidPriv(ThemeSize,$name)"
        }
        if { [lsearch $Themes($name,ThemeSizes) $::GidPriv(ThemeSize)] == -1 } {
            #WarnWinText "lsearch -1"
           set ::GidPriv(ThemeSize) [lindex $Themes($name,ThemeSizes) [expr [llength $Themes($name,ThemeSizes)] / 2]]
        }
        
        array unset ::GidPrivImages ;#to reload images with gid_themes::GetImage
    }
    
    proc gid_themes::SetCurrentThemeAndApplyBackground { name } {         
        global GidPriv
        variable Themes
        #the same is done in tclfileP when NextRestartTheme is set
        foreach key {BackgroundColor BackColorType BackColorTop BackColorBottom BackColorSense ColorPoints
                     ColorNodes ColorLines ColorPolyLines ColorSurfaces ColorSurfsIsoparametric ColorVolumes
                     ColorElements} {
            GiD_Set $key [set Themes($name,$key)]
        }
        gid_themes::SetCurrentTheme $name   
    }
    proc gid_themes::SetCurrentThemeSize { size } { 
         variable Themes
        
        set name [gid_themes::GetCurrentTheme]
    
        set ::GidPriv(ThemeSize,$name) $size
        if { $::GidPriv(ThemeSize) != $::GidPriv(ThemeSize,$name) } {
            set ::GidPriv(ThemeSize) $::GidPriv(ThemeSize,$name)            
            if { [lsearch $Themes($name,ThemeSizes) $::GidPriv(ThemeSize)] == -1 } {
                set ::GidPriv(ThemeSize) [lindex $Themes($name,ThemeSizes) [expr [llength $Themes($name,ThemeSizes)] / 2]]
            }  
            array unset ::GidPrivImages
            array unset icon_chooser::_default_macro_images
            icon_chooser::AfterReadMacros
            UpdateWidgetsLanguage            
        }
    }    
    proc gid_themes::UpdateCurrentThemeSize { } {
        variable Themes
        
        set name [gid_themes::GetCurrentTheme]
        if { ![info exists ::GidPriv(ThemeSize,$name)] } {
            set ::GidPriv(ThemeSize,$name) 0
        }
        if { ![info exists Themes($name,ThemeSizes)] } {
            break;
        }        
        if { $::GidPriv(ThemeSize) != $::GidPriv(ThemeSize,$name) } {
            set ::GidPriv(ThemeSize) $::GidPriv(ThemeSize,$name)            
            if { [lsearch $Themes($name,ThemeSizes) $::GidPriv(ThemeSize)] == -1 } {
                set ::GidPriv(ThemeSize) [lindex $Themes($name,ThemeSizes) [expr [llength $Themes($name,ThemeSizes)] / 2]]
            }  
            array unset ::GidPrivImages
            array unset icon_chooser::_default_macro_images
            icon_chooser::AfterReadMacros            
        }
    }
    #Things to check for correct working
    # DONE:
    #  The image its in all looks
    #  If it ask for a size, it have to be in booth sizes
    # NOT DONE:
    #  Inside a size folder all images have the same size at least the same width or the same height
    #  All images .png
    #  If in root look folder there is a image the same size as in size folder, its almost sure an error    
    proc gid_themes::CheckThemes {} {        
        package require BWidget
        variable Themes
        
        proc openfile { dir } { 
            global tcl_platform
            if { $::tcl_platform(platform) == "windows" } { 
                if { [file isfile $dir] } {
                    set dir [file attributes $dir -shortname]
                }
                catch {eval exec [auto_execok start] $dir >& NUL: &}
            } else {
                #catch {exec netscape $dir >& /dev/null &} 
                if { [ catch {exec netscape $dir &}]} {
                    catch {exec konqueror $dir &}
                } 
            }
            
        }
        proc howmanydifferences { listdir } {
            set allimagesfull ""
            foreach imgdir $listdir {
                if { [catch {append allimagesfull " [glob -directory $imgdir *.*]"} err] } {
                    WarnWinText "Error reading $imgdir ($err)"
                }
            }
            set allimages ""
            foreach file $allimagesfull {
                lappend allimages [file tail $file]
            }
            set allimagesfull ""
            set allimages [lsort -dictionary $allimages]
            #all images must be the same number of times as number of listdir
            
            set previmg ""
            set counterror 0
            set numdir [llength $listdir]
            set numberoftime $numdir
            foreach img $allimages {
                if { $img != $previmg } {
                    if {$numberoftime != $numdir } {
                        set counterror [expr $counterror+1]
                    }
                    set previmg $img
                    set numberoftime 1
                } else {
                    set numberoftime [expr $numberoftime+1]
                }
            }
            return $counterror                                        
        }
        
        proc compareimages { listdir } {
            set i 0
            while { [winfo exists .infoimages$i] } { incr i } 
            
            toplevel .infoimages$i                        
            set sw [ScrolledWindow .infoimages$i.sw]
            
            pack $sw -fill both -expand true
            
            set sf [ScrollableFrame $sw.sf]
            
            $sw setwidget $sf
            
            set uf [$sf getframe]
            
            #sorting list
            set allimagesfull ""
            foreach imgdir $listdir {
                append allimagesfull " [glob -directory $imgdir *.*]"
            }
            set allimages ""
            foreach file $allimagesfull {
                lappend allimages [file tail $file]
            }
            set allimagesfull ""
            set allimages [lsort -dictionary $allimages]
            #titles of columns
            set numcolumn 1
            foreach imgdir $listdir {
                set nm [format "%s.b_%s_%s" $uf $numcolumn "Title"]
                
                set splitimgdir [file split $imgdir]
                set theme [lindex $splitimgdir [expr [lsearch $splitimgdir themes]+1]]
                set folder [file tail [file rootname $imgdir]]
                
                set b [label $nm -text "theme:$theme\nfolder:$folder" -font BigFont ]
                grid $b -row 1 -column $numcolumn
                
                set numcolumn [expr $numcolumn+1]
            }
            #images
            set previmg ""
            set numrow 2
            set background(1) #292929
            set foreground(1) #c2c5ca
            set background(2) #f0f0f0
            set foreground(2) #000000
            foreach img $allimages {
                if { $img != $previmg } {
                    set previmg $img
                    set numcolumn 1
                    foreach imgdir $listdir {
                        set file [file join $imgdir $img]
                        regsub -all {\.} $img {_} imgname
                        
                        set nm [format "%s.b_%s_%s" $uf $numcolumn [string tolower $imgname]]
                        
                        if { [catch {set I($imgname) [image create photo -file $file]}] } {
                            set b [label $nm -text $img -background #ff0000]
                        } else {                                                
                            set b [tk::button $nm -image $I($imgname) -text $img -compound left \
                                    -command {} -background $background($numcolumn) \
                                    -foreground $foreground($numcolumn) -borderwidth 4 -relief flat]
                        }
                        
                        grid $b -row $numrow -column $numcolumn -sticky nwes
                        
                        set numcolumn [expr $numcolumn+1]
                    }
                    set numrow [expr $numrow+1]
                }
            }        
        }
        
        
        toplevel .askwhattocompare -takefocus 1
        
        foreach name [gid_themes::GetThemes] {
            lappend basefolderfiles [file join $::GIDDEFAULT themes $Themes($name,folder) images]
            lappend smallfolderfiles [file join $::GIDDEFAULT themes $Themes($name,folder) images $Themes($name,SizeDefinition,$Themes($name,IconCategories,small_icons),folder)]
            lappend largefolderfiles [file join $::GIDDEFAULT themes $Themes($name,folder) images $Themes($name,SizeDefinition,$Themes($name,IconCategories,large_icons),folder)]
            
            set allsizesfiles ""
            foreach sizedef [ array names Themes $name,SizeDefinition,* ] {
                if { [regexp ",folder" $sizedef] } {                                        
                    lappend allsizesfiles [file join $::GIDDEFAULT themes $Themes($name,folder) images $Themes($sizedef)]
                }
            }
            button .askwhattocompare.sizesintheme$name \
                -text "Compare Sizes in theme $name (Number of errors [howmanydifferences $allsizesfiles])" \
                -command "gid_themes::compareimages [list $allsizesfiles]"
            
            grid .askwhattocompare.sizesintheme$name -columnspan 3
        }        
        label .askwhattocompare.label -text "Between themes:"        
        button .askwhattocompare.basediffthemes \
            -text "Compare base dir (Number of errors [howmanydifferences $basefolderfiles])" \
            -command "gid_themes::compareimages [list $basefolderfiles]"
        
        button .askwhattocompare.smalldiffthemes \
            -text "Compare small_icons dir (Number of errors [howmanydifferences $smallfolderfiles])" \
            -command "gid_themes::compareimages [list $smallfolderfiles]"
        
        button .askwhattocompare.largediffthemes \
            -text "Compare large_icons dir (Number of errors [howmanydifferences $largefolderfiles])" \
            -command "gid_themes::compareimages [list $largefolderfiles]"
        
        grid .askwhattocompare.label -columnspan 3 
        grid .askwhattocompare.basediffthemes .askwhattocompare.smalldiffthemes .askwhattocompare.largediffthemes                
    }   
    
    
    
    #set ::get_image_mode "debug" ;#raise errors including stack trace
    set ::get_image_mode "release" ;#not raise error messages
    
    #relative images are expected to be inside the GiD images folder, else must provide a full path
    #Look decision its defined outside
    #Empty class implies images inside root look folder
    #To select sizes there are 4 options, forced small: small_icons, forced large: large_icons, menu and toolbar => Automatic size decision
    
    #Things to check for correct working
    #  Not accept full path or partial path pointing to GiD image folder 
    proc gid_themes::GetImage { filename {IconCategory "generic"} {Owner "gid"}} {  
        variable IconCategories
        
        if { $Owner != "gid" } {
            return [gid_themes::GetImageModule $filename $IconCategory]
        }
        
        if { $::get_image_mode == "debug" } {                        
            if { [lsearch $IconCategories $IconCategory] == -1 } {
                WarnWinText [_ "Error, IconCategory %s not found. Valid names are %s" $IconCategory $IconCategories]
                set IconCategory "generic"
            }
        }   
        
        if { ![info exists ::GidPrivImages($IconCategory,$filename)] } {    
            if { [llength [file split $filename]] == 1 } {
            set auxthemesize 0
            if { [info exists ::GidPriv(ThemeSize)] } {            
                set auxthemesize $::GidPriv(ThemeSize)
            }   
            if { [info exists ::GidPriv(CurrentTheme,path,$auxthemesize,$IconCategory)] } { 
                set fullname [file join $::GidPriv(CurrentTheme,path,$auxthemesize,$IconCategory) $filename]
            } else {
                set fullname $filename
            }
            } elseif { [file pathtype $filename] == "absolute" } {
            set fullname $filename
            } else {
            #relative to GiD working directory        
            set fullname [file join $::GIDDEFAULT $filename]           
        }        
        #Inteligence gid_themes::GetImage: Search the image asked for, if not search alternative if not, find inside bitmaps else return the imagenotfound.png"
        set fail 0
            if { [catch {set ::GidPrivImages($IconCategory,$filename) [image create photo -file $fullname]} msg] } {
                
                if { [info exists ::GidPriv(CurrentTheme,path_alternative,$auxthemesize,$IconCategory)] } {          
                    set alternativefullname [file join $::GidPriv(CurrentTheme,path_alternative,$auxthemesize,$IconCategory) $filename]     
                    if { [catch {set ::GidPrivImages($IconCategory,$filename) [image create photo -file $alternativefullname]} msg2] } {
                        set fail 1
                        set alternativefullname2 [file join $::GIDDEFAULT bitmaps $filename]
                        if { [catch {set ::GidPrivImages($IconCategory,$filename) [image create photo -file $alternativefullname2]} msg3] } {                                
                            #try finding in other categories
                            set found_other_category 0
                            foreach category $IconCategories {
                                if { $category != $IconCategory } {
                                    set alternativefullname3 [file join $::GidPriv(CurrentTheme,path,$auxthemesize,$category) $filename]
                                    if { ![catch {set ::GidPrivImages($IconCategory,$filename) [image create photo -file $alternativefullname3]} msg3] } {                                    
                                        set msg_alternative "bad categoy $IconCategory, used $category. $alternativefullname3"
                                        set found_other_category 1
                                        break
                                    }                                
                                }                            
                            }
                            if { !$found_other_category } {
                                set ::GidPrivImages($IconCategory,$filename) [gid_themes::GetImage erase-cross.png small_icons]
                                set msg_alternative  "erase-cross.png"
                            }
                            
                        } else {      
                            set msg_alternative $alternativefullname2
                        }
                    } else {                        
                        set msg_alternative $alternativefullname
                    }
                }
            }      
            if { $fail } {
                if { $::get_image_mode == "debug" } {            
                    WarnWinText "Could not find image '$fullname' (IconCategory=$IconCategory), '$msg_alternative' used instead"                        
                    for { set i [info level]} { $i >0 } { incr i -1 } {
                        WarnWinText [info level $i]
                    }
                    WarnWinText "( To check themes correctness \"-np- gid_themes::CheckThemes\" )"
                } else {
                    
                }
            }
        }
        return $::GidPrivImages($IconCategory,$filename)
    }
    
    proc gid_themes::GetImageModule { filename {IconCategory "generic"} } {        
        variable IconCategories
        
        if { ![info exists ::ModulePrivImages($IconCategory,$filename)] } {    
            if { $::ModulePriv($::GidPriv(CurrentTheme),valid) } {            
                if { [llength [file split $filename]] == 1 } {
                    set fullname [file join $::ModulePriv($::GidPriv(CurrentTheme),path,$IconCategory) $filename]
                } elseif { [file pathtype $filename] == "absolute" } {
                    set fullname $filename
                } else {
                    #relative to GiD working directory        
                    set fullname [file join $::ModulePriv(path) $filename]           
                }
                #Inteligence gid_themes::GetImageModule: Search the image asked for, using ModulePrivImages and ModulePriv variables
                #if not found inside module it uses gid_themes::GetImage instate.
                set fail 0
                if { [catch {set ::ModulePrivImages($IconCategory,$filename) [image create photo -file $fullname]} msg] } {
                    if { [info exists ::ModulePriv($::GidPriv(CurrentTheme),path_alternative,$IconCategory)] } {          
                        set alternativefullname [file join $::ModulePriv($::GidPriv(CurrentTheme),path_alternative,$IconCategory) $filename]     
                        if { [catch {set ::ModulePrivImages($IconCategory,$filename) [image create photo -file $alternativefullname]} msg2] } {
                            set fail 1
                            set found_other_category 0
                            foreach category $IconCategories {
                                if { $category != $IconCategory } {
                                    set alternativefullname3 [file join $::ModulePriv($::GidPriv(CurrentTheme),path,$category) $filename]
                                    if { ![catch {set ::ModulePrivImages($IconCategory,$filename) [image create photo -file $alternativefullname3]} msg3] } {                                    
                                        set msg_alternative "bad categoy $IconCategory, used $category. $alternativefullname3"
                                        set found_other_category 1
                                        break
                                    }                                
                                }                            
                            }
                            if { !$found_other_category } {
                                set ::ModulePrivImages($IconCategory,$filename) [gid_themes::GetImage $filename $IconCategory ]
                                set msg_alternative  "used [gid_themes::GetImage $filename $IconCategory ] instate"
                            }                        
                        } else {                        
                            set msg_alternative $alternativefullname
                        }
                    } else {
                        set ::ModulePrivImages($IconCategory,$filename) [gid_themes::GetImage $filename $IconCategory ]
                    }
                }
            } else {
                set ::ModulePrivImages($IconCategory,$filename) [gid_themes::GetImage $filename $IconCategory ]
            }
        }
        return $::ModulePrivImages($IconCategory,$filename)
    }
    
    
    proc gid_themes::GetImageSize { IconCategory  } {         
        if { [lsearch {"small_icons" "large_icons" "menu" "toolbar"} $IconCategory] == -1 } {
            return 0
        }
        set img [gid_themes::GetImage blank_horizontal.png $IconCategory]
        return [image width $img]
    }

    #to force to rebuild the image
    proc gid_themes::ResetCacheImage { filename {IconCategory "generic"} } {        
        if { [info exists ::GidPrivImages($IconCategory,$filename)] } {
            image delete $::GidPrivImages($IconCategory,$filename)
            array unset ::GidPrivImages($IconCategory,$filename)        
        }
    }

    #------------------------------------------------------------------------------
    # tablelist::newgidTheme
    # declare this procedure here instead to do it inside the tablelist package
    # to easily update this third part package without need to modify the standard package
    # with this feature that has only sense for themed GiD
    #------------------------------------------------------------------------------
    namespace eval tablelist {}
    proc tablelist::newgidTheme {} {
        variable themeDefaults
        
        array set colors {
            -toolbars_bg "#282829"
            -frame_bg "#b1b4b9"        
            -text_bg "#c2c5ca"        
            -colortext_fg "#000000"
            -select_fg "#000000"
            -disabled_fg "#5d5d5d"
            -highlight "#63a5b5"
        }
        
        array set themeDefaults [list \
                                     -background              $colors(-text_bg) \
                                     -foreground              $colors(-colortext_fg) \
                                     -disabledforeground      $colors(-disabled_fg) \
                                     -stripebackground        "" \
                                     -selectbackground        $colors(-highlight) \
                                     -selectforeground        $colors(-select_fg) \
                                     -selectborderwidth       0 \
                                     -font                    TkTextFont \
                                     -labelbackground         $colors(-frame_bg) \
                                     -labeldisabledBg         $colors(-frame_bg) \
                                     -labelactiveBg           $colors(-highlight) \
                                     -labelpressedBg          $colors(-highlight) \
                                     -labelforeground         $colors(-colortext_fg) \
                                     -labeldisabledFg         $colors(-disabled_fg) \
                                     -labelactiveFg           $colors(-colortext_fg) \
                                     -labelpressedFg          $colors(-colortext_fg) \
                                     -labelfont               TkDefaultFont \
                                     -labelborderwidth        0 \
                                     -labelpady               1 \
                                     -arrowcolor              "" \
                                     -arrowstyle              flat8x5 \
                                     -treestyle               winnative \
                                    ] 
    }        
    #------------------------------------------------------------------------------
    
}

