package provide gidmonitoring 1.6

if { $::tcl_platform(platform) == "windows" } {
    package require twapi
}

namespace eval gidmonitoring {
    variable request "" ;#to only process and return the asked variables, without the overhead of compute all possible data
    set expr_procedures "" ;#to ask for defined body procedures
}

proc gidmonitoring::set_request { variable_names } {
    variable request
    set request $variable_names
}

proc gidmonitoring::set_expr_procedures { procedures } {
    variable expr_procedures
    set expr_procedures $procedures
}

proc gidmonitoring::getinfo { } {
    variable request
    variable expr_procedures
    array unset MonitoringInfo

    #these process related variables are always returned, because are not expensive
    set pid [pid]
    if { $::tcl_platform(platform) == "windows" } {
	foreach item {workingset workingsetpeak elapsedtime} {
	    set MonitoringInfo($item) [lindex [twapi::get_process_info $pid -$item] 1]	    
	}
    } else {
	#use Linux/Unix commands...
	foreach item {workingset workingsetpeak elapsedtime} {
	    set MonitoringInfo($item) 0   
	}
    }    

    #from here only asked variables are get and returned

    #######################
    #preprocess information
    #######################
    
    foreach item {numpoints numlines numsurfaces numvolumes numdimensions} {
	if { [lsearch $request $item]!=-1 } { 
	    set MonitoringInfo($item) [GiD_Info Geometry $item]
	}
    }

    set ask_surface_type 0
    foreach item {nurbssurface plsurface coonsurface meshsurface} {
	if { [lsearch $request numsurfaces_$item]!=-1 } {
	    set ask_surface_type 1
	    break
	}
    }    
    if { $ask_surface_type && [GiD_Info Geometry numsurfaces] } {
	foreach num [GiD_Geometry list surface 1:end] {
	    incr count([lindex [GiD_Geometry get surface $num] 0])
	}
	foreach item {nurbssurface plsurface coonsurface meshsurface} {
	    if { [lsearch $request numsurfaces_$item]!=-1 } { 
		if { [catch { set MonitoringInfo(numsurfaces_$item) $count($item) } err] } {
		    array unset MonitoringInfo numsurfaces_$item
		}
	    }
	}
    }
    if { [GidUtils::VersionCmp 10.1.8d] >=0 } {  
	set item numsurfaces_unrendered
	if { [lsearch $request $item]!=-1 } { 
	    set MonitoringInfo($item) [llength [GiD_Geometry list surface unrendered]]
	}
    }
    foreach item {numnodes numelements} {
	if { [lsearch $request $item]!=-1 } { 
	    set MonitoringInfo($item) [GiD_Info Mesh $item]
	}
    }

    set ask_element_type 0
    foreach item {linear triangle quadrilateral tetrahedra hexahedra prism pyramid sphere circle point} {
	if { [lsearch $request numelements_$item]!=-1 } {     
	    set ask_element_type 1
	    break
	}
    }
    if { $ask_element_type && [GiD_Info Mesh numelements] } {
	foreach item {linear triangle quadrilateral tetrahedra hexahedra prism pyramid sphere circle point} {
	    if { [lsearch $request numelements_$item]!=-1 } { 
		if { [catch { set MonitoringInfo(numelements_$item) [GiD_Info Mesh numelements $item] } err] } {
		    array unset MonitoringInfo numelements_$item
		}
	    }
	}
    }

    foreach item {linear triangle quadrilateral tetrahedra hexahedra prism pyramid sphere circle point} {
	if { [lsearch $request minjacobian_$item]!=-1 } { 
	    if { [GiD_Info Mesh numelements $item] } {
		#ask also for its minimum jacobian quality
		if { [catch { set MonitoringInfo(minjacobian_$item) [lindex [GiD_Info MeshQuality ElemMinJacobian $item 0 0 1] end-1] } err] } {		    
		    array unset MonitoringInfo minjacobian_$item
		}		   		
	    }
	}
    }

    set item numlayers
    if { [lsearch $request $item]!=-1 } { 
	set MonitoringInfo($item) [llength [GiD_Info layers]]
    }
    set item numgroups
    if { [lsearch $request $item]!=-1 } { 
	set MonitoringInfo($item) [llength [GiD_Groups list]]
    }

    #######################
    #postproces information
    #######################

    foreach item {postnumnodes postnumelements} {
	if { [lsearch $request $item]!=-1 } { 
	    if { $item == "postnumnodes" } {
		set MonitoringInfo($item) [GiD_Info mesh -post numnodes]
	    } elseif { $item == "postnumelements" } {
		set MonitoringInfo($item) [GiD_Info mesh -post numelements]
	    }
	}
    }

    set ask_element_type 0
    foreach item {linear triangle quadrilateral tetrahedra hexahedra prism pyramid sphere point} {
	if { [lsearch $request postnumelements_$item]!=-1 } {     
	    set ask_element_type 1
	    break
	}
    }
    if { $ask_element_type && [GiD_Info Mesh -post numelements] } {
	foreach item {linear triangle quadrilateral tetrahedra hexahedra prism pyramid sphere point} {
	    if { [lsearch $request postnumelements_$item]!=-1 } { 
		if { [catch { set MonitoringInfo(postnumelements_$item) [GiD_Info Mesh -post numelements $item] } err] } {
		    array unset MonitoringInfo postnumelements_$item
		}
	    }
	}
    }
    
    foreach item {numanalysis numresultscurrentanalysis numstepscurrentanalysis stepcurrent} {
	if { [lsearch $request $item]!=-1 } { 
	    if { $item == "numanalysis" } {
		set MonitoringInfo($item) [llength [GiD_Info postprocess get all_analysis]]
	    } elseif { $item == "numresultscurrentanalysis" } {
		set MonitoringInfo($item) [llength [GiD_Info postprocess get cur_results_list Contour_Fill]]
	    } elseif { $item == "numstepscurrentanalysis" } {
		set MonitoringInfo($item) [llength [GiD_Info postprocess get all_steps [GiD_Info postprocess get cur_analysis]]]
	    } elseif { $item == "stepcurrent" } {
		set MonitoringInfo($item) [GiD_Info postprocess get cur_step]
	    }
	}
    }

    
    ### generic expr procedures  ###
    set cont 0
    foreach procedure $expr_procedures {
	incr cont
	if { [catch { set value [expr $procedure] } err] } {
	    set MonitoringInfo(expression-$cont) $err
	} else {
	    set MonitoringInfo(expression-$cont) $value
	}
    }

    set res ""
    foreach item [array names MonitoringInfo] {
	lappend res $item $MonitoringInfo($item)
    }
    return $res
}

proc gidmonitoring::saveinfo { filename } {
    set fp [open $filename w]
    puts -nonewline $fp [gidmonitoring::getinfo]
    close $fp
}
