
proc ChangeBackgroundColourSync { wid } {
    if { $::ChangeVarInfo(BackColorType)} {
	# two colour background
	set old_colour $::ChangeVarInfo(BackColorTop)
    } else {
	# single colour background
	set old_colour $::ChangeVarInfo(BackgroundColor)
    }

    set w [ winfo toplevel $wid].change_col
    
    set new_colour [ GIDChooseColor $w -title [_ "Select colour"] -color $old_colour]
    if { [ string length $new_colour]} {
	set ::ChangeVarInfo(BackColorTop) $new_colour
	set ::ChangeVarInfo(BackgroundColor) $new_colour
	$wid configure -background $new_colour
    }
}

proc ChangeColorFondoGiD { idx_var { actualiza_color 0}} {
    global ChangeVarInfo

    set wb $ChangeVarInfo($idx_var,wid)
    if { ![ winfo exists $wb]} {
	return
    }
    if { $actualiza_color} {
	$wb configure -background $ChangeVarInfo($idx_var)
    } else {
	set nuevo [ GIDChooseColor $wb.mc -title [_ "Select colour"] -color $ChangeVarInfo($idx_var)]
	if { [ string length $nuevo]} {
	    set ChangeVarInfo($idx_var) $nuevo
	    $wb configure -background $nuevo
	}
    }
}

proc ChangeColorSenseFondoGiD { sentido} {
    global ChangeVarInfo

    set ChangeVarInfo(BackColorSense) $sentido
    set wb $ChangeVarInfo(BackColorSense,wid)
    if { ![ winfo exists $wb]} {
	return
    }
    $wb configure -image $ChangeVarInfo(BackColorSense,$sentido)

    $ChangeVarInfo(BackColorSense,TxtArriba,wid) configure -text $ChangeVarInfo(BackColorSense,TxtArriba,$sentido)

    $ChangeVarInfo(BackColorSense,TxtAbajo,wid) configure -text $ChangeVarInfo(BackColorSense,TxtAbajo,$sentido)
}

proc ChangeColorEnableDisableWidgets { } {
    global ChangeVarInfo

    if { ![ info exists ChangeVarInfo(BackColorType,lst_wid)]} { return}
    foreach wb $ChangeVarInfo(BackColorType,lst_wid) {
	if { ![ winfo exists $wb]} {
	    continue
	}
	if { [ winfo class $wb] == "Label"} {
	    if { $ChangeVarInfo(BackColorType)} {
		# two colour background
		$wb configure -foreground black
	    } else {
		# single colour background
		$wb configure -foreground grey60
	    }
	} else {
	    if { $ChangeVarInfo(BackColorType)} {
		# two colour background
		$wb configure -state normal
	    } else {
		# single colour background
		$wb configure -state disabled
	    }
	}
    }

    set label_id $::ChangeVarInfo(BackColorSense,TxtArriba,wid)
    set button_id $::ChangeVarInfo(BackColorTop,wid)

    if { ![ info exists ::ChangeVarInfo(BackgroundColor)]} {
	set ::ChangeVarInfo(BackgroundColor) [ GiDColorToTkColor [ GiD_Set BackgroundColor]]
    }
    if { $::ChangeVarInfo(BackColorType)} {
	# two colour background
	$label_id configure -foreground black
	if { [ info exists ::ChangeVarInfo(BackColorSense,TxtArriba,old_text)]} {
	    $label_id configure -text $::ChangeVarInfo(BackColorSense,TxtArriba,old_text)
	} else {
	    $label_id configure -text [_ "Top"]:
	}
	$button_id configure -state normal -command [ list ChangeBackgroundColourSync $button_id]

	# make sure both bg colors are the same:
	set ::ChangeVarInfo(BackgroundColor) $::ChangeVarInfo(BackColorTop)
    } else {
	# single colour background
	$label_id configure -foreground black
	set ::ChangeVarInfo(BackColorSense,TxtArriba,old_text) [ $label_id cget -text]
	$label_id configure -text [_ "Background colour"]:
	$button_id configure -state normal -command [ list ChangeBackgroundColourSync $button_id]

	# make sure both bg colors are the same:
	set ::ChangeVarInfo(BackColorTop) $::ChangeVarInfo(BackgroundColor)
    }
    # actualize button's colour
    ChangeColorFondoGiD BackColorTop 1
}

proc PopUpRestard { w } {
    set retval [tk_dialogRAM $w.restartgidavdice [_ "Warning"] \
    [_ "In order to change GiD theme, GiD needs to be reinitiated. Restart GiD?"] \
    warning 0 [_ "Ok"] [_ "Cancel"]]    
    if { $retval == 0 } {     
	GiD_Set Theme_Configured 0	
	# if we are in Post mode, change to pre      
	if { [GetCurrentPrePostMode] == "POST" } {
	    GiD_Process escape escape escape escape Preprocess
	}
	# wait at max. 15 seconds to solve for all issues: in post, save changes, ....
	# if not, abort restart
	set ::GidPriv(restartMaxAttempts) 15
	set ::GidPriv(restartCurrentProject) [GiD_Info Project ModelName]
	checkIfInPreAndRestart    
    }    
}

proc pref_graphical_ui {base_base args} {
    global ChangeVarInfo GidPriv

    set c [ CreateScrolledCanvas $base_base.sc]
    $base_base.sc configure -relief flat
    set base [ ttk::frame $c.f]

    ttk::labelframe $base.fgraphicalappearence -text [_ "Graphical appearence"]
    ttk::labelframe $base.fgraphicalsystem -text [_ "Graphical system"]
    
    ttk::labelframe $base.fsoftwareopengl -text [_ "OpenGL options"]
    ttk::frame $base.frame#3 -borderwidth 2 -style ridge.TFrame
    ttk::frame $base.frmBackgroundColor
    ttk::labelframe $base.frame#1 -text [_ "Render precision"]
    #ttk::labelframe $base.frame#1 -text [_ "Curve precision"]
    #ttk::labelframe $base.frame#1b -text [_ "Surface precision (Render)"]
	if { $::GidPriv(ThemeSelection) == "1" } { 
		ttk::labelframe $base.frame#top -text [_ "GiD Theme"]
		ttk::button $base.themechange -text [_ "Change GiD Theme"] \
		-command "PopUpRestard $base"

		grid $base.themechange -in $base.frame#top -row 1 -column 1 -sticky nwes
		grid $base.frame#top  -columnspan 2 -padx 2 -sticky ew  
    }
	
    ttk::radiobutton $base.fsoftwareopengl.rd1 -text [_ "Fast visualization mode."] \
	-value 0 -variable ::ChangeVarInfo(SoftwareOpenGL)
    setTooltip $base.fsoftwareopengl.rd1 \
	[_ "Models will be drawn quicker but some issues could appear. 
I'm confident about my Graphic Card and I want to use its capabilities."]
    
    ttk::radiobutton $base.fsoftwareopengl.rd2 -text [_ "Safe visualization mode."] \
	-value 1 -variable ::ChangeVarInfo(SoftwareOpenGL)
    setTooltip $base.fsoftwareopengl.rd2 \
	[_ "Models will be drawn slower but the artifacts would disappear. 
I'm experiencing troubles with my Graphic Card and I want to use OpenGL by software."]

    #"Emulate Front Buffer ( dynamic lines hack)"
    if { ![info exists ChangeVarInfo(OGL_emulateFrontBuffer)] } { set ChangeVarInfo(OGL_emulateFrontBuffer) 0 }
    ttk::checkbutton $base.fsoftwareopengl.cbEmulateFB -text [_ "Selection lines by software (emulateFrontBuffer)."] \
	-variable ChangeVarInfo(OGL_emulateFrontBuffer)
    setTooltip $base.fsoftwareopengl.cbEmulateFB \
[_ "On some graphic cards, when a selection is done or a line is created, 
the used dynamic lines are drawn with problems, or are not drawn at all 
( for instance, on Intel GMA graphic cards).
To avoid this problem select this option which does not use the 
OpenGL FrontBuffer to drawn these kind of lines."]

    grid $base.fsoftwareopengl.rd1 $base.fsoftwareopengl.rd2 -sticky ew -padx 2
    grid $base.fsoftwareopengl.cbEmulateFB -sticky w -padx 2 -columnspan 2


    # on Mac OS X, if the software renderer is selected, better to use emulateFrontBuffer too
    # because there are some problems with dynamic lines
    if { $::tcl_platform(os) == "Darwin"} {
	$base.fsoftwareopengl.rd1 configure -command {
	    set ::ChangeVarInfo(OGL_emulateFrontBuffer) $::ChangeVarInfo(SoftwareOpenGL)
	}
	$base.fsoftwareopengl.rd2 configure -command {
	    set ::ChangeVarInfo(OGL_emulateFrontBuffer) $::ChangeVarInfo(SoftwareOpenGL)
	}
    }
	if { ![info exists ChangeVarInfo(Lightsmoothedelements)] } { set ChangeVarInfo(Lightsmoothedelements) 0 }
    ttk::checkbutton $base.checkbutton#1 -text [_ "Smoothed elems(deg)"]: \
	-variable ChangeVarInfo(Lightsmoothedelements)

    ttk::entry $base.entry#1 -textvariable ChangeVarInfo(AngleSmoothed) -width 8

    GidHelp "$base.checkbutton#1 $base.entry#1" [_ "If set and when rendering\
	    a mesh, the intersection between elements with a small angle\
	    between their normals will be illuminated as if it were\
	    a continuous solid. If not, illumination is made considering\
	    every element as planar.\
	    \nThe angle (in degrees) is the maximum angle between normals\
	    of two elements to allow smooth lighting between them.\
	    If not, one edge is drawn between them."]

    ttk::label $base.label#35 -text [_ "Surface drawing type"]: -justify left

    TTKComboBox $base.menubutton#1 -textvariable ::ChangeVarInfo(DrawSurfaceMode) \
	-labels [ list [_ "None"] [_ "Boundary lines"] [_ "Isoparametric lines"] [_ "Both modes"]] \
	-values {None "Boundary lines" "Isoparametric lines" "Both modes"} \
	-state readonly

    GidHelp "$base.label#35 $base.menubutton#1" [_ "Lets the user choose how to\
	    draw the surfaces when in wireframe (normal) mode. Options are:\
	    \n'None': Surfaces are not drawn.\
	    \n'Boundary lines': One magenta line is drawn for every\
	    contour line. This set of lines have a small offset towards\
	    the interior of the surface.\
	    \n'Isoparametric lines': Two yellow lines are drawn for every\
	    NURBS surface. The one for 'u=0.5' and the one for 'v=0.5'.\
	    \n'Both': Draws both modes."]

    ttk::label $base.labelQuadratic -text [_ "Draw quadratic elements"]: -justify left
    TTKComboBox $base.menubuttonQuadratic -textvariable ::ChangeVarInfo(DrawQuadratic) \
	-labels [ list [_ "No#C#Nothing, None"] [_ "As lines"] [_ "As curves"]] \
	-values {No AsLines AsCurve} \
	-state readonly
    GidHelp "$base.labelQuadratic $base.menubuttonQuadratic"  [_ "With this option \
	    user can specify how to visualize quadratic elements: as linear (No), as \
	    quadratic but with straight edges between element nodes (As lines), or \
	    as quadratic curved edges (As curves)."]
    ttk::label $base.labelLabelOptions -text [_ "Entity labels show"]: -justify left
    TTKComboBox $base.menubuttonLabelOptions -textvariable ::ChangeVarInfo(LabelOptions) \
	-labels [ list [_ "Num"] [_ "Layer"] [_ "Num & layer"]] \
	-values {1 2 3} \
	-state readonly
    GidHelp "$base.labelLabelOptions $base.menubuttonLabelOptions"  [_ "With this option \
	    user can specify the information showed by entity labels: entity number and/or its layer name."]

    global MenuColorsForceChecked
    set MenuColorsForceChecked 1

    ttk::menubutton $base.menuEntityColor \
	-menu $base.menuEntityColor.m -image [gid_themes::GetImage color.png small_icons]
    
    ttk::label $base.labelEntityColor -text [_ "Entities colour"]:
    menu $base.menuEntityColor.m -tearoff 0
    set idx -1
    # $base.menuEntityColor.m add checkbutton -label [_ "Background"]... \
    #     -variable MenuColorsForceChecked \
    #     -command "SelectColorEntity BackgroundColor [ incr idx] ; set MenuColorsForceChecked 1"
    $base.menuEntityColor.m add checkbutton -label [_ "Points"]... \
	-variable MenuColorsForceChecked \
	-command "SelectColorEntity ColorPoints [ incr idx] ; set MenuColorsForceChecked 1"
    $base.menuEntityColor.m add checkbutton -label [_ "Lines"]... \
	-variable MenuColorsForceChecked \
	-command "SelectColorEntity ColorLines [ incr idx] ; set MenuColorsForceChecked 1"
    $base.menuEntityColor.m add checkbutton -label [_ "PolyLines"]... \
	-variable MenuColorsForceChecked \
	-command "SelectColorEntity ColorPolyLines [ incr idx] ; set MenuColorsForceChecked 1"
    $base.menuEntityColor.m add checkbutton -label [_ "Surfaces"]... \
	-variable MenuColorsForceChecked \
	-command "SelectColorEntity ColorSurfaces [ incr idx] ; set MenuColorsForceChecked 1"
    $base.menuEntityColor.m add checkbutton -label [_ "Surfaces Iso"]... \
	-variable MenuColorsForceChecked \
	-command "SelectColorEntity ColorSurfsIsoparametric [ incr idx] ; set MenuColorsForceChecked 1"
    $base.menuEntityColor.m add checkbutton -label [_ "Volumes"]... \
	-variable MenuColorsForceChecked \
	-command "SelectColorEntity ColorVolumes [ incr idx] ; set MenuColorsForceChecked 1"
    $base.menuEntityColor.m add checkbutton -label [_ "Nodes"]... \
	-variable MenuColorsForceChecked \
	-command "SelectColorEntity ColorNodes [ incr idx] ; set MenuColorsForceChecked 1"
    $base.menuEntityColor.m add checkbutton -label [_ "Elements"]... \
	-variable MenuColorsForceChecked \
	-command "SelectColorEntity ColorElements [ incr idx] ; set MenuColorsForceChecked 1"

    GidHelp "$base.menuEntityColor" \
	[_ "This option set the default colour of the geometrical and mesh entities."]

    ttk::frame $base.tcg
    if { ![info exists ChangeVarInfo(BackColorType)] } { set ChangeVarInfo(BackColorType) 0 }
    ttk::checkbutton $base.tcg.checkbutton#4b -text [_ "Two colours background"] \
	-variable ChangeVarInfo(BackColorType) \
	-command ChangeColorEnableDisableWidgets

    GidHelp "$base.tcg.checkbutton#4b" \
	[_ "Shows the background with a two colours degradation"]

    ttk::label $base.tcg.lArriba -text [_ "Top"]:
    tk::button $base.tcg.colorArriba -width 3 -command {ChangeColorFondoGiD BackColorTop}
    set ChangeVarInfo(BackColorTop,wid) $base.tcg.colorArriba

    ttk::button $base.tcg.intercambio \
	-image [gid_themes::GetImage "ArrowLeftRight.png" large_icons] \
	-command {
	set tmp $::ChangeVarInfo(BackColorTop)
	set ::ChangeVarInfo(BackColorTop) $::ChangeVarInfo(BackColorBottom)
	set ::ChangeVarInfo(BackColorBottom) $tmp
	ChangeColorFondoGiD BackColorTop 1
	ChangeColorFondoGiD BackColorBottom 1
    }
    # -borderwidth 0

    ttk::label $base.tcg.lAbajo -text [_ "Bottom"]:
    tk::button $base.tcg.colorAbajo -width 3 -command {ChangeColorFondoGiD BackColorBottom}
    set ChangeVarInfo(BackColorBottom,wid) $base.tcg.colorAbajo



    ttk::label $base.tcg.sentido -text [_ "Sense"]:
    ttk::menubutton $base.tcg.cbs -menu $base.tcg.cbs.m
    menu $base.tcg.cbs.m -tearoff no -borderwidth 0 -activeborderwidth 0
    set ChangeVarInfo(BackColorSense,wid) $base.tcg.cbs
    set ChangeVarInfo(BackColorSense,TxtArriba,wid) $base.tcg.lArriba
    set ChangeVarInfo(BackColorSense,TxtAbajo,wid) $base.tcg.lAbajo

    set ChangeVarInfo(BackColorType,lst_wid) [ list $base.tcg.lArriba $base.tcg.colorArriba \
	    $base.tcg.lAbajo $base.tcg.colorAbajo \
	    $base.tcg.intercambio \
	    $base.tcg.sentido $base.tcg.cbs \
	    ]

    set primera 0
    foreach img "ArrowDown.png ArrowUp.png ArrowRight.png ArrowLeft.png ArrowUpLeft.png ArrowUpRight.png ArrowDownRight.png ArrowDownLeft.png" \
	val "d u r l ul ur dr dl" \
	txtArriba [list [_ "Top"]: [_ "Bottom"]: [_ "Left"]: [_ "Right"]: \
	    [_ "Bottom right"]: [_ "Bottom left"]: [_ "Top left"]: [_ "Top right"]:] \
	txtAbajo  [list [_ "Bottom"]: [_ "Top"]: [_ "Right"]: [_ "Left"]: \
	    [_ "Top left"]: [_ "Top right"]: [_ "Bottom right"]: [_ "Bottom left"]:] {
	set img_id [gid_themes::GetImage $img small_icons]
	$base.tcg.cbs.m add command -image $img_id -hidemargin 1 -command [list ChangeColorSenseFondoGiD $val]
	if { !$primera} {
	    $base.tcg.cbs configure -image $img_id
	    set primera 1
	}
	set ChangeVarInfo(BackColorSense,$val) $img_id
	set ChangeVarInfo(BackColorSense,TxtArriba,$val) $txtArriba
	set ChangeVarInfo(BackColorSense,TxtAbajo,$val) $txtAbajo
    }

    grid $base.tcg.checkbutton\#4b -columnspan 5
    grid $base.tcg.lArriba $base.tcg.colorArriba $base.tcg.intercambio \
	$base.tcg.lAbajo $base.tcg.colorAbajo \
	$base.tcg.sentido $base.tcg.cbs -padx 2
   
    if { ![info exists ::ChangeVarInfo(RenderPrecision)] } {
	set ::ChangeVarInfo(RenderPrecision) 0        
    }
    gidscale $base.scale#4 -variable ::ChangeVarInfo(RenderPrecision) \
	-from 0.0 -to 1.0 -resolution 0.1 -orient horizontal
    
    GidHelp "$base.frame#1 $base.scale#4" \
	[_ "This option gives the precision used to\
	    draw curves and surfaces in render mode"]

    ttk::frame $base.rotation
	if { ![info exists ChangeVarInfo(FastRotation)] } { set ChangeVarInfo(FastRotation) 0 }
    ttk::checkbutton $base.rotation.checkbutton#2 -text [_ "Fast rotation"] \
	-variable ChangeVarInfo(FastRotation)

    GidHelp $base.rotation.checkbutton#2 [_ "When 'Fast rotation' is set, drawing will be\
	    faster for both the geometry and the mesh when rotating the\
	    model. The way of increasing speed depends on the options."]

    if { ![info exists ChangeVarInfo(AutomaticRotationCenter)] } { set ChangeVarInfo(AutomaticRotationCenter) 0 }
    ttk::checkbutton $base.rotation.checkbutton#22 -text [_ "Automatic rotation center"] \
	-variable ChangeVarInfo(AutomaticRotationCenter)

    GidHelp $base.rotation.checkbutton#22 [_ "When 'Automatic rotation center' is set,\
	    the nearest geometrical/mesh point to the screen center will be used\
	    as rotation center after each 'Zoom In' or 'Zoom Out'."]

    grid $base.rotation.checkbutton#2 $base.rotation.checkbutton#22 -padx 2

    #set pp [ $base cget -background]

    if { ![info exists ChangeVarInfo(FRPoints)] } { set ChangeVarInfo(FRPoints) 0 }
    ttk::checkbutton $base.checkbutton\#15 -image [gid_themes::GetImage "point.png" small_icons] \
	-style Toolbutton \
	-variable ChangeVarInfo(FRPoints)
	# -borderwidth 1 -selectcolor $pp

    if { ![info exists ChangeVarInfo(FRLines)] } { set ChangeVarInfo(FRLines) 0 }
    ttk::checkbutton $base.checkbutton\#13 \
	-image [gid_themes::GetImage "line.png" small_icons] \
	-style Toolbutton \
	-variable ChangeVarInfo(FRLines)
	# -borderwidth 1 -selectcolor $pp
	#-disabledforeground grey60

	if { ![info exists ChangeVarInfo(FRSurfaces)] } { set ChangeVarInfo(FRSurfaces) 0 }
    ttk::checkbutton $base.checkbutton\#7 \
	-image [gid_themes::GetImage "surface.png" small_icons] \
	-style Toolbutton \
	-variable ChangeVarInfo(FRSurfaces)
	# -borderwidth 1 -selectcolor $pp
	#-disabledforeground grey60

	if { ![info exists ChangeVarInfo(FRVolumes)] } { set ChangeVarInfo(FRVolumes) 0 }
    ttk::checkbutton $base.checkbutton\#11 \
    -image [gid_themes::GetImage "volume.png" small_icons] \
	-style Toolbutton \
	-variable ChangeVarInfo(FRVolumes)
	# -borderwidth 1 -selectcolor $pp
	#-disabledforeground grey60

    GidHelp "$base.checkbutton\#15 $base.checkbutton#13 $base.checkbutton#7 $base.checkbutton\#11" \
	[_ "To draw or not to draw this type of entity when rotating.
In Postprocess: 'lines' will drawn boundary lines only, 'surfaces' will drawn a simplified mesh of the model."]

	if { ![info exists ChangeVarInfo(AlwaysGeom)] } { set ChangeVarInfo(AlwaysGeom) 0 }
    ttk::checkbutton $base.checkbutton\#10 \
	-text [_ "Always geometry"] \
	-variable ChangeVarInfo(AlwaysGeom)
	#-disabledforeground grey60

    GidHelp $base.checkbutton\#10 \
	[_ "If set, when viewing the mesh and rotating, the geometry is drawn instead."]

	if { ![info exists ChangeVarInfo(DrawGraphicObjects)] } { set ChangeVarInfo(DrawGraphicObjects) 0 }
    ttk::checkbutton $base.checkbutton\#16 \
	-text [_ "Graphic objects"] \
	-variable ChangeVarInfo(DrawGraphicObjects)
	#-disabledforeground grey60

    GidHelp $base.checkbutton\#16 [_ "If not set, when rotating the geometry,\
	    some graphical and temporal objects like normals or materials\
	    or conditions symbols are not drawn. "]

    

    set opengl_version [drawopengl draw -getstring version]
    if { [scan $opengl_version %d.%d opengl_major opengl_minor] == 2 } {
	set opengl_num ${opengl_major}.${opengl_minor}
    } else {
	set opengl_num 1.0
    }
    
    ttk::labelframe $base.fOGL -text [_ "Drawing method"]

    
    set labels ""
    set values ""
    foreach txt [list [_ "Vertex Buffer Objects"] [_ "Vertex Arrays"] [_ "Display lists"] [_ "Immediate mode"]] \
	    req [list 1.5 1.1 1.0 1.0] \
	    idx [ list 3 2 1 0] {
		# may be internal variable is set to other values
	# if { $req <= $opengl_num } {
	    lappend labels $txt
	    lappend values $idx
	# }        
    }        
    ttk::frame $base.fOGL.rd
    ttk::label $base.fOGL.rd.lbl -text [_ "Method"]:
    TTKComboBox $base.fOGL.rd.cb -textvariable ::ChangeVarInfo(OGL_useMeshObjects) \
	-labels $labels -values $values -state readonly
	
    setTooltip $base.fOGL.rd \
	[_ "Different methods to draw the mesh on the screen, from fastest to slowest.
	    'Vertex Buffer Objects' require set 'Fast visualization mode' and OpenGL 1.5 or higher,
	    and 'Vertex Arrays' require OpenGL 1.1"]
    
    grid $base.fOGL.rd.lbl $base.fOGL.rd.cb -sticky ew
    grid $base.fOGL.rd -sticky nsew
    grid columnconfigure $base.fOGL.rd {0 1} -weight 1    

    # init variable....
    # ttk::checkbutton $base.fOGL.cb_pre_smooth -text [_ "Render preprocess mesh with smoothed normals"] \
    #         -onvalue 1 -offvalue 0 -variable ::ChangeVarInfo(MeshPreRenderSmooth) \
    #         -command {
    #             if { ( $::ChangeVarInfo(MeshPreRenderSmooth) == 1) && 
    #                  ( ( $::ChangeVarInfo(OGL_usePreMeshObjects) == 0) || 
    #                    ( $::ChangeVarInfo(OGL_usePreMeshObjects) == 1))} {
    #                 # if using inmediate or Display listsm change to VA
    #                 set ::ChangeVarInfo(OGL_usePreMeshObjects) 2
    #             }
    #         }
    # 
    # grid $base.fOGL.cb_pre_smooth -sticky w -pady 4 -padx 4
    # setTooltip $base.fOGL.cb_pre_smooth \
    #     [_ "Draws the preprocess mesh using smoothed normals at the vertices of the mesh. Smoothed normals only works with 'Vertex Arrays' or 'Vertex Buffer Objects' in preprocess."]

    if { ![info exists ::ChangeVarInfo(MeshPreColourByLayer)] } { set ::ChangeVarInfo(MeshPreColourByLayer) 0 }
    ttk::checkbutton $base.fOGL.cb_pre_col_by_lay -text [_ "Render normal: preprocess mesh coloured by layer."] \
	    -onvalue 1 -offvalue 0 -variable ::ChangeVarInfo(MeshPreColourByLayer) 

    grid $base.fOGL.cb_pre_col_by_lay -sticky w -pady 4 -padx 4
    setTooltip $base.fOGL.cb_pre_col_by_lay \
	[_ "Draws the preprocess mesh wiht a single colour or coloured by layers.\n
	    Coloring by layers only works with 'Vertex Arrays' or 'Vertex Buffer Objects' in preprocess."]

    # SphereDetailPre values are 1 = INTERNAL TEXTURES or 2 = Nice Mesh Spheres
    # there will be perhaps a third mode 0 = As Points
    if { ![info exists ::ChangeVarInfo(SphereDetailPre)] } { set ::ChangeVarInfo(SphereDetailPre) 1 }
    ttk::checkbutton $base.fOGL.cb_pre_sphere -text [_ "Use textures for spheres in preprocess mesh"] \
        -onvalue 1 -offvalue 2 -variable ::ChangeVarInfo(SphereDetailPre)

    grid $base.fOGL.cb_pre_sphere -sticky w -pady 4 -padx 4
    setTooltip $base.fOGL.cb_pre_sphere -text \
	[_ "Draw spheres in preprocess using textures for a faster drawing.
Disabling this option, sphere will be drawn nicely using triangle meshes,
but depending on the render precision, EACH sphere will use between 4 and 630 triangles."]

    if { ![info exists ::ChangeVarInfo(OGL_useCFillTexture)] } { set ::ChangeVarInfo(OGL_useCFillTexture) 0 }
    ttk::checkbutton $base.fOGL.cb_cfill -text [_ "Use textures for contour fills"] \
	    -onvalue 1 -offvalue 0 -variable ::ChangeVarInfo(OGL_useCFillTexture) \
	    -command {
		if { ( $::ChangeVarInfo(OGL_useCFillTexture) == 0) && 
		     ( ( $::ChangeVarInfo(OGL_useMeshObjects) == 2) || 
		       ( $::ChangeVarInfo(OGL_useMeshObjects) == 3))} {
		    # ifusing VA or VBO, set it to Display lists
		    set ::ChangeVarInfo(OGL_useMeshObjects) 1
		}
	    }

    grid $base.fOGL.cb_cfill -sticky w -pady 4 -padx 4
    setTooltip $base.fOGL.cb_cfill -text \
	[_ "Faster but bit less acurate contour fill. 
Uses an image to draw the colours of the contour fills instead of 
subdividing each element and draw each part with a single colour. 
Vertex Arrays' and 'Vertex Buffer Objects' only works with Contour fill with textures."]
    
    
    # Geometry management
    grid $base.fgraphicalappearence -sticky ew -padx 2
    grid columnconfigure $base.fgraphicalappearence 1 -weight 1
    grid $base.fgraphicalsystem -sticky ew -columnspan 2 -padx 2
    grid columnconfigure $base.fgraphicalsystem 0 -weight 1
    
    grid $base.scale#4 -in $base.frame#1 -sticky ew


    grid $base.labelEntityColor -in $base.frmBackgroundColor -row 1 -column 0
    grid $base.menuEntityColor -in $base.frmBackgroundColor -row 1 -column 1
    grid $base.tcg -in $base.frmBackgroundColor -row 3 -column 0 -columnspan 3

    grid $base.checkbutton#15 $base.checkbutton#13 $base.checkbutton#10 -in $base.frame#3 -sticky w
    grid $base.checkbutton#7 $base.checkbutton#11 $base.checkbutton#16 -in $base.frame#3 -sticky w
    
    grid $base.checkbutton#1 $base.entry#1 -in $base.fgraphicalappearence -sticky w -padx 2
    grid configure $base.entry#1 -sticky ew
    grid $base.label#35 $base.menubutton#1 -in $base.fgraphicalappearence -sticky w -padx 2
    grid configure $base.menubutton#1 -sticky ew
    grid $base.labelQuadratic $base.menubuttonQuadratic -in $base.fgraphicalappearence -sticky w -padx 2
    grid configure $base.menubuttonQuadratic -sticky ew
    grid $base.labelLabelOptions $base.menubuttonLabelOptions -in $base.fgraphicalappearence -sticky w -padx 2
    grid configure $base.menubuttonLabelOptions -sticky ew
    grid $base.frmBackgroundColor -in $base.fgraphicalappearence -columnspan 2 -padx 2 -sticky w
    grid $base.frame#1 -in $base.fgraphicalappearence -columnspan 2 -padx 2 -sticky ew
#     grid $base.frame#1b -in $base.fgraphicalappearence -columnspan 2 -padx 2 -sticky ew
    grid $base.rotation -in $base.fgraphicalappearence -columnspan 2 -padx 2 -sticky w
    grid $base.frame#3 -in $base.fgraphicalappearence -sticky w
    
    grid $base.fsoftwareopengl -in $base.fgraphicalsystem -sticky ew -columnspan 2 -padx 2
    grid columnconfigure $base.fsoftwareopengl "0 1" -weight 1
    
    grid $base.fOGL -in $base.fgraphicalsystem -sticky nsew -padx 2
    grid columnconfigure $base.fOGL 0 -weight 1
    
    
    # Resize behavior management
    grid columnconfigure $base.frame#3 3 -weight 1

    grid columnconfigure $base 0 -weight 1

    grid columnconfigure $base.frame#1 0 -weight 1
    #grid columnconfigure $base.frame#1b 2 -weight 1

    AddToScrolledCanvas $base_base.sc $c.f
    grid $base_base.sc -sticky news
    grid rowconfigure $base_base 0 -weight 1
    grid columnconfigure $base_base 0 -weight 1

    set ChangeVarInfo(PG) $base
    set ChangeVarInfo(FRFrame) $base.frame#3

    ChangeColorEnableDisableWidgets

    trace variable ChangeVarInfo(FastRotation) w ChangeFastRotation

    trace variable ChangeVarInfo(BackgroundColor) w ChangeColor
    trace variable ChangeVarInfo(ColorPoints) w ChangeColor
    trace variable ChangeVarInfo(ColorLines) w ChangeColor
    trace variable ChangeVarInfo(ColorPolyLines) w ChangeColor
    trace variable ChangeVarInfo(ColorSurfaces) w ChangeColor
    trace variable ChangeVarInfo(ColorSurfsIsoparametric) w ChangeColor
    trace variable ChangeVarInfo(ColorVolumes) w ChangeColor
    trace variable ChangeVarInfo(ColorNodes) w ChangeColor
    trace variable ChangeVarInfo(ColorElements) w ChangeColor

    trace variable ChangeVarInfo(Lightsmoothedelements) w "LightsmoothedelementsProc ;#"   
}

proc LightsmoothedelementsProc {} {
    global ChangeVarInfo
    
    if { $ChangeVarInfo(Lightsmoothedelements) } {
	$ChangeVarInfo(PG).entry#1 configure -state normal
    } else {
	$ChangeVarInfo(PG).entry#1 configure -state disabled
    }
}

proc ChangeFastRotation {name1 name2 op} {
    global ChangeVarInfo
    
    set slaves [grid slaves $ChangeVarInfo(FRFrame)]
    foreach i $slaves {
	set slaves [concat $slaves [grid slaves $i]]
    }
    
    foreach i $slaves {
	if { $ChangeVarInfo(FastRotation) == 0 } {
	    catch {
		$i configure -state disabled
		if { [winfo class $i] == "Scale" } { $i configure -foreground grey60 }
	    }
	} else {
	    catch {
		$i configure -state normal
		if { [winfo class $i] == "Scale" } { $i configure -foreground black }
	    }
	}
    }
}

proc selectEntityColor { entity } {
    global ChangeVarInfo
    
    # Select and Update interface
    #set retcol [tk_chooseColor -initialcolor $ChangeVarInfo($entity)]
    set retcol [GIDChooseColor $ChangeVarInfo(PG).selectcolor -title [_ "Select color"] -color $ChangeVarInfo($entity)]
    
    # Set preference
    if { $retcol != "" } {
	set  ChangeVarInfo($entity) $retcol
	set res 1
    } else {
	set res 0
    }
    
    return $res
}

proc ChangeColor  {name elem op} {
    global ChangeVarInfo
    set indice 0
    foreach i "ColorPoints ColorLines ColorPolyLines \
	ColorSurfaces ColorSurfsIsoparametric ColorVolumes \
	ColorNodes ColorElements" {
	if [ info exists ChangeVarInfo($i) ] {
	    $ChangeVarInfo(PG).menuEntityColor.m  entryconfigure $indice -selectcolor $ChangeVarInfo($i)
	}
	incr indice
    }
}

proc SelectColorEntity { entity indice} {
    global ChangeVarInfo
    if { [selectEntityColor $entity] } {
	$ChangeVarInfo(PG).menuEntityColor.m  entryconfigure $indice -selectcolor $ChangeVarInfo($entity)
    }
}


