
proc ChangeGrowLaw { w args } {
    global ChangeVarInfo
    set canvas $w.canvas#1
    set sb $w.spinbox#1
    if { ![winfo exists $sb] } return
    switch $ChangeVarInfo(BoundaryLayer,GrowLaw) {
	0 {
	    $sb configure -from 1.0
	    set ChangeVarInfo(BoundaryLayer,GrowFactor) 2.0
	    GidHelp $w.menubutton\#30 [_ "This function is the geometric progression, represented by: h_i = h_0*(r^0+r^1+r^2++r^(i-1)\
		    , where 'h_i' is the height at level 'i' and 'r' is the grow factor"]
	}
	1 {
	    $sb configure -from 0.1
	    set ChangeVarInfo(BoundaryLayer,GrowFactor) 1.0
	    GidHelp $w.menubutton\#30 [_ "This function exponential, represented by: h_i = exp(r*i + ln(h_0))\
		    , where 'h_i' is the height at level 'i' and 'r' is the grow factor."]
	}
	2 {
	    $sb configure -from 0.1
	    set ChangeVarInfo(BoundaryLayer,GrowFactor) 1.0
	    GidHelp $w.menubutton\#30 [_ "This function is represented by: h_i = h_0 * (1 + i*(1+r*(1+r)*i)))\
		    , where 'h_i' is the height at level 'i' and 'r' is the grow factor. This function typically grows faster\
		    than the Geometric one."]
	}
    }

    RedrawDivisions_blm $canvas
}

proc ChangeGrowFactor { w args } {
    global ChangeVarInfo
    set canvas $w.canvas#1
    set sb $w.spinbox#1
    if { ![winfo exists $sb] } return
    switch $ChangeVarInfo(BoundaryLayer,GrowLaw) {
	0 {
	    if { $ChangeVarInfo(BoundaryLayer,GrowFactor) < 1 } {
		#$sb configure -background red
		$canvas delete temp
		return
	    }
	}
	1 {
	    if { $ChangeVarInfo(BoundaryLayer,GrowFactor) <= 0 } {
		#$sb configure -background red
		$canvas delete temp
		return
	    }
	}
	2 {
	    if { $ChangeVarInfo(BoundaryLayer,GrowFactor) <= 0 } {
		#$sb configure -background red
		$canvas delete temp
		return
	    }
	}
    }
    RedrawDivisions_blm $canvas
}

proc RedrawDivisions_blm { canvas args} {
    if { ![winfo exists $canvas] } {
	return
    }
	
    global ChangeVarInfo

    $canvas delete temp
    set maxwidth 300


    #First distance = 4 pixel
    set DistanceFirst 4

    set layerX [expr 2+$DistanceFirst]
    switch $ChangeVarInfo(BoundaryLayer,GrowLaw) {
	0 {
	    #geometric
	    set i 1
	    while { $layerX<$maxwidth } {
		$canvas create line $layerX 0 $layerX 8 -fill black -width 1  -tags temp
		if { $ChangeVarInfo(BoundaryLayer,GrowFactor) == 1 } {
		    set layerX [expr 2+round($DistanceFirst*($i+1)*$ChangeVarInfo(BoundaryLayer,GrowFactor))]
		} else {
		    set layerX [expr 2+round($DistanceFirst*(1-pow($ChangeVarInfo(BoundaryLayer,GrowFactor),$i+1))/(1-$ChangeVarInfo(BoundaryLayer,GrowFactor)))]
		}
		incr i
	    }
	}
	1 {
	    #exponential
	    set i 1
	    while { $layerX<$maxwidth } {
		$canvas create line $layerX 0 $layerX 8 -fill black -width 1  -tags temp
		set layerX [expr 2+round(exp(log($DistanceFirst)+($ChangeVarInfo(BoundaryLayer,GrowFactor)*$i)))]
		incr i
	    }
	}
	2 {
	    #geometric_mod
	    set i 1
	    while { $layerX<$maxwidth } {
		$canvas create line $layerX 0 $layerX 8 -fill black -width 1  -tags temp
		set layerX [expr 2+round($DistanceFirst*(1+$i*(1+$ChangeVarInfo(BoundaryLayer,GrowFactor)*(1+$ChangeVarInfo(BoundaryLayer,GrowFactor))*$i)))]
		incr i
	    }
	}
    }
}

proc pref_meshing_ui {base_base args} {
    global GidPriv ChangeVarInfo

    #set col_bg [ $base_base cget -background]

    set use_bwidget 0
    if { $use_bwidget } {        
	#more standard scrollbars are tk!!
	set sw [ScrolledWindow $base_base.sw -borderwidth 0]
	set sf [ScrollableFrame $base_base.f]
	set base [$sf getframe]
    } else {
	#the same without BWidget, with some special procs CreateScrolledCanvas, AddToScrolledCanvas,...
	set c [ CreateScrolledCanvas $base_base.sc]
	$base_base.sc configure -relief flat
	set base [ ttk::frame $c.f]
    }
    

    #set growlaw [GiD_Set BoundaryLayer(GrowLaw)]
    #set growfactor [GiD_Set BoundaryLayer(GrowFactor)]

    set frm_quadratic [ttk::labelframe $base.frm_quadratic -text [_ "Default quadratic type"]]
    ttk::labelframe $base.frame#2 -text [_ "Unstructured mesher"]
    ttk::labelframe $base.frame#4 -text [_ "Automatic correct sizes"]
    ttk::labelframe $base.frame#1 -text [_ "Unstructured size transitions"]
    ttk::labelframe $base.frame#8 -text [_ "Chordal error"]
    ttk::labelframe $base.frame#3 -text [_ "Structured mesh"]
    ttk::labelframe $base.frame#10 -text [_ "Boundary Layer Mesh"]
    ttk::labelframe $base.framerball -text [_ "Sphere mesher"]
    ttk::labelframe $base.framecartesian -text [_ "Cartesian mesher"]
    ttk::labelframe $base.frame#5 -text [_ "Others"]
    
    ttk::radiobutton $frm_quadratic.radiobutton#0 -text [_ "Normal"] \
	-variable ChangeVarInfo(Preferences,QuadraticType) -value 0    
    ttk::radiobutton $frm_quadratic.radiobutton#1 -text [_ "Quadratic"] \
	-variable ChangeVarInfo(Preferences,QuadraticType) -value 1    
    ttk::radiobutton $frm_quadratic.radiobutton#2 -text [_ "Quadratic9"] \
	-variable ChangeVarInfo(Preferences,QuadraticType) -value 2    
    
    set msg [_ "The same quadratic type is aplied to the whole mesh. It can be:\
	    \n'Normal': Linear elements are generated.\
	    \n'Quadratic': Quadratic elements are generated, with a node in the middle of each element edge.\
	    \n'Quadratic': The same as Quadratic, but in quadrilateral and hexahedra meshes, a quadratic node \
	    is placed in the center of the element."]
    append msg \n[_ "First time default setting. Does not modify actual mesh type."]
    append msg \n[_ "To set mesh type go to 'Mesh --> Quadratic type' at the top menu."]
    GidHelp "$frm_quadratic $frm_quadratic.radiobutton#0 \
	$frm_quadratic.radiobutton#1 $frm_quadratic.radiobutton#2" $msg           
    
    
    ttk::frame $base.frame#11
    ttk::label $base.label#75 -text [_ "Surface"]:       
    ttk::radiobutton $base.radiobutton#1 -text [_ "RFast"] \
	-variable ChangeVarInfo(SurfaceMesher) -value 0
    ttk::radiobutton $base.radiobutton#2 -text [_ "RSurf"] \
	-variable ChangeVarInfo(SurfaceMesher) -value 1
    ttk::radiobutton $base.radiobutton#4 -text [_ "RJump"] \
	-variable ChangeVarInfo(SurfaceMesher) -value 4

    GidHelp "$base.frame#11 $base.label#75 $base.radiobutton#1 \
	$base.radiobutton#2 $base.radiobutton#4 " \
	[_ "Surface meshers can be:\
	    \n'Rfast': The most efficient in speed and reliability. With\
	    deformed surfaces can give distorted elements.\
	    \n'Rsurf': Meshes are generated directly in the space.\
	    Quality is better but it is slower and can fail for distorted\
	    surfaces.\
	    \n'Rjump': Meshes are generated directly in the space, but trying to jump the lines between surfaces."]
		
    
    ttk::frame $base.frame#12
    ttk::label $base.label#76 -text [_ "Volume"]:
    #ChangeVarInfo(VolumeMesher) "Advancing front"=0 "Delaunay"=1 "Isosurface"=2 "OctTree"=3 "Tetgen"=4
    ttk::radiobutton $base.radiobutton#50 -text [_ "Advancing front"] -variable ChangeVarInfo(VolumeMesher) -value 0      
    ttk::radiobutton $base.radiobutton#51 -text "Tetgen" -variable ChangeVarInfo(VolumeMesher) -value 4
     ttk::radiobutton $base.radiobutton#52 -text "OctTree" -variable ChangeVarInfo(VolumeMesher) -value 3

    GidHelp "$base.frame#12 $base.label#76 $base.radiobutton#50 \
	$base.radiobutton#51 $base.radiobutton#52 " \
	[_ "Volume meshers can be:\
	    \n'Advancing front': Based in advancing front method. It gives good quality elements.\
	    \n'Tetgen': Based in Delaunay algorithm. It is faster, but some elements may be distorted. \
	    \n'OctTree': Based in Octree algorithm (cartesian grid with cutting patterns applied)."]
#     GidHelp "$base.frame#12 $base.label#76 $base.radiobutton#50 \
#         $base.radiobutton#51 " \
#         [_ "Volume meshers can be:\
#             \n'Advancing front': Based in advancing front method. It gives good quality elements.\
#             \n'Tetgen': Based in Delaunay algorithm. It is faster, but some elements may be distorted."]
    
    ttk::checkbutton $base.checkbutton#mixedmesh \
	-text [_ "Structured mesh in the inner part of volumes"] \
	-variable ChangeVarInfo(OctreeMesherForInnerPartOfVolumes)
    GidHelp $base.checkbutton#mixedmesh [_ "If this preference is set, the inner part of the volumes is meshed with a \
	    tetrahedra patterns from an octree, which often makes the meshing process faster."]
    
    ttk::radiobutton $base.radiobutton#6 -text [_ "None#C#AutomaticCorrectSizes"] \
	-variable ChangeVarInfo(AutomaticCorrectSizes) -value 0
    ttk::radiobutton $base.radiobutton#7 -text [_ "Normal"] \
	-variable ChangeVarInfo(AutomaticCorrectSizes) -value 1
    ttk::radiobutton $base.radiobutton#8 -text [_ "Hard#C#AutomaticCorrectSizes"] \
	-variable ChangeVarInfo(AutomaticCorrectSizes) -value 2

    GidHelp "$base.frame#4 $base.radiobutton#6 $base.radiobutton#7 $base.radiobutton#8 " \
	[_ "This preference let to make an automatic mesh sizes correction when meshing begins. \
	    If None is set, no sizes correction is made; if Normal is set, a sizes correction is made, according to \
	    the geometrical entities sizes, and to the compatibility between meshing sizes of neighbour entities; \
	    if Hard is set, the Normal correction is made and, furthermore, an automatic chordal error is \
	    applied to assign sizes to surfaces which are contour of volumes, so as to improve volume meshes."]

    if { ![info exists ChangeVarInfo(SizeTransitions)] } {
	set ChangeVarInfo(SizeTransitions) 0
    }

    gidscale $base.scale#4 -from 0.0 -to 1.0 -orient horizontal -resolution 0.1 \
	-variable ChangeVarInfo(SizeTransitions)
	
    GidHelp "$base.frame#1 $base.scale#4" [_ "It controls whether the transitions between\
	    different element sizes are slow (near to 0) or fast (near to 1)."]

    
    

    if { ![info exists ChangeVarInfo(BoundaryWeightedTransition)] } { set ChangeVarInfo(BoundaryWeightedTransition) 0 }
    ttk::checkbutton $base.checkbutton#12 \
	-text [_ "Regular transition near boundary"] \
	-variable ChangeVarInfo(BoundaryWeightedTransition)
    GidHelp $base.checkbutton#12 [_ "If this preference is set, sizes transition in unstructured\
	    mesh will be boundary weighted. Otherwise will be a more uniform sizes transition. "]

    if { ![info exists ChangeVarInfo(AllowAutomaticStructured)] } { set ChangeVarInfo(AllowAutomaticStructured) 0 }
    ttk::checkbutton $base.checkbutton#4 -text [_ "Allow automatic structured"] \
	-variable ChangeVarInfo(AllowAutomaticStructured)

    GidHelp $base.checkbutton#4 [_ "If this preference is set, functions\
	    like 'Assign sizes by Chordal Error' will define some surfaces\
	    as structured with highly distorted elements over them."]

    ttk::label $base.label#70 -text [_ "Symmetrical structured"]:

    if { ![info exists ChangeVarInfo(SymmetricalStructuredTriangles)] } { set ChangeVarInfo(SymmetricalStructuredTriangles) 0 }
    ttk::checkbutton $base.checkbutton#8 -text [_ "triangles"] \
	-variable ChangeVarInfo(SymmetricalStructuredTriangles)

    GidHelp $base.checkbutton#8 [_ "If this preference is set, structured\
	    triangles meshes will be topologically symmetrical. \
	    Four triangles will be created from each quadrilateral"]

    if { ![info exists ChangeVarInfo(SymmetricalStructuredTetrahedra)] } { set ChangeVarInfo(SymmetricalStructuredTetrahedra) 0 }
    ttk::checkbutton $base.checkbutton#9 -text [_ "tetrahedra"] \
	-variable ChangeVarInfo(SymmetricalStructuredTetrahedra)

    GidHelp $base.checkbutton#9 [_ "If this preference is set, structured\
	    tetrahedra meshes will be topologically symmetrical. \
	    Twenty four tetrahedra will be created from each hexahedron"]

    if { ![info exists ChangeVarInfo(BoundaryLayer,MeshInSpecificLayer)] } { set ChangeVarInfo(BoundaryLayer,MeshInSpecificLayer) 0 }
    ttk::checkbutton $base.checkbutton#15 \
	-text [_ "Boundary layer mesh in separated Layer"] \
	-variable ChangeVarInfo(BoundaryLayer,MeshInSpecificLayer)

    GidHelp $base.checkbutton#15 [_ "If this preference is set, the nodes and elements of boundary layer mesh\
	    will own to new Layers. These new Layers will have BLM_ as a prefix. "]

    ttk::frame $base.frame#25
    ttk::label $base.label#10 -text [_ "Stretching function"]:

    #ComboBoxRAM $base.menubutton#30 ChangeVarInfo(BoundaryLayer,GrowLaw) \
    #        [list [_ "Function 1 (Geometric)"] [_ "Function 2 (Exponential)"] [_ "Function 3"]] {"0" "1" "2"}
    if { ![info exists ::ChangeVarInfo(BoundaryLayer,GrowLaw) ] } {
	set ::ChangeVarInfo(BoundaryLayer,GrowLaw) "0"
    }
    [ TTKMB_CreateOrConfigure $base.menubutton#30] $base.menubutton#30 -textvariable ::ChangeVarInfo(BoundaryLayer,GrowLaw) \
	    -labels [list [_ "Function 1 (Geometric)"] [_ "Function 2 (Exponential)"] [_ "Function 3"]] \
	    -values {"0" "1" "2"}        

    ttk::label $base.label#30 -text [_ "Grow factor"]:
    ttk::spinbox $base.spinbox#1 -from 0 -to 1000 -increment 0.1 -width 6 \
	-textvariable ChangeVarInfo(BoundaryLayer,GrowFactor)
    #$base.spinbox#1 configure -text [format "%.2f" [GiD_Set BoundaryLayer(GrowFactor)]]
    GidHelp $base.spinbox#1 [_ "Factor applied to compute the layers"]

    canvas $base.canvas#1 -height 7 -width 300

    $base.canvas#1 create line 2 5 250 5 -fill red -width 2
    $base.canvas#1 create line 255 5 260 5 -fill red -width 2
    $base.canvas#1 create line 265 5 270 5 -fill red -width 2
    $base.canvas#1 create line 275 5 277 5 -fill red -width 2
    $base.canvas#1 create line 284 5 286 5 -fill red -width 2
    $base.canvas#1 create line 2 0 2 8 -fill black -width 1

    #ChangeGrowLaw $base
    #ChangeGrowFactor $base

    ttk::frame $base.frame#7
    ttk::label $base.label#8 -text [_ "Smoothing"]: 
    #ComboBoxRAM $base.menubutton#1 ChangeVarInfo(HighQualitySmoothing) \
    #        [list [_ "Normal"] [_ "HighAngle"] [_ "HighGeom"]] \
    #        {"Normal" "HighAngle" "HighGeom"}
    if { ![info exists ::ChangeVarInfo(HighQualitySmoothing) ] } {
	set ::ChangeVarInfo(HighQualitySmoothing) "Normal"
    }
    [ TTKMB_CreateOrConfigure $base.menubutton#1] $base.menubutton#1 -textvariable ::ChangeVarInfo(HighQualitySmoothing) \
	    -labels [list [_ "Normal"] [_ "HighAngle"] [_ "HighGeom"]] \
	    -values {"Normal" "HighAngle" "HighGeom"}

    GidHelp  "$base.label#8 $base.menubutton#1" [_ "Options for smoothing in order to enhance\
	    the mesh after the generation. Options are:\
	    \nNormal: only the standard smooth is performed\
	    \nHighAngle: an additional smooth with angle criteria is performed\
	    \nHighGeom: an additional smooth with chordal error criteria is performed"]

	
    ttk::frame $base.frame#22
    if { ![info exists ChangeVarInfo(MeshUntilEnd)] } { set ChangeVarInfo(MeshUntilEnd) 0 }
    ttk::checkbutton $base.checkbutton#1 -text [_ "Mesh until end"] \
	-variable ChangeVarInfo(MeshUntilEnd)
    GidHelp $base.checkbutton#1 [_ "If this preference is set, the mesh\
	    generator will continue until end although there are surfaces\
	    or volumes that cannot be meshed."]

	if { ![info exists ChangeVarInfo(NoMeshFrozenLayers)] } { set ChangeVarInfo(NoMeshFrozenLayers) 0 }
    ttk::checkbutton $base.checkbutton#3 -text [_ "No mesh frozen layers"] \
	-variable ChangeVarInfo(NoMeshFrozenLayers)
    GidHelp $base.checkbutton#3 [_ "If this preference is set, no mesh will\
	    be generated over entities that belong to frozen layers. "]

    if { ![info exists ChangeVarInfo(AvoidElementsInBoundary)] } { set ChangeVarInfo(AvoidElementsInBoundary) 0 }
    ttk::checkbutton $base.checkbutton#10 \
	-text [_ "Avoid elements with all its nodes in boundary"] \
	-variable ChangeVarInfo(AvoidElementsInBoundary)

    GidHelp $base.checkbutton#10 [_ "If this preference is set, the mesh won't have\
	    linear elements with all its nodes in boundary."]

    if { ![info exists ChangeVarInfo(AllowQuadTriMeshes)] } { set ChangeVarInfo(AllowQuadTriMeshes) 0 }
    ttk::checkbutton $base.checkbutton#16 \
	-text [_ "Allow quadrilateral dominant meshes"] \
	-variable ChangeVarInfo(AllowQuadTriMeshes)

    GidHelp $base.checkbutton#16 [_ "If this preference is set, the insertion of triangles\
	    is allowed when meshing unstructured quadrilateral meshes, in order to improve\
	    the quality of the elements."]
   
    ttk::frame $base.frame#20
    if { ![info exists ChangeVarInfo(ChordalError,ApplyChordalErrorToModel)] } { set ChangeVarInfo(ChordalError,ApplyChordalErrorToModel) 0 }
    ttk::checkbutton $base.checkbutton#11 \
	-text [_ "Maximum chordal error in model"]. \
	-variable ChangeVarInfo(ChordalError,ApplyChordalErrorToModel)
    GidHelp "$base.frame#8 $base.checkbutton#11 \
	$base.frame#20 " \
	[_ "This criteria is applied to all the model mesh, and tends \
	    \nto adapt element sizes to avoid the existence of elements \
	    with more chordal error than the one specified."]

    ttk::label $base.label#11 -text [_ "relative (%)"]:
    ttk::entry $base.entry#10 \
	-textvariable ChangeVarInfo(ChordalError,RelativeChordalError) \
	-width 5
    GidHelp "$base.label#11 $base.entry#10 " \
	[_ "This value is a relative value (in %) of the chordal error allowed in the model,\
	    \nrefered to the mean size of the element. Value equal to 0.0 won't be taken into account."]

    ttk::label $base.label#12 -text [_ "absolute"]:
    ttk::entry $base.entry#11 \
	-textvariable ChangeVarInfo(ChordalError,MaxChordalError) \
	-width 10
    GidHelp "$base.label#12 $base.entry#11 " \
	[_ "This value is an absolut value of the maximum chordal error allowed in the model. \
	    \nValue equal to 0.0 won't be taken into account."]

    ttk::frame $base.frame#21
    ttk::label $base.label#13 -text [_ "Minimum element size"]:
    ttk::entry $base.entry#12 \
	-textvariable ChangeVarInfo(ChordalError,MinElemSize) \
	-width 10
    GidHelp "$base.label#13 $base.entry#12 " \
	[_ "This value is an absolut value of the minimum element size allowed. Althought the \
	    \nchordal error is above the limit specified."]


    ttk::label $base.framerball.lblalgorithm -text [_ "Algorithm"]:
    TTKComboBox $base.framerball.mbalgorithm -textvariable ::ChangeVarInfo(Rball,Implicit) \
	-labels [list [_ "Optimization Radius"] [_ "Porosity Predefined"] [_ "Explicit"] \
		     [_ "Radius Expansion"] [_ "Optimization Radius Fast"] \
		     [_ "Position Correction & Inflate"] [_ "Optimization Radius Corrected"]] \
	-values {1 2 0 3 4 7 8} -state readonly
    ttk::label $base.framerball.lbliterations -text [_ "Max. iterations"]:
    ttk::entry $base.framerball.eiterations -textvariable ChangeVarInfo(Rball,MaxIter)
    ttk::button $base.framerball.bmoreoptions -text [_ "More options"]... -command OpenRballWindow

    ttk::frame $base.framecartesian.fgriduniform -style flat.TFrame
    ttk::label $base.framecartesian.fgriduniform.lgriduniform -text [_ "Grid spacing"]:
    TTKComboBox $base.framecartesian.fgriduniform.mbgriduniform -textvariable ::ChangeVarInfo(Cartesian,GridUniform) \
	-labels [list [_ "Variable"] [_ "Uniform"]] -values {0 1} -state readonly    
    ttk::frame $base.framecartesian.ffromrendermesh -style flat.TFrame
    ttk::label $base.framecartesian.ffromrendermesh.lfromrendermesh -text [_ "Shape"]:
    TTKComboBox $base.framecartesian.ffromrendermesh.mbfromrendermesh -textvariable ::ChangeVarInfo(Cartesian,FromRenderMesh) \
	-labels [list [_ "From geometry"] [_ "From mesh"]] -values {1 0} -state readonly
    
    #ttk::radiobutton $base.framecartesian.condfaceelem0 -text [_ "Condition to body"]: -value 0 -variable ChangeVarInfo(Cartesian,CondFaceElem)
    #ttk::radiobutton $base.framecartesian.condfaceelem1 -text [_ "Condition to face"]: -value 1 -variable ChangeVarInfo(Cartesian,CondFaceElem)   
    #init variable
    #ttk::checkbutton $base.framecartesian.createhexahedra -text [_ "Create hexahedra"] -variable ChangeVarInfo(Cartesian,CreateHexahedra)
    
    ttk::frame $base.framecartesian.fminsizemaingrid -style flat.TFrame
    ttk::label $base.framecartesian.fminsizemaingrid.lminsizemaingrid -text [_ "Minimum size"]
    ttk::entry $base.framecartesian.fminsizemaingrid.eminsizemaingrid -textvariable ChangeVarInfo(Cartesian,MinSizeMainGrid) -width 10
    ttk::frame $base.framecartesian.fsizeratio -style flat.TFrame
    ttk::label $base.framecartesian.fsizeratio.l -text [_ "Size ratio"]
    ttk::entry $base.framecartesian.fsizeratio.e -textvariable ChangeVarInfo(Cartesian,SizeRatio) -width 10
   
    
    ttk::frame $base.frame#6
    ttk::label $base.label#7 -text [_ "Mesh always by default"]:
    # -wraplength 20m -justify left

	if { ![info exists ChangeVarInfo(ForceMeshPoints)] } { set ChangeVarInfo(ForceMeshPoints) 0 }
    ttk::checkbutton $base.checkbutton#5 \
	-image [gid_themes::GetImage "point.png" small_icons] \
	-variable ChangeVarInfo(ForceMeshPoints) \
	-style Toolbutton
	# -borderwidth 1 -selectcolor $col_bg

    if { ![info exists ChangeVarInfo(ForceMeshLines)] } { set ChangeVarInfo(ForceMeshLines) 0 }
    ttk::checkbutton $base.checkbutton#6 \
	-image [gid_themes::GetImage "line.png" small_icons] \
	-variable ChangeVarInfo(ForceMeshLines) \
	-style Toolbutton
	# -borderwidth 1 -selectcolor $col_bg

    if { ![info exists ChangeVarInfo(ForceMeshSurfaces)] } { set ChangeVarInfo(ForceMeshSurfaces) 0 }
    ttk::checkbutton $base.checkbutton#7 \
	-image [gid_themes::GetImage "surface.png" small_icons] \
	-variable ChangeVarInfo(ForceMeshSurfaces) \
	-style Toolbutton
	# -borderwidth 1 -selectcolor $col_bg

    GidHelp "$base.label#7 $base.checkbutton#5 $base.checkbutton#6 $base.checkbutton#7" \
	[_ "Changes the default meshing criteria. Entities will be always\
	    meshed by default, although they have higher entities over them.\nExample:\
	    If surfaces are checked, when meshing a volume, both volume elements and\
	    surface elements will be obtained."]
	
    grid $frm_quadratic.radiobutton#0 $frm_quadratic.radiobutton#1 $frm_quadratic.radiobutton#2 -sticky w
    
    grid $frm_quadratic -sticky nesw -padx 2      
    grid $base.frame#2 -sticky nesw -padx 2
    grid $base.frame#4 -sticky nesw -padx 2
    grid $base.frame#1 -sticky nesw -padx 2
    #grid $base.frame#8 -sticky nesw -padx 2
    grid $base.frame#3 -sticky nesw -padx 2
    grid $base.frame#10 -sticky nesw -padx 2
    grid $base.frame#8 -sticky nesw -padx 2
    grid $base.framerball -sticky nesw -padx 2
    grid $base.framecartesian -sticky nesw -padx 2
    grid $base.frame#5 -sticky nesw -padx 2  
    
    grid $base.frame#11 -in $base.frame#2 -row 1 -column 1 -sticky nw
    grid $base.label#75 -in $base.frame#11 -row 1 -column 1 
    grid $base.radiobutton#1 -in $base.frame#11 -row 1 -column 2 
    grid $base.radiobutton#2 -in $base.frame#11 -row 1 -column 3
    #grid $base.radiobutton#3 -in $base.frame#11 -row 2 -column 4
    grid $base.radiobutton#4 -in $base.frame#11 -row 1 -column 4
	
    grid $base.frame#12 -in $base.frame#2 -row 2 -column 1 -sticky nw
    grid $base.label#76 $base.radiobutton#50 $base.radiobutton#51 $base.radiobutton#52 -in $base.frame#12   
#   grid $base.label#76 $base.radiobutton#50 $base.radiobutton#51 -in $base.frame#12 
    grid $base.checkbutton#mixedmesh  -in $base.frame#2 -row 3 -column 1 -sticky nw
    
    
    grid $base.frame#20 -in $base.frame#8 -row 0 -column 1  -padx 2 -sticky news
#     grid $base.checkbutton#11 -in $base.frame#8 -row 1 -column 1  -sticky ew
#     grid $base.entry#10 -in $base.frame#8 -row 1 -column 2  -sticky ew
    grid $base.checkbutton#11 -in $base.frame#20 -row 0 -column 1 -sticky nw
    grid $base.label#11 -in $base.frame#20 -row 0 -column 2 -sticky nw
    grid $base.entry#10 -in $base.frame#20 -row 0 -column 3 -sticky nw
    grid $base.label#12 -in $base.frame#20 -row 0 -column 4 -padx 2 -sticky nw
    grid $base.entry#11 -in $base.frame#20 -row 0 -column 5 -sticky nw

    grid $base.frame#21 -in $base.frame#8 -row 1 -column 1 -padx 2 -sticky news
    grid $base.label#13 -in $base.frame#21 -row 0 -column 1 -sticky nw
    grid $base.entry#12 -in $base.frame#21 -row 0 -column 2 -sticky nw

    grid $base.framerball.lblalgorithm $base.framerball.mbalgorithm -sticky nw
    grid configure $base.framerball.mbalgorithm -sticky ew
    grid $base.framerball.lbliterations $base.framerball.eiterations -sticky nw
    grid $base.framerball.bmoreoptions -row 0 -column 3 -sticky e
	    
    grid $base.framecartesian.fgriduniform.lgriduniform $base.framecartesian.fgriduniform.mbgriduniform -sticky w
    grid $base.framecartesian.ffromrendermesh.lfromrendermesh $base.framecartesian.ffromrendermesh.mbfromrendermesh -sticky w
    grid $base.framecartesian.fgriduniform $base.framecartesian.ffromrendermesh -sticky ew -padx 2
    grid $base.framecartesian.fminsizemaingrid.lminsizemaingrid $base.framecartesian.fminsizemaingrid.eminsizemaingrid -sticky w    
    grid $base.framecartesian.fsizeratio.l $base.framecartesian.fsizeratio.e -sticky w
    grid $base.framecartesian.fminsizemaingrid $base.framecartesian.fsizeratio -sticky ew -padx 2
    #grid $base.framecartesian.condfaceelem0 $base.framecartesian.condfaceelem1 -sticky ew  
    #grid $base.framecartesian.createhexahedra -sticky e

    #grid columnconfigure $base.framecartesian.fsizeratio "1" -weight 1

    grid $base.frame#25 -in $base.frame#10 -row 0 -column 1 -padx 2 -sticky news
    grid $base.checkbutton#15 -in $base.frame#10 -row 1 -column 1 -padx 2 -sticky nw

    grid $base.label#10 -in $base.frame#25 -row 0 -column 0  -padx 2 -sticky w
    grid $base.menubutton#30 -in $base.frame#25 -row 0 -column 1 -padx 2 -sticky w
    grid $base.label#30 -in $base.frame#25 -row 0 -column 2  -padx 2 -sticky w
    grid $base.spinbox#1 -in $base.frame#25 -row 0 -column 3  -padx 2 -sticky nsw
    grid $base.canvas#1 -in $base.frame#25 -row 1 -column 0  -padx 0 -sticky nesw -columnspan 4

    
    grid $base.scale#4  -in $base.frame#1         -row 1 -column 1 -sticky nesw    
    grid $base.checkbutton#12 -in $base.frame#1   -row 2 -column 1 -sticky nesw

    grid $base.frame#22 -in $base.frame#5 -row 1 -column 1  -padx 2 -sticky news
    grid $base.checkbutton#1 -in $base.frame#22 -row 0 -column 1 -sticky nw
    grid $base.checkbutton#3 -in $base.frame#22 -row 0 -column 2 -padx 2 -sticky nw
    grid $base.checkbutton#10 -in $base.frame#5 -row 2 -column 1 -sticky nw
    grid $base.checkbutton#16 -in $base.frame#5 -row 3 -column 1 -sticky nw
#     grid $base.label#31 -in $base.frame#5 -row 4 -column 1 -sticky nw
#     grid $base.spinbox#2 -in $base.frame#5 -row 4 -column 2 -sticky nw
    

    grid $base.radiobutton#6 -in $base.frame#4  -row 1 -column 0
    grid $base.radiobutton#7 -in $base.frame#4  -row 1 -column 1
    grid $base.radiobutton#8 -in $base.frame#4  -row 1 -column 2

    grid $base.checkbutton#4 -in $base.frame#3 -sticky w -columnspan 3
    grid $base.label#70 -in $base.frame#3 -sticky w -row 1 -columnspan 1 -column 0
    grid $base.checkbutton#8 -in $base.frame#3 -sticky w -row 1 -columnspan 1 -column 1
    grid $base.checkbutton#9 -in $base.frame#3 -sticky w -row 1 -columnspan 1 -column 2

    grid $base.frame#6 -in $base.frame#5 -row 5 -column 1 -sticky nw -padx 2
    grid $base.label#7 -in $base.frame#6 -row 1 -column 1 -sticky nw -padx 2
    grid $base.checkbutton#5 -in $base.frame#6 -row 1 -column 2 -sticky nw -padx 2
    grid $base.checkbutton#6 -in $base.frame#6 -row 1 -column 3 -sticky nw -padx 2
    grid $base.checkbutton#7 -in $base.frame#6 -row 1 -column 4 -sticky nw -padx 2

    grid $base.frame#7 -in $base.frame#5 -row 0 -column 1 -sticky nesw -padx 2
    grid $base.label#8 -in $base.frame#7 -row 1 -column 1 -sticky w -padx 2
    grid $base.menubutton#1 -in $base.frame#7 -row 1 -column 2 -sticky w -padx 2

    #grid $base.frame#20 -in $base.frame#5 -row 4 -column 1 -sticky nw -padx 2

    # Resize behavior management

    grid rowconfigure $base.frame#2 1 -weight 0 -minsize 2
    #grid rowconfigure $base.frame#2 2 -weight 0 -minsize 2
    grid columnconfigure $base.frame#2 1 -weight 0
    grid columnconfigure $base.frame#2 2 -weight 0
    grid columnconfigure $base.frame#2 3 -weight 0
    grid columnconfigure $base.frame#2 4 -weight 0

    grid rowconfigure $base.frame#4 0 -weight 0 -minsize 2
    #grid rowconfigure $base.frame#4 2 -weight 0 -minsize 2
    #grid columnconfigure $base.frame#4 1 -weight 1 -minsize 2
    #grid columnconfigure $base.frame#4 2 -weight 0 -minsize 2

    grid columnconfigure $base.frame#3 0 -weight 0
#     grid columnconfigure $base.frame#3 1 -weight 1

    grid columnconfigure $base.frame#10 1 -weight 1
    grid rowconfigure $base.frame#10 1 -weight 1 -minsize 2

    grid rowconfigure $base.frame#25 0 -weight 0
    grid rowconfigure $base.frame#25 1 -weight 0
    grid rowconfigure $base.frame#25 2 -weight 0

    grid columnconfigure $base.frame#25 0 -weight 0
    grid columnconfigure $base.frame#25 1 -weight 0
    grid columnconfigure $base.frame#25 2 -weight 0

    grid rowconfigure $base.framerball 0 -weight 0
    grid columnconfigure $base.framerball "1 2" -weight 1
    
    grid rowconfigure $base.framecartesian 0 -weight 0
    grid columnconfigure $base.framecartesian "0 1" -weight 1

    grid rowconfigure $base.frame#5 1 -weight 1
    grid columnconfigure $base.frame#8 1 -weight 1

    grid columnconfigure $base.frame#7 1 -weight 0 -minsize 30
    grid columnconfigure $base.frame#7 2 -weight 0 -minsize 30

    #grid rowconfigure $base "1 2 3 4 5" -weight 0  -minsize 2

    grid rowconfigure $base.frame#8 6 -weight 0 -minsize 2
    
    grid rowconfigure $base 10 -weight 1
    grid columnconfigure $base 0 -weight 1 -minsize 2

    grid rowconfigure $base.frame#1 1 -weight 0
    grid rowconfigure $base.frame#1 2 -weight 1 -minsize 2   
    grid columnconfigure $base.frame#1 1 -weight 1 -minsize 13    
   

    if { $use_bwidget } { 
	$sw setwidget $sf
	grid $sw -sticky nsew
    } else {
	AddToScrolledCanvas $base_base.sc $c.f
	grid $base_base.sc -sticky news
    }
    
    grid rowconfigure $base_base 0 -weight 1
    grid columnconfigure $base_base 0 -weight 1

    
    UpdateQuadraticVisibility
    
    trace add variable ChangeVarInfo(BoundaryLayer,GrowFactor) write "ChangeGrowFactor $base"
    trace add variable ChangeVarInfo(BoundaryLayer,GrowLaw) write "ChangeGrowLaw $base"
}

proc UpdateQuadraticVisibility { } {
    set base .gid.wvar.f.nb.fMeshing.f.frame
    set frm_quadratic $base.frm_quadratic
    if { [winfo exists $frm_quadratic] } {
	set basename $frm_quadratic.radiobutton#
	foreach bit {0 1 2} integer {1 2 4} {
	    set name ${basename}${bit}
	    if { [expr {$::GidPriv(HideQuadraticTypeLevel) & $integer}] } {
		grid remove $name
	    } else {
		grid $name
	    }
	}               
	if { $::GidPriv(HideQuadraticTypeLevel) == 7 } {
	    grid remove $frm_quadratic
	} else {
	    grid $frm_quadratic
	}
    }
}

trace add variable ::GidPriv(HideQuadraticTypeLevel) write "UpdateQuadraticVisibility ;#" ;#to update preferences window and other


proc OpenRballWindowFillingVariablesFromGiD { } {
    set RballVars {  Implicit MaxIter MaxIterLocal Tolerance \
		         DeltaPosition DeltaRadius MinRadius MaxRadius \
		         Preprocess Porosity Postprocess Overlap \
		         Filter Search SearchType ContactFactor \
		         NeighborFactor FacetFactor OverlapFactor Verbose \
		     }
    foreach item $RballVars {
	set ::ChangeVarInfo(Rball,$item) [format "%g" [GiD_Set Rball($item)]]
    }
    OpenRballWindow
}

proc OpenRballWindow { } {
    set w .gid.spheremesheroptions
    InitWindow $w [_ "Sphere mesher options"] SphereMesherOptionsWindowGeom OpenRballWindow
    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0

    ttk::frame $w.buts -style BottomFrame.TFrame
    # -bg [CCColorActivo [$w cget -background]]
    ttk::button $w.buts.ok -text [_ "Accept"] -command Rball_AcceptValues -underline 0 -style BottomFrame.TButton
    ttk::button $w.buts.rs -text [_ "Reset"] -command  Rball_FillValuesDefault -underline 0 -style BottomFrame.TButton
    ttk::button $w.buts.cn -text [_ "Close"] -command [list  destroy $w] -underline 0 -style BottomFrame.TButton

    #SetWidgetsWidthFromText $w.buts.ok $w.buts.rs $w.buts.cn

    grid $w.buts.ok -row 0 -column 0 -padx 5 -pady 5
    grid $w.buts.rs -row 0 -column 1 -padx 5 -pady 5
    grid $w.buts.cn -row 0 -column 2 -padx 5 -pady 5

    GidHelp $w.buts.ok [_ "With this button, changed values of the active tab are\
    accepted. If this button is not pressed, values changed by the user are not\
    computed."]
    GidHelp $w.buts.rs [_ "This button sets all the options of the active tab to its\
	    default values."]
    GidHelp $w.buts.cn [_ "This button closes the window without accepting any changed\
	    value. Press 'Accept' not to loose the changed options."]
    
    focus $w.buts.ok

    

    #not show option [_ "Verbose"] 
    set RballTexts [list \
		        [_ "Main"] \
		        [_ "Algorithm"] [_ "Max. iterations"] [_ "Max. local iterations"] [_ "Tolerance"] \
		        [_ "Delta position factor"] [_ "Delta radius factor"] [_ "Min. radius factor"]  [_ "Max. radius factor"] \
		        [_ "Preprocess"] [_ "Porosity"] [_ "Postprocess"] [_ "Overlap"] \
		        [_ "Filters"] \
		        [_ "Filter iterations"] [_ "Search iterations"] [_ "Search type"] [_ "Factor contact"] \
		        [_ "Factor neighbor"] [_ "Factor facet"] [_ "Factor overlap"] \
		       ]
    set RballVars { \
		        1 \
		        Implicit MaxIter MaxIterLocal Tolerance \
		        DeltaPosition DeltaRadius MinRadius MaxRadius \
		        Preprocess Porosity Postprocess Overlap \
		        2 \
		        Filter Search SearchType ContactFactor \
		        NeighborFactor FacetFactor OverlapFactor \
		    }
    set RballWidgets { \
		           title \
		           combobox entry entry entry \
		           entry entry entry entry \
		           combobox entry checkbutton checkbutton \
		           title \
		           entry entry combobox entry \
		           entry entry entry \
		       }
    set RballComboboxInfo [list \
		               "" \
		               [list [list [_ "Optimization Radius"] [_ "Porosity Predefined"] [_ "Explicit"] \
		                          [_ "Radius Expansion"] [_ "Optimization Radius Fast"] \
		                          [_ "Position Correction & Inflate"] [_ "Optimization Radius Corrected"] ] {1 2 0 3 4 7 8}] "" "" "" \
		               "" "" "" "" \
		               [list [list [_ "Non porosity"] [_ "Porosity"]] {1 2}] "" "" "" \
		               "" \
		               "" "" [list [list [_ "Delaunay remeshing"] [_ "Spatial search"]] {2 1}] "" \
		               "" "" "" \
		              ]

    CreateWidgetsFromInformation $w $RballTexts $RballVars $RballWidgets $RballComboboxInfo
    ChangeRballPreprocess $w ;#to initially set state according to the trace dependency
    trace add variable ::ChangeVarInfo(Rball,Implicit) write "ChangeRballImplicit $w"
    trace add variable ::ChangeVarInfo(Rball,Preprocess) write "ChangeRballPreprocess $w"   

    grid $w.buts -sticky nsew
    if { $::tcl_version >= 8.5 } { grid anchor $w.buts center }

    grid rowconfigure $w 0 -weight 1
    grid columnconfigure $w 0 -weight 1

    bind $w <Escape> "$w.buts.cn invoke"
    bind $w <Destroy> [list +DestroyRballWindow %W $w]
}

proc DestroyRballWindow { W w } {
    global ChangeVarInfo
    if { $W != $w } return
    trace remove variable ::ChangeVarInfo(Rball,Implicit) write "ChangeRballImplicit $w"
    trace remove variable ::ChangeVarInfo(Rball,Preprocess) write "ChangeRballPreprocess $w"
}

proc ChangeRballImplicit { w args } {
    return
    #return because now this variables are used for all methods
    global ChangeVarInfo
    foreach item {DeltaRadius MinRadius MaxRadius} {
	set e $w.nb.f1.e$item
	if { [winfo exists $e ] } {
	    if { $ChangeVarInfo(Rball,Implicit) == 1 || $ChangeVarInfo(Rball,Implicit) == 4 } {
		$e configure -state normal
	    } else {
		$e configure -state disabled
	    }
	}
    }
}

proc ChangeRballPreprocess { w args } {
    global ChangeVarInfo
    set e $w.nb.f1.ePorosity
    if { [winfo exists $e ] } {
	if { $ChangeVarInfo(Rball,Preprocess) == 2 } {
	    $e configure -state normal
	} else {
	    $e configure -state disabled
	}
    }
}

proc CreateWidgetsFromInformation { w Texts Vars Widgets ComboboxInfo } {
    global ChangeVarInfo
    set nb [ttk::notebook $w.nb]
    ttk::notebook::enableTraversal $nb
    foreach text $Texts item $Vars widget $Widgets comboboxinfo $ComboboxInfo {
	if { $widget == "title" } {
	    set f [ttk::frame $nb.f$item]
	    $nb add $f -text $text -underline 0
	    grid columnconfigure $f {0 1} -weight 1
	} elseif { $widget == "entry" } {
	    ttk::label $f.l$item -text $text
	    ttk::$widget $f.e$item -textvariable ::ChangeVarInfo(Rball,$item)
	} elseif { $widget == "checkbutton" } {
	    ttk::label $f.l$item -text $text
	    ttk::$widget $f.e$item -variable ::ChangeVarInfo(Rball,$item)
	} elseif { $widget == "combobox" } {
	    ttk::label $f.l$item -text $text
	    TTKComboBox $f.e$item -textvariable ::ChangeVarInfo(Rball,$item) \
		-labels [lindex $comboboxinfo 0] -values [lindex $comboboxinfo 1] -state readonly
	} else {
	    ttk::$widget $f.e$item -textvariable ::ChangeVarInfo(Rball,$item)
	}
	if { $widget != "title" } {        
	    grid $f.l$item $f.e$item -sticky ew -padx 2
	}
    }

    grid $w.nb -sticky nsew
}