
proc pref_postprocess_ui {base args} {
    global ChangeVarInfo        
    
    ttk::labelframe $base.fRC -text [_ "Results cache"]
    if { ![info exists ::ChangeVarInfo(PostUseResultCache)] } { set ::ChangeVarInfo(PostUseResultCache) 0 }
    ttk::checkbutton $base.fRC.cb_use -text [_ "Use result cache"] \
	    -onvalue 1 -offvalue 0 -variable ::ChangeVarInfo(PostUseResultCache)
    grid $base.fRC.cb_use -sticky w -padx 4
    setTooltip $base.fRC.cb_use \
	    [_ "Enables the use of a portion of the main memory as results cache avoiding loading all results in memory."]

    ttk::frame $base.fRC.rc -borderwidth 0
    ttk::label $base.fRC.rc.mem_lbl_1 -text [_ "Use a maximum of"]
    ttk::combobox $base.fRC.rc.mem_cb -values [ list 128 256 512 1024 1536 2048 3072] \
	    -textvariable ::ChangeVarInfo(PostMaxMemResultCache) -width 10
    ttk::label $base.fRC.rc.mem_lbl_2 -text [_ "MBytes for results"]
    grid $base.fRC.rc.mem_lbl_1 $base.fRC.rc.mem_cb $base.fRC.rc.mem_lbl_2 -sticky w -columnspan 3 -padx 8

    grid $base.fRC.rc -sticky w

    setTooltip $base.fRC.rc \
	    [_ "Tries to use as much of the specified memory as results cache."]


    # ttk::labelframe $base.fRS -text [_ "Results state"]
    # if { ![info exists ::ChangeVarInfo(PostReadResultState)] } { set ::ChangeVarInfo(PostReadResultState) 0 }
    # ttk::checkbutton $base.fRS.cb_read -text [_ "Read result visualization state"] \
    # 	    -onvalue 1 -offvalue 0 -variable ::ChangeVarInfo(PostReadResultState)
    # grid $base.fRS.cb_read -sticky w -pady 4 -padx 4
    # setTooltip $base.fRS.cb_read \
    # 	    [_ "If checked, tries to draw the last visualized result for the model."]

    ttk::labelframe $base.fFHP -text [_ "Healthy filter"]
    if { ![info exists ::ChangeVarInfo(PostFilterHealthyPoints)] } { set ::ChangeVarInfo(PostFilterHealthyPoints) 0 }
    ttk::checkbutton $base.fFHP.cb_filter -text [_ "Filter coordinates greater than"]: \
	    -onvalue 1 -offvalue 0 -variable ::ChangeVarInfo(PostFilterHealthyPoints)
    ttk::entry $base.fFHP.e_filter -width 8 -textvariable ::ChangeVarInfo(PostFltHealthyValue)
    grid $base.fFHP.cb_filter $base.fFHP.e_filter -sticky w -pady 4 -padx 4
    setTooltip $base.fFHP.cb_filter \
	    [_ "If checked, filters all coordinates of the model \nwith absolute value greater than the specified limit."]
    setTooltip $base.fFHP.e_filter \
	    [_ "Any coordinate with absolute value below \nthis limit will be considered as 'healthy'."]
       
    set fFMT [ttk::labelframe $base.rf -text [_ "Results format"]]
    ttk::label $fFMT.l1 -text [_ "Legend format"]:
    ttk::entry $fFMT.e1 -textvariable ChangeVarInfo(PostLegendFormat)
    ttk::label $fFMT.l2 -text [_ "Scalar label format"]:
    ttk::entry $fFMT.e2 -textvariable ChangeVarInfo(PostScalarFormat)
    ttk::label $fFMT.l3 -text [_ "Vector label format"]:
    ttk::entry $fFMT.e3 -textvariable ChangeVarInfo(PostVectorFormat)
    ttk::label $fFMT.l4 -text [_ "Matrix label format"]:
    ttk::entry $fFMT.e4 -textvariable ChangeVarInfo(PostMatrixFormat)
    GidHelp "$fFMT.l1 $fFMT.e1" [_ "Numeric format used to print postprocess legends"]
    GidHelp "$fFMT.l2 $fFMT.e2" [_ "Numeric format used to print postprocess scalar results"]
    GidHelp "$fFMT.l3 $fFMT.e3" [_ "Numeric format used to print postprocess vector results"]
    GidHelp "$fFMT.l4 $fFMT.e4" [_ "Numeric format used to print postprocess matrix results"]

    grid $fFMT.l1 $fFMT.e1 -padx 2 -sticky w
    grid configure $fFMT.e1 -sticky ew
    grid $fFMT.l2 $fFMT.e2 -padx 2 -sticky w
    grid configure $fFMT.e2 -sticky ew
    grid $fFMT.l3 $fFMT.e3 -padx 2 -sticky w
    grid configure $fFMT.e3 -sticky ew
    grid $fFMT.l4 $fFMT.e4 -padx 2 -sticky w
    grid configure $fFMT.e4 -sticky ew

    grid rowconfigure $fFMT 4 -weight 1
    grid columnconfigure $fFMT 1 -weight 1

    ttk::labelframe $base.fSimp -text [_ "Simplified view ( Fast rotation)"]

    ttk::frame $base.fSimp.gs -borderwidth 0
    ttk::label $base.fSimp.gs.l1 -text [_ "Grid size"]
    ## ttk::entry $base.fSimp.e1 -textvariable ::ChangeVarInfo(PostSimpGridSize)
    ttk::label $base.fSimp.gs.e1 -width 4 -anchor e \
	-textvariable ::ChangeVarInfo(PostSimpGridSize)
    gidscale $base.fSimp.gs.e1sc -orient horizontal \
	-from 64 -to 1024 \
	-showbuttons 1 -symbolbuttons "< >" \
	-showvalue 0 \
	-variable ::ChangeVarInfo(PostSimpGridSize)
    grid $base.fSimp.gs.l1 $base.fSimp.gs.e1 $base.fSimp.gs.e1sc -padx 2 -sticky w
    grid configure  $base.fSimp.gs.e1 -sticky e
    grid configure  $base.fSimp.gs.e1sc -sticky ew
    grid columnconfigure $base.fSimp.gs 2 -weight 1

    ttk::frame $base.fSimp.me -borderwidth 0
    ttk::label $base.fSimp.me.l2 -text [_ "Minimum number of elements to apply:"]
    ttk::entry $base.fSimp.me.e2 -textvariable ::ChangeVarInfo(PostSimpMinElem) -width 12
    grid $base.fSimp.me.l2 $base.fSimp.me.e2 -padx 2 -sticky w
    grid configure $base.fSimp.me.e2 -sticky w

    grid $base.fSimp.gs -sticky ew
    grid $base.fSimp.me -sticky ew
    grid columnconfigure $base.fSimp 0 -weight 1
    
    grid $base.fRC -padx 2 -sticky news
    # grid $base.fRS -padx 2 -sticky news
    grid $base.fFHP -padx 2 -sticky news
    grid $fFMT -padx 2 -sticky news
    grid $base.fSimp -padx 2 -sticky news
    # grid rowconfigure $base "0 1" -weight 1
    grid columnconfigure $base 0 -weight 1
    
    set ChangeVarInfo(BasePostWin) $base         
}
