# -*- mode: Tcl; tcl-indent-level :  4 -*-

GiDMenu::DisableUpdateMenus

proc WarnWinDirect { text { parent .gid} { icn warning}} {
    if { ( $parent != "") && [ winfo exists $parent]} {
        tk_messageBox -message $text -parent $parent \
            -title Warning -icon $icn -type ok
    } else {
        tk_messageBox -message $text \
            -title Warning -icon $icn -type ok
    }
}

# set ::SPW_ChangesPerUpdate 5
# 
# proc ShowPackagesWindow { txt} {
#     set w .show_packages_window
#     if { ![ winfo exists $w]} {
#         toplevel $w
#         wm title $w "Loading Tcl/Tk packages..."
#         # wm geometry $w 400x100+300+300
#         wm geometry $w +300+300
#         wm attributes $w -topmost 1
#         wm overrideredirect $w 1
#         label $w.l -text ""
#         grid $w.l -sticky news
#         grid columnconfigure $w 0 -weight 1
#         grid rowconfigure $w 0 -weight 1
#     }
#     $w.l configure -text $txt
# }



set ::MPLstPackages {}
set ::MPNumPackages 0

proc MyPackage { cmd args} {
    switch $cmd {
        require {
            set len_args [ llength $args]
            if { $len_args >= 1} {
                set pkg_name [ lindex $args 0]
                if { $len_args > 1} {
                    # skip -exact flag
                    if { $pkg_name == "-exact"} {
                        set pkg_name [ lindex $args 1]
                    }
                }
                lappend ::MPLstPackages $pkg_name
                incr ::MPNumPackages
                # ShowPackagesWindow "Loading $pkg_name package ..."
                GiDSplashMessage "Loading $pkg_name package ( $::MPNumPackages ) ..."                               
            }
        }
        showlist {
            # set list ""
            # foreach pkg $::MPLstPackages {
            #         append list "$pkg\n"
            # }
            set num_uniq [ llength [ lsort -unique $::MPLstPackages]]
            WarnWinDirect "Num = [ llength $::MPLstPackages] ( $num_uniq)\n$::MPLstPackages"
            return
        }
        default {
        }
    }
    return [ eval packageOriginal $cmd $args]
}

#----------------------Configuration section-----------------------------

# configuration is now in a separated file

source [file join $::GIDDEFAULTTCL ConfigureProgram.tcl]

#----------------------End configuration section-------------------------

if { $::GidPriv(ShowPackages) } {
    rename package packageOriginal
    rename MyPackage package
}


if { $::GIDDEFAULT == "" } {
    puts [_ " (in tclfile) to run 'gid is necessary to set the environment variable: GIDDEFAULT to the path of the executables"]
    destroy .
}


set winole 0
catch { set winole [ gidolewin]}
if { $winole == "0" } {
    toplevel .gid -class GiD
} else {
    toplevel .gid -use $winole
}
wm withdraw .

if { ( $::tcl_platform(os) == "Darwin")} {
    catch {
        console hide
    }
}

# RAMSAN: it is important not to have external dependencies
#IsSplashWindowOnInDefault esta definido en tclfileP.tcl
# y hay que leer el tclIndex
set auto_path [list $GIDDEFAULTTCL [file join $GIDDEFAULTTCL tk]]

proc GiDSplash { } {
    global GidPriv

    set mainversion [GiD_Info GiDVersion] ;
    set txt [concat [_ "Version"] $mainversion]

    set w .splash
    if { [winfo exists $w] } { destroy $w }
    toplevel $w
    if { $::tcl_platform(platform) == "windows" } {       
        wm attributes $w -toolwindow 1
    }
    set im [gid_themes::GetImage $::GidPriv(SplashWindow) ]
    set x [expr [winfo screenwidth .splash]/2-[image width $im]/2]
    set y [expr [winfo screenheight .splash]/2-[image height $im]/2]

    wm withdraw .gid

    wm geom $w +$x+$y
    wm overrideredirect .splash 1
    set base $w.l
    tk::label $base -image $im -relief ridge -borderwidth 2
    #draw version txt over the image
    tk::label $base.lv -text $txt -font "Helvetica 10 bold" \
        -foreground #9d9d9d -background #010101 -borderwidth 0 -highlightthickness 0        
    place $base.lv -x 385 -y 75 -in $base
    #place $base.lv -x 160 -y 416 -in $base

    pack $base
    update
    bind $w <1> "destroy $w"

    set after_delay 3000
    if { $::tcl_platform(platform) == "windows" } {       
        set after_delay 5000
    }
    after $after_delay "if { [ winfo exists $w] } { destroy $w }"
}

proc GiDSplashMessage { txt} {
    set w .splash
    if { $::GidPriv(ShowPackages) } {
        packageOriginal require Tk
    } else {
        package require Tk
    }
    if { ![ winfo exists $w]} {
        # WarnWinText "$txt"
        return        
    }
    #wm attributes $w -topmost 1 ;#splash can hide the error message or window to select opengl by software
    wm overrideredirect $w 1
    set lbl $w.show_packages_label
    set max_len 70
    if { ![ winfo exists $lbl]} {
        tk::label $lbl -text [ string repeat " " $max_len] -font "Helvetica 8" \
        -foreground #fefefe -background #010101
        # -x 2 because the image is drawn with a border 2 ( ridge)
        place $lbl -x 2 -y 233 -in $w.l
    }
    set len_spc [ expr $max_len - [ string length $txt]]
    if { $len_spc > 0} {
        append txt [ string repeat " " $len_spc]
    }
    $lbl configure -text $txt -font "Helvetica 10"
}

proc CheckFileOwner { file} {
    if { $::tcl_platform(platform) == "windows" } { 
        return
    }
    set owner_file [ file attributes $file -owner]
    set owner_dir [ file attributes [ file dirname $file] -owner]
    set group_file [ file attributes $file -group]
    set group_dir [ file attributes [ file dirname $file] -group]
    if { "$owner_file" != "$owner_dir"} {
        # let's change owner of the file to owner of the directory where the file is
        # to avoid problems such that the owner of the preferences file is root but we're in a user directory
        set err [ catch {
            file attributes $file -owner $owner_dir -group $group_dir
        } err_txt ]
        if { $err} {
            WarnWinDirect "Sorry but could not change owner of \nfile '$file' to '$owner_dir': \n$err_txt"
        }
    }
}

proc InitialGiDRestart { { first_time 1}} {
    set changedVariables {}

    if { [info exists ::UseLocalOpenGL] } {
        if { $::GidPriv(SoftwareOpenGL) != $::UseLocalOpenGL} {
            set ::GidPriv(SoftwareOpenGL) $::UseLocalOpenGL
            if { $::GidPriv(SoftwareOpenGL)} {
                WarnWinDirect [_ "Changing to software rendering."] .themechangewin
            } else {
                WarnWinDirect [_ "Changing to accelerated OpenGL."] .themechangewin
            }
            # if { $::tcl_platform(os) == "Darwin"} {
            after 1000 GiD_Set OGL_emulateFrontBuffer $::UseLocalOpenGL
            # }
        }
        lappend changedVariables [ list SoftwareOpenGL $::GidPriv(SoftwareOpenGL) ] \
            [ list OGL_configured 1 ]
    }
    if { [info exists ::UseLocalTheme] } {        
        #allways apply begining color options (example change from black to black)
        gid_themes::SetCurrentThemeAndApplyBackground $::UseLocalTheme
        # this is done after reading user preferents (readdefaults $gid_defaults 0 ;#kernel values) 
        lappend changedVariables [ list Theme $::GidPriv(CurrentTheme) ] [ list Theme_configured 1 ]
        foreach v { BackgroundColor BackColorType BackColorTop BackColorBottom BackColorSense ColorPoints ColorNodes ColorLines \
        ColorPolyLines ColorSurfaces ColorSurfsIsoparametric ColorVolumes ColorElements } {
            lappend changedVariables [ list $v [ GiD_Set $v ] ]
        }
    }
    destroy .iniconf
    if { !$first_time} {
	RestartGiD
    }
}

proc InitialConfiguration { { first_time 1} {configogl 1} {configtheme 1} } {
    if { $first_time} {
        wm withdraw .gid
    }
    
    toplevel .iniconf -class OverlayWindow
    
    set sx [ winfo screenwidth .]
    set sy [ winfo screenheight .]

    set txt_width 450    ;# win width is about 500 pixels ( decorations, frames, ...)
    set win_height 450
    set width [ expr ( $sx - $txt_width) / 2]
    wm geometry .iniconf +$width+[ expr ( $sy - $win_height) / 2]
    set width $txt_width
    
    if { $configogl && $configtheme } {
        wm title .iniconf [_ "Initial themes and graphics configuration"]
    } elseif { $configogl } {
        wm title .iniconf [_ "Initial graphics configuration ( OpenGL)"]
    } elseif { $configtheme } {
        wm title .iniconf [_ "Initial themes configuration"]    
    }
    
    if { $configogl } { 
        set ::UseLocalOpenGL $::GidPriv(SoftwareOpenGL)   
        set baseogl .iniconf.ogl
        ttk::frame $baseogl -style groove.TFrame
        
        ttk::label $baseogl.lbl -text [_ "Initial graphics configuration ( OpenGL)"]:
        ttk::radiobutton $baseogl.rd1 \
            -text [_ "Fast visualization mode."] \
            -value 0 -variable ::UseLocalOpenGL
        ttk::label $baseogl.rd1txt -wraplength $width \
            -text [_ "( Models will be drawn quicker but some issues could appear. I'm confident about my Graphic Card and I want to use its capabilities)."] \
            -state disabled
        
        ttk::radiobutton $baseogl.rd2 \
            -text [_ "Safe visualization mode."] \
            -value 1 -variable ::UseLocalOpenGL
        ttk::label $baseogl.rd2txt -wraplength $width \
            -text [_ "( Models will be drawn slower but the artifacts would disappear. I'm experiencing troubles with my Graphic Card and I want to use OpenGL by software)."] \
            -state disabled
        
        ttk::label $baseogl.txt -wraplength $width \
            -text [_ "(This preference can always be accessed through the 'OpenGL' part in the 'Graphical' panel in the 'Utilities-->Preferences' window, or writting 'gid -openglconfig' in the command line.)"] \
            -state disabled
        
        grid $baseogl.lbl -sticky w -padx 4 -pady 2
        grid $baseogl.rd1 -sticky w -padx 4 -pady 2
        grid $baseogl.rd1txt -sticky news -padx 24 -pady 0
        grid $baseogl.rd2 -sticky w -padx 4 -pady 2
        grid $baseogl.rd2txt -sticky news -padx 24 -pady 0
        grid $baseogl.txt -sticky news -padx 8 -pady 2
        grid $baseogl -sticky news -padx 4 -pady 2 
        grid columnconfigure $baseogl 0 -weight 1    
        grid rowconfigure $baseogl "2 4 5" -weight 1
    }
    
    
    if { $configtheme } {
        set ::UseLocalTheme $::GidPriv(CurrentTheme)        
        set basetheme .iniconf.theme
        ttk::frame $basetheme -style groove.TFrame
        
        ttk::label $basetheme.lbl -text [_ "Initial themes configuration"]:
        ttk::radiobutton $basetheme.rd1 \
            -text [_ "Classic GiD Theme"] \
            -value GiD_classic -variable ::UseLocalTheme
        ttk::label $basetheme.rd1txt -wraplength $width \
            -text [_ "(Colors integrated on operating system, similar look to previous version of the program)."] \
            -state disabled
        
        ttk::radiobutton $basetheme.rd2 \
            -text [_ "Dark GiD Theme"] \
            -value GiD_black -variable ::UseLocalTheme
        ttk::label $basetheme.rd2txt -wraplength $width \
            -text [_ "(Modern look, professional GUI combining dark colours and blue)."] \
            -state disabled
        
        ttk::label $basetheme.txt -wraplength $width \
            -text [_ "(This preference, and others related with the graphics configuration, can always be accessed through 'Graphical' panel in the 'Utilities-->Preferences' window.)"] \
            -state disabled
        
        grid $basetheme.lbl -sticky w -padx 4 -pady 2
        grid $basetheme.rd1 -sticky w -padx 4 -pady 2
        grid $basetheme.rd1txt -sticky news -padx 24 -pady 0
        grid $basetheme.rd2 -sticky w -padx 4 -pady 2
        grid $basetheme.rd2txt -sticky news -padx 24 -pady 0
        grid $basetheme.txt -sticky news -padx 8 -pady 2
        grid $basetheme -sticky news -padx 4 -pady 2 
        grid columnconfigure $basetheme 0 -weight 1    
        grid rowconfigure $basetheme "2 4 5" -weight 1
    }
    

    ttk::frame .iniconf.frmbuttons -style BottomFrame.TFrame
    ttk::button .iniconf.b1 -text [_ "Ok"] -command "InitialGiDRestart $first_time" -style BottomFrame.TButton
    ttk::button .iniconf.b2 -text [_ "Cancel"] -command "destroy .iniconf" -style BottomFrame.TButton
    
    grid .iniconf.frmbuttons -sticky news -pady 4
    grid .iniconf.b1 .iniconf.b2 -in .iniconf.frmbuttons -pady 4   
    
    grid columnconfigure .iniconf.frmbuttons "0 1" -weight 1        
    grid columnconfigure .iniconf 0 -weight 1    
    grid rowconfigure .iniconf "0 1" -weight 1    


    tkwait window .iniconf
}


proc CreateIcon {} {
    set rootname [file rootname $::GidPriv(LogosGiDBitmapName)]
    set extension [file extension $::GidPriv(LogosGiDBitmapName)]
    set logos {}
    set lst_sizes [ list 512 256 128 64 48 32 16]
    # if { ( $::tcl_platform(os) eq "Linux") && ( $::tcl_platform(pointerSize) == 8)} {
    #         set lst_sizes [ list 64 32 16]
    # }
    foreach size $lst_sizes {
        lappend logos [ gid_themes::GetImage ${rootname}${size}${extension}]
    }
    #On windows 7 takes size 32 in toolbar (works well)
    # but also takes size 32 for icon window (must be size 16) ( works wrong)
    wm iconphoto .gid -default {*}$logos
    
}

#must read themes before splash, because find images is needed
set err [ catch {
    package require gid_themes
} err_txt]
if { $err} {
    WarnWinDirect $err_txt
}

set err [ catch {
    gid_themes::ReadThemes
} err_txt]
if { $err} {
    WarnWinDirect $err_txt
}
if { $::GidPriv(ThemeSelection) != "1" } {
  if { [info exists ThemeConfig] } {
    unset ThemeConfig
    gid_themes::SetCurrentThemeAndApplyBackground $::GidPriv(ThemeSelection)
  } else {
    gid_themes::SetCurrentTheme $::GidPriv(ThemeSelection)
  }
} else {
    #this implies Splash and Logos from GiD_classic (easy to change)
    gid_themes::SetCurrentTheme GiD_classic
}

CreateIcon

# WarnWinDirect [ info library]
# WarnWinDirect -$auto_path-
if { ![ info exists OpenGLConfig] && ![ info exists ThemeConfig] } {
    if { [info exists GidPriv(SplashWindow)] && [IsSplashWindowOnInDefaults] } {
        GiDSplash
        GiDSplashMessage "Initializing $::GidPriv(ProgName,Default) ..."
    }
}

if { [info exists ::GIDDEFAULT] } {
    gid_themes::SetFontsDefault [winfo screenheight .gid]
} else {
    gid_themes::SetFontsDefault 1024
}

# until here splash
#to debug errors in gid_theme
#set err [ catch {
#    gid_themes::SetOptions
#} err_msg}]
#if { $err} {
#    WarnWinDirect "Error in gid_themes::SetOptions: \n$err_msg"
#}

#set tcl_interactive 0
set tk_strictMotif 0
# precision in floating point operations
set tcl_precision 17

package require BWidget
Widget::theme 0
package require dialogwin
package require tdom
package require gidscale

if { [ tk windowingsystem] eq "aqua"} {
    set ::acceleratorKey Command
    set ::acceleratorText Command
} else {
    set ::acceleratorKey Control
    set ::acceleratorText Ctrl
}


proc bindThemeChanged { } {

    #calculating apropiate color of style BottomFrame.TFrame
    set colordefaultframe [ttk::style lookup $ttk::currentTheme -background]                
    set framebuttoncolor $colordefaultframe        
    set valueerror ""
    if { [ info exists ::bindThemeChanged(inside)] && $::bindThemeChanged(inside) } {
        # WarnWinText "bindThemeChanged - already inside"
        return
    }
    set ::bindThemeChanged(inside) 1
    if [catch {                                        
        set n [ scan $colordefaultframe #%2x%2x%2x r g b]
        if { $n != 3} {
            set rgb [ winfo rgb . $colordefaultframe]        
            set r [ expr int( [ lindex $rgb 0]/256.0)]
            set g [ expr int( [ lindex $rgb 1]/256.0)]
            set b [ expr int( [ lindex $rgb 2]/256.0)]                                
        }
        set factor 17                        
        set r [ expr $r + $factor]
        if { $r > 255} { set r 255}
        set g [ expr $g + $factor]
        if { $g > 255} { set g 255}
        set b [ expr $b + $factor]
        if { $b > 255} { set b 255}                        
        set framebuttoncolor [ format #%02x%02x%02x $r $g $b]                                                
    } valueerror ] {
        #if error                                 
        set framebuttoncolor #fdfae9
    }
    #to let testcolors work only change color if background color is default
    if { [ttk::style configure BottomFrame.TFrame -background] == "#fdfae9" } {
        ttk::style configure "BottomFrame.TButton" -background $framebuttoncolor
        ttk::style map "BottomFrame.TButton" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.TFrame" -background $framebuttoncolor
        ttk::style map "BottomFrame.TFrame" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.TLabel" -background $framebuttoncolor
        ttk::style map "BottomFrame.TLabel" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.TEntry" -background $framebuttoncolor
        ttk::style map "BottomFrame.TEntry" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.TCheckbutton" -background $framebuttoncolor
        ttk::style map "BottomFrame.TCheckbutton" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.TCombobox" -background $framebuttoncolor
        ttk::style map "BottomFrame.TCombobox" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.TLabelframe" -background $framebuttoncolor
        ttk::style map "BottomFrame.TLabelframe" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.TMenubutton" -background $framebuttoncolor
        ttk::style map "BottomFrame.TMenubutton" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.TNotebook" -background $framebuttoncolor
        ttk::style map "BottomFrame.TNotebook" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.TPanedwindow" -background $framebuttoncolor
        ttk::style map "BottomFrame.TPanedwindow" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.Horizontal.TProgressbar" -background $framebuttoncolor
        ttk::style map "BottomFrame.Horizontal.TProgressbar" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.Vertical.TProgressbar" -background $framebuttoncolor
        ttk::style map "BottomFrame.Vertical.TProgressbar" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.TRadiobutton" -background $framebuttoncolor
        ttk::style map "BottomFrame.TRadiobutton" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.Horizontal.TScrollbar" -background $framebuttoncolor
        ttk::style map "BottomFrame.Horizontal.TScrollbar" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.Vertical.TScrollbar" -background $framebuttoncolor
        ttk::style map "BottomFrame.Vertical.TScrollbar" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.TSeparator" -background $framebuttoncolor
        ttk::style map "BottomFrame.TSeparator" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.TSizegrip" -background $framebuttoncolor
        ttk::style map "BottomFrame.TSizegrip" -background [list disabled $framebuttoncolor active $framebuttoncolor]
        ttk::style configure "BottomFrame.Treeview" -background $framebuttoncolor
        ttk::style map "BottomFrame.Treeview" -background [list disabled $framebuttoncolor active $framebuttoncolor]
    }

    # WarnWinText "[ ttk::style theme use] <----> [ gid_themes::GetCurrentTheme]"
    #CHANGE TK COLORS HERE

    set ::bindThemeChanged(inside) 0
} ;# bind themechanges

# after idle [list after 0 [list event generate . <<ThemeChanged>>]]


#eval font configure NormalFont [font configure TkTextFont] ;#usada en pruebas en DataWindows con tile??


# this is to avoid send commands
tk appname gid
#catch { rename send "" }

# wm protocol .gid WM_DELETE_WINDOW {GiD_Process Mescape Quit}
wm protocol .gid WM_DELETE_WINDOW QuitFullScreen
ChangeWindowTitle UNNAMED

#now only read one of both: .ini or default best file in \scripts
#and in this second case read alo the 'kernel' C++ interpreted part, calling readdefaults
set gid_defaults [GiveGidDefaultsFile];#gid.ini
if { ![file exists $gid_defaults] } {
    set gid_defaults [GiveBestGidDefaults];#get default file from scripts folder
}
ReadDefaultValues $gid_defaults;#tcl values 

# InitialOpenGLConfiguration
# if { ( $tcl_platform(platform) == "windows") || ( $::tcl_platform(os) == "Darwin") } {
if { ![ info exists ::GidPriv(SoftwareOpenGL)] } {
    set ::GidPriv(SoftwareOpenGL) 0
}

if { ![ info exists ThemeConfig] } {
    #apply apropiated colors to window
    gid_themes::SetCurrentTheme [gid_themes::GetCurrentTheme]
    # WarnWinText "reseting theme again"
}

# set default colours & co.
bind . <<ThemeChanged>> bindThemeChanged
after idle bindThemeChanged

if { [ info exists OpenGLConfig] || [ info exists ThemeConfig] } {            
    InitialConfiguration 1 [ info exists OpenGLConfig] [ info exists ThemeConfig]
    #careful theme background and color entities aplied after readdefaults
}


set err [ catch {
    gid_themes::SetOptions
} err_msg } ]
if { $err } {
    WarnWinDirect "Error in gid_themes::SetOptions: \n $err_msg"
}

# readdefaults $gid_defaults 0 ;#kernel values. 
#WARNING, its too early to read the kernel values, togle is not initialized, call 'readdefaults' later

if { ![ info exists MainWindowGeom]} {
    # something bad happened with the user's defaults
    ReadDefaultValues [ GiveBestGidDefaults]
    # then complement with user's defaults
    ReadDefaultValues $gid_defaults;#tcl values    
}




#set menus after read defaults, to set the appropiated language
SetAllMenusVariables
DWInitUserDataOptions

wm minsize .gid 1 1

#deprecated code:
#set aa "exec xprop -root | grep SERVER_OVERLAY_VISUAL | grep -v CUT_BUFFER"
# # unused (old) code
# if { $tcl_platform(os) != "SunOS" } {
#     set overlay_flag 1
# } else {
#     set overlay_flag 0
# }
# 
# if { $overlay_flag && ![catch { eval $aa } bb] } {
#     regexp {.*=[ ]*([^ ,]+).*} $bb trash GidPriv(SERVER_OVERLAY_VISUALS)
#     option add *OverlayWindowN*Visual $GidPriv(SERVER_OVERLAY_VISUALS)
# } else { set GidPriv(SERVER_OVERLAY_VISUALS) ""}

# set GidPriv(SERVER_OVERLAY_VISUALS) ""

#proc grab { args } { }


#option add *Menu*overlay true
ttk::frame .gid.phantom -class OverlayWindowN

if { [winfo cells .gid.phantom] < 6 } {
    option add *Menu*selectColor black
    option add *Menu*disabledForeground white
} else {
    option add *Menu*disabledForeground grey60
}

menu .gid.phantom.display3dbutton -tearoff no
Create3rdButtonMenu .gid.phantom.display3dbutton

menu .gid.display3dbutton_graph -tearoff no
Create3rdButtonMenuGraph .gid.display3dbutton_graph

if { $tcl_platform(platform) != "windows" } {
    wm colormapwindows .gid .gid.phantom
}

# update idletasks is needed so that gid's main window can be positioned in the second monitor.
wm withdraw .gid
update idletasks
WmGidGeom .gid $MainWindowGeom

#wm deiconify .gid
#wm withdraw .gid

if { [winfo exists .splash] } {
    wm deiconify .gid
    raise .splash .gid
}
###puts "update idletasks"
update idletasks
#wm withdraw .
###puts "frame .gid.central"

if { ![info exists GidPriv(MenuType)] } {
    if { ( $tcl_platform(platform) == "windows") || ( $tcl_platform(os) == "Darwin") } {
        set GidPriv(MenuType) "native"
    } else {
        set GidPriv(MenuType) "generic"
    }
} else {
    if { $GidPriv(MenuType) == "native" &&  ($tcl_platform(platform) != "windows" && $tcl_platform(os) != "Darwin") } {
        #native menus not available for this platform
        set GidPriv(MenuType) "generic"
    }
}

if { $GidPriv(MenuType) == "generic" } {
    ttk::frame .gid.topm
    set ::GiDTBLayout(.gid.topm) [list]
}

foreach _f_ {top left right rightb bottom bottomc} {
    set ::GiDTBLayout([ttk::frame .gid.$_f_]) [list]
}

ttk::frame .gid.bottominfo -style ForcedFrame
ttk::separator .gid.bottominfo.separator -orient horizontal -style ForcedSeparator
grid .gid.bottominfo.separator -in .gid.bottominfo -columnspan 100 -sticky "news" -pady {2 0}
grid columnconfigure .gid.bottominfo 0 -weight 1 




if { $tcl_platform(platform) != "windows" } {
    ttk::frame .gid.central
} else {
    ttk::frame .gid.central -style sunken.TFrame
}

if { [ info exists ::env(GLX_ERROR)]} {
    WarnWinDirect [_ "ERROR: GLX configuration is not valid.\nCheck if the graphics driver installation is correct.\nUsing software rendering mode!"]    
    set ::GidPriv(SoftwareOpenGL) 1
}


#name is the widget name, its parent must exists
#ident GID, sharelist must be 1 for more than one togls, else axes are shown wrongly
proc CreateNewToglGID { name sharelist } {
    set ident GID
    if { $::GidPriv(SoftwareOpenGL)} {
        set extra_flags [list -softrender true]
        set alphaFlag false
    } else {
        set extra_flags [list ]
        set alphaFlag true
    }
    lappend extra_flags -stereo true
    if { $sharelist } {
        lappend extra_flags -sharelist $ident
	# -sharecontext $ident
    }

    #-alpha true -> aunque sea lo correcto en windows con algunos drivers "no certificados" da problemas al hacer el c.fill
    set bpp [winfo screendepth .]
    if { ( $bpp == 24) ||  ( $bpp == 32)} {
        set rgbFlags [list -redsize 8 -greensize 8 -bluesize 8 -alphasize 8 -alpha $alphaFlag]   
    } elseif { $bpp == 16} {
        set rgbFlags [list -redsize 5 -greensize 6 -bluesize 5 -alphasize 8]
        # } elseif { $bpp == 15} {
        #     set rgbFlags [list -redsize 5 -greensize 5 -bluesize 5 -alphasize 8]
        # } elseif { $bpp == 8} {
        #     set rgbFlags [list -redsize 3 -greensize 3 -bluesize 2]
    } else {
        set rgbFlags ""
    }
    set depthBits [winfo depth .]
    lappend rgbFlags -depthsize $depthBits -stencil true -stencilsize 8
    # deprecated:  -overlay $overlay_flag
    #set rgbFlags [list -redsize 8 -greensize 8 -bluesize 8 -alphasize 8 -depthsize 24 -stencilsize 0]
    if { [catch { togl $name -rgba true -double true -depth true -ident $ident -privatecmap false {*}$rgbFlags {*}$extra_flags} err_msg] } {
        WarnWinDirect "Couldn't create a graphical window with '$rgbFlags' and '$extra_flags': $err_msg"
        puts "Couldn't create a graphical window with '$rgbFlags' and '$extra_flags': $err_msg"
    }
    return $name
}

CreateNewToglGID .gid.central.s 0

#ReadDefaultValues $gid_defaults;#tcl values
# read defaults from c kernel
readdefaults $gid_defaults 0 ;#kernel values

if { [ info exists ThemeConfig] } {
    #has been choosed theme
    gid_themes::SetCurrentThemeAndApplyBackground $::GidPriv(CurrentTheme)
}

if { ( $::tcl_platform(os) == "Linux") && \
         ( [gid_themes::GetCurrentTheme] == "GiD_classic") } {
    # GiD_classic too ugly
    ttk::style theme use clam
}

if { [.gid.central.s disable windows] } {
    wm withdraw .gid
} else {
    wm deiconify .gid
}

proc MouseWheelEventToButtonEvent { w increment } {
    if { $increment > 0 } {
	$w button 4 $increment 0
    } else {
	$w button 5 $increment 0
    }
}

proc Togl_Button-1 { W x y } {
    set ::DynamicEightDirections($W,LastPostition,x) $x
    set ::DynamicEightDirections($W,LastPostition,y) $y
    $W gidmakecurrent
    focus -force $W   
    set ::GidPriv(PressingButton1) 1
    $W button 1 $x $y
    after 300 [ list catch {  unset -nocomplain ::GidPriv(PressingButton1) } ]	
}

proc Togl_ButtonRelease-1 { W x y } {
    if { [info exists ::GidPriv(PressingButton1)] } {
	$W button -r -f 1 $x $y
	#break
    } else {
	$W button -r 1 $x $y       
    }
}

proc Togl_Motion { W x y } { 
    .gid.central.s motion $x $y
}

proc GetDynamicEightDirectionsPosition { W new_x new_y} {
    if { [ info exists ::DynamicEightDirections($W,LastPostition,x)] && 
	 [ info exists ::DynamicEightDirections($W,LastPostition,y)]} {
	set old_x $::DynamicEightDirections($W,LastPostition,x)
	set old_y $::DynamicEightDirections($W,LastPostition,y)
	set dif_x [ expr abs( $new_x - $old_x)]
	set dif_y [ expr abs( $new_y - $old_y)]
	# four directions: axis aligned
	# if { $dif_x > $dif_y} {
	#     # make y axis aligned
	#     set new_y $old_y
	# } else {
	#     # make x axis aligned
	#     set new_x $old_x
	# }
	# eight directions
	if { $dif_x > $dif_y} {
	    # make y axis aligned
	    if { [catch { set div [expr double($dif_y)/double($dif_x)] } err_txt] } {	    
		#WarnWinText "GetDynamicEightDirectionsPosition: $err_txt\ndoing double( $dif_y) / double( $dif_x)"
	    } else {
		if { $div < 0.4142135623731} {
		    set new_y $old_y
		} else {
		    # at 45 degrees
		    #scale bigger dif_x
		    set avg_dif [ expr 0.5 * ( $dif_x + $dif_y)]
		    set dif_y [ expr $new_y - $old_y]
		    if { $dif_y < 0} { 
			set dif_y [ expr -$avg_dif]
		    } else { 
			set dif_y $avg_dif
		    }
		    set new_y [ expr $old_y + $dif_y]
		    set dif_x [ expr $new_x - $old_x]
		    if { $dif_x < 0} {
			set dif_x [ expr -$avg_dif]
		    } else { 
			set dif_x $avg_dif
		    }
		    set new_x [ expr $old_x + $dif_x]
		}
	    }
	} else {
	    # make x axis aligned
	    if { [catch {set div [expr double($dif_x)/double($dif_y)]} err_txt] } {	    
		#WarnWinText "GetDynamicEightDirectionsPosition: $err_txt\ndoing double( $dif_x) / double( $dif_y)"
	    } else {
		if { $div < 0.4142135623731} {
		    set new_x $old_x
		} else {
		    # at 45 degrees
		    set avg_dif [ expr 0.5 * ( $dif_x + $dif_y)]
		    set dif_x [ expr $new_x - $old_x]
		    if { $dif_x < 0} {
			set dif_x [ expr -$avg_dif]
		    } else { 
			set dif_x $avg_dif 
		    }
		    set new_x [ expr $old_x + $dif_x]
		    set dif_y [ expr $new_y - $old_y]
		    if { $dif_y < 0} {
			set dif_y [ expr -$avg_dif]
		    } else { 
			set dif_y $avg_dif
		    }
		    set new_y [ expr $old_y + $dif_y]
		}
	    }
	}
    }
    return [ list $new_x $new_y]
}

proc Togl_Alt-Motion { W x y } {
    lassign [ GetDynamicEightDirectionsPosition $W $x $y] new_x new_y
    .gid.central.s motion $new_x $new_y
}

proc Togl_Alt-Button-1 { W x y } { 
    # W "Togl_Alt-Button-1 $W $x $y"
    lassign [ GetDynamicEightDirectionsPosition $W $x $y] new_x new_y
    Togl_Button-1 $W $new_x $new_y
}

proc Togl_Alt-ButtonRelease-1 { W x y } {
    # W "Togl_Alt-ButtonRelease-1 $W $x $y"
    lassign [ GetDynamicEightDirectionsPosition $W $x $y] new_x new_y
    Togl_ButtonRelease-1 $W $new_x $new_y
}


# %k = The keycode field from the event ( KeyPress & KeyRelease)
# %N = The keysym corresponding to the event, substituted as a decimal number  ( KeyPress & KeyRelease)
# %s = The state field from the event
proc Togl_KeyPress { k N s} {
    #in theory by the Tk manual until 122 are lower letters
    #but in my machine lowercase and uppercase provide the same k !!
    #and then 90 is the last letter
    #(if set 122 letters like F5 ->k=116 and is raised twice the command associated!!
    # above is incorrect:
    # --> %k is keycode ( code of key taste in keyboard, nor the ascii letter !!!
    # 123 = F12
    # 145 = ScrollLock
    if { $N >=32 && $N <=122 && $s==0 } {	
	#only do this if s is zero to avoid for example send the 'a' when pressing <Ctrl-a>
	if { [winfo exists .gid.comm.e] } {		    
	    focus -force .gid.comm.e
	    #event generate .gid.comm.e <KeyPress> -keysym $K -when tail
	    event generate .gid.comm.e <KeyPress> -keycode $k -when tail	    
	}
    }
}

proc Togl_Escape {} {    
    if { [winfo exists .gid.comm.e] } {
        .gid.comm.e delete 0 end
    }
    GiD_Process escape
}

proc ::WarnEvent { what w x y } {
    WarnWinText "$what - $w - $x $y"
    $what $w $x $y
}

bind .gid.central.s <1> {Togl_Button-1 %W %x %y}
bind .gid.central.s <ButtonRelease-1> {Togl_ButtonRelease-1 %W %x %y}
# %k = The keycode field from the event ( KeyPress & KeyRelease)
# %N = The keysym corresponding to the event, substituted as a decimal number  ( KeyPress & KeyRelease)
# %s = The state field from the event
bind .gid.central.s <KeyPress> {Togl_KeyPress %k %N %s}

set ::gid_central_button 2
set ::gid_right_button 3

if { $::tcl_platform(os) == "Darwin" } {
    # http://wiki.tcl.tk/12987 FAQ 7.7:
    # For historical reasons, MacMice buttons 2 and 3 refer to the right and middle buttons
    # respectively, which is indeed the opposite way round from Windows and *nix systems.
    set ::gid_central_button 3
    set ::gid_right_button 2
}


bind .gid.central.s <$::gid_central_button> { .gid.central.s button 2 %x %y }
##### to handle mouse wheel
 
catch {
    bind .gid.central.s <4> { .gid.central.s button 4 %x %y }
    bind .gid.central.s <5> { .gid.central.s button 5 %x %y }
}

bind .gid.central.s <MouseWheel> [list MouseWheelEventToButtonEvent .gid.central.s %D]   

bind .gid.central.s <Motion> { Togl_Motion %W %x %y }
bind .gid.central.s <Alt-Motion> { Togl_Alt-Motion %W %x %y }
bind .gid.central.s <Alt-1> { Togl_Alt-Button-1 %W %x %y}   
bind .gid.central.s <Alt-ButtonRelease-1> {Togl_Alt-ButtonRelease-1 %W %x %y}

bind .gid.central.s <Leave> { .gid.central.s leave %x %y }
bind .gid.central.s <Enter> { .gid.central.s enter %x %y }
bind .gid <Unmap> {UnMapAllWindows %W}
bind .gid <Map> {MapAllWindows %W}
bind .gid <${::acceleratorKey}-a> SelectJoinOrNoJoin
bind .gid <${::acceleratorKey}-A> SelectJoinOrNoJoin
#bind .gid <${::acceleratorKey}-h> SelectionOnlyVisibleOrAll

bind .gid <${::acceleratorKey}-Key-1> {GiD_Process 'Rotate Angle 270 90}
bind .gid <${::acceleratorKey}-Key-2> {GiD_Process 'Rotate Angle 270 0}
bind .gid <${::acceleratorKey}-Key-3> {GiD_Process 'Rotate Angle 0 0}
bind .gid <${::acceleratorKey}-Key-4> {GiD_Process 'Rotate Angle 45 45}
bind .gid <${::acceleratorKey}-t> {focus .gid.comm.e}

bind .gid <Escape> Togl_Escape


set mxbutwidth 5 ; set PostButStack ""

proc GiDShowContextualMenu { x y } {    
    if { [GiD_Info project ViewMode] == "GRAPHUSE" } {
        GiD_PopupMenu .gid.display3dbutton_graph $x $y
    } else {
        GiD_PopupMenu .gid.phantom.display3dbutton $x $y
    }
}

bind .gid.central.s <$::gid_right_button> {
    #%W gidmakecurrent
    set x [expr [winfo rootx .gid.central.s]+%x+2]
    set y [expr [winfo rooty .gid.central.s]+%y]
    
    GiDShowContextualMenu $x $y    
}

#bind .gid.central.s <${::acceleratorKey}-ButtonRelease-1> "ContextualEntity::Activate %W %x %y; break"
bind .gid.central.s <${::acceleratorKey}-ButtonPress-1> "ContextualEntity::Activate %W %x %y; break"


bind Listbox <4> {
    %W yview scroll -3 units
    break
}

bind Listbox <5> {
    %W yview scroll 3 units
    break
}

bind Text <4> {
    %W yview scroll -3 units
    break
}

bind Text <5> {
    %W yview scroll 3 units
    break
}

# top
if { $GidPriv(MenuType) == "native" } {
    #reserve the row 0 to handle swapping at runtime between native and generic like menus
    set row 1
} else {
    grid .gid.topm -row 0 -column 0 -columnspan 4 -sticky "snew"
    grid columnconfigure .gid.topm 0 -weight 1
    set row 1
}
grid .gid.top -row $row -column 0 -columnspan 4 -sticky "snew"
grid columnconfigure .gid.top 0 -weight 1

incr row

set crow $row
# left
grid .gid.left -row $row -column 0 -sticky "ns"
grid rowconfigure .gid.left 0 -weight 1
# central
grid .gid.central -row $row -column 1 -sticky "snew"
grid .gid.central.s -sticky "snew"
grid rowconfigure .gid.central 0 -weight 1
grid columnconfigure .gid.central 0 -weight 1
# right toolbars
grid .gid.right -row $row -column 2 -sticky "ns"
grid rowconfigure .gid.right 0 -weight 1
# right buttons
grid .gid.rightb -row $row -column 3 -sticky "ns"
grid rowconfigure .gid.rightb 0 -weight 1
incr row
# bottom
grid .gid.bottom -row $row -column 0 -columnspan 4 -sticky "snew"
grid columnconfigure .gid.bottom 0 -weight 1
incr row
grid .gid.bottomc -row $row -column 0 -columnspan 4 -sticky "snew"
grid columnconfigure .gid.bottomc 0 -weight 1
#lowerbottom
incr row
grid .gid.bottominfo -row $row -column 0 -columnspan 4 -sticky "snew"


grid rowconfigure .gid $crow -weight 1
grid columnconfigure .gid 1 -weight 1

grid remove .gid.bottominfo
#configuration is stored, real grid on proc BottomStatusFrame

set WarnLineVar ""
set ActionsMenuVar ""
set ActionsMenuVarType "INTERP"
set EntryVar ""
set EntryVarHilit ""

#test to speech texts using Microsoft Windows Speech com object
set SPEECH_WARNLINE 0
if { $SPEECH_WARNLINE == 1 && $::tcl_platform(platform) == "windows" } {
    set ::GiD_Voice ""
    catch {
        package require tcom
        set ::GiD_Voice [::tcom::ref createobject Sapi.SpVoice]
    }
} else {
    set ::GiD_Voice ""
}

proc ChangeWarnLineVar { val } {
    #with after idle some messages could be shown unordered, but without after idle someting failed in the past!
    #after idle [list ChangeWarnLineVar_after $val]
    ChangeWarnLineVar_after $val
}

proc ChangeWarnLineVar_after { val } {       
    update idletasks
    if { ![winfo exists .gid.comm.fwarn.list] } {
        return
    }
    global WarnLineVar uparrow vecToUseInVectors
    
    if { [string index $val 0] == "\r" } {
        set WarnLineVar "[.gid.comm.fwarn.list get end] [string range $val 1 end]"
        .gid.comm.fwarn.list delete end
    } else {
        set WarnLineVar $val
    }    
    .gid.comm.fwarn.list insert end $WarnLineVar   


    if { $::GiD_Voice != "" } {
        $::GiD_Voice Speak $val 1
    }
    set uparrow [.gid.comm.fwarn.list size]
    .gid.comm.fwarn.list yview [expr $uparrow-2]
    if { [info exists vecToUseInVectors] } { 
        GetVarVecs $val 
    }    
}

proc procTraceVar { name elm op } {
    global uparrow vecToUseInVectors
    upvar $name x
    
    if { $name ==  "ActionsMenuVar" } {
        SetButtons
    }
    if { $name ==  "WarnLineVar" } {
        .gid.comm.fwarn.list insert end $x
        set uparrow [.gid.comm.fwarn.list size]
        .gid.comm.fwarn.list yview [expr $uparrow-2]
        if { [info exists vecToUseInVectors] } { 
            GetVarVecs $x 
        }
    }
    if { $name ==  "EntryVar"} { .gid.comm.e delete 0 end ; .gid.comm.e insert 0 $x }
    if { $name ==  "EntryVarHilit" } { .gid.comm.e select range 0 end }
}
trace variable ActionsMenuVar w procTraceVar
trace variable EntryVar w procTraceVar
trace variable EntryVarHilit w procTraceVar

if { [GiD_Info Project Debug] == "debug" } {

    bind .gid <${::acceleratorKey}-x><${::acceleratorKey}-d> DebugMesherPost
    bind .gid <${::acceleratorKey}-x><${::acceleratorKey}-l> { auto_reset }

}
bind all <Triple-Button-1> {grab release [grab current .gid] }

proc register_dnd_event { win_dnd } {
    # OLE commands in windows
    if { [info commands olednd] != "" } {
	#olednd is a deprecated internal GiD command, removed from 7.5.10b version
	olednd bindtarget $win_dnd FileName <Drop> "LoadFileInGid"
    } else {
	#if {$::tcl_platform(platform) eq "windows"} {
	catch { package require tkdnd 2.6 } ;# only if it is compiled
	if { [info commands dnd] != "" } {
	    proc DropMultipleFiles { files } {
		foreach i $files {
		    LoadFileInGid $i
		}
	    }
	    ##dnd bindtarget .gid FileName <Drop> "LoadFileInGid %D"
	    #dnd bindtarget .gid text/uri-list <Drop> [ list after idle "DropMultipleFiles %D"]
	    #updated syntax tkdnd 2.6 (but deprecated syntax also works, at least on Windows)
	    tkdnd::drop_target register $win_dnd DND_Files
	    bind $win_dnd <<Drop:DND_Files>> [list after idle "DropMultipleFiles %D"]
	    bind $win_dnd <<Drop>> "WarnWinText dnd-%D"
	}
	#}
    }
}

if { $::tcl_platform(os) == "Darwin"} {
    register_dnd_event .gid.central.s
} elseif { $::tcl_platform(os) == "Linux"} {
    register_dnd_event .gid.central
} else {
    #if {$::tcl_platform(platform) eq "windows"} {
    catch { package require tkdnd } ;# only if it is compiled
    if { [info commands dnd] != "" } {
        proc DropMultipleFiles { files } {
            foreach i $files {
                LoadFileInGid $i
            }
        }
        #dnd bindtarget .gid FileName <Drop> "LoadFileInGid %D"
        dnd bindtarget .gid text/uri-list <Drop> [ list after idle "DropMultipleFiles %D"]
    }
    #}
}

set ramdebuggerinside 1 ;#1 if is included with GiD, all packages are inside scripts
if { $ramdebuggerinside } {
    if { [catch {
        package require commR ;#modification (commR) of tcllib comm package
        comm::register gid 1    
    } err_txt] } {    
        set GidPriv(RamdebuggerMenuState) 0 ;#to hide it in menu
        #bgerror "ramdebugger not available: $err_txt"
    } else {
        bind all <F12> {
            package require RamDebugger           
        }
    }
} else {
    #donde esta RamDebuger y tcl?
    if { $tcl_platform(platform) == "windows" } {
        lappend auto_path "E:/Programas/ActiveTcl8.4.9.0/lib"
        set where_RamDebugger "C:/gid project/scripts/ramdebugger"
        lappend auto_path ~/mytcltk
        set where_ActiveTcl_lib {E:/Programas/ActiveTcl8.4.9.0/lib/tcl8.4}
    } else {
        set where_RamDebugger "/home/miguel/GID/stuff/RamDebugger"
        set where_ActiveTcl_lib ""
    }

    
    # enter here the path where you have installed RamDebugger, subdirectory addons
    # for example: c:\Program files\RamDebugger\addons
    lappend auto_path [file join $where_RamDebugger addons]
    package require commR ;#modification (commR) of tcllib package
    comm::register gid 1
    
    bind all <F12> {
        # enter here the path where you have installed RamDebugger
        # for example: c:\Program files\RamDebugger
        lappend auto_path $where_RamDebugger
        if  { $where_ActiveTcl_lib != ""} {
            lappend auto_path $where_ActiveTcl_lib
            lappend auto_path [file join $where_ActiveTcl_lib tcl8.4]
        }
        package require RamDebugger
    }
}


#-----------------------------------------------Debug funcs---------------------------------

proc SetBindingsForMouse { w ibutton cmd x y } {    
    GiD_Process {*}$cmd
    $w button 1 $x $y
    set ::GidPriv(OldButtonRelease-$ibutton) [bind $w <ButtonRelease-$ibutton>]
    bind $w <ButtonRelease-$ibutton> "%W button -r 1 %x %y ;\
            GiD_Process escape ;\
            bind %W <ButtonRelease-$ibutton> \$::GidPriv(OldButtonRelease-$ibutton) ;\
            break"
}

proc SetButtonMeaning { w } {   
    global GidPriv
    if { $GidPriv(ChangeButtonMeaning) == "0" } {
        foreach "mod cmd" [list Alt "'DynamicPan" Shift "'Zoom Dynamic" Alt-Shift "'Rotate Trackball"] {
            bind $w <$mod-ButtonPress-1> "[list SetBindingsForMouse $w 1 $cmd %x %y] ; break"
        }
    } else {

        set ::gid_central_button 2
        set ::gid_right_button 3
        
        if { $::tcl_platform(os) == "Darwin" } {
            # http://wiki.tcl.tk/12987 FAQ 7.7:
            # For historical reasons, MacMice buttons 2 and 3 refer to the right and middle buttons
            # respectively, which is indeed the opposite way round from Windows and *nix systems.
            set ::gid_central_button 3
            set ::gid_right_button 2
        }

        #new bindings
        bind $w <Shift-1> "[list SetBindingsForMouse $w 1 {'Rotate Trackball} %x %y] ; break"
        bind $w <Shift-$::gid_central_button> "[list SetBindingsForMouse $w 2 {'Zoom Dynamic} %x %y] ; break"
        bind $w <Shift-$::gid_right_button> "[list SetBindingsForMouse $w 3 'DynamicPan %x %y] ; break"        
    }
}

SetButtonMeaning .gid.central.s

#LoadGidAddons must be called after load Tk, to modify some bindings, etc
LoadGidAddons

after idle ChangeTopMenusAndWindows

# to load problemtype icons ... needed in linux for the file browser
after idle [ list FastCacheFromProblemtypeMenuDir [ file join $::GIDDEFAULT problemtypes]]

proc restoreOriginalPackage {} {
    # rename it back
    set ::GidPriv(ShowPackages) 0
    if { [ info proc MyPackage] == ""} {
	rename package MyPackage
	rename packageOriginal package
    }
}

if { $::GidPriv(ShowPackages) } {
    # still problem with autmatic loading of models
    after idle restoreOriginalPackage
}

# to avoid printing anything
return
