# Visualization Toolkit (VTK) Tcl package configuration.

proc VtkWriteDebug { txt} {
    catch {
	if { ![ info exists VtkWriteDebugFirstTime]} {
	    set VtkWriteDebugFirstTime 1
	    #delete previous contents
	    set fo [ open /tmp/vtk_dbg.txt a]
	} else {
	    set fo [ open /tmp/vtk_dbg.txt a]
	}
	puts $fo $txt
	close $fo
    }
}

proc VtkPrintError { txt} {
    if { [ catch "puts $txt"]} {
	catch {
	    tk_messageBox -icon error -message $txt
	}
    }
}

package ifneeded vtkinit {5.8} {
    namespace eval ::vtk::init {
	proc load_library_package {libName libPath {libPrefix {lib}} {version {5.8}}} {
	    set libExt [info sharedlibextension]
	    set currentDirectory [pwd]
	    if { $::tcl_platform(platform) == "windows" } {
		set libPrefix ""	 
	    } else {
		append libExt .$version
	    }
	    set libFile [file join $libPath "$libPrefix$libName$libExt"]
	    
	    # ::VtkWriteDebug "currentDirectory = $currentDirectory\nlibPath = $libPath\nloading $libFile"
	    if {[catch "cd {$libPath}; load {$libFile}" errorMessage]} {
		::VtkPrintError "load_library_package $libName: $errorMessage"
	    }
	    cd $currentDirectory
	}
	proc require_package {name {version {5.8}}} {
	    if {[catch "package require -exact $name $version" errorMessage]} {
		# puts $errorMessage
		::VtkPrintError "require_package $name $version: $errorMessage"
		return 0
	    } else {
		return 1
	    }
	}
	set version {5.8}
	set kits {}
	foreach kit { base Common Filtering IO Imaging Graphics
	    
	    
	    
	} {
	    lappend kits [string tolower "${kit}"]
	}
    }
    package provide vtkinit {5.8}
}

foreach kit { Common Filtering IO Imaging Graphics
    
    
    
} {
    package ifneeded "vtk${kit}TCL" {5.8} "
    package require -exact vtkinit {5.8}
    ::vtk::init::load_library_package {vtk${kit}TCL} {[file dirname [info script]]}
  "

    package ifneeded "vtk[string tolower ${kit}]" {5.8} "
    package require -exact vtkinit {5.8}
    if {\[catch {source \[file join {[file dirname [info script]]/tcl} {vtk[string tolower ${kit}]} {vtk[string tolower ${kit}].tcl}\]} errorMessage\]} {
      ::VtkPrintError \$errorMessage
    }
  "
}

foreach src {vtk vtkbase vtkinteraction vtktesting} {
    package ifneeded ${src} {5.8} "
    package require -exact vtkinit {5.8}
    if {\[catch {source \[file join {[file dirname [info script]]/tcl} {$src} {$src.tcl}\]} errorMessage\]} {
      ::VtkPrintError \$errorMessage
    }
  "
}
