namespace eval ::xmlprograms {

   #after 3000 GiD_PGF_Fonts get

    proc FormatG { var } {
      return [format %g $var]
    }   

    proc TclColour { var } {
      set var {*}$var
      if { $var == "0" } {
        set var "#ff0000"
      } else {
        set rgb [split $var #]
        if { [llength $rgb] == "3" } {      
          #accepting color format 22#200#100
          set var [::CreateWidgetsFromXml::GiDColorToTkColor $var]
        }
      }
      return $var
    }

    proc FromCosToDegree { var } {
      if { [expr $var>1.0] || [expr -1.0>$var] } {
        WarnWinText "$var not inside -1 to 1, imposible to calculate cosinus"
      }
      return [format %g [expr 57.295779513082323*acos($var)]]
    }

    proc FromDegreeToCos { var } {
      return [format %g [expr cos($var*0.017453292519943295)]]
    }


    proc FontFromGiDToTcl { fontgid } {
        # GiD give 'PGF_DefaultFont'={{C:\Windows\Fonts\cour.ttf} 0 14 {{Courier New} {Regular} {Fixed width}}}
        # On preprocess variable its transform to: Input format {{Courier New (Fixed width)} {Regular} 14}
        lassign [lindex $fontgid 0] fontgid_name fontgid_type_id fontgid_size fontgid_types
        lassign $fontgid_types font_name font_type font_size
        if { $font_size == {Fixed width} } {
            set font_name "$font_name (Fixed width)"           
        }
        return [list $font_name $font_type $fontgid_size]
    }

    proc FontFromTclToGiD { fonttcl } {
        # tcl format {{Courier New} {Regular} 14}
        #'On postrocess variable its transform to: {C:\Windows\Fonts\cour.ttf} 0 14
        lassign [lindex $fonttcl 0] font_name font_type font_size
        set font_key [list $font_name $font_type]
        if { [info exists ::PGFFonts_new($font_key)] } {
            lassign $::PGFFonts_new($font_key) fontgid_name fontgid_type_id            
        } else {
            #trick to try to found if exists the same font name but not exactly for this type (e.g exists Magneto Bold but not Magneto Regular)
            set alternative_font_key [lindex [array names ::PGFFonts_new  [list $font_name *]] 0]
            if { $alternative_font_key != "" } {
                lassign $::PGFFonts_new($alternative_font_key) fontgid_name fontgid_type_id        
            } else {                
                error "Font '$fonttcl' not found"                
                set fontgid_name ""
                set fontgid_type_id ""
            }
        }
        return [list $fontgid_name $fontgid_type_id $font_size]
    }

    proc GetFonts {} {
      set lst_ttf_fonts [ GiD_PGF_Fonts get]
      set lst_valores_combo ""
      foreach ifont $lst_ttf_fonts {
        if { [lindex [ lindex $ifont 2] 2] == "Fixed width" } {        
          set txt_combo [list "[lindex [ lindex $ifont 2] 0] (Fixed width)" [list [lindex [ lindex $ifont 2] 1]]]    
        } else {
          set txt_combo [list [lindex [ lindex $ifont 2] 0] [list [lindex [ lindex $ifont 2] 1]]]  
        }       
        lappend lst_valores_combo [lindex $txt_combo 0]
        set ::PGFFonts_new($txt_combo) [ lrange $ifont 0 1]        
      }
      set lst_valores_combo [lsort -unique $lst_valores_combo]
      return [list $lst_valores_combo]
    }

    proc RebuildFontList { baseframe } {
        set w {}
        foreach w [ winfo children $baseframe] {
            if { [ winfo class $w] == "TButton"} {
                break
            }
        }
        GidUtils::WaitState $baseframe
        if { $w != {}} {
            set old_label [ $w cget -text]
            $w configure -text [_ "Rebuilding font list ..."]
            update
        }
        GiD_PGF_Fonts resetfontlist
        if { $w != {}} {
            $w configure -text $old_label
        }
        GidUtils::EndWaitState $baseframe
        # needs to be done the cache of tcl fonts, which can be appened to
        # TODO
        # GiD_PGF_Fonts fontlistfilename
        # need to refill fonts comboboxed:
        # GetFonts
        # GetPmfonts
        InfoWin [_ "Font list rebuild. \nNew fonts will appear the next time the preferences window is opened."] $baseframe
    }
    proc FontFromPmGiDToTcl { fontgid } {
      set fonttcl [lindex [lindex {*}$fontgid 3] 0]
      return $fonttcl
    }
    proc FontFromTclToPmGiD { fonttcl } {
      return $fonttcl
    } 
    proc GetPmfonts {} {
      set fonts [ GiD_PGF_Fonts get -pmfont]
      return [list $fonts $fonts ]
    }
    proc GetLanguages {} {
      set langs [::GidUtils::GetAllLanguages]
      #reorder langs by translated strings, not by langs
      set langsandlabels [list]
      foreach i $langs {
        lappend langsandlabels [list [_ "%s" [::GIDi18n::language $i]] $i]
      }
      set langsandlabels [lsort -index 0 $langsandlabels]
      set langlabels ""
      set langs ""
      foreach item $langsandlabels {
        lappend langlabels [lindex $item 0]
        lappend langs [lindex $item 1]
      }
      return [list $langs $langlabels ]
    }
    proc GetThemesAvailable {} {
        set names [gid_themes::GetThemes]
        set translatedthemenames ""
        foreach name $names {
            #translations applied to all themes
            switch $name {
                GiD_classic {
                    lappend translatedthemenames [_ "Classic GiD Theme"]
                }
                GiD_black {
                    lappend translatedthemenames [_ "Dark GiD Theme"]
                }
                default {
                    lappend translatedthemenames [_ "Theme %s" $name]
                }
            }
        }
        return [list $names $translatedthemenames]
    }
    proc GetThemeSizesCurrent {} {      
        set tranlatedsizes ""
        foreach size $::gid_themes::Themes([gid_themes::GetCurrentTheme],ThemeSizes) {
            #translations applied to all themes
            switch $size {
                -2 {
                    lappend tranlatedsizes [_ "Super small"]
                }
                -1 {
                    lappend tranlatedsizes [_ "Small"]
                }
                0 {
                    lappend tranlatedsizes [_ "Medium"]
                }
                +1 -
                1 {
                    lappend tranlatedsizes [_ "Large"]
                }
                +2 -
                2 {
                    lappend tranlatedsizes [_ "Super large"]
                }
                default {
                    lappend tranlatedsizes [_ "Size %s" $size]
                }
            }
        }
        return [list $::gid_themes::Themes([gid_themes::GetCurrentTheme],ThemeSizes) $tranlatedsizes]
    }

    proc OpenBrowserForBatch { baseframe variable} {      
      set types [list \
        [list [_ "Batch file"] [list .bch]] \
        [list [_ "All files"] .*]]
      set $variable [Browser-ramR file write .gid [_ "write batch file"] \
      [set $variable] $types .bch 0]      
      return variable
    }
       
    proc RedrawBoundaryMesh {baseframe validpath} {
      canvas $validpath.canvas#1 -height 7 -width 300
      $validpath.canvas#1 create line 2 5 250 5 -fill red -width 2
      $validpath.canvas#1 create line 255 5 260 5 -fill red -width 2
      $validpath.canvas#1 create line 265 5 270 5 -fill red -width 2
      $validpath.canvas#1 create line 275 5 277 5 -fill red -width 2
      $validpath.canvas#1 create line 284 5 286 5 -fill red -width 2
      $validpath.canvas#1 create line 2 0 2 8 -fill black -width 1
      grid $validpath.canvas#1 -in $baseframe -sticky w
      
      #RedrawDivisions_blm $canvas
    }
    
    proc RedrawDivisions_blm { baseframe validpath BoundaryLayerGrowLaw BoundaryLayerGrowFactor } {
      set canvas $validpath.canvas#1
      if { ![winfo exists $canvas] } {
        return
      }      
      $canvas delete temp
      set maxwidth 300

      #First distance = 4 pixel
      set DistanceFirst 4

      set layerX [expr 2+$DistanceFirst]
      switch $BoundaryLayerGrowLaw {
      0 {
        #geometric
        set i 1
        while { $maxwidth > $layerX } {
          $canvas create line $layerX 0 $layerX 8 -fill black -width 1  -tags temp
          if { $BoundaryLayerGrowFactor == 1 } {
            set layerX [expr 2+round($DistanceFirst*($i+1)*$BoundaryLayerGrowFactor)]
          } else {
            set layerX [expr 2+round($DistanceFirst*(1-pow($BoundaryLayerGrowFactor,$i+1))/(1-$BoundaryLayerGrowFactor))]
          }
          incr i
        }
      }
      1 {
        #exponential
        set i 1
        while { $maxwidth > $layerX } {
          $canvas create line $layerX 0 $layerX 8 -fill black -width 1  -tags temp
          set layerX [expr 2+round(exp(log($DistanceFirst)+($BoundaryLayerGrowFactor*$i)))]
          incr i
        }
      }
      2 {
        #geometric_mod
        set i 1
        while { $maxwidth > $layerX } {
          $canvas create line $layerX 0 $layerX 8 -fill black -width 1  -tags temp
          set layerX [expr 2+round($DistanceFirst*(1+$i*(1+$BoundaryLayerGrowFactor*(1+$BoundaryLayerGrowFactor)*$i)))]
          incr i
        }
      }
      }
    }
    #---------not used -----------
    proc DrawNormalFont  {baseframe validpath} {
      tk::label $validpath.normalfontlabel -text "Normal font"
      grid $validpath.normalfontlabel -in $baseframe -sticky ew
    }    
    proc DrawCommentsLegendFont  {baseframe validpath} {
      tk::label $validpath.commentsfontlabel -text "Comments / Legend font"
      grid $validpath.commentsfontlabel -in $baseframe -sticky ew
    }
    proc GetSize {} {
      #return a list of 2 list {listvalues listlabels}
      return { {10 11 12} {10 11 12} }
    }
    proc FontFromTclToGiD_asian { fonttcl } {
      # tcl format {{Courier New} {Regular} 14}
      #'On postrocess variable its transform to: {C:\Windows\Fonts\cour.ttf} 0 14
      return [list {*}$::PGFFonts_asian([list [lindex {*}$fonttcl 0] [list [lindex {*}$fonttcl 1]]]) [lindex {*}$fonttcl 2]]      
    }
    proc GetAsianfonts {} {
      set lst_fnts [ GiD_Fonts get AsianFonts]      
      set lst_valores_combo ""
      foreach ifont $lst_fnts {
        if { [lindex [ lindex $ifont 2] 2] == "Fixed width" } {        
          set txt_combo [list "[lindex [ lindex $ifont 2] 0] (Fixed width)" [list [lindex [ lindex $ifont 2] 1]]]    
        } else {
          set txt_combo [list [lindex [ lindex $ifont 2] 0] [list [lindex [ lindex $ifont 2] 1]]]  
        }       
        lappend lst_valores_combo [lindex $txt_combo 0]
        set ::PGFFonts_asian($txt_combo) [ lrange $ifont 0 1]        
      }
      set lst_valores_combo [lsort -unique $lst_valores_combo]
      return [list $lst_valores_combo]
    }
}